#line 1 "./TensorMeson2PScalarDecayer.cc"
// -*- C++ -*-
//
// TensorMeson2PScalarDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the TensorMeson2PScalarDecayer class.
//

#include "TensorMeson2PScalarDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/TensorWaveFunction.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/PDT/DecayMode.h"
#include "Herwig/Decay/TwoBodyDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void TensorMeson2PScalarDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    for(unsigned int ix=0;ix<_incoming.size();++ix)
      if(mode(ix)) _maxweight[ix] = mode(ix)->maxWeight();
  }
}

TensorMeson2PScalarDecayer::TensorMeson2PScalarDecayer() 
  : _incoming(48), _outgoing1(48), _outgoing2(48), 
    _coupling(48), _maxweight(48) {
  // a_2 -> eta pi
  _incoming[0] = 115; _outgoing1[0] =  221; _outgoing2[0] = 111; 
  _coupling[0] = 10.90/GeV; _maxweight[0] = 1.7; 
  _incoming[1] = 215; _outgoing1[1] =  221; _outgoing2[1] = 211; 
  _coupling[1] = 10.90/GeV; _maxweight[1] = 1.7; 
  // a_2 -> eta' pi
  _incoming[2] = 115; _outgoing1[2] =  331; _outgoing2[2] = 111; 
  _coupling[2] = 9.92/GeV; _maxweight[2] = 1.9; 
  _incoming[3] = 215; _outgoing1[3] =  331; _outgoing2[3] = 211; 
  _coupling[3] = 9.92/GeV; _maxweight[3] = 1.9; 
  // a_2 -> K K
  _incoming[4] = 115; _outgoing1[4] =  311; _outgoing2[4] = -311; 
  _coupling[4] = 7.36/GeV; _maxweight[4] = 1.7; 
  _incoming[5] = 115; _outgoing1[5] =  321; _outgoing2[5] = -321; 
  _coupling[5] = 7.36/GeV; _maxweight[5] = 1.7; 
  _incoming[6] = 215; _outgoing1[6] =  321; _outgoing2[6] = -311; 
  _coupling[6] = 10.41/GeV; _maxweight[6] = 1.7; 
  // f_2 -> pi pi
  _incoming[7] = 225; _outgoing1[7] =  211; _outgoing2[7] = -211; 
  _coupling[7] = 18.73/GeV; _maxweight[7] = 1.7; 
  _incoming[8] = 225; _outgoing1[8] =  111; _outgoing2[8] = 111; 
  _coupling[8] = 13.24/GeV; _maxweight[8] = 1.7; 
  // f_2 -> eta eta
  _incoming[9] = 225; _outgoing1[9] =  221; _outgoing2[9] = 221; 
  _coupling[9] = 8.362/GeV; _maxweight[9] = 1.8; 
  // f_2 -> KK
  _incoming[10] = 225; _outgoing1[10] =  321; _outgoing2[10] = -321; 
  _coupling[10] = 11.03/GeV; _maxweight[10] = 2.; 
  _incoming[11] = 225; _outgoing1[11] =  311; _outgoing2[11] = -311; 
  _coupling[11] = 11.38/GeV; _maxweight[11] = 2.; 
  // f'_2 -> KK
  _incoming[12] = 335; _outgoing1[12] =  321; _outgoing2[12] = -321; 
  _coupling[12] = 14.65/GeV; _maxweight[12] = 1.65; 
  _incoming[13] = 335; _outgoing1[13] =  311; _outgoing2[13] = -311; 
  _coupling[13] = 14.65/GeV; _maxweight[13] = 1.65; 
  // f'_2 -> eta eta
  _incoming[14] = 335; _outgoing1[14] =  221; _outgoing2[14] = 221; 
  _coupling[14] = 9.15/GeV; _maxweight[14] = 1.7; 
  // f'_2 -> pi pi 
  _incoming[15] = 335; _outgoing1[15] =  211; _outgoing2[15] = -211; 
  _coupling[15] = 0.860/GeV; _maxweight[15] = 1.7; 
  _incoming[16] = 335; _outgoing1[16] =  111; _outgoing2[16] = 111; 
  _coupling[16] = 0.608/GeV; _maxweight[16] = 1.7; 
  // K_2 -> K eta
  _incoming[17] =  325; _outgoing1[17] =  321; _outgoing2[17] = 221; 
  _coupling[17] = 1.52/GeV; _maxweight[17] = 1.8; 
  _incoming[18] =  315; _outgoing1[18] =  311; _outgoing2[18] = 221; 
  _coupling[18] = 1.52/GeV; _maxweight[18] = 1.8; 
  // K_2 -> K pi
  _incoming[19] =  325; _outgoing1[19] =  321; _outgoing2[19] =  111; 
  _coupling[19] = 8.30/GeV;_maxweight[19] = 1.65; 
  _incoming[20] =  325; _outgoing1[20] =  311; _outgoing2[20] =  211; 
  _coupling[20] = 11.74/GeV; _maxweight[20] = 1.65; 
  _incoming[21] =  315; _outgoing1[21] =  311; _outgoing2[21] =  111; 
  _coupling[21] = 8.68/GeV; _maxweight[21] = 1.65; 
  _incoming[22] =  315; _outgoing1[22] =  321; _outgoing2[22] = -211; 
  _coupling[22] = 12.28/GeV; _maxweight[22] = 1.65; 
  // B_2 -> B pi
  _incoming[23] =  525; _outgoing1[23] =  521; _outgoing2[23] = 111; 
  _coupling[23] = 27.23/GeV; _maxweight[23] = 1.65; 
  _incoming[24] =  525; _outgoing1[24] =  511; _outgoing2[24] = 211; 
  _coupling[24] = 38.52/GeV; _maxweight[24] = 1.65; 
  _incoming[25] =  515; _outgoing1[25] =  511; _outgoing2[25] = 111; 
  _coupling[25] = 27.16/GeV; _maxweight[25] = 1.65; 
  _incoming[26] =  515; _outgoing1[26] =  521; _outgoing2[26] = -211; 
  _coupling[26] = 38.62/GeV; _maxweight[26] = 1.65; 
  // D_2 -> D pi
  _incoming[27] =  425; _outgoing1[27] =  421; _outgoing2[27] = 111; 
  _coupling[27] = 18.07/GeV; _maxweight[27] = 1.7; 
  _incoming[28] =  425; _outgoing1[28] =  411; _outgoing2[28] = -211; 
  _coupling[28] = 25.56/GeV; _maxweight[28] = 1.7; 
  _incoming[29] =  415; _outgoing1[29] =  411; _outgoing2[29] = 111; 
  _coupling[29] = 14.91/GeV; _maxweight[29] = 1.7; 
  _incoming[30] =  415; _outgoing1[30] =  421; _outgoing2[30] = 211; 
  _coupling[30] = 21.09/GeV; _maxweight[30] = 1.7; 
  // D_s2
  _incoming[31] =  435; _outgoing1[31] =  421; _outgoing2[31] =  321; 
  _coupling[31] = 23.39/GeV; _maxweight[31] = 1.7; 
  _incoming[32] =  435; _outgoing1[32] =  411; _outgoing2[32] =  311; 
  _coupling[32] = 23.39/GeV; _maxweight[32] = 1.7; 
  // B_s2
  _incoming[33] =  535; _outgoing1[33] =  521; _outgoing2[33] = -321; 
  _coupling[33] = 45.43/GeV; _maxweight[33] = 1.7; 
  _incoming[34] =  535; _outgoing1[34] =  511; _outgoing2[34] = -311; 
  _coupling[34] = 48.84/GeV; _maxweight[34] = 1.7; 
  // chi_c2 to pi pi
  _incoming[35] =  445; _outgoing1[35] =  211; _outgoing2[35] = -211; 
  _coupling[35] = 0.0200/GeV; _maxweight[35] = 1.7; 
  _incoming[36] =  445; _outgoing1[36] =  111; _outgoing2[36] =  111; 
  _coupling[36] = 0.0141/GeV; _maxweight[36] = 1.7; 
  // chi_c2 to K K
  _incoming[37] =  445; _outgoing1[37] =  321; _outgoing2[37] = -321; 
  _coupling[37] = 0.056/GeV; _maxweight[37] = 1.7; 
  _incoming[38] =  445; _outgoing1[38] =  311; _outgoing2[38] = -311; 
  _coupling[38] = 0.056/GeV; _maxweight[38] = 1.7; 
  // f_2 to sigma sigma
  _incoming[39] = 225; _outgoing1[39] = 9000221; _outgoing2[39] = 9000221; 
  _coupling[39] = 104.1/GeV; _maxweight[39] = 60.; 
  // pi_2 to sigma pi
  _incoming[40] = 10115; _outgoing1[40] = 9000221; _outgoing2[40] = 111; 
  _coupling[40] = 15.3/GeV; _maxweight[40] = 10.; 
  _incoming[41] = 10215; _outgoing1[41] = 9000221; _outgoing2[41] = 211; 
  _coupling[41] = 15.3/GeV; _maxweight[41] = 10.; 
  // eta_2 to a_0 pi
  _incoming[42] = 10225; _outgoing1[42] = 9000111; _outgoing2[42] = 111; 
  _coupling[42] = 11.3/GeV; _maxweight[42] = 10.; 
  _incoming[43] = 10225; _outgoing1[43] = 9000211; _outgoing2[43] = -211; 
  _coupling[43] = 11.3/GeV; _maxweight[43] = 10.; 
  // eta'_2 to a_0 pi
  _incoming[44] = 10335; _outgoing1[44] = 9000111; _outgoing2[44] = 111; 
  _coupling[44] = 4.43/GeV; _maxweight[44] = 10.; 
  _incoming[45] = 10335; _outgoing1[45] = 9000211; _outgoing2[45] = -211; 
  _coupling[45] = 4.43/GeV; _maxweight[45] = 10.; 
  // chi_c2(2P) to D D
  _incoming[46] =  100445; _outgoing1[46] =  411; _outgoing2[46] = -411; 
  _coupling[46] = 22.3/GeV; _maxweight[46] = 1.7; 
  _incoming[47] =  100445; _outgoing1[47] =  421; _outgoing2[47] = -421; 
  _coupling[47] = 22.3/GeV; _maxweight[47] = 1.7; 
  // initial size of the vectors
  _initsize=_incoming.size();
  // intermediates
  generateIntermediates(false);
}

void TensorMeson2PScalarDecayer::doinit() {
  DecayIntegrator::doinit();
  // check consistence of the parameters
  unsigned int isize=_incoming.size();
  if(isize!=_outgoing1.size()||isize!=_outgoing2.size()||
     isize!=_maxweight.size()||isize!=_coupling.size())
    throw InitException() << "Inconsistent parameters TensorMeson2PScalarDecayer" 
			  << Exception::abortnow;
  // set up the integration channels
  vector<double> wgt(0);
  DecayPhaseSpaceModePtr mode;
  tPDVector extpart(3);
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    extpart[0]=getParticleData(_incoming[ix]);
    extpart[1]=getParticleData(_outgoing1[ix]);
    extpart[2]=getParticleData(_outgoing2[ix]);
    if(extpart[0]&&extpart[1]&&extpart[2]) 
      mode=new_ptr(DecayPhaseSpaceMode(extpart,this));
    else
      mode=DecayPhaseSpaceModePtr();
    addMode(mode,_maxweight[ix],wgt);
  }
}

int TensorMeson2PScalarDecayer::modeNumber(bool & cc,tcPDPtr parent,
					   const tPDVector & children) const {
  if(children.size()!=2) return -1;
  int id(parent->id());
  int idbar = parent->CC() ? parent->CC()->id() : id;
  int id1(children[0]->id());
  int id1bar = children[0]->CC() ? children[0]->CC()->id() : id1;
  int id2(children[1]->id());
  int id2bar = children[1]->CC() ? children[1]->CC()->id() : id2;
  int imode(-1);
  unsigned int ix(0);
  cc=false;
  do {
    if(id   ==_incoming[ix]) {
      if((id1   ==_outgoing1[ix]&&id2   ==_outgoing2[ix])||
	 (id2   ==_outgoing1[ix]&&id1   ==_outgoing2[ix])) imode=ix;
    }
    if(idbar==_incoming[ix]) {
      if((id1bar==_outgoing1[ix]&&id2bar==_outgoing2[ix])||
	 (id2bar==_outgoing1[ix]&&id1bar==_outgoing2[ix])) {
	imode=ix;
	cc=true;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  return imode;
}

void TensorMeson2PScalarDecayer::persistentOutput(PersistentOStream & os) const  {
  os << _incoming << _outgoing1 << _outgoing2 << _maxweight 
     << ounit(_coupling,1/GeV);
}

void TensorMeson2PScalarDecayer::persistentInput(PersistentIStream & is, int)  {
  is >> _incoming >> _outgoing1 >> _outgoing2 >> _maxweight 
     >> iunit(_coupling,1/GeV);
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<TensorMeson2PScalarDecayer,DecayIntegrator>
describeHerwigTensorMeson2PScalarDecayer("Herwig::TensorMeson2PScalarDecayer", "HwTMDecay.so");

void TensorMeson2PScalarDecayer::Init() {

  static ClassDocumentation<TensorMeson2PScalarDecayer> documentation
    ("The TensorMeson2PScalarDecayer class is designed for the decay"
     " of a tensor meson to two (pseudo)-scalar mesons.");

  static ParVector<TensorMeson2PScalarDecayer,int> interfaceIncoming
    ("Incoming",
     "The PDG code for the incoming particle",
     &TensorMeson2PScalarDecayer::_incoming,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<TensorMeson2PScalarDecayer,int> interfaceOutcoming1
    ("FirstOutgoing",
     "The PDG code for the first outgoing particle",
     &TensorMeson2PScalarDecayer::_outgoing1,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<TensorMeson2PScalarDecayer,int> interfaceOutcoming2
    ("SecondOutgoing",
     "The PDG code for the second outgoing particle",
     &TensorMeson2PScalarDecayer::_outgoing2,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<TensorMeson2PScalarDecayer,InvEnergy> interfaceCoupling
    ("Coupling",
     "The coupling for the decay mode",
     &TensorMeson2PScalarDecayer::_coupling,
     1/GeV, 0, ZERO, ZERO, 1000./GeV, false, false, true);

  static ParVector<TensorMeson2PScalarDecayer,double> interfaceMaxWeight
    ("MaxWeight",
     "The maximum weight for the decay mode",
     &TensorMeson2PScalarDecayer::_maxweight,
     0, 0, 0, 0., 100000., false, false, true);
}

// matrix elememt for the process
double TensorMeson2PScalarDecayer::me2(const int, const Particle & inpart,
				       const ParticleVector & decay,
				       MEOption meopt) const {
  if(!ME())
    ME(new_ptr(TwoBodyDecayMatrixElement(PDT::Spin2,PDT::Spin0,PDT::Spin0)));
  // stuff for incoming particle
  if(meopt==Initialize) {
    _rho = RhoDMatrix(PDT::Spin2);
    TensorWaveFunction::
      calculateWaveFunctions(_tensors,_rho,const_ptr_cast<tPPtr>(&inpart),
			     incoming,false);
  }
  if(meopt==Terminate) {
    TensorWaveFunction::constructSpinInfo(_tensors,const_ptr_cast<tPPtr>(&inpart),
					  incoming,true,false);
    // set up the spin information for the decay products
    for(unsigned int ix=0;ix<decay.size();++ix)
      ScalarWaveFunction::constructSpinInfo(decay[ix],outgoing,true);
    return 0.;
  }
  // calculate the matrix element
  for(unsigned int ix=0;ix<5;++ix) {
    (*ME())(ix,0,0) = Complex(_coupling[imode()]/inpart.mass()*
			      ((_tensors[ix]*decay[1]->momentum())*decay[0]->momentum()));
  }
//   // test of the answer
//   double me = newME.contract(_rho).real();
//   Energy pcm = Kinematics::pstarTwoBodyDecay(inpart.mass(),decay[0]->mass(),
// 					     decay[1]->mass());
//   double test = Energy4(pow<4,1>(2*pcm))*sqr( _coupling[imode()]/inpart.mass())/120.;
//   cout << "testing matrix element for " << inpart.PDGName() << " -> " 
//        << decay[0]->PDGName() << " " << decay[1]->PDGName() << " " 
//        << me << " " << test << " " << (me-test)/(me+test) << endl;
  // return the answer
  return ME()->contract(_rho).real();
}

bool TensorMeson2PScalarDecayer::twoBodyMEcode(const DecayMode & dm,int & mecode,
					       double & coupling) const {
  int imode(-1);
  int id(dm.parent()->id());
  int idbar = dm.parent()->CC() ? dm.parent()->CC()->id() : id;
  ParticleMSet::const_iterator pit(dm.products().begin());
  int id1((**pit).id());
  int id1bar = (**pit).CC() ? (**pit).CC()->id() : id1;
  ++pit;
  int id2((**pit).id());
  int id2bar = (**pit).CC() ? (**pit).CC()->id() : id2;
  unsigned int ix(0); bool order(false);
  do {
    if(id   ==_incoming[ix]) {
      if(id1==_outgoing1[ix]&&id2==_outgoing2[ix]) {
	imode=ix;
	order=true;
      }
      if(id2==_outgoing1[ix]&&id1==_outgoing2[ix]) {
	imode=ix;
	order=false;
      }
    }
    if(idbar==_incoming[ix]&&imode<0) {
      if(id1bar==_outgoing1[ix]&&id2bar==_outgoing2[ix]) {
	imode=ix;
	order=true;
      }
      if(id2bar==_outgoing1[ix]&&id1bar==_outgoing2[ix]) {
	imode=ix;
	order=false;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  coupling=_coupling[imode]*dm.parent()->mass();
  mecode=7;
  return order;
}

void TensorMeson2PScalarDecayer::dataBaseOutput(ofstream & output,
						bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  // the rest of the parameters
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":Incoming " << ix << " " 
	     << _incoming[ix] << "\n";
      output << "newdef " << name() << ":FirstOutgoing " << ix << " " 
	     << _outgoing1[ix] << "\n";
      output << "newdef " << name() << ":SecondOutgoing " << ix << " " 
	     << _outgoing2[ix] << "\n";
      output << "newdef " << name() << ":Coupling " << ix << " " 
	     << _coupling[ix]*GeV << "\n";
      output << "newdef " << name() << ":MaxWeight " << ix << " " 
	     << _maxweight[ix] << "\n";
    }
    else {
      output << "insert " << name() << ":Incoming " << ix << " " 
	     << _incoming[ix] << "\n";
      output << "insert " << name() << ":FirstOutgoing " << ix << " " 
	     << _outgoing1[ix] << "\n";
      output << "insert " << name() << ":SecondOutgoing " << ix << " " 
	     << _outgoing2[ix] << "\n";
      output << "insert " << name() << ":Coupling " << ix << " " 
	     << _coupling[ix]*GeV << "\n";
      output << "insert " << name() << ":MaxWeight " << ix << " " 
	     << _maxweight[ix] << "\n";
    }
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}
#line 1 "./TensorMesonVectorPScalarDecayer.cc"
// -*- C++ -*-
//
// TensorMesonVectorPScalarDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the TensorMesonVectorPScalarDecayer class.
//

#include "TensorMesonVectorPScalarDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Helicity/WaveFunction/TensorWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "ThePEG/Helicity/epsilon.h"
#include "Herwig/Decay/TwoBodyDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void TensorMesonVectorPScalarDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()){
    for(unsigned int ix=0;ix<_incoming.size();++ix)
      if(mode(ix)) _maxweight[ix] = mode(ix)->maxWeight();
  }
}

TensorMesonVectorPScalarDecayer::TensorMesonVectorPScalarDecayer() 
  :  _incoming(31), _outgoingV(31), _outgoingP(31), 
     _coupling(31), _maxweight(31) {
  // a_2 -> rho pi
  _incoming[0] =  115; _outgoingV[0] =  213; _outgoingP[0] = -211; 
  _coupling[0] = 21.1/GeV2; _maxweight[0] = 10.; 
  _incoming[1] =  215; _outgoingV[1] =  113; _outgoingP[1] =  211; 
  _coupling[1] = 21.1/GeV2; _maxweight[1] = 9.; 
  _incoming[2] =  215; _outgoingV[2] =  213; _outgoingP[2] =  111; 
  _coupling[2] = 21.1/GeV2; _maxweight[2] = 9.; 
  // a_2+/- -> gamma pi+/-
  _incoming[3] =  215; _outgoingV[3] =  22; _outgoingP[3] =  211; 
  _coupling[3] = 0.551/GeV2; _maxweight[3] = 2.; 
  // k_2 -> K_2 omega
  _incoming[4] =  315; _outgoingV[4] = 223; _outgoingP[4] =  311; 
  _coupling[4] = 11.66/GeV2; _maxweight[4] = 17.; 
  _incoming[5] =  325; _outgoingV[5] = 223; _outgoingP[5] =  321; 
  _coupling[5] = 11.66/GeV2; _maxweight[5] = 20.5; 
  // k_2+/- -> K+/- gamma
  _incoming[6] =  325; _outgoingV[6] = 22; _outgoingP[6] =  321; 
  _coupling[6] = 0.553/GeV2; _maxweight[6] = 2.2; 
  // B_c2 -> B_c gamma
  _incoming[7] =  545; _outgoingV[7] = 22; _outgoingP[7] =  541; 
  _coupling[7] = 0.651/GeV2; _maxweight[7] = 2.; 
  // K_2 -> K rho
  _incoming[8] =  325; _outgoingV[8] =  113; _outgoingP[8] =  321; 
  _coupling[8] = 10.14/GeV2; _maxweight[8] = 9.; 
  _incoming[9] =  325; _outgoingV[9] =  213; _outgoingP[9] =  311; 
  _coupling[9] = 14.33/GeV2; _maxweight[9] = 9.; 
  _incoming[10] =  315; _outgoingV[10] =  113; _outgoingP[10] =  311; 
  _coupling[10] = 10.14/GeV2; _maxweight[10] = 9.; 
  _incoming[11] =  315; _outgoingV[11] = -213; _outgoingP[11] =  321; 
  _coupling[11] = 14.33/GeV2; _maxweight[11] = 9.; 
  // K_2 -> K* pi 
  _incoming[12] =  325; _outgoingV[12] =  323; _outgoingP[12] =  111; 
  _coupling[12] = 9.733/GeV2; _maxweight[12] = 13; 
  _incoming[13] =  325; _outgoingV[13] =  313; _outgoingP[13] =  211; 
  _coupling[13] = 13.77/GeV2; _maxweight[13] = 11; 
  _incoming[14] =  315; _outgoingV[14] =  313; _outgoingP[14] =  111; 
  _coupling[14] = 9.733/GeV2; _maxweight[14] = 8.; 
  _incoming[15] =  315; _outgoingV[15] =  323; _outgoingP[15] = -211; 
  _coupling[15] = 13.77/GeV2; _maxweight[15] = 8.; 
  // D_2 -> D* pi 
  _incoming[16] =  425; _outgoingV[16] =  423; _outgoingP[16] =  111; 
  _coupling[16] = 8.035/GeV2; _maxweight[16] = 2.2; 
  _incoming[17] =  425; _outgoingV[17] =  413; _outgoingP[17] = -211; 
  _coupling[17] = 11.670/GeV2; _maxweight[17] = 2.4; 
  _incoming[18] =  415; _outgoingV[18] =  413; _outgoingP[18] =  111; 
  _coupling[18] = 6.801/GeV2; _maxweight[18] = 2.4; 
  _incoming[19] =  415; _outgoingV[19] =  423; _outgoingP[19] =  211; 
  _coupling[19] = 9.527/GeV2; _maxweight[19] = 2.; 
  // D_s2 -> D* K
  _incoming[20] =  435; _outgoingV[20] =  423; _outgoingP[20] =  321; 
  _coupling[20] = 13.10/GeV2; _maxweight[20] = 2.2; 
  _incoming[21] =  435; _outgoingV[21] =  413; _outgoingP[21] =  311; 
  _coupling[21] = 13.10/GeV2; _maxweight[21] = 2.5; 
  // B_2 -> B* pi 
  _incoming[22] =  525; _outgoingV[22] =  523; _outgoingP[22] =  111; 
  _coupling[22] = 4.99/GeV2; _maxweight[22] = 2.1; 
  _incoming[23] =  525; _outgoingV[23] =  513; _outgoingP[23] =  211; 
  _coupling[23] = 7.059/GeV2; _maxweight[23] = 2.1; 
  _incoming[24] =  515; _outgoingV[24] =  513; _outgoingP[24] =  111; 
  _coupling[24] = 4.99/GeV2; _maxweight[24] = 2.1; 
  _incoming[25] =  515; _outgoingV[25] =  523; _outgoingP[25] = -211; 
  _coupling[25] = 7.059/GeV2; _maxweight[25] = 2.1; 
  // D_s2
  _incoming[26] =  435; _outgoingV[26] =  423; _outgoingP[26] =  321; 
  _coupling[26] = 13.09/GeV2; _maxweight[26] = 2.2; 
  _incoming[27] =  435; _outgoingV[27] =  413; _outgoingP[27] =  311; 
  _coupling[27] = 13.09/GeV2; _maxweight[27] = 2.5; 
  // B_s2
  _incoming[28] =  535; _outgoingV[28] =  523; _outgoingP[28] = -321; 
  _coupling[28] = 7.29/GeV2; _maxweight[28] = 2.4; 
  _incoming[29] =  535; _outgoingV[29] =  513; _outgoingP[29] = -311; 
  _coupling[29] = 9.43/GeV2; _maxweight[29] = 2.1; 
  // upsilon_2(1d) to chi_b gamma
  _incoming[30] = 20555; _outgoingV[30] =   22; _outgoingP[30] = 10551; 
  _coupling[30] = 1.11/GeV2; _maxweight[30] = 2.4; 
  // initial size of the arrays
  _initsize=_incoming.size();
  // intermediates
  generateIntermediates(false);
}

void TensorMesonVectorPScalarDecayer::doinit() {
  DecayIntegrator::doinit();
  // check consistence of the parameters
  unsigned int isize=_incoming.size();
  if(isize!=_outgoingV.size()||isize!=_outgoingP.size()||
     isize!=_maxweight.size()||isize!=_coupling.size())
    throw InitException() << "Inconsistent parameters TensorMesonVectorPScalarDecayer" 
			  << Exception::abortnow;
  // set up the integration channels
  vector<double> wgt;
  DecayPhaseSpaceModePtr mode;
  tPDVector extpart(3);
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    extpart[0] = getParticleData(_incoming[ix]);
    extpart[1] = getParticleData(_outgoingV[ix]);
    extpart[2] = getParticleData(_outgoingP[ix]);
    if(extpart[0]&&extpart[1]&&extpart[2]) 
      mode=new_ptr(DecayPhaseSpaceMode(extpart,this));
    else
      mode=DecayPhaseSpaceModePtr();
    addMode(mode,_maxweight[ix],wgt);
  }
}

int TensorMesonVectorPScalarDecayer::modeNumber(bool & cc, tcPDPtr parent, 
						const tPDVector & children) const {
  if(children.size()!=2) return -1;
  int id(parent->id());
  int idbar = parent->CC() ? parent->CC()->id() : id;
  int id1(children[0]->id());
  int id1bar = children[0]->CC() ? children[0]->CC()->id() : id1;
  int id2(children[1]->id());
  int id2bar = children[1]->CC() ? children[1]->CC()->id() : id2;
  int imode(-1);
  unsigned int ix(0);
  cc=false;
  do {
    if(id   ==_incoming[ix]) {
      if((id1   ==_outgoingP[ix]&&id2   ==_outgoingV[ix])||
	 (id2   ==_outgoingP[ix]&&id1   ==_outgoingV[ix])) imode=ix;
    }
    if(idbar==_incoming[ix]) {
      if((id1bar==_outgoingP[ix]&&id2bar==_outgoingV[ix])||
	 (id2bar==_outgoingP[ix]&&id1bar==_outgoingV[ix])) {
	imode=ix;
	cc=true;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  return imode;
}

void TensorMesonVectorPScalarDecayer::persistentOutput(PersistentOStream & os) const {
  os << _incoming << _outgoingV << _outgoingP << _maxweight << ounit(_coupling,1/GeV2);
}

void TensorMesonVectorPScalarDecayer::persistentInput(PersistentIStream & is, int) {
  is >> _incoming >> _outgoingV >> _outgoingP >> _maxweight >> iunit(_coupling,1/GeV2);
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<TensorMesonVectorPScalarDecayer,DecayIntegrator>
describeHerwigTensorMesonVectorPScalarDecayer("Herwig::TensorMesonVectorPScalarDecayer", "HwTMDecay.so");

void TensorMesonVectorPScalarDecayer::Init() {

  static ClassDocumentation<TensorMesonVectorPScalarDecayer> documentation
    ("The TensorMesonVectorPScalarDecayer class implements the"
     " decay of a tensor meson to a spin-1 particle and a pseduoscalar meson");

  static ParVector<TensorMesonVectorPScalarDecayer,int> interfaceIncoming
    ("Incoming",
     "The PDG code for the incoming particle",
     &TensorMesonVectorPScalarDecayer::_incoming,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<TensorMesonVectorPScalarDecayer,int> interfaceOutcomingV
    ("OutgoingVector",
     "The PDG code for the outgoing spin-1particle",
     &TensorMesonVectorPScalarDecayer::_outgoingV,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<TensorMesonVectorPScalarDecayer,int> interfaceOutcomingP
    ("OutgoingScalar",
     "The PDG code for the outgoing pseudoscalar meson",
     &TensorMesonVectorPScalarDecayer::_outgoingP,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<TensorMesonVectorPScalarDecayer,InvEnergy2> interfaceCoupling
    ("Coupling",
     "The coupling for the decay mode",
     &TensorMesonVectorPScalarDecayer::_coupling,
     1/GeV2, 0, ZERO, ZERO, 100./GeV2, false, false, true);

  static ParVector<TensorMesonVectorPScalarDecayer,double> interfaceMaxWeight
    ("MaxWeight",
     "The maximum weight for the decay mode",
     &TensorMesonVectorPScalarDecayer::_maxweight,
     0, 0, 0, 0., 1000., false, false, true);

}

// matrix elememt for the process
double TensorMesonVectorPScalarDecayer::me2(const int,const Particle & inpart,
					    const ParticleVector & decay,
					    MEOption meopt) const {
  if(!ME())
    ME(new_ptr(TwoBodyDecayMatrixElement(PDT::Spin2,PDT::Spin1,PDT::Spin0)));
  // check for photons
  bool photon(_outgoingV[imode()]==ParticleID::gamma);
  // stuff for incoming particle
  if(meopt==Initialize) {
    _rho = RhoDMatrix(PDT::Spin2);
    TensorWaveFunction::
      calculateWaveFunctions(_tensors,_rho,const_ptr_cast<tPPtr>(&inpart),
			     incoming,false);
  }
  if(meopt==Terminate) {
    TensorWaveFunction::constructSpinInfo(_tensors,const_ptr_cast<tPPtr>(&inpart),
					  incoming,true,false);
    // set up the spin information for the decay products
    VectorWaveFunction::constructSpinInfo(_vectors,decay[0],outgoing,true,photon);
    ScalarWaveFunction::constructSpinInfo(decay[1],outgoing,true);
    return 0.;
  }
  VectorWaveFunction::calculateWaveFunctions(_vectors,decay[0],outgoing,photon);
  InvEnergy3 fact(_coupling[imode()]/inpart.mass());
  // calculate the matrix element
  for(unsigned int inhel=0;inhel<5;++inhel) {
    for(unsigned int vhel=0;vhel<3;++vhel){
      if(vhel==1&&photon) (*ME())(inhel,vhel,0)=0.;
      else {
	LorentzVector<complex<InvEnergy> > vtemp=
	  fact*epsilon(decay[0]->momentum(),_vectors[vhel],decay[1]->momentum());
	(*ME())(inhel,vhel,0)= Complex((decay[1]->momentum()*_tensors[inhel]).dot(vtemp));
      }
    }
  }
//   // test of the answer
//   double me = ME()->contract(_rho).real();
//   Energy pcm = Kinematics::pstarTwoBodyDecay(inpart.mass(),decay[0]->mass(),
// 					     decay[1]->mass());
//   double test = Energy4(pow<4,1>(2*pcm))*sqr( _coupling[imode()])/80.;
//   cout << "testing matrix element for " << inpart.PDGName() << " -> " 
//        << decay[0]->PDGName() << " " << decay[1]->PDGName() << " " 
//        << me << " " << test << " " << (me-test)/(me+test) << endl;
  // return the answer
  return ME()->contract(_rho).real();
}

bool TensorMesonVectorPScalarDecayer::twoBodyMEcode(const DecayMode & dm,int & mecode,
						   double & coupling) const {
  int imode(-1);
  int id(dm.parent()->id());
  int idbar = dm.parent()->CC() ? dm.parent()->CC()->id() : id;
  ParticleMSet::const_iterator pit(dm.products().begin());
  int id1((**pit).id());
  int id1bar = (**pit).CC() ? (**pit).CC()->id() : id1;
  ++pit;
  int id2((**pit).id());
  int id2bar = (**pit).CC() ? (**pit).CC()->id() : id2;
  unsigned int ix(0); 
  bool order(false);
  do {
    if(id   ==_incoming[ix]) {
      if(id1==_outgoingP[ix]&&id2==_outgoingV[ix]) {
	imode=ix;
	order=true;
      }
      if(id2==_outgoingP[ix]&&id1==_outgoingV[ix]) {
	imode=ix;
	order=false;
      }
    }
    if(idbar==_incoming[ix]&&imode<0) {
      if(id1bar==_outgoingP[ix]&&id2bar==_outgoingV[ix]) {
	imode=ix;
	order=true;
      }
      if(id2bar==_outgoingP[ix]&&id1bar==_outgoingV[ix]) {
	imode=ix;
	order=false;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  coupling=_coupling[imode]*sqr(dm.parent()->mass());
  mecode=8;
  return order;
}

void TensorMesonVectorPScalarDecayer::dataBaseOutput(ofstream & output,
						     bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  // the rest of the parameters
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":Incoming " << ix << " " 
	     << _incoming[ix] << "\n";
      output << "newdef " << name() << ":OutgoingVector " << ix << " " 
	     << _outgoingV[ix] << "\n";
      output << "newdef " << name() << ":OutgoingScalar " << ix << " " 
	     << _outgoingP[ix] << "\n";
      output << "newdef " << name() << ":Coupling " << ix << " " 
	     << _coupling[ix]*GeV2 << "\n";
      output << "newdef " << name() << ":MaxWeight " << ix << " " 
	     << _maxweight[ix] << "\n";
    }
    else {
      output << "insert " << name() << ":Incoming " << ix << " " 
	     << _incoming[ix] << "\n";
      output << "insert " << name() << ":OutgoingVector " << ix << " " 
	     << _outgoingV[ix] << "\n";
      output << "insert " << name() << ":OutgoingScalar " << ix << " " 
	     << _outgoingP[ix] << "\n";
      output << "insert " << name() << ":Coupling " << ix << " " 
	     << _coupling[ix]*GeV2 << "\n";
      output << "insert " << name() << ":MaxWeight " << ix << " " 
	     << _maxweight[ix] << "\n";
    }
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}
#line 1 "./TensorMesonVectorVectorDecayer.cc"
// -*- C++ -*-
//
// TensorMesonVectorVectorDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the TensorMesonVectorVectorDecayer class.
//

#include "TensorMesonVectorVectorDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/TensorWaveFunction.h"
#include "Herwig/Decay/TwoBodyDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void TensorMesonVectorVectorDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    for(unsigned int ix=0;ix<_incoming.size();++ix)
      if(mode(ix)) _maxweight[ix] = mode(ix)->maxWeight();
  }
}

void TensorMesonVectorVectorDecayer::doinit() {
  DecayIntegrator::doinit();
  // check consistence of the parameters
  unsigned int isize=_incoming.size();
  if(isize!=_outgoing1.size()||isize!=_outgoing2.size()||
     isize!=_maxweight.size()||isize!=_coupling.size())
    throw InitException() << "Inconsistent parameters TensorMesonVectorVectorDecayer" 
			  << Exception::abortnow;
  // set up the integration channels
  vector<double> wgt(0);
  tPDVector extpart(3);
  DecayPhaseSpaceModePtr mode;
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    extpart[0]=getParticleData(_incoming[ix]);
    extpart[1]=getParticleData(_outgoing1[ix]);
    extpart[2]=getParticleData(_outgoing2[ix]);
    if(extpart[0]&&extpart[1]&&extpart[2]) 
      mode=new_ptr(DecayPhaseSpaceMode(extpart,this));
    else
      mode=DecayPhaseSpaceModePtr();
    addMode(mode,_maxweight[ix],wgt);
  }
}

TensorMesonVectorVectorDecayer::TensorMesonVectorVectorDecayer() 
  : _incoming(22), _outgoing1(22), _outgoing2(22), 
    _coupling(22), _maxweight(22) {
  // a_2 -> gamma gamma
  _incoming[0] = 115; _outgoing1[0] =  22; _outgoing2[0] = 22; 
  _coupling[0] = 0.00727/GeV; _maxweight[0] = 1.7; 
  // f_2 -> gamma gamma
  _incoming[1] = 225; _outgoing1[1] =  22; _outgoing2[1] = 22; 
  _coupling[1] = 0.01253/GeV; _maxweight[1] = 1.7; 
  // f'_2 -> gamma gamma
  _incoming[2] = 335; _outgoing1[2] =  22; _outgoing2[2] = 22; 
  _coupling[2] = 0.00161/GeV; _maxweight[2] = 1.7; 
  // chi_b(2P) decays
  _incoming[3] = 100555; _outgoing1[3] = 553; _outgoing2[3] = 223; 
  _coupling[3] = 0.0118/GeV; _maxweight[3] = 1.8; 
  _incoming[4] = 100555; _outgoing1[4] = 553; _outgoing2[4] = 22; 
  _coupling[4] = 0.0172/GeV; _maxweight[4] = 1.7; 
  _incoming[5] = 100555; _outgoing1[5] = 100553; _outgoing2[5] = 22; 
  _coupling[5] = 0.145/GeV; _maxweight[5] = 1.7; 
  _incoming[6] = 100555; _outgoing1[6] = 333; _outgoing2[6] = 333; 
  _coupling[6] = 0.00483/GeV; _maxweight[6] = 18.0; 
  // chi_c decays
  _incoming[7] = 445; _outgoing1[7] = 443; _outgoing2[7] = 22; 
  _coupling[7] = 0.243/GeV; _maxweight[7] = 1.7; 
  _incoming[8] = 445; _outgoing1[8] = 323; _outgoing2[8] = -323; 
  _coupling[8] = 0.00560/GeV; _maxweight[8] = 15.; 
  _incoming[9] = 445; _outgoing1[9] = 313; _outgoing2[9] = -313; 
  _coupling[9] = 0.00560/GeV; _maxweight[9] = 20.; 
  _incoming[10] = 445; _outgoing1[10] = 333; _outgoing2[10] = 333; 
  _coupling[10] = 0.00418/GeV; _maxweight[10] = 10.; 
  _incoming[11] = 445; _outgoing1[11] = 22; _outgoing2[11] = 22; 
  _coupling[11] = 0.00122/GeV; _maxweight[11] = 1.7; 
  // chi_b(1P) decays
  _incoming[12] = 555; _outgoing1[12] = 553; _outgoing2[12] = 22; 
  _coupling[12] = 0.0683/GeV; _maxweight[12] = 1.8; 
  // a_2 omega rho 
  _incoming[13] =  115; _outgoing1[13] =  223; _outgoing2[13] =  113; 
  _coupling[13] = 23.1/GeV; _maxweight[13] = 15.; 
  _incoming[14] =  215; _outgoing1[14] =  223; _outgoing2[14] =  213; 
  _coupling[14] = 23.1/GeV; _maxweight[14] = 21.; 
  // f_2 rho rho
  _incoming[15] =  225; _outgoing1[15] =  113; _outgoing2[15] =  113; 
  _coupling[15] = 11.7/GeV; _maxweight[15] = 26.; 
  _incoming[16] =  225; _outgoing1[16] =  213; _outgoing2[16] = -213; 
  _coupling[16] = 16.5/GeV; _maxweight[16] = 26.; 
  // K_2-> K* rho
  _incoming[17] =  315; _outgoing1[17] =  113; _outgoing2[17] =  313; 
  _coupling[17] = 13.42/GeV; _maxweight[17] = 30.; 
  _incoming[18] =  315; _outgoing1[18] = -213; _outgoing2[18] =  323; 
  _coupling[18] = 18.98/GeV; _maxweight[18] = 30.; 
  _incoming[19] =  325; _outgoing1[19] =  113; _outgoing2[19] =  323; 
  _coupling[19] = 13.42/GeV; _maxweight[19] = 30.; 
  _incoming[20] =  325; _outgoing1[20] =  213; _outgoing2[20] =  313; 
  _coupling[20] = 18.98/GeV; _maxweight[20] = 30.; 
  _incoming[21] = 445; _outgoing1[21] = 223; _outgoing2[21] = 223; 
  _coupling[21] = 0.00389/GeV; _maxweight[21] = 12.; 
  // initial size of the vectors
  _initsize = _incoming.size();
  // intermediates
  generateIntermediates(false);
}

int TensorMesonVectorVectorDecayer::modeNumber(bool & cc, tcPDPtr parent, 
						const tPDVector & children) const {
  if(children.size()!=2) return -1;
  int id(parent->id());
  int idbar = parent->CC() ? parent->CC()->id() : id;
  int id1(children[0]->id());
  int id1bar = children[0]->CC() ? children[0]->CC()->id() : id1;
  int id2(children[1]->id());
  int id2bar = children[1]->CC() ? children[1]->CC()->id() : id2;
  int imode(-1);
  unsigned int ix(0);
  cc=false;
  do {
    if(id   ==_incoming[ix]) {
      if((id1   ==_outgoing1[ix]&&id2   ==_outgoing2[ix])||
	 (id2   ==_outgoing1[ix]&&id1   ==_outgoing2[ix])) imode=ix;
    }
    if(idbar==_incoming[ix]) {
      if((id1bar==_outgoing1[ix]&&id2bar==_outgoing2[ix])||
	 (id2bar==_outgoing1[ix]&&id1bar==_outgoing2[ix])) {
	imode=ix;
	cc=true;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  return imode;
}


void TensorMesonVectorVectorDecayer::persistentOutput(PersistentOStream & os) const  {
  os << _incoming << _outgoing1 << _outgoing2 << _maxweight << ounit(_coupling,1/GeV);
}

void TensorMesonVectorVectorDecayer::persistentInput(PersistentIStream & is, int) {
  is >> _incoming >> _outgoing1 >> _outgoing2 >> _maxweight >> iunit(_coupling,1/GeV);
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<TensorMesonVectorVectorDecayer,DecayIntegrator>
describeHerwigTensorMesonVectorVectorDecayer("Herwig::TensorMesonVectorVectorDecayer", "HwTMDecay.so");

void TensorMesonVectorVectorDecayer::Init() {

  static ClassDocumentation<TensorMesonVectorVectorDecayer> documentation
    ("The TensorMesonVectorVectorDecayer class performs the"
     " decay of a tensor meson to two scalar mesons.");

  static ParVector<TensorMesonVectorVectorDecayer,int> interfaceIncoming
    ("Incoming",
     "The PDG code for the incoming particle",
     &TensorMesonVectorVectorDecayer::_incoming,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<TensorMesonVectorVectorDecayer,int> interfaceOutcoming1
    ("FirstOutgoing",
     "The PDG code for the first outgoing particle",
     &TensorMesonVectorVectorDecayer::_outgoing1,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<TensorMesonVectorVectorDecayer,int> interfaceOutcoming2
    ("SecondOutgoing",
     "The PDG code for the second outgoing particle",
     &TensorMesonVectorVectorDecayer::_outgoing2,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<TensorMesonVectorVectorDecayer,InvEnergy> interfaceCoupling
    ("Coupling",
     "The coupling for the decay mode",
     &TensorMesonVectorVectorDecayer::_coupling,
     1/GeV, 0, ZERO, ZERO, 1000./GeV, false, false, true);

  static ParVector<TensorMesonVectorVectorDecayer,double> interfaceMaxWeight
    ("MaxWeight",
     "The maximum weight for the decay mode",
     &TensorMesonVectorVectorDecayer::_maxweight,
     0, 0, 0, 0., 100., false, false, true);

}

// matrix elememt for the process
double TensorMesonVectorVectorDecayer::me2(const int,const Particle & inpart,
					   const ParticleVector & decay,
					   MEOption meopt) const {
  if(!ME())
    ME(new_ptr(TwoBodyDecayMatrixElement(PDT::Spin2,PDT::Spin1,PDT::Spin1)));
  // photons ??
  bool photon[2];
  for(unsigned int ix=0;ix<2;++ix)
    photon[ix]=decay[ix]->id()==ParticleID::gamma;
  // stuff for incoming particle
  if(meopt==Initialize) {
    _rho = RhoDMatrix(PDT::Spin2);
    TensorWaveFunction::
      calculateWaveFunctions(_tensors,_rho,const_ptr_cast<tPPtr>(&inpart),
			     incoming,false);
  }
  if(meopt==Terminate) {
    TensorWaveFunction::constructSpinInfo(_tensors,const_ptr_cast<tPPtr>(&inpart),
					  incoming,true,false);
    // set up the spin information for the decay products
    for(unsigned int ix=0;ix<2;++ix)
      VectorWaveFunction::constructSpinInfo(_vectors[ix],decay[ix],
					    outgoing,true,photon[ix]);
    return 0.;
  }
  for(unsigned int ix=0;ix<2;++ix)
    VectorWaveFunction::calculateWaveFunctions(_vectors[ix],decay[ix],
					       outgoing,photon[ix]);
  // compute some useful dot products etc
  complex<Energy> p1eps2[3],p2eps1[3];
  Energy2 p1p2(decay[0]->momentum()*decay[1]->momentum());
  for(unsigned int ix=0;ix<3;++ix) {
    p1eps2[ix]=_vectors[1][ix]*decay[0]->momentum();
    p2eps1[ix]=_vectors[0][ix]*decay[1]->momentum();
  }
  // compute the traces and useful dot products
  Complex trace[5];
  complex<Energy2> pboth[5];
  LorentzPolarizationVectorE pleft[2][5],pright[2][5];
  for(unsigned int ix=0;ix<5;++ix) {
    trace[ix]=_tensors[ix].xx() + _tensors[ix].yy() + _tensors[ix].zz() + _tensors[ix].tt();
    pleft[0][ix] =(-decay[0]->momentum())*_tensors[ix];
    pleft[1][ix] =(-decay[1]->momentum())*_tensors[ix];
    pright[0][ix]=_tensors[ix]*(-decay[0]->momentum());
    pright[1][ix]=_tensors[ix]*(-decay[1]->momentum());
    pboth[ix]=((pleft[0][ix]+pright[0][ix])*decay[1]->momentum());
  }
  // loop to compute the matrix element
  Complex e1e2;
  LorentzTensor<Energy2> te1e2;
  InvEnergy2 fact(_coupling[imode()]/inpart.mass());
  complex<Energy2> me;
  for(unsigned int ix=0;ix<3;++ix) {
    for(unsigned iy=0;iy<3;++iy) {
      e1e2=_vectors[0][ix].dot(_vectors[1][iy]);
      te1e2=complex<Energy2>(p1p2)*
	(LorentzTensor<double>(_vectors[0][ix],_vectors[1][iy])+
	 LorentzTensor<double>(_vectors[1][iy],_vectors[0][ix]));
      for(unsigned int inhel=0;inhel<5;++inhel) {
	me = (_tensors[inhel]*te1e2
	      -p2eps1[ix]*(_vectors[1][iy].dot(pleft[0][inhel]+pright[0][inhel])) 
	      -p1eps2[iy]*(_vectors[0][ix].dot(pleft[1][inhel]+pright[1][inhel]))
	      +pboth[inhel]*e1e2
	      +(p2eps1[ix]*p1eps2[iy]-e1e2*p1p2)*trace[inhel]);
	(*ME())(inhel,ix,iy)=Complex(fact*me);
      }    
    }
  }
//   // testing the matrix element
//   double metest = ME()->contract(_rho).real();
//   Energy2 m02(sqr(inpart.mass())),m12(sqr(decay[0]->mass())),m22(sqr(decay[1]->mass()));
//   Energy pcm = Kinematics::pstarTwoBodyDecay(inpart.mass(),decay[0]->mass(),
// 					     decay[1]->mass());
//   Energy2 pcm2(sqr(pcm));
//   double test = 4./15./m02/m02*sqr(_coupling[imode()])*
//     (3.*m02*(8.*pcm2*pcm2+5.*(m12*m22+pcm2*(m12+m22)))-5.*(m12-m22)*(m12-m22)*pcm2);
//   cout << "testing the matrix element VV " << inpart.PDGName() << " -> " 
//        << decay[0]->PDGName() << " " << decay[1]->PDGName() << " " 
//        << metest << " " << test <<  " " << (metest-test)/(metest+test) << endl;
  // return the answer
  return ME()->contract(_rho).real();
}

bool TensorMesonVectorVectorDecayer::twoBodyMEcode(const DecayMode & dm,int & mecode,
						   double & coupling) const {
  int imode(-1);
  int id(dm.parent()->id());
  int idbar = dm.parent()->CC() ? dm.parent()->CC()->id() : id;
  ParticleMSet::const_iterator pit(dm.products().begin());
  int id1((**pit).id());
  int id1bar = (**pit).CC() ? (**pit).CC()->id() : id1;
  ++pit;
  int id2((**pit).id());
  int id2bar = (**pit).CC() ? (**pit).CC()->id() : id2;
  unsigned int ix(0);
  do {
    if(id   ==_incoming[ix]) {
      if(id1==_outgoing1[ix]&&id2==_outgoing2[ix]) {
	imode=ix;
      }
      if(id2==_outgoing1[ix]&&id1==_outgoing2[ix]) {
	imode=ix;
      }
    }
    if(idbar==_incoming[ix]&&imode<0) {
      if(id1bar==_outgoing1[ix]&&id2bar==_outgoing2[ix]) {
	imode=ix;
      }
      if(id2bar==_outgoing1[ix]&&id1bar==_outgoing2[ix]) {
	imode=ix;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  coupling=_coupling[imode]*dm.parent()->mass();
  mecode=9;
  return id1==_outgoing1[imode]&&id2==_outgoing2[imode];
}

void TensorMesonVectorVectorDecayer::dataBaseOutput(ofstream & output,
						    bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  // the rest of the parameters
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":Incoming " << ix << " " 
	     << _incoming[ix] << "\n";
      output << "newdef " << name() << ":FirstOutgoing " << ix << " " 
	     << _outgoing1[ix] << "\n";
      output << "newdef " << name() << ":SecondOutgoing " << ix << " " 
	     << _outgoing2[ix] << "\n";
      output << "newdef " << name() << ":Coupling " << ix << " " 
	     << _coupling[ix]*GeV << "\n";
      output << "newdef " << name() << ":MaxWeight " << ix << " " 
	     << _maxweight[ix] << "\n";
    }
    else {
      output << "insert " << name() << ":Incoming " << ix << " " 
	     << _incoming[ix] << "\n";
      output << "insert " << name() << ":FirstOutgoing " << ix << " " 
	     << _outgoing1[ix] << "\n";
      output << "insert " << name() << ":SecondOutgoing " << ix << " " 
	     << _outgoing2[ix] << "\n";
      output << "insert " << name() << ":Coupling " << ix << " " 
	     << _coupling[ix]*GeV << "\n";
      output << "insert " << name() << ":MaxWeight " << ix << " " 
	     << _maxweight[ix] << "\n";
    }
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}
