#line 1 "./a1ThreePionCLEODecayer.cc"
// -*- C++ -*-
//
// a1ThreePionCLEODecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the a1ThreePionCLEODecayer class.
//

#include "a1ThreePionCLEODecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Interface/Switch.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "Herwig/PDT/ThreeBodyAllOnCalculator.h"
#include "Herwig/Decay/GeneralDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;
using Constants::pi;
 
a1ThreePionCLEODecayer::a1ThreePionCLEODecayer() 
  : _rhomass(2), _rhowidth(2), _f2mass(1.275*GeV), _f2width(0.185*GeV), 
    _pf2cc(ZERO), _pf200(ZERO), _f0mass(1.186*GeV), _f0width(0.350*GeV),
    _pf0cc(ZERO), _pf000(ZERO), _sigmamass(0.860*GeV), _sigmawidth(0.880*GeV), 
    _psigmacc(ZERO), _psigma00(ZERO), _mpi0(ZERO), _mpic(ZERO),
    _coupling(45.57/GeV), _rhomagP(2), _rhophaseP(2), _rhomagD(2),
    _rhophaseD(2), _f2mag(0.71/GeV2), _f2phase(0.56*pi), _f2coup(ZERO,ZERO),
    _f0mag(0.77), _f0phase(-0.54*pi), _f0coup(0.,0.), _sigmamag(2.10),
    _sigmaphase(0.23*pi), _sigmacoup(0.,0.), _localparameters(true),
    _zerowgts(9), _onewgts(9), _twowgts(9), _threewgts(12), _zeromax(13.0704),
    _onemax(6.91104), _twomax(6.94654), _threemax(6.40086) {
  // rho masses and widths
  _rhomass[0] = 0.7743*GeV; _rhowidth[0] = 0.1491*GeV;
  _rhomass[1] = 1.370 *GeV; _rhowidth[1] = 0.386 *GeV;
  // p-wave rho and rho prime
  _rhomagP[0] = 1.  ; _rhophaseP[0] = 0.;
  _rhomagP[1] = 0.12; _rhophaseP[1] = 0.99*pi;
  // d-wave rho and rho prime
  _rhomagD[0] = 0.37/GeV2; _rhophaseD[0] = -0.15*pi;
  _rhomagD[1] = 0.87/GeV2; _rhophaseD[1] =  0.53*pi;
  // set up the integration channels
  _zerowgts[0]  = 0.132162;_zerowgts[1]  = 0.116638;_zerowgts[2]  = 0.121088;
  _zerowgts[3]  = 0.10656 ;_zerowgts[4]  = 0.102577;_zerowgts[5]  = 0.101169;
  _zerowgts[6]  = 0.104587;_zerowgts[7]  = 0.104663;_zerowgts[8]  = 0.110557;
  _onewgts[0]   = 0.177017;_onewgts[1]   = 0.176011;_onewgts[2]   = 0.110129;
  _onewgts[3]   = 0.108023;_onewgts[4]   = 0.110553;_onewgts[5]   = 0.109976;
  _onewgts[6]   = 0.088634;_onewgts[7]   = 0.059104;_onewgts[8]   = 0.060553;
  _twowgts[0]   = 0.173357;_twowgts[1]   = 0.172283;_twowgts[2]   = 0.116031;
  _twowgts[3]   = 0.114642;_twowgts[4]   = 0.109058;_twowgts[5]   = 0.114073;
  _twowgts[6]   = 0.080946;_twowgts[7]   = 0.060135;_twowgts[8]   = 0.059477;
  _threewgts[0] = 0.125022;_threewgts[1] = 0.129911;_threewgts[2] = 0.074165;
  _threewgts[3] = 0.075813;_threewgts[4 ]= 0.071154;_threewgts[5 ]= 0.077730;
  _threewgts[6] = 0.082255;_threewgts[7 ]= 0.086761;_threewgts[8 ]= 0.067106;
  _threewgts[9] = 0.070171;_threewgts[10]= 0.070146;_threewgts[11]= 0.069767;
  // generation of intermediates
  generateIntermediates(true);
}
  
void a1ThreePionCLEODecayer::doinit() {
  DecayIntegrator::doinit();
  // pointers to the particles we need as external particles
  tPDPtr a1p = getParticleData(ParticleID::a_1plus);
  tPDPtr a10 = getParticleData(ParticleID::a_10);
  tPDPtr pip = getParticleData(ParticleID::piplus);
  tPDPtr pim = getParticleData(ParticleID::piminus);
  tPDPtr pi0 = getParticleData(ParticleID::pi0);
  // possible intermediate particles
  // the different rho resonances
  tPDPtr rhop[3] = {getParticleData(213),getParticleData(100213),
		    getParticleData(30213)};
  tPDPtr rho0[3] = {getParticleData(113),getParticleData(100113),
		    getParticleData(30113)};
  tPDPtr rhom[3] = {getParticleData(-213),getParticleData(-100213),
		    getParticleData(-30213)};
  // the sigma
  tPDPtr sigma = getParticleData(9000221);
  // the f_2
  tPDPtr f2=getParticleData(225);
  // the f_0
  tPDPtr f0=getParticleData(10221);
  // set up the integration channels
  tPDVector extpart(4);
  DecayPhaseSpaceChannelPtr newchannel;
  DecayPhaseSpaceModePtr mode;
  // decay mode a_10 -> pi0 pi0 pi0
  extpart[0]=a10;
  extpart[1]=pi0;
  extpart[2]=pi0;
  extpart[3]=pi0;
  mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
  // there are six sigma channels
  tPDPtr temp;
  for(unsigned int ix=0;ix<3;++ix) {
    if(ix==0)      temp = sigma;
    else if(ix==1) temp = f2;
    else if(ix==2) temp = f0;
    if(temp) {
      newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
      newchannel->addIntermediate(a10,0,0.0,-1,1);
      newchannel->addIntermediate(temp,0,0.0,2,3);
      mode->addChannel(newchannel);
      newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
      newchannel->addIntermediate(a10,0,0.0,-1,2);
      newchannel->addIntermediate(temp,0,0.0,1,3);
      mode->addChannel(newchannel);
      newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
      newchannel->addIntermediate(a10,0,0.0,-1,3);
      newchannel->addIntermediate(temp,0,0.0,1,2);
      mode->addChannel(newchannel);
    }
  }
  if(_zerowgts.size()!=mode->numberChannels()) 
    _zerowgts=vector<double>(mode->numberChannels(),1./mode->numberChannels());
  addMode(mode,_zeromax,_zerowgts);
  // decay mode a_1+ -> pi+ pi0 pi0
  extpart[0]=a1p;
  extpart[1]=pi0;
  extpart[2]=pi0;
  extpart[3]=pip;
  mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
  for(unsigned int ix=0;ix<3;++ix) {
    if(!rhop[ix]) continue;
    // first rho+ channel
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,1);
    newchannel->addIntermediate(rhop[ix],0,0.0,2,3);
    mode->addChannel(newchannel);
    // second rho+ channel
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,2);
    newchannel->addIntermediate(rhop[ix],0,0.0,1,3);
    mode->addChannel(newchannel);
  }
  // the sigma channel
  if(sigma) {
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,3);
    newchannel->addIntermediate(sigma,0,0.0,1,2);
    mode->addChannel(newchannel);
  }
  //  the f_2  channel
  if(f2) {
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,3);
    newchannel->addIntermediate(f2,0,0.0,1,2);
    mode->addChannel(newchannel);
  }
  // the f_0 channel
  if(f0) {
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,3);
    newchannel->addIntermediate(f0,0,0.0,1,2);
    mode->addChannel(newchannel);
  }
  if(_onewgts.size()!=mode->numberChannels()) 
    _onewgts=vector<double>(mode->numberChannels(),1./mode->numberChannels());
  addMode(mode,_onemax,_onewgts);
  // decay mode a_10 -> pi+ pi- pi0
  extpart[0]=a10;
  extpart[1]=pip;
  extpart[2]=pim;
  extpart[3]=pi0;
  mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
  for(unsigned int ix=0;ix<3;++ix) {
    if(!rhop[ix]) continue;
    // first rho channel
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a10,0,0.0,-1,1);
    newchannel->addIntermediate(rhom[ix],0,0.0,2,3);
    mode->addChannel(newchannel);
    // second channel
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a10,0,0.0,-1,2);
    newchannel->addIntermediate(rhop[ix],0,0.0,1,3);
    mode->addChannel(newchannel);
  }
  // sigma channel
  if(sigma) {
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a10,0,0.0,-1,3);
    newchannel->addIntermediate(sigma,0,0.0,1,2);
    mode->addChannel(newchannel);
  }
  // f_2 channel
  if(f2) {
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a10,0,0.0,-1,3);
    newchannel->addIntermediate(f2,0,0.0,1,2);
    mode->addChannel(newchannel);
  }
  // f_0 channel
  if(f0) {
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a10,0,0.0,-1,3);
    newchannel->addIntermediate(f0,0,0.0,1,2);
    mode->addChannel(newchannel);
  }
  if(_twowgts.size()!=mode->numberChannels()) 
    _twowgts=vector<double>(mode->numberChannels(),1./mode->numberChannels());
  addMode(mode,_twomax,_twowgts);
  // decay mode a_1+ -> pi+ pi+ pi-
  extpart[0]=a1p;
  extpart[1]=pip;
  extpart[2]=pip;
  extpart[3]=pim;
  mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
  for(unsigned int ix=0;ix<3;++ix) {
    // the neutral rho channels
    if(!rho0[ix]) continue;
    // first channel
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,1);
    newchannel->addIntermediate(rho0[ix],0,0.0,2,3);
    mode->addChannel(newchannel);
    // interchanged channel
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,2);
    newchannel->addIntermediate(rho0[ix],0,0.0,1,3);
    mode->addChannel(newchannel);
  }
  // the sigma channels
  if(sigma) {
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,1);
    newchannel->addIntermediate(sigma,0,0.0,2,3);
    mode->addChannel(newchannel);
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,2);
    newchannel->addIntermediate(sigma,0,0.0,1,3);
    mode->addChannel(newchannel);
  }
  // the f_2 channels
  if(f2) {
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,1);
    newchannel->addIntermediate(f2,0,0.0,2,3);
    mode->addChannel(newchannel);
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,2);
    newchannel->addIntermediate(f2,0,0.0,1,3);
    mode->addChannel(newchannel);
  }
  // the f_0 channel
  if(f0) {
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,1);
    newchannel->addIntermediate(f0,0,0.0,2,3);
    mode->addChannel(newchannel);
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,2);
    newchannel->addIntermediate(f0,0,0.0,1,3);
    mode->addChannel(newchannel);
  }
  if(_threewgts.size()!=mode->numberChannels()) 
    _threewgts=vector<double>(mode->numberChannels(),1./mode->numberChannels());
  addMode(mode,_threemax,_threewgts);
  // if using local parameters set the values in the phase space channels
  if(_localparameters) {
    for(unsigned int iy=0;iy<_rhomass.size();++iy) {
      resetIntermediate(rho0[iy],_rhomass[iy],_rhowidth[iy]);
      resetIntermediate(rhop[iy],_rhomass[iy],_rhowidth[iy]);
      resetIntermediate(rhom[iy],_rhomass[iy],_rhowidth[iy]);
    }
    resetIntermediate(sigma,_sigmamass,_sigmawidth);
    resetIntermediate(f2,_f2mass,_f2width);
    resetIntermediate(f0,_f0mass,_f0width);
    // make sure the rho array has enough masses
    if(_rhomass.size()<3) {
      for(unsigned int ix=_rhomass.size();ix<3;++ix) {
	_rhomass.push_back(rhop[ix]->mass());
	_rhowidth.push_back(rhop[ix]->width());
      }
    }
  }
  // set the local variables if needed
  else {
    // masses and widths for the particles
    _rhomass.resize(3);_rhowidth.resize(3);
    for(unsigned int ix=0;ix<3;++ix) {
      _rhomass[ix]=rhop[ix]->mass();
      _rhowidth[ix]=rhop[ix]->width();
    }
    if(f2) {
      _f2mass=f2->mass();
      _f2width=f2->width();
    }
    if(f0) {
      _f0mass=f0->mass();
      _f0width=f0->width();
    }
    if(sigma) {
      _sigmamass=sigma->mass();
      _sigmawidth=sigma->width();
    }
  }
  // parameters for the breit-wigners
  _mpic=pip->mass();
  _mpi0=pi0->mass();
  // momenta of the decay products for on-shell particles
  _psigmacc = Kinematics::pstarTwoBodyDecay(_sigmamass,_mpic,_mpic);
  _psigma00 = Kinematics::pstarTwoBodyDecay(_sigmamass,_mpi0,_mpi0);
  _pf2cc    = Kinematics::pstarTwoBodyDecay(_f2mass   ,_mpic,_mpic);
  _pf200    = Kinematics::pstarTwoBodyDecay(_f2mass   ,_mpi0,_mpi0);
  _pf0cc    = Kinematics::pstarTwoBodyDecay(_f0mass   ,_mpic,_mpic);
  _pf000    = Kinematics::pstarTwoBodyDecay(_f0mass   ,_mpi0,_mpi0); 
  _prhocc.resize(3);_prhoc0.resize(3);
  for(unsigned int ix=0;ix<3;++ix) {
    _prhocc[ix] = Kinematics::pstarTwoBodyDecay(_rhomass[ix],_mpic,_mpic);
    _prhoc0[ix] = Kinematics::pstarTwoBodyDecay(_rhomass[ix],_mpic,_mpi0);
  }
  // couplings for the different modes
  Complex ii(0.,1.);
  _rhocoupP.resize(_rhomagP.size());
  for(unsigned int ix=0;ix<_rhomagP.size();++ix)
    _rhocoupP[ix]=_rhomagP[ix]*(cos(_rhophaseP[ix])+ii*sin(_rhophaseP[ix]));
  _rhocoupD.resize(_rhomagD.size());
  for(unsigned int ix=0;ix<_rhomagD.size();++ix)
    _rhocoupD[ix]=_rhomagD[ix]*(cos(_rhophaseD[ix])+ii*sin(_rhophaseD[ix]));
  _f0coup=_f0mag*(cos(_f0phase)+ii*sin(_f0phase));
  _f2coup=_f2mag*(cos(_f2phase)+ii*sin(_f2phase));
  _sigmacoup=_sigmamag*(cos(_sigmaphase)+ii*sin(_sigmaphase));
}


inline void a1ThreePionCLEODecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    // get the weights for the different channels
    for(unsigned int ix=0;ix<_zerowgts.size();++ix)
      _zerowgts[ix]=mode(0)->channelWeight(ix);
    for(unsigned int ix=0;ix<_onewgts.size();++ix)
      _onewgts[ix]=mode(1)->channelWeight(ix);
    for(unsigned int ix=0;ix<_twowgts.size();++ix)
      _twowgts[ix]=mode(2)->channelWeight(ix);
    for(unsigned int ix=0;ix<_threewgts.size();++ix)
      _threewgts[ix]=mode(3)->channelWeight(ix);
    // get the maximum weight
    _zeromax  = mode(0)->maxWeight();
    _onemax   = mode(1)->maxWeight();
    _twomax   = mode(2)->maxWeight();
    _threemax = mode(3)->maxWeight();
  }
}

int a1ThreePionCLEODecayer::modeNumber(bool & cc,tcPDPtr parent,
				       const tPDVector & children) const {
  if(children.size()!=3) return -1;
  int id(parent->id());
  // check the pions
  tPDVector::const_iterator pit  = children.begin();
  tPDVector::const_iterator pend = children.end();
  int idtemp,npi0(0),npiplus(0),npiminus(0);
  for( ; pit!=pend;++pit) {
    idtemp=(**pit).id();
    if(idtemp==ParticleID::piplus)       ++npiplus;
    else if(idtemp==ParticleID::piminus) ++npiminus;
    else if(idtemp==ParticleID::pi0)     ++npi0;
  }
  int imode(-1);
  // a_1+ decay modes
  if(id==ParticleID::a_1plus) {
    cc=false;
    if(npiplus==1&&npi0==2)          imode=1;
    else if(npiplus==2&&npiminus==1) imode=3;
    }
  // a_1- modes
  else if(id==ParticleID::a_1minus) {
    cc=true;
    if(npiminus==1&&npi0==2)         imode=1;
    else if(npiminus==2&&npiplus==1) imode=3;
  }
  // a_0 modes
  else if(id==ParticleID::a_10) {
    cc=false;
    if(npiminus==1&&npiplus==1&&npi0==1) imode=2;
    else if(npi0==3)                     imode=0;
  }
  return imode;
}

void a1ThreePionCLEODecayer::persistentOutput(PersistentOStream & os) const {
  os << ounit(_rhomass,GeV) << ounit(_rhowidth,GeV) 
     << ounit(_prhocc,GeV) << ounit(_prhoc0,GeV) 
     << ounit(_f2mass,GeV) << ounit(_f2width,GeV) 
     << ounit(_pf2cc,GeV) 
     << ounit(_pf200,GeV) << ounit(_f0mass,GeV) 
     << ounit(_f0width,GeV) << ounit(_pf0cc,GeV) << ounit(_pf000,GeV) 
     << ounit(_sigmamass,GeV) << ounit(_sigmawidth,GeV)
     << ounit(_psigmacc,GeV) << ounit(_psigma00,GeV) 
     << ounit(_mpi0,GeV) << ounit(_mpic,GeV) 
     << ounit(_coupling,1/GeV) << _rhomagP << _rhophaseP 
     << _rhocoupP << ounit(_rhomagD ,1/GeV2)<< _rhophaseD 
     << ounit(_rhocoupD,1/GeV2) << ounit(_f2mag,1/GeV2)
     << _f2phase << ounit(_f2coup,1/GeV2)
     << _f0mag << _f0phase << _f0coup 
     << _sigmamag << _sigmaphase << _sigmacoup 
     << _localparameters 
     << _zerowgts << _onewgts << _twowgts << _threewgts 
     << _zeromax << _onemax << _twomax << _threemax;
}
  
void a1ThreePionCLEODecayer::persistentInput(PersistentIStream & is, int) {
  is >> iunit(_rhomass,GeV) >> iunit(_rhowidth,GeV) 
     >> iunit(_prhocc,GeV) >> iunit(_prhoc0,GeV) 
     >> iunit(_f2mass,GeV) >> iunit(_f2width,GeV) 
     >> iunit(_pf2cc,GeV)
     >> iunit(_pf200,GeV) >> iunit(_f0mass,GeV) 
     >> iunit(_f0width,GeV) >> iunit(_pf0cc,GeV) >> iunit(_pf000,GeV) 
     >> iunit(_sigmamass,GeV) >> iunit(_sigmawidth,GeV) 
     >> iunit(_psigmacc,GeV) >> iunit(_psigma00,GeV) 
     >> iunit(_mpi0,GeV) >> iunit(_mpic,GeV) 
     >> iunit(_coupling,1/GeV) >> _rhomagP >> _rhophaseP
     >> _rhocoupP >> iunit(_rhomagD,1/GeV2) >> _rhophaseD 
     >> iunit(_rhocoupD,1/GeV2)>>iunit(_f2mag,1/GeV2) 
     >> _f2phase >> iunit(_f2coup,1/GeV2)
     >> _f0mag >> _f0phase >> _f0coup
     >> _sigmamag >> _sigmaphase >> _sigmacoup
     >> _localparameters 
     >> _zerowgts >> _onewgts >> _twowgts >> _threewgts
     >> _zeromax >> _onemax >> _twomax >> _threemax;
}
  
// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<a1ThreePionCLEODecayer,DecayIntegrator>
describeHerwiga1ThreePionCLEODecayer("Herwig::a1ThreePionCLEODecayer", "HwVMDecay.so");
  
void a1ThreePionCLEODecayer::Init() {
  
  static ClassDocumentation<a1ThreePionCLEODecayer> documentation
    ("The a1ThreePionCLEODecayer class performs the decay of the "
     "a_1 to three pions using the model of CLEO",
     "The decay of a_1 to three pions was modelled after \\cite{Asner:1999kj}.",
     "%\\cite{Asner:1999kj}\n"
     "\\bibitem{Asner:1999kj}\n"
     "  D.~M.~Asner {\\it et al.}  [CLEO Collaboration],\n"
     "   ``Hadronic structure in the decay tau- --> nu/tau pi- pi0 pi0 and the  sign\n"
     "  %of the tau neutrino helicity,''\n"
     "  Phys.\\ Rev.\\  D {\\bf 61}, 012002 (2000)\n"
     "  [arXiv:hep-ex/9902022].\n"
     "  %%CITATION = PHRVA,D61,012002;%%\n"
     );

  static ParVector<a1ThreePionCLEODecayer,Energy> interfacerhomass
    ("RhoMasses",
     "The masses of the different rho resonnaces",
     &a1ThreePionCLEODecayer::_rhomass,
     GeV, 0, ZERO, -10000*GeV, 10000*GeV, false, false, true);

  static ParVector<a1ThreePionCLEODecayer,Energy> interfacerhowidth
    ("RhoWidths",
     "The widths of the different rho resonnaces",
     &a1ThreePionCLEODecayer::_rhowidth,
     GeV, 0, ZERO, -10000*GeV, 10000*GeV, false, false, true);

  static Parameter<a1ThreePionCLEODecayer,Energy> interfacef_2Mass
    ("f_2Mass",
     "The mass of the f_2 meson",
     &a1ThreePionCLEODecayer::_f2mass, GeV, 1.275*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static Parameter<a1ThreePionCLEODecayer,Energy> interfacef_2Width
    ("f_2Width",
     "The width of the f_2 meson",
     &a1ThreePionCLEODecayer::_f2width, GeV, 0.185*GeV, ZERO, 1.0*GeV,
     false, false, true);

  static Parameter<a1ThreePionCLEODecayer,Energy> interfacef_0Mass
    ("f_0Mass",
     "The mass of the f_0 meson",
     &a1ThreePionCLEODecayer::_f0mass, GeV, 1.186*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static Parameter<a1ThreePionCLEODecayer,Energy> interfacef_0Width
    ("f_0Width",
     "The width of the f_0 meson",
     &a1ThreePionCLEODecayer::_f0width, GeV, 0.350*GeV, ZERO, 1.0*GeV,
     false, false, true);

  static Parameter<a1ThreePionCLEODecayer,Energy> interfacesigmaMass
    ("sigmaMass",
     "The mass of the sigma meson",
     &a1ThreePionCLEODecayer::_sigmamass, GeV, 0.860*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static Parameter<a1ThreePionCLEODecayer,Energy> interfacesigmaWidth
    ("sigmaWidth",
     "The width of the sigma meson",
     &a1ThreePionCLEODecayer::_sigmawidth, GeV, 0.880*GeV, ZERO, 2.0*GeV,
     false, false, true);

  static Parameter<a1ThreePionCLEODecayer,InvEnergy> interfaceCoupling
    ("Coupling",
     "The overall coupling for the decay",
     &a1ThreePionCLEODecayer::_coupling, 1./GeV, 45.57/GeV, -0./GeV, 1000./GeV,
     false, false, false);

  static ParVector<a1ThreePionCLEODecayer,double> interfacerhomagP
    ("RhoPWaveMagnitude",
     "The magnitude of the couplings for the p-wave rho currents",
     &a1ThreePionCLEODecayer::_rhomagP,
     0, 0, 0, 0, 10000, false, false, true);

  static ParVector<a1ThreePionCLEODecayer,double> interfacerhophaseP
    ("RhoPWavePhase",
     "The phase of the couplings for the p-wave rho currents",
     &a1ThreePionCLEODecayer::_rhophaseP,
     0, 0, 0, -pi, pi, false, false, true);

  static ParVector<a1ThreePionCLEODecayer,InvEnergy2> interfacerhomagD
    ("RhoDWaveMagnitude",
     "The magnitude of the couplings for the d-wave rho currents",
     &a1ThreePionCLEODecayer::_rhomagD,
     1/MeV2, 0, ZERO, ZERO, 10000/MeV2, false, false, true);

  static ParVector<a1ThreePionCLEODecayer,double> interfacerhophaseD
    ("RhoDWavePhase",
     "The phase of the couplings for the d-wave rho currents",
     &a1ThreePionCLEODecayer::_rhophaseD,
     0, 0, 0, -pi, pi, false, false, true);

  static Parameter<a1ThreePionCLEODecayer,double> interfacef0Phase
    ("f0Phase",
     "The phase of the f_0 scalar current",
     &a1ThreePionCLEODecayer::_f0phase, 0.54*pi, -pi, pi,
     false, false, Interface::limited);

  static Parameter<a1ThreePionCLEODecayer,double> interfacef2Phase
    ("f2Phase",
     "The phase of the f_2 tensor current",
     &a1ThreePionCLEODecayer::_f2phase, 0.56*pi, -pi, pi,
     false, false, Interface::limited);

  static Parameter<a1ThreePionCLEODecayer,double> interfacesigmaPhase
    ("sigmaPhase",
     "The phase of the sigma scalar current",
     &a1ThreePionCLEODecayer::_sigmaphase, 0.23*pi, -pi, pi,
     false, false, Interface::limited);

  static Parameter<a1ThreePionCLEODecayer,double> interfacef0Magnitude
    ("f0Magnitude",
     "The magnitude of the f_0 scalar current",
     &a1ThreePionCLEODecayer::_f0mag, 0.77, 0.0, 10,
     false, false, true);

  static Parameter<a1ThreePionCLEODecayer,InvEnergy2> interfacef2Magnitude
    ("f2Magnitude",
     "The magnitude of the f_2 tensor current",
     &a1ThreePionCLEODecayer::_f2mag, 1./GeV2, 0.71/GeV2, ZERO, 10./GeV2,
     false, false, true);

  static Parameter<a1ThreePionCLEODecayer,double> interfacesigmaMagnitude
    ("sigmaMagnitude",
     "The magnitude of the sigma scalar current",
     &a1ThreePionCLEODecayer::_sigmamag, 2.1, 0.0, 10,
     false, false, true);

  static Switch<a1ThreePionCLEODecayer,bool> interfaceLocalParameters
    ("LocalParameters",
     "Use local values of the intermediate resonances masses and widths",
     &a1ThreePionCLEODecayer::_localparameters, true, false, false);
  static SwitchOption interfaceLocalParametersLocal
    (interfaceLocalParameters,
     "Local",
     "Use the local values",
     true);
  static SwitchOption interfaceLocalParametersDefault
    (interfaceLocalParameters,
     "ParticleData",
     "Use the values from the particleData objects",
     false);

  static ParVector<a1ThreePionCLEODecayer,double> interfacezerowgts
    ("AllNeutralWeights",
     "The weights of the different channels to use for the integration of"
     " the decay a_1^0->pi0pi0pi0",
     &a1ThreePionCLEODecayer::_zerowgts,
     0, 0, 0, -10000, 10000, false, false, true);

  static ParVector<a1ThreePionCLEODecayer,double> interfaceonewgts
    ("OneChargedWeights",
     "The weights of the different channels to use for the integration of"
     " the decay a_1^+->pi+pi0pi0",
     &a1ThreePionCLEODecayer::_onewgts,
     0, 0, 0, -10000, 10000, false, false, true);

  static ParVector<a1ThreePionCLEODecayer,double> interfacetwowgts
    ("TwoChargedWeights",
     "The weights of the different channels to use for the integration of"
     " the decay a_1^0->pi+pi-pi0",
     &a1ThreePionCLEODecayer::_twowgts,
     0, 0, 0, -10000, 10000, false, false, true);

  static ParVector<a1ThreePionCLEODecayer,double> interfacethreewgts
    ("ThreeChargedWeights",
     "The weights of the different channels to use for the integration of"
     " the decay a_1^+->pi+pi+pi-",
     &a1ThreePionCLEODecayer::_threewgts,
     0, 0, 0, -10000, 10000, false, false, true);

  static Parameter<a1ThreePionCLEODecayer,double> interfaceZeroMax
    ("ZeroMax",
     "The maximum weight for the integration fo the channel a_1^0->pi0pi0pi0",
     &a1ThreePionCLEODecayer::_zeromax, 0.0716349E3, 0.0, 10000.0,
     false, false, true);

  static Parameter<a1ThreePionCLEODecayer,double> interfaceOneMax
    ("OneMax",
     "The maximum weight for the integration fo the channel a_1^+->pi+pi0pi0",
     &a1ThreePionCLEODecayer::_onemax,1.23756E3, 0.0, 10000.0,
     false, false, true);

  static Parameter<a1ThreePionCLEODecayer,double> interfaceTwoMax
    ("TwoMax",
     "The maximum weight for the integration fo the channel a_1^0->pi+pi-pi0",
     &a1ThreePionCLEODecayer::_twomax,2.43819E3, 0.0, 10000.0,
     false, false, true);

  static Parameter<a1ThreePionCLEODecayer,double> interfaceThreeMax
    ("ThreeMax",
     "The maximum weight for the integration fo the channel a_1^+->pi+pi+pi-",
     &a1ThreePionCLEODecayer::_threemax, 1.38754E3, 0.0, 10000.0,
     false, false, true);
}

double a1ThreePionCLEODecayer::me2(const int ichan,
				   const Particle & inpart,
				   const ParticleVector & decay,
				   MEOption meopt) const {
  useMe();
  if(!ME())
    ME(new_ptr(GeneralDecayMatrixElement(PDT::Spin1,PDT::Spin0,PDT::Spin0,PDT::Spin0)));
  if(meopt==Initialize) {
    VectorWaveFunction::calculateWaveFunctions(_vectors,_rho,
						const_ptr_cast<tPPtr>(&inpart),
						incoming,false);
  }
  if(meopt==Terminate) {
    VectorWaveFunction::constructSpinInfo(_vectors,const_ptr_cast<tPPtr>(&inpart),
					  incoming,true,false);
    // set up the spin information for the decay products
    for(unsigned int ix=0;ix<3;++ix)
      ScalarWaveFunction::constructSpinInfo(decay[ix],outgoing,true);
    return 0.;
  }
  // momentum of the incoming particle
  Lorentz5Momentum Q=inpart.momentum();
  Energy2 q2=Q.mass2();
  // identify the mesons
  // calculate the invariants and form factors
  Lorentz5Momentum ps1=decay[1]->momentum()+decay[2]->momentum();
  Lorentz5Momentum ps2=decay[0]->momentum()+decay[2]->momentum();
  Lorentz5Momentum ps3=decay[0]->momentum()+decay[1]->momentum();
  ps1.rescaleMass();
  ps2.rescaleMass();
  ps3.rescaleMass();
  Energy2 s1=ps1.mass2(),s2=ps2.mass2(),s3=ps3.mass2();
  complex<InvEnergy> F1,F2,F3;
  formFactors(imode(),ichan,q2,s1,s2,s3,F1,F2,F3);
  // use the form-factors to compute the current
  LorentzPolarizationVector output=
    LorentzPolarizationVector(F1*decay[1]->momentum())-LorentzPolarizationVector(F1*decay[2]->momentum())
    -LorentzPolarizationVector(F2*decay[2]->momentum())+LorentzPolarizationVector(F2*decay[0]->momentum())
    +LorentzPolarizationVector(F3*decay[0]->momentum())-LorentzPolarizationVector(F3*decay[1]->momentum());
  // compute the matrix element
  for(unsigned int ix=0;ix<3;++ix) 
    (*ME())(ix,0,0,0)=output.dot(_vectors[ix]);
  // answer
  double out = ME()->contract(_rho).real();
  // test of the answer
//   double test = threeBodyMatrixElement(imode(),sqr(inpart.mass()),
// 				       s3,s2,s1,decay[0]->mass(),decay[1]->mass(), 
// 				       decay[2]->mass());
//   if(ichan<0) cerr << "testing matrix element " << inpart.PDGName() << " -> "
//        << decay[0]->PDGName() << " " << decay[1]->PDGName() << " "
//        << decay[2]->PDGName() << out << " " << test << " " 
//        << (out-test)/(out+test) << "\n";  
  // return the answer
  return out;
}

// matrix element for the running a_1 width
double a1ThreePionCLEODecayer::
threeBodyMatrixElement(const int iopt,const Energy2 q2, const Energy2 s3,
		       const Energy2 s2,const Energy2 s1, const Energy m1, 
		       const Energy m2 ,const Energy m3) const {
  Energy2 m12=m1*m1,m22=m2*m2,m32=m3*m3;
  // calculate the form factors
  complex<InvEnergy> F1,F2,F3;
  formFactors(iopt,-1,q2,s1,s2,s3,F1,F2,F3);
  // analytic calculation of the matrix element
  double dot1=( F1*conj(F1)*(2.*m22+2.*m32-s1)+F2*conj(F2)*(2.*m12+2.*m32-s2)
		+F3*conj(F3)*(2.*m12+2.*m22-s3)-F1*conj(F2)*( s1+s2-s3-4.*m32)
		+F1*conj(F3)*( s1-s2+s3-4.*m22)-F2*conj(F3)*(-s1+s2+s3-4.*m12)).real();
  complex<Energy> dot2 = 0.5*(F1*(s3-m32-s2+m22)-F2*(s1-m12-s3+m32)+F3*(s2-m22-s1+m12));
  return (-dot1+(dot2*conj(dot2)).real()/q2)/3.;
}

WidthCalculatorBasePtr 
a1ThreePionCLEODecayer::threeBodyMEIntegrator(const DecayMode & dm) const {
  ParticleMSet::const_iterator pit  = dm.products().begin();
  ParticleMSet::const_iterator pend = dm.products().end();
  int ncharged=0;
  for( ; pit!=pend;++pit) {
    if(abs((**pit).id())==ParticleID::piplus) ++ncharged;
  }
  // integrator to perform the integral
  vector<double> inweights;inweights.push_back(0.5);inweights.push_back(0.5);
  vector<int> intype;intype.push_back(2);intype.push_back(3);
  vector<Energy> inmass(2,_rhomass[0]),inwidth(2,_rhowidth[0]);
  vector<double> inpow(2,0.0);
  Energy m[3];
  m[0] = ncharged<2 ? _mpi0 : _mpic;
  m[1] = m[0];
  m[2] = (ncharged==0||ncharged==2) ? _mpi0 : _mpic;
  return new_ptr(ThreeBodyAllOnCalculator<a1ThreePionCLEODecayer>
		 (inweights,intype,inmass,inwidth,inpow,*this,ncharged,m[0],m[1],m[2]));
}

// calculate the form factos
void a1ThreePionCLEODecayer::formFactors(int iopt,int ichan,
					 Energy2 q2,Energy2 s1,Energy2 s2,
					 Energy2 s3,
					 complex<InvEnergy> & FF1,
					 complex<InvEnergy> & FF2,
					 complex<InvEnergy> & FF3) const {
  Complex F1, F2, F3;
  InvEnergy fact = _coupling;
  // a_1^0 pi0 pi0 pi0 mode
  if(iopt==0) {
    fact*=1./sqrt(6.);
    // compute the breit wigners we need
    Complex sigbws1 = sigmaBreitWigner(s1,1);
    Complex sigbws2 = sigmaBreitWigner(s2,1);
    Complex sigbws3 = sigmaBreitWigner(s3,1);
    Complex f0bws1  = f0BreitWigner(s1,1);
    Complex f0bws2  = f0BreitWigner(s2,1);
    Complex f0bws3  = f0BreitWigner(s3,1);
    Complex f2bws1  = f2BreitWigner(s1,1);
    Complex f2bws2  = f2BreitWigner(s2,1);
    Complex f2bws3  = f2BreitWigner(s3,1);
    if(ichan<0) {
      // the scalar terms
      F1=2./3.*(_sigmacoup*sigbws3+_f0coup*f0bws3)
	-2./3.*(_sigmacoup*sigbws2+_f0coup*f0bws2);
      F2=2./3.*(_sigmacoup*sigbws3+_f0coup*f0bws3)
	-2./3.*(_sigmacoup*sigbws1+_f0coup*f0bws1);
      F3=-2./3.*(_sigmacoup*sigbws1+_f0coup*f0bws1)
	+2./3.*(_sigmacoup*sigbws2+_f0coup*f0bws2);
      // the tensor terms
      complex<Energy2> Dfact1 = 1./18.*(4.*_mpi0*_mpi0-s1)*(q2+s1-_mpi0*_mpi0)/s1*f2bws1;
      complex<Energy2> Dfact2 = 1./18.*(4.*_mpi0*_mpi0-s2)*(q2+s2-_mpi0*_mpi0)/s2*f2bws2;
      complex<Energy2> Dfact3 = 1./18.*(4.*_mpi0*_mpi0-s3)*(q2-_mpi0*_mpi0+s3)/s3*f2bws3;
      F1 += Complex(_f2coup*( 0.5*(s3-s2)*f2bws1-Dfact2+Dfact3));
      F2 += Complex(_f2coup*( 0.5*(s3-s1)*f2bws2-Dfact1+Dfact3));
      F3 += Complex(_f2coup*(-0.5*(s1-s2)*f2bws3-Dfact1+Dfact2));
    }
    else if(ichan==0) {
      F2=-2./3.*_sigmacoup*sigbws1;
      F3=-2./3.*_sigmacoup*sigbws1;
    }
    else if(ichan==1) {
      F1=-2./3.*_sigmacoup*sigbws2;
      F3=+2./3.*_sigmacoup*sigbws2;
    }
    else if(ichan==2) {
      F1= 2./3.*_sigmacoup*sigbws3;
      F2= 2./3.*_sigmacoup*sigbws3;
    }
    else if(ichan==3) {
      complex<Energy2> Dfact1 = 1./18.*(4.*_mpi0*_mpi0-s1)*(q2+s1-_mpi0*_mpi0)/s1*f2bws1;
      F1+=Complex(_f2coup*0.5*(s3-s2)*f2bws1);
      F2-=Complex(_f2coup*Dfact1); 
      F3-=Complex(_f2coup*Dfact1);
    }
    else if(ichan==4) {
      complex<Energy2> Dfact2 = 1./18.*(4.*_mpi0*_mpi0-s2)*(q2+s2-_mpi0*_mpi0)/s2*f2bws2;
      F2+=Complex(_f2coup*0.5*(s3-s1)*f2bws2);
      F1-=Complex(_f2coup*Dfact2);
      F3+=Complex(_f2coup*Dfact2);
    }
    else if(ichan==5) {
      complex<Energy2> Dfact3 = 1./18.*(4.*_mpi0*_mpi0-s3)*(q2-_mpi0*_mpi0+s3)/s3*f2bws3;
      F3+=Complex(-_f2coup*0.5*(s1-s2)*f2bws3);
      F1+=Complex(_f2coup*Dfact3);
      F2+=Complex(_f2coup*Dfact3);
    }
    else if(ichan==6) {
      F2=-2./3.*_f0coup*f0bws1;
      F3=-2./3.*_f0coup*f0bws1;
    }
    else if(ichan==7) {
      F1=-2./3.*_f0coup*f0bws2;
      F3=+2./3.*_f0coup*f0bws2;
    }
    else if(ichan==8) {
      F1= 2./3.*_f0coup*f0bws3;
      F2= 2./3.*_f0coup*f0bws3;
    }
  }
  // a_1^+ -> pi0 pi0 pi+
  else if(iopt==1) {
    fact *= 1./sqrt(2.);
    // compute the breit wigners we need
    Complex rhos1bw[3],rhos2bw[3],f0bw,sigbw,f2bw;
    for(unsigned int ix=0,N=max(_rhocoupP.size(),_rhocoupD.size());ix<N;++ix) {
      rhos1bw[ix]=rhoBreitWigner(ix,s1,1);
      rhos2bw[ix]=rhoBreitWigner(ix,s2,1);
    }
    f0bw  = f0BreitWigner(s3,1);
    sigbw = sigmaBreitWigner(s3,1);
    f2bw  = f2BreitWigner(s3,1);
    if(ichan<0) {
      // the p-wave rho terms
      for(unsigned int ix=0;ix<_rhocoupP.size();++ix) {
	F1+=_rhocoupP[ix]*rhos1bw[ix];
	F2+=_rhocoupP[ix]*rhos2bw[ix];
      }
      // the D-wave rho terms
      Energy2 Dfact1=-1./3.*((s3-_mpic*_mpic)-(s1-_mpi0*_mpi0));
      Energy2 Dfact2=-1./3.*((s3-_mpic*_mpic)-(s2-_mpi0*_mpi0));
      for(unsigned int ix=0;ix<_rhocoupD.size();++ix) {
	F1+=Complex(Dfact1*_rhocoupD[ix]*rhos2bw[ix]);
	F2+=Complex(Dfact2*_rhocoupD[ix]*rhos1bw[ix]);
	F3+=Complex(_rhocoupD[ix]*(Dfact2*rhos1bw[ix]-Dfact1*rhos2bw[ix]));
      }
      // the scalar terms
      Complex scalar=2./3.*(_sigmacoup*sigbw+_f0coup*f0bw);
      F1+=scalar;
      F2+=scalar;
      // the tensor terms
      Complex Dfact3=1./18./s3*_f2coup*(q2-_mpic*_mpic+s3)*(4.*_mpi0*_mpi0-s3)*f2bw;
      F1+=Dfact3;F2+=Dfact3;
      F3-=Complex(0.5*_f2coup*(s1-s2)*f2bw);
    }
    else if(ichan%2==0&&ichan<=4) {
      unsigned int ires=ichan/2;
      if(ires<_rhocoupP.size()) F1+=_rhocoupP[ires]*rhos1bw[ires];
      Energy2 Dfact2=-1./3.*((s3-_mpic*_mpic)-(s2-_mpi0*_mpi0));
      if(ires<_rhocoupD.size()) {
	F2+=Complex(Dfact2*_rhocoupD[ires]*rhos1bw[ires]);
	F3+=Complex(_rhocoupD[ires]*Dfact2*rhos1bw[ires]);
      }
    }
    else if(ichan%2==1&&ichan<=5) {
      unsigned int ires=(ichan-1)/2;
      if(ires<_rhocoupP.size()) F2+=_rhocoupP[ires]*rhos2bw[ires];
      Energy2 Dfact1=-1./3.*((s3-_mpic*_mpic)-(s1-_mpi0*_mpi0));
      if(ires<_rhocoupD.size()) {
	F1+=Complex(Dfact1*_rhocoupD[ires]*rhos2bw[ires]);
	F3-=Complex(_rhocoupD[ires]*Dfact1*rhos2bw[ires]);
      }
    }
    else if(ichan==6) {
      F1+=2./3.*_sigmacoup*sigbw;
      F2+=2./3.*_sigmacoup*sigbw;
    }
    else if(ichan==7) {
      Complex Dfact3=1./18./s3*_f2coup*(q2-_mpic*_mpic+s3)*(4.*_mpi0*_mpi0-s3)*f2bw;
      F1+=Dfact3;
      F2+=Dfact3;
      F3-=Complex(0.5*_f2coup*(s1-s2)*f2bw);
    }
    else if(ichan==8) {
      F1+=2./3.*_f0coup*f0bw;
      F2+=2./3.*_f0coup*f0bw;
    }
  }
  // a_1^0 ->pi+pi-pi0
  else if(iopt==2) {
    // compute the breit wigners we need
    Complex rhos1bw[3],rhos2bw[3],f0bw,sigbw,f2bw;
    for(unsigned int ix=0,N=max(_rhocoupP.size(),_rhocoupD.size());ix<N;++ix) {
      rhos1bw[ix]=rhoBreitWigner(ix,s1,1);
      rhos2bw[ix]=rhoBreitWigner(ix,s2,1);
    }
    f0bw  =f0BreitWigner(s3,0);
    sigbw =sigmaBreitWigner(s3,0);
    f2bw  =f2BreitWigner(s3,0);
    if(ichan<0) {
      // the p-wave rho terms
      for(unsigned int ix=0;ix<_rhocoupP.size();++ix) {
	F1+=_rhocoupP[ix]*rhos1bw[ix];
	F2+=_rhocoupP[ix]*rhos2bw[ix];
      }
      // the D-wave rho terms
      Energy2 Dfact1=-1./3.*(s3-_mpi0*_mpi0-s1+_mpic*_mpic);
      Energy2 Dfact2=-1./3.*(s3-_mpi0*_mpi0-s2+_mpic*_mpic);
      for(unsigned int ix=0;ix<_rhocoupD.size();++ix) {
	F1+=Complex(Dfact1*_rhocoupD[ix]*rhos2bw[ix]);
	F2+=Complex(Dfact2*_rhocoupD[ix]*rhos1bw[ix]);
	F3+=Complex(_rhocoupD[ix]*(Dfact2*rhos1bw[ix]-Dfact1*rhos2bw[ix]));
      }
      // the scalar terms
      Complex scalar=2./3.*(_sigmacoup*sigbw+_f0coup*f0bw);
      F1+=scalar;
      F2+=scalar;
      // the tensor terms
      Complex Dfact3=1./18./s3*_f2coup*(q2-_mpi0*_mpi0+s3)*(4.*_mpic*_mpic-s3)*f2bw;
      F1+=Dfact3;
      F2+=Dfact3;
      F3-=Complex(0.5*_f2coup*(s1-s2)*f2bw);
    }
    else if(ichan%2==0&&ichan<=4) {
      unsigned int ires=ichan/2;
      if(ires<_rhocoupP.size()) F1+=_rhocoupP[ires]*rhos1bw[ires];
      Energy2 Dfact2=-1./3.*(s3-_mpi0*_mpi0-s2+_mpic*_mpic);
      if(ires<_rhocoupD.size()) {
	F2+=Complex(Dfact2*_rhocoupD[ires]*rhos1bw[ires]);
	F3+=Complex(_rhocoupD[ires]*Dfact2*rhos1bw[ires]);
      }
    }
    else if(ichan%2==1&&ichan<=5) {
      unsigned int ires=(ichan-1)/2;
      if(ires<_rhocoupP.size()) F2+=_rhocoupP[ires]*rhos2bw[ires];
      Energy2 Dfact1=-1./3.*(s3-_mpi0*_mpi0-s1+_mpic*_mpic);
      if(ires<_rhocoupD.size()) {
	F1+=Complex(Dfact1*_rhocoupD[ires]*rhos2bw[ires]);
	F3-=Complex(_rhocoupD[ires]*-Dfact1*rhos2bw[ires]);
      }
    }
    else if(ichan==6) {
      F1+=2./3.*_sigmacoup*sigbw;
      F2+=2./3.*_sigmacoup*sigbw;
    }
    else if(ichan==7) {
      Complex Dfact3=1./18./s3*_f2coup*(q2-_mpi0*_mpi0+s3)*(4.*_mpic*_mpic-s3)*f2bw;
      F1+=Dfact3;
      F2+=Dfact3;
      F3-=Complex(0.5*_f2coup*(s1-s2)*f2bw);
    }
    else if(ichan==8) {
      F1+=2./3.*_f0coup*f0bw;
      F2+=2./3.*_f0coup*f0bw;
    }
  }
  // a_1^+ -> pi+ pi+ pi- mode
  else {
    fact *= 1./sqrt(2.);
    // compute the breit wigners we need
    Complex rhos1bw[3],rhos2bw[3],f0bws1,sigbws1,f2bws1,f0bws2,sigbws2,f2bws2;
    for(unsigned int ix=0,N=max(_rhocoupP.size(),_rhocoupD.size());ix<N;++ix) {
      rhos1bw[ix]=rhoBreitWigner(ix,s1,0);
      rhos2bw[ix]=rhoBreitWigner(ix,s2,0);
    }
    f0bws1  =f0BreitWigner(s1,0);
    sigbws1 =sigmaBreitWigner(s1,0);
    f2bws1  =f2BreitWigner(s1,0);
    f0bws2  =f0BreitWigner(s2,0);
    sigbws2 =sigmaBreitWigner(s2,0);
    f2bws2  =f2BreitWigner(s2,0);
    if(ichan<0) {
      // the p-wave rho terms
      for(unsigned int ix=0;ix<_rhocoupP.size();++ix) {
	F1-=_rhocoupP[ix]*rhos1bw[ix];
	F2-=_rhocoupP[ix]*rhos2bw[ix];
      }
      // the D-wave rho terms
      Energy2 Dfact1=1./3.*(s1-s3);
      Energy2 Dfact2=1./3.*(s2-s3);
      for(unsigned int ix=0;ix<_rhocoupD.size();++ix) {
	F1-=Complex(Dfact1*_rhocoupD[ix]*rhos2bw[ix]);
	F2-=Complex(Dfact2*_rhocoupD[ix]*rhos1bw[ix]);
	F3-=Complex(_rhocoupD[ix]*(Dfact2*rhos1bw[ix]-Dfact1*rhos2bw[ix]));
      }
      // the scalar terms
      F1-=2./3.*(_sigmacoup*sigbws2+_f0coup*f0bws2);
      F2-=2./3.*(_sigmacoup*sigbws1+_f0coup*f0bws1);
      F3+=-2./3.*(_sigmacoup*sigbws1+_f0coup*f0bws1)
	+2./3.*(_sigmacoup*sigbws2+_f0coup*f0bws2);
      // the tensor terms
      complex<Energy2> sfact1 
	= 1./18.*(4.*_mpic*_mpic-s1)*(q2+s1-_mpic*_mpic)/s1*f2bws1;
      complex<Energy2> sfact2 
	= 1./18.*(4.*_mpic*_mpic-s2)*(q2+s2-_mpic*_mpic)/s2*f2bws2;
      F1+=Complex(_f2coup*(0.5*(s3-s2)*f2bws1-sfact2));
      F2+=Complex(_f2coup*(0.5*(s3-s1)*f2bws2-sfact1));
      F3+=Complex(_f2coup*(-sfact1+sfact2));
    }
    else if(ichan%2==0&&ichan<=4) {
      unsigned int ires=ichan/2;
      Energy2 Dfact2=1./3.*(s2-s3);
      if(ires<_rhocoupP.size()) F1-=_rhocoupP[ires]*rhos1bw[ires];
      if(ires<_rhocoupD.size()){
	F2-=Complex(Dfact2*_rhocoupD[ires]*rhos1bw[ires]);
	F3-=Complex(_rhocoupD[ires]*Dfact2*rhos1bw[ires]);
      }
    }
    else if(ichan%2==1&&ichan<=5) {
      unsigned int ires=(ichan-1)/2;
      Energy2 Dfact1=1./3.*(s1-s3);
      if(ires<_rhocoupP.size()) F2-=_rhocoupP[ires]*rhos2bw[ires];
      if(ires<_rhocoupD.size()) {
	F1-=Complex(Dfact1*_rhocoupD[ires]*rhos2bw[ires]);
	F3+=Complex(_rhocoupD[ires]*Dfact1*rhos2bw[ires]);
      }
    }
    else if(ichan==6) {
      F2-=2./3.*_sigmacoup*sigbws1;
      F3-=2./3.*_sigmacoup*sigbws1;
    }
    else if(ichan==7) {
      F1-=2./3.*_sigmacoup*sigbws2;
      F3+=2./3.*_sigmacoup*sigbws2;
    }
    else if(ichan==8) {
      complex<Energy2> sfact1 = 1./18.*(4.*_mpic*_mpic-s1)*(q2+s1-_mpic*_mpic)/s1*f2bws1;
      F1+=Complex(_f2coup*0.5*(s3-s2)*f2bws1);
      F2-=Complex(_f2coup*sfact1);
      F3-=Complex(_f2coup*sfact1);
    }
    else if(ichan==9) {
      complex<Energy2> sfact2 = 1./18.*(4.*_mpic*_mpic-s2)*(q2+s2-_mpic*_mpic)/s2*f2bws2;
      F1-=Complex(_f2coup*sfact2);
      F2+=Complex(_f2coup*0.5*(s3-s1)*f2bws2);
      F3+=Complex(_f2coup*sfact2);
    }
    else if(ichan==10) {
      F2-=2./3.*_f0coup*f0bws1;
      F3-=2./3.*_f0coup*f0bws1;
    }
    else if(ichan==11) {
      F1-=2./3.*_f0coup*f0bws2;
      F3+=2./3.*_f0coup*f0bws2;
    }
  }
  FF1 = F1 * fact;
  FF2 = F2 * fact;
  FF3 = F3 * fact;
} 

// output the setup information for the particle database
void a1ThreePionCLEODecayer::dataBaseOutput(ofstream & output,
					    bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  // masses and widths of the intermediate particles
  output << "newdef " << name() << ":f_2Mass "    << _f2mass/GeV     << "\n";
  output << "newdef " << name() << ":f_2Width "   << _f2width/GeV    << "\n";
  output << "newdef " << name() << ":f_0Mass "    << _f0mass/GeV     << "\n";
  output << "newdef " << name() << ":f_0Width "   << _f0width/GeV    << "\n";
  output << "newdef " << name() << ":sigmaMass "  << _sigmamass/GeV  << "\n";
  output << "newdef " << name() << ":sigmaWidth " << _sigmawidth/GeV << "\n";
  for(unsigned int ix=0;ix<_rhomass.size();++ix) {
    if(ix<2) output << "newdef    " << name() << ":RhoMasses " << ix << " " 
		    << _rhomass[ix]/GeV << "\n";
    else     output << "insert " << name() << ":RhoMasses " << ix << " " 
		    << _rhomass[ix]/GeV << "\n";
  }
  for(unsigned int ix=0;ix<_rhowidth.size();++ix) {
    if(ix<2) output << "newdef    " << name() << ":RhoWidths " << ix << " " 
		    << _rhowidth[ix]/GeV << "\n";
    else     output << "insert " << name() << ":RhoWidths " << ix << " " 
		    << _rhowidth[ix]/GeV << "\n";
  }
  // couplings and phases for different channels
  output << "newdef " << name() << ":f0Phase " << _f0phase << "\n";
  output << "newdef " << name() << ":f2Phase " << _f2phase<< "\n";
  output << "newdef " << name() << ":sigmaPhase " << _sigmaphase<< "\n";
  output << "newdef " << name() << ":f0Magnitude " << _f0mag<< "\n";
  output << "newdef " << name() << ":f2Magnitude " << _f2mag*GeV2 << "\n";
  output << "newdef " << name() << ":sigmaMagnitude " << _sigmamag << "\n";
  output << "newdef " << name() << ":Coupling " << _coupling*GeV << "\n";
  for(unsigned int ix=0;ix<_rhomagP.size();++ix) {
    if(ix<2) output << "newdef    " << name() << ":RhoPWaveMagnitude " << ix << " " 
		    << _rhomagP[ix] << "\n";
    else     output << "insert " << name() << ":RhoPWaveMagnitude " << ix << " " 
		    << _rhomagP[ix] << "\n";
  }
  for(unsigned int ix=0;ix<_rhophaseP.size();++ix) {
    if(ix<2) output << "newdef    " << name() << ":RhoPWavePhase " << ix << " " 
		    << _rhophaseP[ix] << "\n";
    else     output << "insert " << name() << ":RhoPWavePhase " << ix << " " 
		    << _rhophaseP[ix] << "\n";
  }  
  for(unsigned int ix=0;ix<_rhomagD.size();++ix) {
    if(ix<2) output << "newdef    " << name() << ":RhoDWaveMagnitude " << ix << " " 
		    << _rhomagD[ix]*MeV2 << "\n";
    else     output << "insert " << name() << ":RhoDWaveMagnitude " << ix << " " 
		    << _rhomagD[ix]*MeV2 << "\n";
  }
  for(unsigned int ix=0;ix<_rhophaseD.size();++ix) {
    if(ix<2) output << "newdef    " << name() << ":RhoDWavePhase " << ix << " " 
		    << _rhophaseD[ix] << "\n";
    else     output << "insert " << name() << ":RhoDWavePhase " << ix << " " 
		    << _rhophaseD[ix] << "\n";
  }
  // use local values of the masses etc.
  output << "newdef " << name() << ":LocalParameters " << _localparameters << "\n";
  // integration weights for the different channels
  for(unsigned int ix=0;ix<_zerowgts.size();++ix) {
    output << "newdef " << name() << ":AllNeutralWeights " 
	   << ix << " " << _zerowgts[ix] << "\n";
  }
  for(unsigned int ix=0;ix<_onewgts.size();++ix) {
    output << "newdef " << name() << ":OneChargedWeights " 
	   << ix << " " << _onewgts[ix] << "\n";
  }
  for(unsigned int ix=0;ix<_twowgts.size();++ix) {
    output << "newdef " << name() << ":TwoChargedWeights " 
	   << ix << " " << _twowgts[ix] << "\n";
  }
  for(unsigned int ix=0;ix<_threewgts.size();++ix) {
    output << "newdef " << name() << ":ThreeChargedWeights " 
	   << ix << " " << _threewgts[ix] << "\n";
  }
  // maximum weights for the different  channels
  output << "newdef " << name() << ":ZeroMax "  << _zeromax  << "\n";
  output << "newdef " << name() << ":OneMax "   << _onemax   << "\n";
  output << "newdef " << name() << ":TwoMax "   << _twomax   << "\n";
  output << "newdef " << name() << ":ThreeMax " << _threemax << "\n";
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}
#line 1 "./a1SimpleDecayer.cc"
// -*- C++ -*-
//
// a1SimpleDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the a1SimpleDecayer class.
//

#include "a1SimpleDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/Switch.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "Herwig/PDT/WidthCalculatorBase.h"
#include "Herwig/PDT/ThreeBodyAllOnCalculator.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "Herwig/Decay/GeneralDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

a1SimpleDecayer::a1SimpleDecayer() 
  : _rhomass(3), _rhowidth(3), _rhowgts(3),_localparameters(true), 
    _coupling(47.95/GeV), _onemax(5.4474), _twomax(5.47784), 
    _threemax(5.40185), _onewgts(6), 
    _twowgts(6), _threewgts(6), _mpi(ZERO) {
  // rho masses, widths and weights
  _rhomass[0] = 0.773*GeV; _rhowidth[0] = 0.145*GeV; _rhowgts[0] =  1.0;  
  _rhomass[1] = 1.370*GeV; _rhowidth[1] = 0.510*GeV; _rhowgts[1] = -0.145;
  _rhomass[2] = 1.750*GeV; _rhowidth[2] = 0.120*GeV; _rhowgts[2] =  0.;
  // integration weights
  _onewgts[0] = 0.235562; _twowgts[0] = 0.236208; _threewgts[0] = 0.234259;
  _onewgts[1] = 0.231098; _twowgts[1] = 0.229481; _threewgts[1] = 0.233634;
  _onewgts[2] = 0.131071; _twowgts[2] = 0.131169; _threewgts[2] = 0.135922;
  _onewgts[3] = 0.131135; _twowgts[3] = 0.133604; _threewgts[3] = 0.129231;
  _onewgts[4] = 0.135841; _twowgts[4] = 0.132685; _threewgts[4] = 0.133949;
  _onewgts[5] = 0.135294; _twowgts[5] = 0.136854; _threewgts[5] = 0.133005;
  generateIntermediates(true);
}

void a1SimpleDecayer::doinit() {
  DecayIntegrator::doinit();
  // pointers to the particles we need as external particles
  tPDPtr a1p = getParticleData(ParticleID::a_1plus);
  tPDPtr a10 = getParticleData(ParticleID::a_10);
  tPDPtr pip = getParticleData(ParticleID::piplus);
  tPDPtr pim = getParticleData(ParticleID::piminus);
  tPDPtr pi0 = getParticleData(ParticleID::pi0);
  // the different rho resonances
  tPDPtr rhop[3] = {getParticleData(213),getParticleData(100213),
		    getParticleData(30213)};
  tPDPtr rho0[3] = {getParticleData(113),getParticleData(100113),
		    getParticleData(30113)};
  tPDPtr rhom[3] = {getParticleData(-213),getParticleData(-100213),
		    getParticleData(-30213)};
  tPDVector extpart(4);
  DecayPhaseSpaceChannelPtr newchannel;
  DecayPhaseSpaceModePtr mode;
  // decay mode a_1+ -> pi+ pi0 pi0
  extpart[0]=a1p;
  extpart[1]=pi0;
  extpart[2]=pi0;
  extpart[3]=pip;
  mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
  for(unsigned int ix=0;ix<3;++ix) {
    if(!rhop[ix]) continue;
    // first rho+ channel
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,2);
    newchannel->addIntermediate(rhop[ix],0,0.0,1,3);
    mode->addChannel(newchannel);
    // second rho+ channel
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,1);
    newchannel->addIntermediate(rhop[ix],0,0.0,2,3);
    mode->addChannel(newchannel);
  }
  if(_onewgts.size()!=mode->numberChannels()) 
    _onewgts=vector<double>(mode->numberChannels(),1./mode->numberChannels());
  addMode(mode,_onemax,_onewgts);
  // decay mode a_10 -> pi+ pi- pi0
  extpart[0]=a10;
  extpart[1]=pip;
  extpart[2]=pim;
  extpart[3]=pi0;
  mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
  for(unsigned int ix=0;ix<3;++ix) {
    if(!rhop[ix]) continue;
    // first rho channel
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a10,0,0.0,-1,2);
    newchannel->addIntermediate(rhop[ix],0,0.0,1,3);
    mode->addChannel(newchannel);
    // second channel
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a10,0,0.0,-1,1);
    newchannel->addIntermediate(rhom[ix],0,0.0,2,3);
    mode->addChannel(newchannel);
  }
  if(_twowgts.size()!=mode->numberChannels()) 
    _twowgts=vector<double>(mode->numberChannels(),1./mode->numberChannels());
  addMode(mode,_twomax,_twowgts);
  // decay mode a_1+ -> pi+ pi+ pi-
  extpart[0]=a1p;
  extpart[1]=pip;
  extpart[2]=pip;
  extpart[3]=pim;
  mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
  for(unsigned int ix=0;ix<3;++ix) {
    if(!rho0[ix]) continue;
    // the neutral rho channels
    // first channel
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,2);
    newchannel->addIntermediate(rho0[ix],0,0.0,1,3);
    mode->addChannel(newchannel);
    // interchanged channel
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,1);
    newchannel->addIntermediate(rho0[ix],0,0.0,2,3);
    mode->addChannel(newchannel);      
  }
  if(_threewgts.size()!=mode->numberChannels()) 
    _threewgts=vector<double>(mode->numberChannels(),1./mode->numberChannels());
  addMode(mode,_threemax,_threewgts);
  // if using local parameters set the values in the phase space channels
  if(_localparameters) {
    for(unsigned int iy=0;iy<_rhomass.size();++iy) {
      resetIntermediate(rho0[iy],_rhomass[iy],_rhowidth[iy]);
      resetIntermediate(rhop[iy],_rhomass[iy],_rhowidth[iy]);
      resetIntermediate(rhom[iy],_rhomass[iy],_rhowidth[iy]);
    }
    // make sure the rho array has enough masses
    if(_rhomass.size()<3) {
      for(unsigned int ix=_rhomass.size();ix<3;++ix) {
	_rhomass.push_back(rhop[ix]->mass());
	_rhowidth.push_back(rhop[ix]->width());
      }
    }
  }
  // set the local variables if needed
  else {
    // masses and widths for the particles
    _rhomass.resize(3);_rhowidth.resize(3);
    for(unsigned int ix=0;ix<3;++ix) {
      if(!rhop[ix]) continue;
      _rhomass[ix]=rhop[ix]->mass();
      _rhowidth[ix]=rhop[ix]->width();
    }
  }
  _mpi = pip->mass();
}

void a1SimpleDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    // get the weights for the different channels
    for(unsigned int ix=0;ix<_onewgts.size();++ix)
      _onewgts[ix]=mode(0)->channelWeight(ix);
    for(unsigned int ix=0;ix<_twowgts.size();++ix)
      _twowgts[ix]=mode(1)->channelWeight(ix);
    for(unsigned int ix=0;ix<_threewgts.size();++ix)
      _threewgts[ix]=mode(2)->channelWeight(ix);
    // get the maximum weight
    _onemax   = mode(0)->maxWeight();
    _twomax   = mode(1)->maxWeight();
    _threemax = mode(2)->maxWeight();
  }
}

void a1SimpleDecayer::persistentOutput(PersistentOStream & os) const {
  os << ounit(_rhomass,GeV) << ounit(_rhowidth,GeV) << _rhowgts 
     << _localparameters << ounit(_coupling,1./GeV) << _onemax
     << _twomax << _threemax << _onewgts << _twowgts << _threewgts
     << ounit(_mpi,GeV);
}

void a1SimpleDecayer::persistentInput(PersistentIStream & is, int) {
  is >> iunit(_rhomass,GeV) >> iunit(_rhowidth,GeV) >> _rhowgts 
     >> _localparameters >> iunit(_coupling,1./GeV) >> _onemax
     >> _twomax >> _threemax >> _onewgts >> _twowgts >> _threewgts
     >> iunit(_mpi,GeV);
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<a1SimpleDecayer,DecayIntegrator>
describeHerwiga1SimpleDecayer("Herwig::a1SimpleDecayer", "HwVMDecay.so");

void a1SimpleDecayer::Init() {

  static ClassDocumentation<a1SimpleDecayer> documentation
    ("The a1SimpleDecayer class implements a simple model for the decay of"
     " the a_1 to three pions based on the approach of Kuhn and Santanmaria,"
     " Z.Phys. C48, 445 (1990)",
     "The decays of the $a_1$ were modelled using the approach of "
     "\\cite{Kuhn:1990ad}.\n",
     "\\bibitem{Kuhn:1990ad} J.~H.~Kuhn and A.~Santamaria,\n"
     "Z.\\ Phys.\\  C {\\bf 48} (1990) 445.\n"
     "%%CITATION = ZEPYA,C48,445;%%\n");

  static Switch<a1SimpleDecayer,bool> interfaceLocalParameters
    ("LocalParameters",
     "Use local values of the intermediate resonances masses and widths",
     &a1SimpleDecayer::_localparameters, true, false, false);
  static SwitchOption interfaceLocalParametersLocal
    (interfaceLocalParameters,
     "Local",
     "Use the local values",
     true);
  static SwitchOption interfaceLocalParametersDefault
    (interfaceLocalParameters,
     "ParticleData",
     "Use the values from the particleData objects",
     false);

  static Parameter<a1SimpleDecayer,InvEnergy> interfaceCoupling
    ("Coupling",
     "The overall coupling for the decay",
     &a1SimpleDecayer::_coupling, 1./GeV, 47.95/GeV, ZERO, 100./GeV,
     false, false, Interface::limited);

  static ParVector<a1SimpleDecayer,Energy> interfacerhomass
    ("RhoMasses",
     "The masses of the different rho resonnaces",
     &a1SimpleDecayer::_rhomass,
     MeV, 0, ZERO, ZERO, 10000*MeV, false, false, true);

  static ParVector<a1SimpleDecayer,Energy> interfacerhowidth
    ("RhoWidths",
     "The widths of the different rho resonnaces",
     &a1SimpleDecayer::_rhowidth,
     MeV, 0, ZERO, ZERO, 10000*MeV, false, false, true);

  static ParVector<a1SimpleDecayer,double> interfaceRhoWeights
    ("RhoWeights",
     "Weight for the different rho resonances",
     &a1SimpleDecayer::_rhowgts, -1, 0.0, -10.0, 10.0,
     false, false, Interface::limited);

  static Parameter<a1SimpleDecayer,double> interfaceOneMax
    ("OneMax",
     "The maximum weight for the integration fo the channel a_1^+->pi+pi0pi0",
     &a1SimpleDecayer::_onemax, 5.57613, 0.0, 10000.0,
     false, false, true);

  static Parameter<a1SimpleDecayer,double> interfaceTwoMax
    ("TwoMax",
     "The maximum weight for the integration fo the channel a_1^0->pi+pi-pi0",
     &a1SimpleDecayer::_twomax, 5.61218, 0.0, 10000.0,
     false, false, true);

  static Parameter<a1SimpleDecayer,double> interfaceThreeMax
    ("ThreeMax",
     "The maximum weight for the integration fo the channel a_1^+->pi+pi+pi-",
     &a1SimpleDecayer::_threemax, 5.5384, 0.0, 10000.0,
     false, false, true);
  
  static ParVector<a1SimpleDecayer,double> interfaceonewgts
    ("OneChargedWeights",
     "The weights of the different channels to use for the integration of"
     " the decay a_1^+->pi+pi0pi0",
     &a1SimpleDecayer::_onewgts,
     0, 0, 0, 0., 1., false, false, true);

  static ParVector<a1SimpleDecayer,double> interfacetwowgts
    ("TwoChargedWeights",
     "The weights of the different channels to use for the integration of"
     " the decay a_1^0->pi+pi-pi0",
     &a1SimpleDecayer::_twowgts,
     0, 0, 0, 0., 1., false, false, true);

  static ParVector<a1SimpleDecayer,double> interfacethreewgts
    ("ThreeChargedWeights",
     "The weights of the different channels to use for the integration of"
     " the decay a_1^+->pi+pi+pi-",
     &a1SimpleDecayer::_threewgts,
     0, 0, 0, 0., 1., false, false, true);

}

int a1SimpleDecayer::modeNumber(bool & cc,tcPDPtr parent,
				       const tPDVector & children) const {
  if(children.size()!=3) return -1;
  int id(parent->id());
  // check the pions
  tPDVector::const_iterator pit  = children.begin();
  tPDVector::const_iterator pend = children.end();
  int idtemp,npi0(0),npiplus(0),npiminus(0);
  for( ; pit!=pend;++pit) {
    idtemp=(**pit).id();
    if(idtemp==ParticleID::piplus)       ++npiplus;
    else if(idtemp==ParticleID::piminus) ++npiminus;
    else if(idtemp==ParticleID::pi0)     ++npi0;
  }
  int imode(-1);
  // a_1+ decay modes
  if(id==ParticleID::a_1plus) {
    cc=false;
    if(npiplus==1&&npi0==2)          imode=0;
    else if(npiplus==2&&npiminus==1) imode=2;
  }
  // a_1- modes
  else if(id==ParticleID::a_1minus) {
    cc=true;
    if(npiminus==1&&npi0==2)         imode=0;
    else if(npiminus==2&&npiplus==1) imode=2;
  }
  // a_0 modes
  else if(id==ParticleID::a_10) {
    cc=false;
    if(npiminus==1&&npiplus==1&&npi0==1) imode=1;
  }
  return imode;
}

double a1SimpleDecayer::me2(const int ichan,const Particle & inpart,
			    const ParticleVector & decay,MEOption meopt) const {
  if(!ME())
    ME(new_ptr(GeneralDecayMatrixElement(PDT::Spin1,PDT::Spin0,PDT::Spin0,PDT::Spin0)));
  useMe();
  if(meopt==Initialize) {
    VectorWaveFunction::calculateWaveFunctions(_vectors,_rho,
						const_ptr_cast<tPPtr>(&inpart),
						incoming,false);
  }
  if(meopt==Terminate) {
    VectorWaveFunction::constructSpinInfo(_vectors,const_ptr_cast<tPPtr>(&inpart),
					  incoming,true,false);
    // set up the spin information for the decay products
    for(unsigned int ix=0;ix<3;++ix)
      ScalarWaveFunction::constructSpinInfo(decay[ix],outgoing,true);
    return 0.;
  }
  Lorentz5Vector<complex<Energy> > current;
  Energy2 s1 = (decay[1]->momentum()+decay[2]->momentum()).m2();
  Energy2 s2 = (decay[0]->momentum()+decay[2]->momentum()).m2();
  if(ichan<0) {
    current = rhoFormFactor(s2,-1)*(decay[0]->momentum()-decay[2]->momentum())
    +rhoFormFactor(s1,-1)*(decay[1]->momentum()-decay[2]->momentum());
  }
  else if(ichan<3) {
    current = 
      rhoFormFactor(s2,ichan)*(decay[0]->momentum()-decay[2]->momentum());
  }
  else if(ichan<6) {
    current = 
      rhoFormFactor(s1,-1)*(decay[1]->momentum()-decay[2]->momentum());
  }
  // compute the matrix element
  for(unsigned int ix=0;ix<3;++ix)
    (*ME())(ix,0,0,0) = Complex(_coupling*current.dot(_vectors[ix]));
  // matrix element and identical particle factor
  double output=ME()->contract(_rho).real();
  if(imode()!=1) output*=0.5;
  // test the output
//   double test = threeBodyMatrixElement(imode(),sqr(inpart.mass()),
// 				       s3,s2,s1,decay[0]->mass(),decay[1]->mass(), 
// 				       decay[2]->mass());
//   if(ichan<0) cerr << "testing matrix element " << inpart.PDGName() << " -> "
//        << decay[0]->PDGName() << " " << decay[1]->PDGName() << " "
//        << decay[2]->PDGName() << output << " " << test << " " 
//        << (output-test)/(output+test) << "\n";  
  // return the answer
  return output;
}

double a1SimpleDecayer::
threeBodyMatrixElement(const int iopt,const Energy2 q2, const Energy2 s3,
		       const Energy2 s2,const Energy2 s1, const Energy m1, 
		       const Energy m2 ,const Energy m3) const {
  Energy2 v12  = (s2-2.*sqr(m1)-2.*sqr(m3))+0.25*sqr(s1-s3-sqr(m1)+sqr(m3))/q2;
  Energy2 v22  = (s1-2.*sqr(m2)-2.*sqr(m3))+0.25*sqr(s2-s3-sqr(m2)+sqr(m3))/q2;
  Energy2 v1v2 = (0.5*q2-s3-0.5*(3*sqr(m3)-sqr(m1)-sqr(m2)))
    +0.25*(s1-s3-sqr(m1)+sqr(m3))*(s2-s3-sqr(m2)+sqr(m3))/q2;
  Complex rho1=rhoFormFactor(s2,-1);
  Complex rho2=rhoFormFactor(s1,-1);
  double me = sqr(_coupling)*real(v12*rho1*conj(rho1)+v22*rho2*conj(rho2)
				  +2.*v1v2*rho1*conj(rho2))/3.;
  if(iopt!=1) me *= 0.5;
  return me;
}

WidthCalculatorBasePtr
a1SimpleDecayer::threeBodyMEIntegrator(const DecayMode & dm) const {
  ParticleMSet::const_iterator pit  = dm.products().begin();
  ParticleMSet::const_iterator pend = dm.products().end();
  int ncharged=0;
  for( ; pit!=pend;++pit) {
    if(abs((**pit).id())==ParticleID::piplus) ++ncharged;
  }
  --ncharged;
  // integrator to perform the integral
  vector<double> inweights;inweights.push_back(0.5);inweights.push_back(0.5);
  vector<int> intype;intype.push_back(2);intype.push_back(3);
  vector<Energy> inmass(2,_rhomass[0]),inwidth(2,_rhowidth[0]);
  vector<double> inpow(2,0.0);
  Energy mpi0=getParticleData(ParticleID::pi0)->mass();
  Energy mpic=getParticleData(ParticleID::piplus)->mass();
  Energy m[3];
  m[0] = ncharged<2 ? mpi0 : mpic;
  m[1] = m[0];
  m[2] = (ncharged==0||ncharged==2) ? mpi0 : mpic;
  return new_ptr(ThreeBodyAllOnCalculator<a1SimpleDecayer>
		 (inweights,intype,inmass,inwidth,inpow,*this,ncharged,m[0],m[1],m[2]));
}


void a1SimpleDecayer::dataBaseOutput(ofstream & output,
					    bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  output << "newdef " << name() << ":LocalParameters " << _localparameters << "\n";
  output << "newdef " << name() << ":Coupling " << _coupling*GeV << "\n";
  output << "newdef " << name() << ":OneMax   " <<   _onemax << "\n";
  output << "newdef " << name() << ":TwoMax   " <<   _twomax << "\n";
  output << "newdef " << name() << ":ThreeMax " << _threemax << "\n";
  for(unsigned int ix=0;ix<_rhomass.size();++ix) {
    if(ix<3) output << "newdef    " << name() << ":RhoMasses " << ix << " " 
		    << _rhomass[ix]/MeV << "\n";
    else     output << "insert " << name() << ":RhoMasses " << ix << " " 
		    << _rhomass[ix]/MeV << "\n";
  }
  for(unsigned int ix=0;ix<_rhowidth.size();++ix) {
    if(ix<3) output << "newdef    " << name() << ":RhoWidths " << ix << " " 
		    << _rhowidth[ix]/MeV << "\n";
    else     output << "insert " << name() << ":RhoWidths " << ix << " " 
		    << _rhowidth[ix]/MeV << "\n";
  }
  for(unsigned int ix=0;ix<_rhowgts.size();++ix) {
    if(ix<3) output << "newdef    " << name() << ":RhoWeights " << ix << " " 
		    << _rhowgts[ix] << "\n";
    else     output << "insert " << name() << ":RhoWeights " << ix << " " 
		    << _rhowgts[ix] << "\n";
  }
  for(unsigned int ix=0;ix<_onewgts.size();++ix) {
    output << "newdef " << name() << ":OneChargedWeights " 
	   << ix << " " << _onewgts[ix] << "\n";
  }
  for(unsigned int ix=0;ix<_twowgts.size();++ix) {
    output << "newdef " << name() << ":TwoChargedWeights " 
	   << ix << " " << _twowgts[ix] << "\n";
  }
  for(unsigned int ix=0;ix<_threewgts.size();++ix) {
    output << "newdef " << name() << ":ThreeChargedWeights " 
	   << ix << " " << _threewgts[ix] << "\n";
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}

// functions to return the Breit-Wigners
Complex a1SimpleDecayer::rhoFormFactor(Energy2 q2,int ires) const {
  Complex output(0.),norm(0.);
  for(unsigned int ix=0,N=min(3,int(_rhowgts.size()));ix<N;++ix)
    norm+=_rhowgts[ix];
  if(ires<0) {
    for(unsigned int ix=0,N=min(3,int(_rhowgts.size()));ix<N;++ix) {
      output+=_rhowgts[ix]*rhoBreitWigner(q2,ix);
    }
  }
  else {
    unsigned int temp(ires);
    if(temp<_rhowgts.size()&&temp<3)
      output=_rhowgts[temp]*rhoBreitWigner(q2,temp);
    else
      output=0.;
  }
  return output/norm;
}
#line 1 "./a1ThreePionDecayer.cc"
// -*- C++ -*-
//
// a1ThreePionDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the a1ThreePionDecayer class.
//

#include "a1ThreePionDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Interface/Switch.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "Herwig/PDT/ThreeBodyAllOnCalculator.h"
#include "Herwig/Decay/GeneralDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

a1ThreePionDecayer::a1ThreePionDecayer() 
  : _rhomass(1,0.7761*GeV), _rhowidth(1,0.1445*GeV), _sigmamass(0.8*GeV),
    _sigmawidth(0.8*GeV), _psigma(ZERO), _mpi(ZERO), _mpi2(ZERO),
    _lambda2(1.2*GeV2), _a1mass2(1.23*1.23*GeV2),
    _zsigma(0.), _zmag(1.3998721), _zphase(0.43585036),
    _rhomag(1,1.), _rhophase(1,0.), _coupling(90.44), 
    _localparameters(true), _zerowgts(3), _onewgts(7), _twowgts(7),
    _threewgts(8) ,_zeromax(19.144), _onemax(7.83592), 
    _twomax(6.64804), _threemax(6.66296) {
  // weights for the different channels
  _threewgts[0] = 0.153071;
  _threewgts[1] = 0.165741;
  _threewgts[2] = 0.107509;
  _threewgts[3] = 0.10275 ;
  _threewgts[4] = 0.109738;
  _threewgts[5] = 0.11254 ;
  _threewgts[6] = 0.125344;
  _threewgts[7] = 0.123307;
  _onewgts[0] = 0.19616 ;
  _onewgts[1] = 0.191408;
  _onewgts[2] = 0.12137 ;
  _onewgts[3] = 0.115498;
  _onewgts[4] = 0.12729 ;
  _onewgts[5] = 0.127183;
  _onewgts[6] = 0.12109 ;
  _zerowgts[0] = 0.339108;
  _zerowgts[1] = 0.335601;
  _zerowgts[2] = 0.325291;
  _twowgts[0] = 0.188163;
  _twowgts[1] = 0.192479;
  _twowgts[2] = 0.121658;
  _twowgts[3] = 0.12135 ;
  _twowgts[4] = 0.127298;
  _twowgts[5] = 0.124835;
  _twowgts[6] = 0.124217;
  // generation of intermediates
  generateIntermediates(true);
}

void a1ThreePionDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    // get the weights for the different channels
    for(unsigned int ix=0;ix<_zerowgts.size();++ix)
      _zerowgts[ix]=mode(0)->channelWeight(ix);
    for(unsigned int ix=0;ix<_onewgts.size();++ix)
      _onewgts[ix]=mode(1)->channelWeight(ix);
    for(unsigned int ix=0;ix<_twowgts.size();++ix)
      _twowgts[ix]=mode(2)->channelWeight(ix);
    for(unsigned int ix=0;ix<_threewgts.size();++ix)
      _threewgts[ix]=mode(3)->channelWeight(ix);
    // get the maximum weight
    _zeromax  = mode(0)->maxWeight();
    _onemax   = mode(1)->maxWeight();
    _twomax   = mode(2)->maxWeight();
    _threemax = mode(3)->maxWeight();
  }
}

void a1ThreePionDecayer::doinit() {
  DecayIntegrator::doinit();
  // particles we need for the external state
  tPDPtr a1p = getParticleData(ParticleID::a_1plus);
  tPDPtr a10 = getParticleData(ParticleID::a_10);
  tPDPtr pip = getParticleData(ParticleID::piplus);
  tPDPtr pim = getParticleData(ParticleID::piminus);
  tPDPtr pi0 = getParticleData(ParticleID::pi0);
  // possible intermediate particles
  // the different rho resonances
  tPDPtr rhop[3] = {getParticleData(213),getParticleData(100213),
		    getParticleData(30213)};
  tPDPtr rho0[3] = {getParticleData(113),getParticleData(100113),
		    getParticleData(30113)};
  tPDPtr rhom[3] = {getParticleData(-213),getParticleData(-100213),
		    getParticleData(-30213)};
  // the sigma
  tPDPtr sigma = getParticleData(9000221);
  // set up the phase space integration
  tPDVector extpart(4);
  DecayPhaseSpaceModePtr mode;
  DecayPhaseSpaceChannelPtr newchannel;
  // decay mode a_0 -> pi0 pi0 pi0
  extpart[0]=a10;
  extpart[1]=pi0;
  extpart[2]=pi0;
  extpart[3]=pi0;
  if(sigma) {
    mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a10,0,0.0,-1,1);
    newchannel->addIntermediate(sigma,0,0.0,2,3);
    mode->addChannel(newchannel);
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a10,0,0.0,-1,2);
    newchannel->addIntermediate(sigma,0,0.0,1,3);
    mode->addChannel(newchannel);
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a10,0,0.0,-1,3);
    newchannel->addIntermediate(sigma,0,0.0,1,2);
    mode->addChannel(newchannel);
  }
  if(_zerowgts.size()!=mode->numberChannels()) 
    _zerowgts=vector<double>(mode->numberChannels(),1./mode->numberChannels());
  addMode(mode,_zeromax,_zerowgts);
  // decay mode a_1+ -> pi+ pi0 pi0
  extpart[0]=a1p;
  extpart[1]=pi0;
  extpart[2]=pi0;
  extpart[3]=pip;
  mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
  for(unsigned int ix=0;ix<3;++ix) {
    if(!rhop[ix]) continue;
    // first rho+ channel
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,1);
    newchannel->addIntermediate(rhop[ix],0,0.0,2,3);
    mode->addChannel(newchannel);
    // second rho+ channel
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,2);
    newchannel->addIntermediate(rhop[ix],0,0.0,1,3);
    mode->addChannel(newchannel);
  }
  // the sigma channel
  if(sigma) {
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,3);
    newchannel->addIntermediate(sigma,0,0.0,1,2);
    mode->addChannel(newchannel);
  }
  if(_onewgts.size()!=mode->numberChannels()) 
    _onewgts=vector<double>(mode->numberChannels(),1./mode->numberChannels());
  addMode(mode,_onemax,_onewgts);
  // decay mode a_1 -> pi+ pi- pi0
  extpart[0]=a10;
  extpart[1]=pip;
  extpart[2]=pim;
  extpart[3]=pi0;
  mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
  for(unsigned int ix=0;ix<3;++ix) {
    if(!rhop[ix]) continue;
    // first rho channel
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a10,0,0.0,-1,1);
    newchannel->addIntermediate(rhom[ix],0,0.0,2,3);
    mode->addChannel(newchannel);
    // second channel
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a10,0,0.0,-1,2);
    newchannel->addIntermediate(rhop[ix],0,0.0,1,3);
    mode->addChannel(newchannel);
  }
  // sigma channel
  if(sigma) {
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a10,0,0.0,-1,3);
    newchannel->addIntermediate(sigma,0,0.0,1,2);
    mode->addChannel(newchannel);
  }
  if(_twowgts.size()!=mode->numberChannels()) 
    _twowgts=vector<double>(mode->numberChannels(),1./mode->numberChannels());
  addMode(mode,_twomax,_twowgts);
  // decay mode a_1+ -> pi+ pi+ pi-
  extpart[0]=a1p;
  extpart[1]=pip;
  extpart[2]=pip;
  extpart[3]=pim;
  mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
  for(unsigned int ix=0;ix<3;++ix) {
    // the neutral rho channels
    if(!rho0[ix]) continue;
    // first channel
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,1);
    newchannel->addIntermediate(rho0[ix],0,0.0,2,3);
    mode->addChannel(newchannel);
    // interchanged channel
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,2);
    newchannel->addIntermediate(rho0[ix],0,0.0,1,3);
    mode->addChannel(newchannel);
  }
  // the sigma channels
  if(sigma) {
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,1);
    newchannel->addIntermediate(sigma,0,0.0,2,3);
    mode->addChannel(newchannel);
    newchannel = new_ptr(DecayPhaseSpaceChannel(mode));
    newchannel->addIntermediate(a1p,0,0.0,-1,2);
    newchannel->addIntermediate(sigma,0,0.0,1,3);
    mode->addChannel(newchannel);
  }
  if(_threewgts.size()!=mode->numberChannels()) 
    _threewgts=vector<double>(mode->numberChannels(),1./mode->numberChannels());
  addMode(mode,_threemax,_threewgts);
  // set up the parameters 
  _mpi=getParticleData(ParticleID::piplus)->mass();
  _mpi2=sqr(_mpi);
  if(_localparameters) {
    if(_rhomass.size()<_rhocoupling.size()) {
      unsigned int itemp=_rhomass.size();
      _rhomass.resize(_rhocoupling.size());
      _rhowidth.resize(_rhocoupling.size());
      for(unsigned int ix=itemp;ix<_rhocoupling.size();++ix) {
	_rhomass[ix]=rhop[ix]->mass();
	_rhowidth[ix]=rhop[ix]->width();
      }
      // reset the intermediates in the phase space integration if needed
      resetIntermediate(sigma,_sigmamass,_sigmawidth);
      for(unsigned int iy=0;iy<_rhocoupling.size();++iy) {
	resetIntermediate(rho0[iy],_rhomass[iy],_rhowidth[iy]);
	resetIntermediate(rhop[iy],_rhomass[iy],_rhowidth[iy]);
	resetIntermediate(rhom[iy],_rhomass[iy],_rhowidth[iy]);
      }
    }
  }
  else {
    _a1mass2=sqr(getParticleData(ParticleID::a_1plus)->mass());
    if(sigma) {
      _sigmamass=sigma->mass();
      _sigmawidth=sigma->width();
    }
    _rhomass.resize(_rhocoupling.size());
    _rhowidth.resize(_rhocoupling.size());
    for(unsigned int ix=0;ix<_rhocoupling.size();++ix) {
      _rhomass[ix]=rhop[ix]->mass();
      _rhowidth[ix]=rhop[ix]->width();
    }
  }
  // parameters for the resonances
  // for the sigma
  _psigma=Kinematics::pstarTwoBodyDecay(_sigmamass,_mpi,_mpi);
  // for the rho
  _prho.resize(_rhomass.size());_hm2.resize(_rhomass.size());
  _dhdq2m2.resize(_rhomass.size());_rhoD.resize(_rhomass.size());
  for(unsigned int ix=0;ix<_rhomass.size();++ix) {
    _prho[ix]    = Kinematics::pstarTwoBodyDecay(_rhomass[ix],_mpi,_mpi);
    _hm2[ix]     = hFunction(_rhomass[ix]);
    _dhdq2m2[ix] = dhdq2Parameter(ix);
    _rhoD[ix]    = DParameter(ix);
  }
  // convert the magnitude and phase of z into a phase
  _zsigma = _zmag*Complex(cos(_zphase),sin(_zphase));
  // convert rho couplings
  for(unsigned int ix=0;ix<_rhomag.size();++ix) {
    _rhocoupling.push_back(_rhomag[ix]*Complex(cos(_rhophase[ix]),sin(_rhophase[ix])));
  }
}
  
int a1ThreePionDecayer::modeNumber(bool & cc,tcPDPtr parent,
				       const tPDVector & children) const {
  if(children.size()!=3) return -1;
  int id(parent->id());
  // check the pions
  tPDVector::const_iterator pit  = children.begin();
  tPDVector::const_iterator pend = children.end();
  int idtemp,npi0(0),npiplus(0),npiminus(0);
  for( ; pit!=pend;++pit) {
    idtemp=(**pit).id();
    if(idtemp==ParticleID::piplus)       ++npiplus;
    else if(idtemp==ParticleID::piminus) ++npiminus;
    else if(idtemp==ParticleID::pi0)     ++npi0;
  }
  int imode(-1);
  // a_1+ decay modes
  if(id==ParticleID::a_1plus) {
    cc=false;
    if(npiplus==1&&npi0==2)          imode=1;
    else if(npiplus==2&&npiminus==1) imode=3;
  }
  // a_1- modes
  else if(id==ParticleID::a_1minus) {
    cc=true;
    if(npiminus==1&&npi0==2)         imode=1;
    else if(npiminus==2&&npiplus==1) imode=3;
  }
  // a_0 modes
  else if(id==ParticleID::a_10) {
    cc=false;
    if(npiminus==1&&npiplus==1&&npi0==1) imode=2;
    else if(npi0==3)                     imode=0;
  }
  return imode;
}
  
void a1ThreePionDecayer::persistentOutput(PersistentOStream & os) const {
   os << ounit(_rhomass,GeV) << ounit(_rhowidth,GeV) << ounit(_prho,GeV) 
      << ounit(_hm2,GeV2) << ounit(_rhoD,GeV2) << _dhdq2m2 <<  ounit(_sigmamass,GeV)
      << ounit(_sigmawidth,GeV) << ounit(_psigma,GeV) << ounit(_mpi,GeV)
      << ounit(_mpi2,GeV2) << ounit(_lambda2,GeV2) << ounit(_a1mass2,GeV2) << _zsigma  
      << _rhocoupling << _coupling << _localparameters << _zerowgts << _onewgts 
      << _twowgts << _threewgts << _zeromax << _zmag << _zphase
      << _onemax << _twomax << _threemax << _coupling << _rhomag << _rhophase;
}
  
void a1ThreePionDecayer::persistentInput(PersistentIStream & is, int) {
   is >> iunit(_rhomass,GeV) >> iunit(_rhowidth,GeV) >> iunit(_prho,GeV) 
      >> iunit(_hm2,GeV2) >> iunit(_rhoD,GeV2) >> _dhdq2m2 >>  iunit(_sigmamass,GeV)
      >> iunit(_sigmawidth,GeV) >> iunit(_psigma,GeV) >> iunit(_mpi,GeV) 
      >> iunit(_mpi2,GeV2) >> iunit(_lambda2,GeV2) >> iunit(_a1mass2,GeV2) >> _zsigma
      >> _rhocoupling >> _coupling >> _localparameters >> _zerowgts >> _onewgts 
      >> _twowgts >> _threewgts >> _zeromax >> _zmag >> _zphase
      >> _onemax >> _twomax >> _threemax >> _coupling >> _rhomag >> _rhophase;
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<a1ThreePionDecayer,DecayIntegrator>
describeHerwiga1ThreePionDecayer("Herwig::a1ThreePionDecayer", "HwVMDecay.so");
  
void a1ThreePionDecayer::Init() {
    
  static ClassDocumentation<a1ThreePionDecayer> documentation
    ("The a1ThreePionDecayer class is designed to decay the a_1 "
     "resonance to three pions using a model based on that used in the modelling "
     "of tau->4 pions.",
     "The decay of the $a_1$ resonance to three pions uses a model based on"
     "tau to four pions, \\cite{Bondar:2002mw}.",
     "%\\cite{Bondar:2002mw}\n"
     "\\bibitem{Bondar:2002mw}\n"
     "  A.~E.~Bondar, S.~I.~Eidelman, A.~I.~Milstein, T.~Pierzchala, N.~I.~Root, Z.~Was and M.~Worek,\n"
     "   ``Novosibirsk hadronic currents for tau --> 4pi channels of tau decay\n"
     "  %library TAUOLA,''\n"
     "  Comput.\\ Phys.\\ Commun.\\  {\\bf 146}, 139 (2002)\n"
     "  [arXiv:hep-ph/0201149].\n"
     "  %%CITATION = CPHCB,146,139;%%\n"
     );

  static Switch<a1ThreePionDecayer,bool> interfaceLocalParameters
    ("LocalParameters",
     "Use local values of the intermediate resonances masses and widths",
     &a1ThreePionDecayer::_localparameters, true, false, false);
  static SwitchOption interfaceLocalParametersLocal
    (interfaceLocalParameters,
     "Local",
     "Use the local values",
     true);
  static SwitchOption interfaceLocalParametersDefault
    (interfaceLocalParameters,
     "ParticleData",
     "Use the values from the particleData objects",
     false);

  static Parameter<a1ThreePionDecayer,double> interfaceCoupling
    ("Coupling",
     "The overall coupling for the decay",
     &a1ThreePionDecayer::_coupling, 90.44, 0.0, 1000.0,
     false, false, true);

  static ParVector<a1ThreePionDecayer,Energy> interfacerhomass
    ("RhoMasses",
     "The masses of the different rho resonnaces",
     &a1ThreePionDecayer::_rhomass,
     GeV, 0, ZERO, ZERO, 10000*GeV, false, false, true);

  static ParVector<a1ThreePionDecayer,Energy> interfacerhowidth
    ("RhoWidths",
     "The widths of the different rho resonnaces",
     &a1ThreePionDecayer::_rhowidth,
     GeV, 0, ZERO, ZERO, 10000*GeV, false, false, true);

  static ParVector<a1ThreePionDecayer,double> interfaceRhoMagnitude
    ("RhoMagnitude",
     "The magnitude of the rho couplings",
     &a1ThreePionDecayer::_rhomag, -1, 1.0, 0.0, 10.0,
     false, false, Interface::limited);

  static ParVector<a1ThreePionDecayer,double> interfaceRhoPhase
    ("RhoPhase",
     "The phase of the rho coupling",
     &a1ThreePionDecayer::_rhophase, -1, 0., 0.0, 2.*Constants::pi,
     false, false, Interface::limited);

  static Parameter<a1ThreePionDecayer,Energy2> interfaceLambda2
    ("Lambda2",
     "The value of the mass scale squared to use in the form-factor",
     &a1ThreePionDecayer::_lambda2, GeV2, 1.2*GeV2, 0.0001*GeV2, 10.0*GeV2,
     false, false, true);

  static Parameter<a1ThreePionDecayer,Energy2> interfacea1mass2
    ("a1mass2",
     "The local value of the square of the a_1 mass",
     &a1ThreePionDecayer::_a1mass2, GeV2, 1.5129*GeV2, 0.5*GeV2, 10.0*GeV2,
     false, false, true);

  static Parameter<a1ThreePionDecayer,Energy> interfaceSigmaMass
    ("SigmaMass",
     "The local value of the sigma mass",
     &a1ThreePionDecayer::_sigmamass, GeV, 0.8*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static Parameter<a1ThreePionDecayer,Energy> interfaceSigmaWidth
    ("SigmaWidth",
     "The local value of the sigma width",
     &a1ThreePionDecayer::_sigmawidth, GeV, 0.8*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static ParVector<a1ThreePionDecayer,double> interfacezerowgts
    ("AllNeutralWeights",
     "The weights of the different channels to use for the integration of"
     " the decay a_1^0->pi0pi0pi0",
     &a1ThreePionDecayer::_zerowgts,
     0, 0, 0, -10000, 10000, false, false, true);

  static ParVector<a1ThreePionDecayer,double> interfaceonewgts
    ("OneChargedWeights",
     "The weights of the different channels to use for the integration of"
     " the decay a_1^+->pi+pi0pi0",
     &a1ThreePionDecayer::_onewgts,
     0, 0, 0, -10000, 10000, false, false, true);

  static ParVector<a1ThreePionDecayer,double> interfacetwowgts
    ("TwoChargedWeights",
     "The weights of the different channels to use for the integration of"
     " the decay a_1^0->pi+pi-pi0",
     &a1ThreePionDecayer::_twowgts,
     0, 0, 0, -10000, 10000, false, false, true);

  static ParVector<a1ThreePionDecayer,double> interfacethreewgts
    ("ThreeChargedWeights",
     "The weights of the different channels to use for the integration of"
     " the decay a_1^+->pi+pi+pi-",
     &a1ThreePionDecayer::_threewgts,
     0, 0, 0, -10000, 10000, false, false, true);

  static Parameter<a1ThreePionDecayer,double> interfaceZeroMax
    ("ZeroMax",
     "The maximum weight for the integration fo the channel a_1^0->pi0pi0pi0",
     &a1ThreePionDecayer::_zeromax, 107.793, 0.0, 10000.0,
     false, false, true);

  static Parameter<a1ThreePionDecayer,double> interfaceOneMax
    ("OneMax",
     "The maximum weight for the integration fo the channel a_1^+->pi+pi0pi0",
     &a1ThreePionDecayer::_onemax, 1088.96, 0.0, 10000.0,
     false, false, true);

  static Parameter<a1ThreePionDecayer,double> interfaceTwoMax
    ("TwoMax",
     "The maximum weight for the integration fo the channel a_1^0->pi+pi-pi0",
     &a1ThreePionDecayer::_twomax, 1750.73, 0.0, 10000.0,
     false, false, true);

  static Parameter<a1ThreePionDecayer,double> interfaceThreeMax
    ("ThreeMax",
     "The maximum weight for the integration fo the channel a_1^+->pi+pi+pi-",
     &a1ThreePionDecayer::_threemax, 739.334, 0.0, 10000.0,
     false, false, true);

  static Parameter<a1ThreePionDecayer,double> interfaceSigmaMagnitude
    ("SigmaMagnitude",
     "magnitude of the relative sigma coupling",
     &a1ThreePionDecayer::_zmag, 1.3998721, 0.0, 10.0e20,
     false, false, true);

  static Parameter<a1ThreePionDecayer,double> interfaceSigmaPhase
    ("SigmaPhase",
     "phase of the relative sigma coupling",
     &a1ThreePionDecayer::_zphase, 0.43585036, 0.0, Constants::twopi,
     false, false, true);
}

double a1ThreePionDecayer::me2(const int ichan,
			       const Particle & inpart,
			       const ParticleVector & decay,
			       MEOption meopt) const {
  useMe();
  if(!ME())
    ME(new_ptr(GeneralDecayMatrixElement(PDT::Spin1,PDT::Spin0,PDT::Spin0,PDT::Spin0)));
  if(meopt==Initialize) {
    VectorWaveFunction::calculateWaveFunctions(_vectors,_rho,
						const_ptr_cast<tPPtr>(&inpart),
						incoming,false);
  }
  if(meopt==Terminate) {
    VectorWaveFunction::constructSpinInfo(_vectors,const_ptr_cast<tPPtr>(&inpart),
					  incoming,true,false);
    // set up the spin information for the decay products
    for(unsigned int ix=0;ix<3;++ix)
      ScalarWaveFunction::constructSpinInfo(decay[ix],outgoing,true);
    return 0.;
  }
  // momentum of the incoming particle
  Lorentz5Momentum Q=inpart.momentum();
  // momenta of the intermediates
  Energy2 s1=(decay[1]->momentum()+decay[2]->momentum()).m2();
  Energy2 s2=(decay[0]->momentum()+decay[2]->momentum()).m2();
  Energy2 s3=(decay[0]->momentum()+decay[1]->momentum()).m2();
  Energy2 dot01=Q*decay[0]->momentum();
  Energy2 dot02=Q*decay[1]->momentum();
  Energy2 dot03=Q*decay[2]->momentum();
  // vector for the output
  LorentzVector<complex<Energy3> > output;
  // a_10 -> pi0pi0pi0
  if(imode()==0) {
    //the breit-wigners
    Complex sig1=sigmaBreitWigner(s1);
    Complex sig2=sigmaBreitWigner(s2);
    Complex sig3=sigmaBreitWigner(s3);
    // compute the vector
    LorentzPolarizationVectorE tmpoutput;
    if(ichan<0) {
      tmpoutput= sig1*(decay[0]->momentum())+sig2*(decay[1]->momentum())
	+sig3*(decay[2]->momentum());
    }
    else if(ichan==0) tmpoutput=sig1*(decay[0]->momentum());
    else if(ichan==1) tmpoutput=sig2*(decay[1]->momentum());
    else if(ichan==2) tmpoutput=sig3*(decay[2]->momentum());
    // the coupling z and identical particle factor
    output = tmpoutput * _zsigma* 1./sqrt(6.) *Q.mass2();
  }
  // a_1+ -> pi0pi0pi+
  else if(imode()==1) {
    // scalar propagator
    Complex sig1 = sigmaBreitWigner(s3);
    // sigma terms
    if(ichan<0||ichan==6) 
      output = _zsigma*Q.mass2()*sig1*decay[2]->momentum();
    // the rho terms
    for(int ix=0,N=_rhocoupling.size();ix<N;++ix) {
      Complex rho1=_rhocoupling[ix]*rhoBreitWigner(s1,ix);
      Complex rho2=_rhocoupling[ix]*rhoBreitWigner(s2,ix);
      if(ichan<0||ichan==2*ix) {
	output +=rho1*(dot03*(decay[1]->momentum())-
		       dot02*(decay[2]->momentum()));
      }
      if(ichan<0||ichan==2*ix+1){
	output +=rho2*(dot03*(decay[0]->momentum())-
		       dot01*(decay[2]->momentum()));
      }
    }
    // the identical particle factor
    output *= 1./sqrt(2.);
  }
  // a_10->pi+pi-pi0
  else if(imode()==2) {
    // the sigma terms
    Complex sig1=sigmaBreitWigner(s3);
    if(ichan<0||ichan==6)
      output = _zsigma*Q.mass2()*sig1*decay[2]->momentum();
    // rho terms
    for(int ix=0,N=_rhocoupling.size();ix<N;++ix) {
      Complex rho1=_rhocoupling[ix]*rhoBreitWigner(s1,ix);
      Complex rho2=_rhocoupling[ix]*rhoBreitWigner(s2,ix);
      if(ichan<0||ichan==2*ix) {
	output+=rho1*(dot03*(decay[1]->momentum())
		      -dot02*(decay[2]->momentum()));
      }
      if(ichan<0||ichan==2*ix+1) {
	output+=rho2*(dot03*(decay[0]->momentum())
		      -dot01*(decay[2]->momentum()));
      }
    }
  }
  // a1+ -> pi+pi+pi-
  else if(imode()==3) {
    // the scalar propagators 
    Complex sig1=sigmaBreitWigner(s1);
    Complex sig2=sigmaBreitWigner(s2);
    // sigma terms
    LorentzPolarizationVectorE tmpoutput;
    if(ichan<0||ichan==6) tmpoutput+=sig1*(decay[0]->momentum());
    if(ichan<0||ichan==7) tmpoutput+=sig2*(decay[1]->momentum());
    output = tmpoutput * _zsigma * Q.mass2();
    // rho terms
    for(int ix=0,N=_rhocoupling.size();ix<N;++ix) {
      Complex rho1 = _rhocoupling[ix]*rhoBreitWigner(s1,ix);
      Complex rho2 = _rhocoupling[ix]*rhoBreitWigner(s2,ix);
      if(ichan<0||ichan==2*ix) {
	output-=rho1*( dot03*(decay[1]->momentum())-
		       dot02*(decay[2]->momentum()));
      }
      if(ichan<0||ichan==2*ix+1) {
	output-=rho2*( dot03*(decay[0]->momentum())-
		       dot01*(decay[2]->momentum()));
      }
    }
    // the identical particle factor
    output *= 1./sqrt(2.);
  }
  // form-factor
  LorentzPolarizationVector outputFinal 
    = output * a1FormFactor(Q.mass2())*_coupling/(Q.mass()*sqr(_rhomass[0]));
  // compute the matrix element
  for(unsigned int ix=0;ix<3;++ix)
    (*ME())(ix,0,0,0)=outputFinal.dot(_vectors[ix]);
  // return the answer
  double out = ME()->contract(_rho).real();
  // test of the answer
//   double test = threeBodyMatrixElement(imode(),sqr(inpart.mass()),s3,s2,s1,
// 				       decay[0]->mass(),decay[1]->mass(), 
// 				       decay[2]->mass());
//   if(ichan<0) cerr << "testing matrix element " << inpart.PDGName() << " -> "
// 		   << decay[0]->PDGName() << " " << decay[1]->PDGName() << " "
// 		   << decay[2]->PDGName() << " " << out << " " << test << " " 
// 		   << (out-test)/(out+test) << "\n"; 
  return out;
}

// matrix element for the running a_1 width
double a1ThreePionDecayer::
threeBodyMatrixElement(const int iopt,const Energy2 q2, const Energy2 s3,
		       const Energy2 s2,const Energy2 s1, const Energy m1, 
		       const Energy m2 ,const Energy m3) const {
  Energy6 meout(0.*pow<3,1>(GeV2));
  Energy2 m12(sqr(m1)),m22(sqr(m2)),m32(sqr(m3));
  Energy2 dot01(q2-s1+m12),dot02(q2-s2+m22),dot03(q2-s3+m32),
    dot12(s3-m12-m22),dot13(s2-m12-m32),dot23(s1-m22-m32);
  if(iopt==0) {
    Complex sig1=sigmaBreitWigner(s1);
    Complex sig2=sigmaBreitWigner(s2);
    Complex sig3=sigmaBreitWigner(s3);
    Energy2 metemp = 
      real(0.25*sig1*conj(sig1)*lambda(q2,s1,m12)/q2+
	   0.25*sig2*conj(sig2)*lambda(q2,s2,m22)/q2+
	   0.25*sig3*conj(sig3)*lambda(q2,s3,m32)/q2+
 	   sig1*conj(sig2)*(-dot12+0.5*dot01*dot02/q2)+
 	   sig1*conj(sig3)*(-dot13+0.5*dot01*dot03/q2)+
 	   sig2*conj(sig3)*(-dot23+0.5*dot02*dot03/q2));
    meout = metemp*real(_zsigma*conj(_zsigma))/6.*sqr(q2);
  }
  else if(iopt==1||iopt==2) {
    // the sigma terms
    Complex sig=sigmaBreitWigner(s3);
    Complex rho1,rho2;
    for(int ix=0,N=_rhocoupling.size();ix<N;++ix) {
      rho1 += _rhocoupling[ix]*rhoBreitWigner(s1,ix);
      rho2 += _rhocoupling[ix]*rhoBreitWigner(s2,ix);
    }
    meout =
      0.25*lambda(q2,m32,s3)*q2*norm(_zsigma*sig)+
      0.25*norm(rho1)*(dot23*dot02*dot03-m32*sqr(dot02)-m22*sqr(dot03))+
      0.25*norm(rho2)*(dot13*dot01*dot03-m32*sqr(dot01)-m12*sqr(dot03))-
      0.5*real(_zsigma*sig*conj(rho1))*q2*(dot03*dot23-2.*m32*dot02)-
      0.5*real(_zsigma*sig*conj(rho2))*q2*(dot03*dot13-2.*m32*dot01)-
      0.25*real(rho1*conj(rho2))*(sqr(dot03)*dot12-dot03*dot02*dot13
				  -dot03*dot01*dot23+2.*m32*dot02*dot01);
    if(iopt==1) meout *= 0.5;
  }
  else if(iopt==3) {
    Complex sig1=sigmaBreitWigner(s1);
    Complex sig2=sigmaBreitWigner(s2);
    Complex rho1,rho2;
    for(int ix=0,N=_rhocoupling.size();ix<N;++ix) {
      rho1 += _rhocoupling[ix]*rhoBreitWigner(s1,ix);
      rho2 += _rhocoupling[ix]*rhoBreitWigner(s2,ix);
    }
    meout =
      0.25*lambda(q2,m12,s1)*q2*norm(_zsigma*sig1)+
      0.25*lambda(q2,m22,s2)*q2*norm(_zsigma*sig2)+
      0.25*norm(rho1)*(dot23*dot02*dot03-m32*sqr(dot02)-m22*sqr(dot03))+
      0.25*norm(rho2)*(dot13*dot01*dot03-m32*sqr(dot01)-m12*sqr(dot03))-
      0.25*real(rho1*conj(rho2))*(sqr(dot03)*dot12-dot03*dot02*dot13
				  -dot03*dot01*dot23+2.*m32*dot02*dot01)-
      real(_zsigma*sig1*conj(_zsigma*sig2))*q2*(q2*dot12-0.5*dot02*dot01)+
      0.5*real(_zsigma*sig1*conj(rho1))*q2*(dot03*dot12-dot02*dot13)+
      0.5*real(_zsigma*sig2*conj(rho1))*q2*(2.*dot03*m22-dot02*dot23)+
      0.5*real(_zsigma*sig1*conj(rho2))*q2*(2.*dot03*m12-dot01*dot13)+
      0.5*real(_zsigma*sig2*conj(rho2))*q2*(dot03*dot12-dot01*dot23);
    meout *= 0.5;
  }
  return meout*a1FormFactor(q2)*sqr(_coupling/sqr(_rhomass[0]))/q2/3.;
}

WidthCalculatorBasePtr 
a1ThreePionDecayer::threeBodyMEIntegrator(const DecayMode & dm) const {
  ParticleMSet::const_iterator pit  = dm.products().begin();
  ParticleMSet::const_iterator pend = dm.products().end();
  int ncharged=0;
  for( ; pit!=pend;++pit) {
    if(abs((**pit).id())==ParticleID::piplus) ++ncharged;
  }
  // integrator to perform the integral
  vector<double> inweights(2,0.5);
  vector<int> intype;intype.push_back(2);intype.push_back(3);
  vector<Energy> inmass(2,_rhomass[0]),inwidth(2,_rhowidth[0]);
  vector<double> inpow(2,0.0);
  Energy m[3];
  Energy mpi0=getParticleData(ParticleID::pi0)->mass();
  Energy mpic=getParticleData(ParticleID::piplus)->mass();
  m[0] = ncharged<2 ? mpi0 : mpic;
  m[1] = m[0];
  m[2] = (ncharged==0||ncharged==2) ? mpi0 : mpic;
  return new_ptr(ThreeBodyAllOnCalculator<a1ThreePionDecayer>
		 (inweights,intype,inmass,inwidth,inpow,*this,ncharged,m[0],m[1],m[2]));
}

// output the setup information for the particle database
void a1ThreePionDecayer::dataBaseOutput(ofstream & output,
					    bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  output << "newdef " << name() << ":LocalParameters " << _localparameters << "\n";
  output << "newdef " << name() << ":Coupling " << _coupling     << "\n";
  output << "newdef " << name() << ":Lambda2 "  << _lambda2/GeV2 << "\n";
  output << "newdef " << name() << ":a1mass2 "  << _a1mass2/GeV2 << "\n";
  output << "newdef " << name() << ":SigmaMass "  << _sigmamass/GeV  << "\n";
  output << "newdef " << name() << ":SigmaWidth " << _sigmawidth/GeV << "\n";
  output << "newdef " << name() << ":SigmaMagnitude " << _zmag << "\n";
  output << "newdef " << name() << ":SigmaPhase " << _zphase << "\n";
  for(unsigned int ix=0;ix<_rhomag.size();++ix) {
    if(ix<1) output << "newdef    " << name() << ":RhoMagnitude " << ix << " " 
		    << _rhomag[ix] << "\n";
    else     output << "insert " << name() << ":RhoMagnitude " << ix << " " 
		    << _rhomag[ix] << "\n";
  }
  for(unsigned int ix=0;ix<_rhophase.size();++ix) {
    if(ix<1) output << "newdef    " << name() << ":RhoPhase " << ix << " " 
		    << _rhophase[ix] << "\n";
    else     output << "insert " << name() << ":RhoPhase " << ix << " " 
		    << _rhophase[ix] << "\n";
  }
  for(unsigned int ix=0;ix<_rhomass.size();++ix) {
    if(ix<1) output << "newdef    " << name() << ":RhoMasses " << ix << " " 
		    << _rhomass[ix]/GeV << "\n";
    else     output << "insert " << name() << ":RhoMasses " << ix << " " 
		    << _rhomass[ix]/GeV << "\n";
  }
  for(unsigned int ix=0;ix<_rhowidth.size();++ix) {
    if(ix<1) output << "newdef    " << name() << ":RhoWidths " << ix << " " 
		    << _rhowidth[ix]/GeV << "\n";
    else     output << "insert " << name() << ":RhoWidths " << ix << " " 
		    << _rhowidth[ix]/GeV << "\n";
  }
  // integration weights for the different channels
  for(unsigned int ix=0;ix<_zerowgts.size();++ix) {
    output << "newdef " << name() << ":AllNeutralWeights " 
	   << ix << " " << _zerowgts[ix] << "\n";
  }
  for(unsigned int ix=0;ix<_onewgts.size();++ix) {
    output << "newdef " << name() << ":OneChargedWeights " 
	   << ix << " " << _onewgts[ix] << "\n";
  }
  for(unsigned int ix=0;ix<_twowgts.size();++ix) {
    output << "newdef " << name() << ":TwoChargedWeights " 
	   << ix << " " << _twowgts[ix] << "\n";
  }
  for(unsigned int ix=0;ix<_threewgts.size();++ix) {
    output << "newdef " << name() << ":ThreeChargedWeights " 
	   << ix << " " << _threewgts[ix] << "\n";
  }
  output << "newdef " << name() << ":ZeroMax "  << _zeromax  << "\n";
  output << "newdef " << name() << ":OneMax "   << _onemax   << "\n";
  output << "newdef " << name() << ":TwoMax "   << _twomax   << "\n";
  output << "newdef " << name() << ":ThreeMax " << _threemax << "\n";
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}
#line 1 "./PVectorMesonVectorPScalarDecayer.cc"
// -*- C++ -*-
//
// PVectorMesonVectorPScalarDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the PVectorMesonVectorPScalarDecayer class.
//

#include "PVectorMesonVectorPScalarDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "Herwig/Decay/GeneralDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;
 
void PVectorMesonVectorPScalarDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    for(unsigned int ix=0;ix<_incoming.size();++ix)
      if(mode(ix)) _maxweight[ix] = mode(ix)->maxWeight();
  }
}

void PVectorMesonVectorPScalarDecayer::doinit() {
  DecayIntegrator::doinit();
  // check consistence of the parameters
  unsigned int isize=_incoming.size();
  if(isize!=_outgoingV.size()||isize!=_outgoingP.size()||
     isize!=_maxweight.size()||isize!=_coupling.size())
    throw InitException() << "Inconsistent parameters in "
			  << "PVectorMesonVectorPScalarDecayer::doinit()" 
			  << Exception::abortnow;
  // set up the integration channels
  vector<double> wgt(0);
  tPDVector extpart(3);
  DecayPhaseSpaceModePtr mode;
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    extpart[0]=getParticleData(_incoming[ix]);
    extpart[1]=getParticleData(_outgoingV[ix]);
    extpart[2]=getParticleData(_outgoingP[ix]);
    if(extpart[0]&&extpart[1]&&extpart[2]) 
      mode=new_ptr(DecayPhaseSpaceMode(extpart,this));
    else
      mode=DecayPhaseSpaceModePtr();
    addMode(mode,_maxweight[ix],wgt);
  }
}

PVectorMesonVectorPScalarDecayer::PVectorMesonVectorPScalarDecayer() 
  : _coupling(67), _incoming(67), _outgoingV(67), _outgoingP(67), 
    _maxweight(67) {
  // decay mode h'_1 to K K*
  _incoming[0] =  10333; _outgoingV[0] =  313; _outgoingP[0] = -311; 
  _coupling[0] = 4.889/GeV; _maxweight[0] = 11.; 
  _incoming[1] =  10333; _outgoingV[1] =  323; _outgoingP[1] = -321; 
  _coupling[1] = 4.156/GeV; _maxweight[1] = 11.; 
  // decay mode h_1 to rho
  _incoming[2] = 10223; _outgoingV[2] =  113; _outgoingP[2] =  111; 
  _coupling[2] = 4.411/GeV; _maxweight[2] = 6.; 
  _incoming[3] = 10223; _outgoingV[3] = -213; _outgoingP[3] =  211; 
  _coupling[3] = 4.411/GeV; _maxweight[3] = 6.; 
  // decay mode b_1 to omega pi
  _incoming[4] =  10113; _outgoingV[4] =  223; _outgoingP[4] =  111; 
  _coupling[4] = 3.862/GeV; _maxweight[4] = 7.5; 
  _incoming[5] =  10213; _outgoingV[5] =  223; _outgoingP[5] =  211; 
  _coupling[5] = 3.862/GeV; _maxweight[5] = 7.5; 
  // decay mode b_1^+ to gamma pi
  _incoming[6] =  10213; _outgoingV[6] =   22; _outgoingP[6] =  211; 
  _coupling[6] = 0.195/GeV; _maxweight[6] = 2.; 
  // decay mode D'_s1 to D*K
  _incoming[7] =  20433; _outgoingV[7] =  413; _outgoingP[7] =  311; 
  _coupling[7] = 0.161/GeV; _maxweight[7] = 2.5; 
  _incoming[8] =  20433; _outgoingV[8] =  423; _outgoingP[8] =  321; 
  _coupling[8] = 0.161/GeV; _maxweight[8] = 2.5; 
  // decay mode B'_s1 to B*K
  _incoming[9] =  20533; _outgoingV[9] =  513; _outgoingP[9] = -311; 
  _coupling[9] = 0.389/GeV; _maxweight[9] = 2.1; 
  _incoming[10] =  20533; _outgoingV[10] =  523; _outgoingP[10] = -321; 
  _coupling[10] = 0.389/GeV; _maxweight[10] = 2.1; 
  // decay mode K_1 to rho K
  _incoming[11] =  10323; _outgoingV[11] =  213; _outgoingP[11] =  311; 
  _coupling[11] = 4.98/GeV; _maxweight[11] = 3.5; 
  _incoming[12] =  10323; _outgoingV[12] =  113; _outgoingP[12] =  321; 
  _coupling[12] = 3.40/GeV; _maxweight[12] = 3.5; 
  _incoming[13] =  10313; _outgoingV[13] = -213; _outgoingP[13] =  321; 
  _coupling[13] = 4.87/GeV; _maxweight[13] = 3.5; 
  _incoming[14] =  10313; _outgoingV[14] =  113; _outgoingP[14] =  311; 
  _coupling[14] = 3.55/GeV; _maxweight[14] = 3.5; 
  // decay mode K'_1 to rho K
  _incoming[15] =  20323; _outgoingV[15] =  213; _outgoingP[15] =  311; 
  _coupling[15] = 0.97/GeV; _maxweight[15] = 6.5; 
  _incoming[16] =  20323; _outgoingV[16] =  113; _outgoingP[16] =  321; 
  _coupling[16] = 0.69/GeV; _maxweight[16] = 6.5; 
  _incoming[17] =  20313; _outgoingV[17] = -213; _outgoingP[17] =  321; 
  _coupling[17] = 0.97/GeV; _maxweight[17] = 6.5; 
  _incoming[18] =  20313; _outgoingV[18] =  113; _outgoingP[18] =  311; 
  _coupling[18] = 0.707/GeV; _maxweight[18] = 6.5; 
  // decay mode K_1 to omega K
  _incoming[19] =  10323; _outgoingV[19] =  223; _outgoingP[19] =  321; 
  _coupling[19] = 4.76/GeV; _maxweight[19] = 7.5; 
  _incoming[20] =  10313; _outgoingV[20] =  223; _outgoingP[20] =  311; 
  _coupling[20] = 6.0/GeV; _maxweight[20] = 7.5; 
  // decay mode K'_1 to omega K
  _incoming[21] =  20323; _outgoingV[21] =  223; _outgoingP[21] =  321; 
  _coupling[21] = 0.600/GeV; _maxweight[21] = 8.; 
  _incoming[22] =  20313; _outgoingV[22] =  223; _outgoingP[22] =  311; 
  _coupling[22] = 0.600/GeV; _maxweight[22] = 8.; 
  // decay mode K_1 to K* pi
  _incoming[23] =  10323; _outgoingP[23] =  211; _outgoingV[23] =  313; 
  _coupling[23] = 0.941/GeV; _maxweight[23] = 8.5; 
  _incoming[24] =  10323; _outgoingP[24] =  111; _outgoingV[24] =  323; 
  _coupling[24] = 0.656/GeV; _maxweight[24] = 8.5; 
  _incoming[25] =  10313; _outgoingP[25] = -211; _outgoingV[25] =  323; 
  _coupling[25] = 0.932/GeV; _maxweight[25] = 8.5; 
  _incoming[26] =  10313; _outgoingP[26] =  111; _outgoingV[26] =  313; 
  _coupling[26] = 0.658/GeV; _maxweight[26] = 8.5; 
  // decay mode K'_1 to K* pi
  _incoming[27] =  20323; _outgoingP[27] =  211; _outgoingV[27] =  313; 
  _coupling[27] = 2.845/GeV; _maxweight[27] = 12.; 
  _incoming[28] =  20323; _outgoingP[28] =  111; _outgoingV[28] =  323; 
  _coupling[28] = 1.99/GeV; _maxweight[28] = 12.; 
  _incoming[29] =  20313; _outgoingP[29] = -211; _outgoingV[29] =  323; 
  _coupling[29] = 2.84/GeV; _maxweight[29] = 12.; 
  _incoming[30] =  20313; _outgoingP[30] =  111; _outgoingV[30] =  313; 
  _coupling[30] = 2.00/GeV; _maxweight[30] = 12.; 
  // decaymode D_1 to D* pi
  _incoming[31] =  10423; _outgoingP[31] = -211; _outgoingV[31] =  413; 
  _coupling[31] = 0.489/GeV; _maxweight[31] = 3.; 
  _incoming[32] =  10423; _outgoingP[32] =  111; _outgoingV[32] =  423; 
  _coupling[32] = 0.347/GeV; _maxweight[32] = 3.; 
  _incoming[33] =  10413; _outgoingP[33] =  211; _outgoingV[33] =  423; 
  _coupling[33] = 0.542/GeV; _maxweight[33] = 3.; 
  _incoming[34] =  10413; _outgoingP[34] =  111; _outgoingV[34] =  413; 
  _coupling[34] = 0.383/GeV; _maxweight[34] = 3.; 
  // decaymode D'_1 to D* pi
  _incoming[35] =  20423; _outgoingP[35] = -211; _outgoingV[35] =  413; 
  _coupling[35] = 1.933/GeV; _maxweight[35] = 3.; 
  _incoming[36] =  20423; _outgoingP[36] =  111; _outgoingV[36] =  423; 
  _coupling[36] = 1.367/GeV; _maxweight[36] = 3.; 
  _incoming[37] =  20413; _outgoingP[37] =  211; _outgoingV[37] =  423; 
  _coupling[37] = 1.926/GeV; _maxweight[37] = 3.; 
  _incoming[38] =  20413; _outgoingP[38] =  111; _outgoingV[38] =  413; 
  _coupling[38] = 1.367/GeV; _maxweight[38] = 3.; 
  // decaymode B_1 to B* pi
  _incoming[39] =  10523; _outgoingP[39] =  211; _outgoingV[39] =  513; 
  _coupling[39] = 0.130/GeV; _maxweight[39] = 2.2; 
  _incoming[40] =  10523; _outgoingP[40] =  111; _outgoingV[40] =  523; 
  _coupling[40] = 0.0924/GeV; _maxweight[40] = 2.2; 
  _incoming[41] =  10513; _outgoingP[41] = -211; _outgoingV[41] =  523; 
  _coupling[41] = 0.130/GeV; _maxweight[41] = 2.2; 
  _incoming[42] =  10513; _outgoingP[42] =  111; _outgoingV[42] =  513; 
  _coupling[42] = 0.0924/GeV; _maxweight[42] = 2.2; 
  // decaymode B'_1 to B* pi
  _incoming[43] =  20523; _outgoingP[43] =  211; _outgoingV[43] =  513; 
  _coupling[43] = 0.445/GeV; _maxweight[43] = 2.2; 
  _incoming[44] =  20523; _outgoingP[44] =  111; _outgoingV[44] =  523; 
  _coupling[44] = 0.314/GeV; _maxweight[44] = 2.2; 
  _incoming[45] =  20513; _outgoingP[45] = -211; _outgoingV[45] =  523; 
  _coupling[45] = 0.445/GeV; _maxweight[45] = 2.2; 
  _incoming[46] =  20513; _outgoingP[46] =  111; _outgoingV[46] =  513; 
  _coupling[46] = 0.314/GeV; _maxweight[46] = 2.2; 
  // decaymode D_s1 to D* pi
  _incoming[47] =  10433; _outgoingP[47] =  111; _outgoingV[47] =  433; 
  _coupling[47] = 0.022/GeV; _maxweight[47] = 2.5; 
  // decaymode D_s1 to D gamma
  _incoming[48] =  10433; _outgoingP[48] =  431; _outgoingV[48] = 22; 
  _coupling[48] = 0.0587/GeV; _maxweight[48] = 2.1; 
  // decaymode B_s1 to B gamma
  _incoming[49] =  10533; _outgoingP[49] =  531; _outgoingV[49] = 22; 
  _coupling[49] = 0.142/GeV; _maxweight[49] = 2.;
  // decaymode B_s1 to B* pi
  _incoming[50] =  10533; _outgoingP[50] =  111; _outgoingV[50] =  533; 
  _coupling[50] = 0.0074/GeV; _maxweight[50] = 2.1; 
  // decaymode B_c1 to B_c gamma
  _incoming[51] =  10543; _outgoingP[51] =  541; _outgoingV[51] = 22; 
  _coupling[51] = 0.0759/GeV; _maxweight[51] = 2.2; 
  // decaymode B'_c1 to B_c gamma
  _incoming[52] =  20543; _outgoingP[52] =  541; _outgoingV[52] = 22; 
  _coupling[52] = 0.175/GeV; _maxweight[52] = 2.2; 
  // decaymode h_c to eta_c gamma
  _incoming[53] = 10443; _outgoingP[53] = 441; _outgoingV[53] = 22; 
  _coupling[53] = 0.329/GeV; _maxweight[53] = 4.; 
  // decaymode h_b to eta_b gamma
  _incoming[54] = 10553; _outgoingP[54] = 551; _outgoingV[54] = 22; 
  _coupling[54] = 0.0356/GeV; _maxweight[54] = 3.5; 
  // a_1 to K* K
  _incoming[55] = 20213; _outgoingP[55] = -311; _outgoingV[55] = 323; 
  _coupling[55] = 3.42/GeV; _maxweight[55] = 2.5; 
  _incoming[56] = 20213; _outgoingP[56] = 321; _outgoingV[56] = -313; 
  _coupling[56] = 3.42/GeV; _maxweight[56] = 2.5; 
  _incoming[57] = 20113; _outgoingP[57] = 321; _outgoingV[57] = -323; 
  _coupling[57] = 3.42/GeV; _maxweight[57] = 4.0; 
  _incoming[58] = 20113; _outgoingP[58] = 311; _outgoingV[58] = -313; 
  _coupling[58] = 3.42/GeV; _maxweight[58] = 4.0; 
  // a_1 to gamma pi
  _incoming[59] =  20113; _outgoingP[59] =  111; _outgoingV[59] = 22; 
  _coupling[59] = 0.01/GeV; _maxweight[59] = 2.; 
  _incoming[60] =  20213; _outgoingP[60] =  211; _outgoingV[60] = 22; 
  _coupling[60] = 0.01/GeV; _maxweight[60] = 2.; 
  // f'_1 to K* K
  _incoming[61] = 20333; _outgoingP[61] = 321; _outgoingV[61] = -323; 
  _coupling[61] = 1.637/GeV; _maxweight[61] = 7.; 
  _incoming[62] = 20333; _outgoingP[62] = 311; _outgoingV[62] = -313; 
  _coupling[62] = 1.737/GeV; _maxweight[62] = 7.; 
  // decay mode K_1 to gamma K
  _incoming[63] =  10313; _outgoingV[63] =  22; _outgoingP[63] =  311; 
  _coupling[63] = 0.119/GeV; _maxweight[63] = 7.5; 
  // decay mode K'_1 to gamma K
  _incoming[64] =  20313; _outgoingV[64] =  22; _outgoingP[64] =  311; 
  _coupling[64] = 0.220/GeV; _maxweight[64] = 8.;  
  // decaymode B_s1 to B K
  _incoming[65] =  10533; _outgoingP[65] =  -311; _outgoingV[65] = 513; 
  _coupling[65] = 0.0418/GeV; _maxweight[65] = 2.;
  _incoming[66] =  10533; _outgoingP[66] =  -321; _outgoingV[66] = 523; 
  _coupling[66] = 0.0373/GeV; _maxweight[66] = 2.;  
  // initial size of the arrays
  _initsize = _coupling.size();
  // intermediates
  generateIntermediates(false);
}

int PVectorMesonVectorPScalarDecayer::modeNumber(bool & cc,tcPDPtr parent,
						 const tPDVector & children) const {
  if(children.size()!=2) return -1;
  int id(parent->id());
  int idbar = parent->CC() ? parent->CC()->id() : id;
  int id1(children[0]->id());
  int id1bar = children[0]->CC() ? children[0]->CC()->id() : id1;
  int id2(children[1]->id());
  int id2bar = children[1]->CC() ? children[1]->CC()->id() : id2;
  int imode(-1);
  unsigned int ix(0);
  cc=false;
  do {
    if(id   ==_incoming[ix]) {
      if((id1   ==_outgoingV[ix]&&id2   ==_outgoingP[ix])||
	 (id2   ==_outgoingV[ix]&&id1   ==_outgoingP[ix])) imode=ix;
    }
    if(idbar==_incoming[ix]&&imode<0) {
      if((id1bar==_outgoingV[ix]&&id2bar==_outgoingP[ix])||
	 (id2bar==_outgoingV[ix]&&id1bar==_outgoingP[ix])) {
	imode=ix;
	cc=true;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  return imode;
}


void PVectorMesonVectorPScalarDecayer::persistentOutput(PersistentOStream & os) const {
  os << _incoming << _outgoingV << _outgoingP << _maxweight << ounit(_coupling,1/GeV);
}

void PVectorMesonVectorPScalarDecayer::persistentInput(PersistentIStream & is, int) {
  is >> _incoming >> _outgoingV >> _outgoingP >> _maxweight >> iunit(_coupling,1/GeV);
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<PVectorMesonVectorPScalarDecayer,DecayIntegrator>
describeHerwigPVectorMesonVectorPScalarDecayer("Herwig::PVectorMesonVectorPScalarDecayer", "HwVMDecay.so");

void PVectorMesonVectorPScalarDecayer::Init() {

  static ClassDocumentation<PVectorMesonVectorPScalarDecayer> documentation
    ("The PVectorMesonVectorPScalarDecayer class is designed for the "
     "decay of a pseudovector meson to a vector meson, or the photon, and a "
     "pseudoscalar meson.");

  static ParVector<PVectorMesonVectorPScalarDecayer,int> interfaceIncoming
    ("Incoming",
     "The PDG code for the incoming particle",
     &PVectorMesonVectorPScalarDecayer::_incoming,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<PVectorMesonVectorPScalarDecayer,int> interfaceOutcomingVector
    ("OutgoingVector",
     "The PDG code for the outgoing spin-1 particle",
     &PVectorMesonVectorPScalarDecayer::_outgoingV,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<PVectorMesonVectorPScalarDecayer,int> interfaceOutcomingPScalar
    ("OutgoingPScalar",
     "The PDG code for the outgoing spin-0 particle",
     &PVectorMesonVectorPScalarDecayer::_outgoingP,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<PVectorMesonVectorPScalarDecayer,InvEnergy> interfaceCoupling
    ("Coupling",
     "The coupling for the decay mode",
     &PVectorMesonVectorPScalarDecayer::_coupling,
     1/GeV, 0, ZERO, ZERO, 100./GeV, false, false, true);

  static ParVector<PVectorMesonVectorPScalarDecayer,double> interfaceMaxWeight
    ("MaxWeight",
     "The maximum weight for the decay mode",
     &PVectorMesonVectorPScalarDecayer::_maxweight,
     0, 0, 0, 0., 10000., false, false, true);

}


double PVectorMesonVectorPScalarDecayer::me2(const int,
					     const Particle & inpart,
					     const ParticleVector& decay,
					     MEOption meopt) const {
  if(!ME())
    ME(new_ptr(GeneralDecayMatrixElement(PDT::Spin1,PDT::Spin1,PDT::Spin0)));
  // is the vector massless
  bool photon(_outgoingV[imode()]==ParticleID::gamma);
  if(meopt==Initialize) {
    VectorWaveFunction::calculateWaveFunctions(_vectors[0],_rho,
						const_ptr_cast<tPPtr>(&inpart),
						incoming,false);
  }
  if(meopt==Terminate) {
    VectorWaveFunction::constructSpinInfo(_vectors[0],const_ptr_cast<tPPtr>(&inpart),
					  incoming,true,false);
    VectorWaveFunction::constructSpinInfo(_vectors[1],decay[0],
					  outgoing,true,photon);
    ScalarWaveFunction::constructSpinInfo(decay[1],outgoing,true);
    return 0.;
  }
  VectorWaveFunction::calculateWaveFunctions(_vectors[1],decay[0],outgoing,photon);
  // compute the matrix element
  Energy2 p0dotpv(inpart.momentum()*decay[0]->momentum());
  complex<Energy> epsdot(ZERO);
  InvEnergy2 pre(_coupling[imode()]/inpart.mass());
  for(unsigned ix=0;ix<3;++ix) {
    if(ix==1&&photon) {
      for(unsigned int iy=0;iy<3;++iy) (*ME())(iy,ix,0)=0.;
    }
    else {
      epsdot=_vectors[1][ix]*inpart.momentum();
      for(unsigned int iy=0;iy<3;++iy)
	(*ME())(iy,ix,0)=Complex(pre*_vectors[0][iy].dot(p0dotpv*_vectors[1][ix]
							 -epsdot*decay[0]->momentum()));
    }
  }
  // test of the matrix element
//   double me = newME.contract(rhoin).real();
//   Energy pcm=Kinematics::pstarTwoBodyDecay(inpart.mass(),decay[0]->mass(),
// 					   decay[1]->mass());
//   double test = sqr(_coupling[imode()])/3.*(2.*sqr(pcm)+3.*sqr(decay[0]->mass()));
//   cerr << "testing matrix element for " << inpart.PDGName() << " -> " 
//        << decay[0]->PDGName() << " " << decay[1]->PDGName() << " "
//        << me << " " << test << " " << (me-test)/(me+test) << "\n";
  // return the answer
  return ME()->contract(_rho).real();
}

bool PVectorMesonVectorPScalarDecayer::twoBodyMEcode(const DecayMode & dm,
						     int & mecode,
						     double & coupling) const {
  int id(dm.parent()->id()),idbar(id);
  if(dm.parent()->CC()){idbar=dm.parent()->CC()->id();}
  ParticleMSet::const_iterator pit(dm.products().begin());
  int id1((**pit).id()),id1bar(id1);
  if((**pit).CC()){id1bar=(**pit).CC()->id();}
  ++pit;
  int id2((**pit).id()),id2bar(id2);
  if((**pit).CC()){id2bar=(**pit).CC()->id();}
  unsigned int ix(0); bool order(false);
  int imode(-1);
  do {
    if(id==_incoming[ix]) {
      if(id1   ==_outgoingV[ix]&&id2   ==_outgoingP[ix]) {
	imode=ix;
	order=true;
      }
      if(id2   ==_outgoingV[ix]&&id1   ==_outgoingP[ix]) {
	imode=ix;
	order=false;
      }
    }
    if(idbar==_incoming[ix]&&imode<0) {
      if(id1bar==_outgoingV[ix]&&id2bar==_outgoingP[ix]) {
	imode=ix;
	order=true;
      }
      if(id2bar==_outgoingV[ix]&&id1bar==_outgoingP[ix]) {
	imode=ix;
	order=false;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  coupling = _coupling[imode]*dm.parent()->mass();  
  mecode = 4;
  return order;
}

void PVectorMesonVectorPScalarDecayer::dataBaseOutput(ofstream & output,
						      bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  // the rest of the parameters
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":Incoming " << ix << " "
	     << _incoming[ix] << "\n";
      output << "newdef " << name() << ":OutgoingVector " << ix << " "
	     << _outgoingV[ix] << "\n";
      output << "newdef " << name() << ":OutgoingPScalar " << ix << " "
	     << _outgoingP[ix] << "\n";
      output << "newdef " << name() << ":Coupling " << ix << " "
	     << _coupling[ix]*GeV << "\n";
      output << "newdef " << name() << ":MaxWeight " << ix << " "
	     << _maxweight[ix] << "\n";
    }
    else {
      output << "insert " << name() << ":Incoming "  << ix << " "
	     << _incoming[ix] << "\n";
      output << "insert " << name() << ":OutgoingVector " << ix << " "
	     << _outgoingV[ix] << "\n";
      output << "insert " << name() << ":OutgoingPScalar " << ix << " "
	     << _outgoingP[ix] << "\n";
      output << "insert " << name() << ":Coupling " << ix << " "
	     << _coupling[ix]*GeV << "\n";
      output << "insert " << name() << ":MaxWeight " << ix << " "
	     << _maxweight[ix] << "\n";
    }
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}
#line 1 "./VectorMeson2FermionDecayer.cc"
// -*- C++ -*-
//
// VectorMeson2FermionDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the VectorMeson2FermionDecayer class.
//

#include "VectorMeson2FermionDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Helicity/WaveFunction/SpinorWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/SpinorBarWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "Herwig/Decay/TwoBodyDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void VectorMeson2FermionDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    for(unsigned int ix=0;ix<_incoming.size();++ix) {
      if(mode(ix)) _maxweight[ix] = mode(ix)->maxWeight();
    }
  }
}

void VectorMeson2FermionDecayer::doinit() {
  DecayIntegrator::doinit();
  // check the parameters arew consistent
  unsigned int isize=_coupling.size();
  if(isize!=_incoming.size()  || isize!=_outgoingf.size()||
     isize!=_outgoinga.size() || isize!=_maxweight.size())
    throw InitException() << "Inconsistent parameters in VectorMeson2"
			   << "FermionDecayer::doiin() " << Exception::runerror;
  // set up the integration channels
  vector<double> wgt(0);
  DecayPhaseSpaceModePtr mode;
  tPDVector extpart(3);
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    extpart[0]=getParticleData(_incoming[ix]);
    extpart[1]=getParticleData(_outgoingf[ix]);
    extpart[2]=getParticleData(_outgoinga[ix]);
    if(extpart[0]&&extpart[1]&&extpart[2]) 
      mode=new_ptr(DecayPhaseSpaceMode(extpart,this));
    else
      mode=DecayPhaseSpaceModePtr();
    addMode(mode,_maxweight[ix],wgt);
  }
}

VectorMeson2FermionDecayer::VectorMeson2FermionDecayer() 
  : _coupling(42), _incoming(42), _outgoingf(42), _outgoinga(42), _maxweight(42) {
  // don't include intermediates
  generateIntermediates(false);
  // rho -> e+e-, mu+mu
  _incoming[0] = 113; _outgoingf[0] = 11; _outgoinga[0] = -11; 
  _coupling[0] = 18.524E-3;   _maxweight[0] = 1.61; 
  _incoming[1] = 113; _outgoingf[1] = 13; _outgoinga[1] = -13; 
  _coupling[1] = 18.524E-3;    _maxweight[1] = 1.61; 
  // omega -> e+e-, mu+mu-
  _incoming[2] = 223; _outgoingf[2] = 11; _outgoinga[2] = -11; 
  _coupling[2] = 5.429E-3;  _maxweight[2] = 1.61; 
  _incoming[3] = 223; _outgoingf[3] = 13; _outgoinga[3] = -13; 
  _coupling[3] = 5.429E-3;  _maxweight[3] = 1.61; 
  // phi -> e+e-, mu+mu-
  _incoming[4] = 333; _outgoingf[4] = 11; _outgoinga[4] = -11; 
  _coupling[4] = 6.852E-3;  _maxweight[4] = 1.61; 
  _incoming[5] = 333; _outgoingf[5] = 13; _outgoinga[5] = -13; 
  _coupling[5] = 6.852E-3;  _maxweight[5] = 1.61; 
  // psi(1d) to leptons
  _incoming[6] = 30443; _outgoingf[6] = 11; _outgoinga[6] = -11; 
  _coupling[6] = 1.590E-3;  _maxweight[6] = 1.7; 
  _incoming[7] = 30443; _outgoingf[7] = 13; _outgoinga[7] = -13; 
  _coupling[7] = 1.509E-3;  _maxweight[7] = 1.7; 
  _incoming[8] = 30443; _outgoingf[8] = 15; _outgoinga[8] = -15;  
  _coupling[8] = 1.509E-3;  _maxweight[8] = 1.8; 
  // J/psi decay
  _incoming[9] = 443; _outgoingf[9] = 11; _outgoinga[9] = -11; 
  _coupling[9] = 8.215E-3;  _maxweight[9] = 1.7; 
  _incoming[10] = 443; _outgoingf[10] = 13; _outgoinga[10] = -13; 
  _coupling[10] = 8.215E-3;  _maxweight[10] = 1.7; 
  // psi2s to leptons
  _incoming[11] = 100443; _outgoingf[11] = 11; _outgoinga[11] = -11; 
  _coupling[11] = 5.031E-3;  _maxweight[11] = 1.7; 
  _incoming[12] = 100443; _outgoingf[12] = 13; _outgoinga[12] = -13; 
  _coupling[12] = 5.031E-3;  _maxweight[12] = 1.7; 
  _incoming[13] = 100443; _outgoingf[13] = 15; _outgoinga[13] = -15; 
  _coupling[13] = 5.031E-3;  _maxweight[13] = 1.7; 
  // upsilon to leptons
  _incoming[14] = 553; _outgoingf[14] = 11; _outgoinga[14] = -11; 
  _coupling[14] = 2.315E-3;  _maxweight[14] = 1.7; 
  _incoming[15] = 553; _outgoingf[15] = 13; _outgoinga[15] = -13; 
  _coupling[15] = 2.315E-3;  _maxweight[15] = 1.7; 
  _incoming[16] = 553; _outgoingf[16] = 15; _outgoinga[16] = -15; 
  _coupling[16] = 2.315E-3;  _maxweight[16] = 1.7;
   // upsilon 2s to leptons
  _incoming[17] = 100553; _outgoingf[17] = 11; _outgoinga[17] = -11; 
  _coupling[17] = 1.520E-3;  _maxweight[17] = 1.7; 
  _incoming[18] = 100553; _outgoingf[18] = 13; _outgoinga[18] = -13; 
  _coupling[18] = 1.520E-3;  _maxweight[18] = 1.7; 
  _incoming[19] = 100553; _outgoingf[19] = 15; _outgoinga[19] = -15; 
  _coupling[19] = 1.520E-3;  _maxweight[19] = 1.7;
  // upsilon 3s to leptons
  _incoming[20] = 200553; _outgoingf[20] = 11; _outgoinga[20] = -11; 
  _coupling[20] = 1.270E-3;  _maxweight[20] = 1.7; 
  _incoming[21] = 200553; _outgoingf[21] = 13; _outgoinga[21] = -13; 
  _coupling[21] = 1.270E-3;  _maxweight[21] = 1.7; 
  _incoming[22] = 200553; _outgoingf[22] = 15; _outgoinga[22] = -15; 
  _coupling[22] = 1.270E-3;  _maxweight[22] = 1.7;
  // upsilon 4s to leptons
  _incoming[23] = 300553; _outgoingf[23] = 11; _outgoinga[23] = -11; 
  _coupling[23] = 1.081E-3;  _maxweight[23] = 1.7; 
  _incoming[24] = 300553; _outgoingf[24] = 13; _outgoinga[24] = -13; 
  _coupling[24] = 1.081E-3;  _maxweight[24] = 1.7; 
  _incoming[25] = 300553; _outgoingf[25] = 15; _outgoinga[25] = -15; 
  _coupling[25] = 1.081E-3;  _maxweight[25] = 1.7; 
  // baryonic jpsi decays
  // to neutrons and proton
  _incoming[26] = 443; _outgoingf[26] = 2212; _outgoinga[26] = -2212; 
  _maxweight[26] = 1.7; _coupling[26] = 1.619E-3; 
  _incoming[27] = 443; _outgoingf[27] = 2112; _outgoinga[27] = -2112; 
  _maxweight[27] = 1.7; _coupling[27] = 1.619E-3; 
  // to sigma's
  _incoming[28] = 443; _outgoingf[28] = 3112; _outgoinga[28] = -3112; 
  _maxweight[28] = 1.7; _coupling[28] = 1.342E-3; 
  _incoming[29] = 443; _outgoingf[29] = 3212; _outgoinga[29] = -3212; 
  _maxweight[29] = 1.7; _coupling[29] = 1.342E-3; 
  _incoming[30] = 443; _outgoingf[30] = 3222; _outgoinga[30] = -3222; 
  _maxweight[30] = 1.7; _coupling[30] = 1.342E-3; 
  // to Xi's 
  _incoming[31] = 443; _outgoingf[31] = 3322; _outgoinga[31] = -3322; 
  _maxweight[31] = 1.7; _coupling[31] = 1.196E-3; 
  _incoming[32] = 443; _outgoingf[32] = 3312; _outgoinga[32] = -3312; 
  _maxweight[32] = 1.7; _coupling[32] = 1.196E-3; 
  // to lambda
  _incoming[33] = 443; _outgoingf[33] = 3122; _outgoinga[33] = -3122; 
  _maxweight[33] = 1.7; _coupling[33] = 1.416E-3; 
  // baryonic psi(2s) decays
  // to neutrons and protons
  _incoming[34] = 100443; _outgoingf[34] = 2212; _outgoinga[34] = -2212; 
  _maxweight[34] = 1.7; _coupling[34] = 9.692E-4; 
  _incoming[35] = 100443; _outgoingf[35] = 2112; _outgoinga[35] = -2112; 
  _maxweight[35] = 1.7; _coupling[35] = 9.692E-4; 
  // to sigma's
  _incoming[36] = 100443; _outgoingf[36] = 3112; _outgoinga[36] = -3112; 
  _maxweight[36] = 1.7; _coupling[36] = 9.273E-4; 
  _incoming[37] = 100443; _outgoingf[37] = 3212; _outgoinga[37] = -3212; 
  _maxweight[37] = 1.7; _coupling[37] = 9.273E-4; 
  _incoming[38] = 100443; _outgoingf[38] = 3222; _outgoinga[38] = -3222; 
  _maxweight[38] = 1.7; _coupling[38] = 9.273E-4; 
  // to Xi's
  _incoming[39] = 100443; _outgoingf[39] = 3322; _outgoinga[39] = -3322; 
  _maxweight[39] = 1.7; _coupling[39] = 8.862E-4; 
  _incoming[40] = 100443; _outgoingf[40] = 3312; _outgoinga[40] = -3312; 
  _maxweight[40] = 1.7; _coupling[40] = 8.862E-4; 
  // to lambda
  _incoming[41] = 100443; _outgoingf[41] = 3122; _outgoinga[41] = -3122; 
  _maxweight[41] = 1.7; _coupling[41] = 9.565E-4;   
  // set the initial size
  _initsize=_incoming.size();
}

int VectorMeson2FermionDecayer::modeNumber(bool & cc,tcPDPtr parent,
					   const tPDVector & children) const {
  if(children.size()!=2) return -1;
  int id(parent->id());
  int idbar = parent->CC() ? parent->CC()->id() : id;
  int id1(children[0]->id());
  int id1bar = children[0]->CC() ? children[0]->CC()->id() : id1;
  int id2(children[1]->id());
  int id2bar = children[1]->CC() ? children[1]->CC()->id() : id2;
  int imode(-1);
  unsigned int ix(0);
  cc=false;
  do {
    if(_incoming[ix]==id   ) {
      if((id1   ==_outgoingf[ix]&&id2   ==_outgoinga[ix])||
	 (id2   ==_outgoingf[ix]&&id1   ==_outgoinga[ix])) imode=ix;
    }
    if(_incoming[ix]==idbar) {
      if((id1bar==_outgoingf[ix]&&id2bar==_outgoinga[ix])||
	 (id2bar==_outgoingf[ix]&&id1bar==_outgoinga[ix])) {
	imode=ix;
	cc=true;
      }
    }
    ++ix;
  }
  while(imode<0&&ix<_incoming.size());
  return imode;
}

void VectorMeson2FermionDecayer::persistentOutput(PersistentOStream & os) const {
  os << _coupling << _incoming << _outgoingf << _outgoinga << _maxweight;
}

void VectorMeson2FermionDecayer::persistentInput(PersistentIStream & is, int) {
  is >> _coupling >> _incoming >> _outgoingf >> _outgoinga >> _maxweight;
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<VectorMeson2FermionDecayer,DecayIntegrator>
describeHerwigVectorMeson2FermionDecayer("Herwig::VectorMeson2FermionDecayer", "HwVMDecay.so");

void VectorMeson2FermionDecayer::Init() {

  static ClassDocumentation<VectorMeson2FermionDecayer> documentation
    ("The VectorMeson2FermionDecayer class is designed for the decay "
     "of vectro mesons to fermions. It is mainly used for the decay of vector mesons "
     "to electrons and muons.");

  static ParVector<VectorMeson2FermionDecayer,int> interfaceIncoming
    ("Incoming",
     "The PDG code for the incoming particle",
     &VectorMeson2FermionDecayer::_incoming,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMeson2FermionDecayer,int> interfaceOutcoming1
    ("OutgoingFermion",
     "The PDG code for the outgoing fermion",
     &VectorMeson2FermionDecayer::_outgoingf,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMeson2FermionDecayer,int> interfaceOutcoming2
    ("OutgoingAntiFermion",
     "The PDG code for the second outgoing anti-fermion",
     &VectorMeson2FermionDecayer::_outgoinga,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMeson2FermionDecayer,double> interfaceCoupling
    ("Coupling",
     "The coupling for the decay mode",
     &VectorMeson2FermionDecayer::_coupling,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMeson2FermionDecayer,double> interfaceMaxWeight
    ("MaxWeight",
     "The maximum weight for the decay mode",
     &VectorMeson2FermionDecayer::_maxweight,
     0, 0, 0, -10000000, 10000000, false, false, true);

}

double VectorMeson2FermionDecayer::me2(const int,
				       const Particle & inpart,
				       const ParticleVector& decay,
				       MEOption meopt) const {
  if(!ME())
    ME(new_ptr(TwoBodyDecayMatrixElement(PDT::Spin1,PDT::Spin1Half,PDT::Spin1Half)));
  // fermion and antifermion
  unsigned int iferm(0),ianti(1);
  if(_outgoingf[imode()]!=decay[iferm]->id()) swap(iferm,ianti);
  // initialization
  if(meopt==Initialize) {
    VectorWaveFunction::calculateWaveFunctions(_vectors,_rho,
					       const_ptr_cast<tPPtr>(&inpart),
					       incoming,false);
  }
  if(meopt==Terminate) {
    VectorWaveFunction::constructSpinInfo(_vectors,const_ptr_cast<tPPtr>(&inpart),
					  incoming,true,false);
    SpinorBarWaveFunction::
      constructSpinInfo(_wavebar,decay[iferm],outgoing,true);
    SpinorWaveFunction::
      constructSpinInfo(_wave   ,decay[ianti],outgoing,true);
    return 0.;
  }
  SpinorBarWaveFunction::
    calculateWaveFunctions(_wavebar,decay[iferm],outgoing);
  SpinorWaveFunction::
    calculateWaveFunctions(_wave   ,decay[ianti],outgoing);
  // prefactor
  InvEnergy pre(_coupling[imode()]/inpart.mass());
  // now compute the currents
  LorentzPolarizationVector temp;
  for(unsigned ix=0;ix<2;++ix) {
    for(unsigned iy=0;iy<2;++iy) {
      temp = pre*_wave[ix].vectorCurrent(_wavebar[iy]);
      for(unsigned int iz=0;iz<3;++iz) {
	if(iferm>ianti) (*ME())(iz,ix,iy)=_vectors[iz].dot(temp);
	else            (*ME())(iz,iy,ix)=_vectors[iz].dot(temp);
      }
    }
  }
  // test of the matrix element
//   double me = newME.contract(rhoin).real();
//   double test = 4.*sqr(_coupling[imode()])/3.*
//     (1.+2.*sqr(decay[0]->mass()/inpart.mass()));
//   cerr << "testing matrix element for " << inpart.PDGName() << " -> " 
//        << decay[0]->PDGName() << " " << decay[1]->PDGName() << " "
//        << me << " " << test << " " << (me-test)/(me+test) << "\n";
  // return the answer
  return ME()->contract(_rho).real();
}

bool VectorMeson2FermionDecayer::twoBodyMEcode(const DecayMode & dm,int & mecode,
						double & coupling) const {
  int imode(-1);
  int id(dm.parent()->id()),idbar(id);
  if(dm.parent()->CC()){idbar=dm.parent()->CC()->id();}
  ParticleMSet::const_iterator pit(dm.products().begin());
  int id1((**pit).id()),id1bar(id1);
  if((**pit).CC()){id1bar=(**pit).CC()->id();}
  ++pit;
  int id2((**pit).id()),id2bar(id2);
  if((**pit).CC()){id2bar=(**pit).CC()->id();}
  unsigned int ix(0); bool order(false);
  do {
    if(id   ==_incoming[ix]) {
      if(id1==_outgoingf[ix]&&id2==_outgoinga[ix]) {
	imode=ix;
	order=true;
      }
      if(id2==_outgoingf[ix]&&id1==_outgoinga[ix]) {
	imode=ix;
	order=false;
      }
    }
    if(idbar==_incoming[ix]&&imode<0) {
      if(id1bar==_outgoingf[ix]&&id2bar==_outgoinga[ix]) {
	imode=ix;
	order=true;
      }
      if(id2bar==_outgoingf[ix]&&id1bar==_outgoinga[ix]) {
	imode=ix;
	order=false;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  coupling=_coupling[imode];
  mecode=2;
  return order;
}

// output the setup information for the particle database
void VectorMeson2FermionDecayer::dataBaseOutput(ofstream & output,
						bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  // the rest of the parameters
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":Incoming " << ix << " "
	     << _incoming[ix] << "\n";
      output << "newdef " << name() << ":OutgoingFermion " << ix << " "
	     << _outgoingf[ix] << "\n";
      output << "newdef " << name() << ":OutgoingAntiFermion "  << ix << " "
	     << _outgoinga[ix] << "\n";
      output << "newdef " << name() << ":Coupling " << ix << " "
	     << _coupling[ix] << "\n";
      output << "newdef " << name() << ":MaxWeight " << ix << " "
	     << _maxweight[ix] << "\n";
    }
    else {
      output << "insert " << name() << ":Incoming " << ix << " "
	     << _incoming[ix] << "\n";
      output << "insert " << name() << ":OutgoingFermion "  << ix << " "
	     << _outgoingf[ix] << "\n";
      output << "insert " << name() << ":OutgoingAntiFermion "  << ix << " "
	     << _outgoinga[ix] << "\n";
      output << "insert " << name() << ":Coupling " << ix << " "
	     << _coupling[ix] << "\n";
      output << "insert " << name() << ":MaxWeight " << ix << " "
	     << _maxweight[ix] << "\n";
    }
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}
#line 1 "./VectorMeson2MesonDecayer.cc"
// -*- C++ -*-
//
// VectorMeson2MesonDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the VectorMeson2MesonDecayer class.
//

#include "VectorMeson2MesonDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "Herwig/Decay/TwoBodyDecayMatrixElement.h"

using namespace Herwig; 
using namespace ThePEG::Helicity;
 
void VectorMeson2MesonDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    for(unsigned int ix=0;ix<_incoming.size();++ix)
      if(mode(ix)) _maxweight[ix]= mode(ix)->maxWeight();
  }
}

void VectorMeson2MesonDecayer::doinit() {
  DecayIntegrator::doinit();
  // check consistence of the parameters
  unsigned int isize=_incoming.size();
  if(isize!=_outgoing1.size()||isize!=_outgoing2.size()||isize!=_maxweight.size()||
     isize!=_coupling.size()) {
    throw InitException() << "Inconsistent parameters in "
			  << "VectorMeson2MesonDecayer" << Exception::runerror;
  }
  // set up the integration channels
  vector<double> wgt(0);
  tPDVector extpart(3);
  DecayPhaseSpaceModePtr mode;
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    extpart[0]=getParticleData( _incoming[ix]);
    extpart[1]=getParticleData(_outgoing1[ix]);
    extpart[2]=getParticleData(_outgoing2[ix]);
    if(extpart[0]&&extpart[1]&&extpart[2]) 
      mode=new_ptr(DecayPhaseSpaceMode(extpart,this));
    else
      mode=DecayPhaseSpaceModePtr();
    addMode(mode,_maxweight[ix],wgt);
  }
}

VectorMeson2MesonDecayer::VectorMeson2MesonDecayer() :
  _incoming(64), _outgoing1(64), _outgoing2(64), _maxweight(64), _coupling(64) {
  // matrix element storage
  // don't generate intermediates
  generateIntermediates(false);
  // reserve size of vectors for speed
  // particles and couplings for the different modes
  // rho -> pi pi
  _incoming[0] =  113; _outgoing1[0] =  211; _outgoing2[0] = -211;
  _coupling[0] = 6.; _maxweight[0] = 1.7;
  _incoming[1] =  213; _outgoing1[1] =  111; _outgoing2[1] =  211;
  _coupling[1] = 6.; _maxweight[1] = 1.7;
  // rho' -> pi pi
  _incoming[2] =  100113; _outgoing1[2] =  211; _outgoing2[2] = -211;  
  _coupling[2] = 2.082; _maxweight[2] = 1.7;
  _incoming[3] =  100213; _outgoing1[3] =  111; _outgoing2[3] =  211;
  _coupling[3] = 2.082; _maxweight[3] = 1.7; 
  // rho'' -> pi pi
  _incoming[4] =  30113; _outgoing1[4] =  211; _outgoing2[4] = -211; 
  _coupling[4] = 1.655; _maxweight[4] = 1.7; 
  _incoming[5] =  30213; _outgoing1[5] =  111; _outgoing2[5] =  211; 
  _coupling[5] = 1.655; _maxweight[5] = 1.7;
  // rho'' -> K K
  _incoming[6] =  30113; _outgoing1[6] =  321; _outgoing2[6] = -321; 
  _coupling[6] = 0.194; _maxweight[6] = 1.7;
  _incoming[7] =  30113; _outgoing1[7] =  311; _outgoing2[7] = -311; 
  _coupling[7] = 0.194; _maxweight[7] = 1.7;
  _incoming[8] =  30213; _outgoing1[8] =  321; _outgoing2[8] = -311; 
  _coupling[8] = 0.274; _maxweight[8] = 1.7;
  // rho'' -> pi' pi
  _incoming[ 9] =  30113; _outgoing1[ 9] = 100211; _outgoing2[ 9] = -211; 
  _coupling[ 9] = 7.828; _maxweight[ 9] = 4.1; 
  _incoming[10] =  30213; _outgoing1[10] = 100111; _outgoing2[10] =  211; 
  _coupling[10] = 7.828; _maxweight[10] = 4.1; 
  _incoming[11] =  30213; _outgoing1[11] = 111   ; _outgoing2[11] =  100211; 
  _coupling[11] = 7.828; _maxweight[11] = 4.1;
  // rho' -> pi' pi
  _incoming[12] =  100113; _outgoing1[12] = 100211; _outgoing2[12] = -211; 
  _coupling[12] = 17.9;  _maxweight[12] = 4.2;
  _incoming[13] =  100213; _outgoing1[13] = 100111; _outgoing2[13] =  211; 
  _coupling[13] = 17.9;  _maxweight[13] = 4.2;
  _incoming[14] =  100213; _outgoing1[14] = 111   ; _outgoing2[14] =  100211; 
  _coupling[14] = 17.9;  _maxweight[14] = 4.2;
  // omega -> pi pi
  _incoming[15] = 223; _outgoing1[15] =  211; _outgoing2[15] = -211; 
  _coupling[15] = 0.1847; _maxweight[15] = 1.7;
  // K* decays
  _incoming[16] =  313; _outgoing1[16] =  321; _outgoing2[16] = -211; 
  _coupling[16] = 4.57; _maxweight[16] = 1.7;
  _incoming[17] =  313; _outgoing1[17] =  311; _outgoing2[17] =  111; 
  _coupling[17] = 3.23; _maxweight[17] = 1.7;
  _incoming[18] =  323; _outgoing1[18] =  311; _outgoing2[18] =  211; 
  _coupling[18] = 4.57; _maxweight[18] = 1.7;
  _incoming[19] =  323; _outgoing1[19] =  321; _outgoing2[19] =  111; 
  _coupling[19] = 3.23; _maxweight[19] = 1.7;
  // K*' decays
  _incoming[20] =  100313; _outgoing1[20] =  321; _outgoing2[20] = -211; 
  _coupling[20] = 1.296; _maxweight[20] = 1.7;  
  _incoming[21] =  100313; _outgoing1[21] =  311; _outgoing2[21] =  111; 
  _coupling[21] = 0.916; _maxweight[21] = 1.7;  
  _incoming[22] =  100323; _outgoing1[22] =  311; _outgoing2[22] =  211; 
  _coupling[22] = 1.296; _maxweight[22] = 1.7;  
  _incoming[23] =  100323; _outgoing1[23] =  321; _outgoing2[23] =  111; 
  _coupling[23] = 0.916; _maxweight[23] = 1.7;
  // K*'' decays
  _incoming[24] =  30313; _outgoing1[24] =  321; _outgoing2[24] = -211; 
  _coupling[24] = 3.114;  _maxweight[24] = 1.7;
  _incoming[25] =  30313; _outgoing1[25] =  311; _outgoing2[25] =  111; 
  _coupling[25] = 2.201;  _maxweight[25] = 1.7;
  _incoming[26] =  30323; _outgoing1[26] =  311; _outgoing2[26] =  211; 
  _coupling[26] = 3.114;  _maxweight[26] = 1.7;
  _incoming[27] =  30323; _outgoing1[27] =  321; _outgoing2[27] =  111; 
  _coupling[27] = 2.201;  _maxweight[27] = 1.7;
  // phi decays
  _incoming[28] =  333; _outgoing1[28] =  321; _outgoing2[28] = -321; 
  _coupling[28] = 4.48;      _maxweight[28] = 1.7; 
  _incoming[29] =  333; _outgoing1[29] =  311; _outgoing2[29] = -311; 
  _coupling[29] = 4.59;      _maxweight[29] = 1.7; 
  _incoming[62] =  333; _outgoing1[62] =  130; _outgoing2[62] = 310; 
  _coupling[62] = 4.59;      _maxweight[62] = 1.7; 
  _incoming[30] =  333; _outgoing1[30] =  211; _outgoing2[30] = -211; 
  _coupling[30] = 8.986E-3;  _maxweight[30] = 1.7; 
  // phi' decays
  _incoming[31] =  100333; _outgoing1[31] =  321; _outgoing2[31] = -321; 
  _coupling[31] = 0.912; _maxweight[31] = 1.7;  
  _incoming[32] =  100333; _outgoing1[32] =  311; _outgoing2[32] = -311; 
  _coupling[32] = 0.918; _maxweight[32] = 1.7;   
  _incoming[63] =  100333; _outgoing1[63] =  130; _outgoing2[63] = 310; 
  _coupling[63] = 0.918; _maxweight[63] = 1.7;  
  // excited psi decays
  _incoming[33] = 30443; _outgoing1[33] =  411; _outgoing2[33] = -411; 
  _coupling[33] = 12.83; _maxweight[33] = 1.7; 
  _incoming[34] = 30443; _outgoing1[34] =  421; _outgoing2[34] = -421; 
  _coupling[34] = 12.83; _maxweight[34] = 1.7;
  // D* decays
  _incoming[35] =  423; _outgoing1[35] =  421; _outgoing2[35] = 111; 
  _coupling[35] = 6.309; _maxweight[35] = 1.7; 
  _incoming[36] =  413; _outgoing1[36] =  411; _outgoing2[36] = 111; 
  _coupling[36] = 6.313; _maxweight[36] = 1.7; 
  _incoming[37] =  413; _outgoing1[37] =  421; _outgoing2[37] = 211; 
  _coupling[37] = 8.938; _maxweight[37] = 1.7; 
  // D_s* decays
  _incoming[38] =  433; _outgoing1[38] =  431; _outgoing2[38] = 111; 
  _coupling[38] = 1.398; _maxweight[38] = 1.7; 
  // K_1 decays to K*_0 pion
  _incoming[39] =  10323; _outgoing1[39] =  10321; _outgoing2[39] =  111;  
  _coupling[39] = 14.45; _maxweight[39] = 8.9; 
  _incoming[40] =  10323; _outgoing1[40] =  10311; _outgoing2[40] =  211; 
  _coupling[40] = 20.46; _maxweight[40] = 8.9; 
  _incoming[41] =  10313; _outgoing1[41] =  10311; _outgoing2[41] =  111; 
  _coupling[41] = 14.27; _maxweight[41] = 8.9; 
  _incoming[42] =  10313; _outgoing1[42] =  10321; _outgoing2[42] = -211;
  _coupling[42] = 20.43; _maxweight[42] = 8.9;
  // K_1 decays to f(1370) kaon
  _incoming[43] =  10323; _outgoing1[43] =  321; _outgoing2[43] =  10221; 
  _coupling[43] = 20.; _maxweight[43] = 4.2; 
  _incoming[44] =  10313; _outgoing1[44] =  311; _outgoing2[44] =  10221; 
  _coupling[44] = 20.; _maxweight[44] = 4.2;
  // K'_1 decays to f(1370) kaon
  _incoming[45] =  20323; _outgoing1[45] =  321; _outgoing2[45] =  10221; 
  _coupling[45] = 23.34; _maxweight[45] = 6.; 
  _incoming[46] =  20313; _outgoing1[46] =  311; _outgoing2[46] =  10221; 
  _coupling[46] = 23.34; _maxweight[46] = 6.; 
  // upsilon(4s)
  _incoming[47] = 300553; _outgoing1[47] = 521; _outgoing2[47] = -521; 
  _coupling[47] = 24.188; _maxweight[47] = 1.6; 
  _incoming[48] = 300553; _outgoing1[48] = 511; _outgoing2[48] = -511; 
  _coupling[48] = 24.188; _maxweight[48] = 1.6;
  // jpsi to pions
  _incoming[49] = 443; _outgoing1[49] = 211; _outgoing2[49] = -211; 
  _coupling[49] = 2.568E-3; _maxweight[49] = 1.7; 
  // jpsi to kaons
  _incoming[50] = 443; _outgoing1[50] = 321; _outgoing2[50] = -321; 
  _coupling[50] = 1.111E-3; _maxweight[50] = 1.7; 
  _incoming[51] = 443; _outgoing1[51] = 311; _outgoing2[51] = -311; 
  _coupling[51] = 0.873E-3; _maxweight[51] = 1.7; 
  _incoming[60] = 443; _outgoing1[60] = 130; _outgoing2[60] = 310; 
  _coupling[60] = 0.873E-3; _maxweight[60] = 1.7; 
  // psi(2s) to pions
  _incoming[52] = 100443; _outgoing1[52] = 211; _outgoing2[52] = -211; 
  _coupling[52] = 1.055E-3; _maxweight[52] = 1.7;
  // psi(2s) to kaons
  _incoming[53] = 100443; _outgoing1[53] = 321; _outgoing2[53] = -321; 
  _coupling[53] = 0.905E-3; _maxweight[53] = 1.7; 
  _incoming[54] = 100443; _outgoing1[54] = 311; _outgoing2[54] = -311; 
  _coupling[54] = 0.905E-3; _maxweight[54] = 1.7;  
  _incoming[61] = 100443; _outgoing1[61] = 130; _outgoing2[61] = 310; 
  _coupling[61] = 0.905E-3; _maxweight[61] = 1.7;
  // f_1 to a_0 pi
  _incoming[55] = 20223; _outgoing1[55] =  9000111; _outgoing2[55] =  111; 
  _coupling[55] = 4.169; _maxweight[55] = 10.; 
  _incoming[56] = 20223; _outgoing1[56] =  9000211; _outgoing2[56] = -211; 
  _coupling[56] = 4.169; _maxweight[56] = 10.;
  // f'_1 to a_0 pi
  _incoming[57] = 20333; _outgoing1[57] =  9000111; _outgoing2[57] =  111; 
  _coupling[57] = 0.968; _maxweight[57] = 10.; 
  _incoming[58] = 20333; _outgoing1[58] =  9000211; _outgoing2[58] = -211; 
  _coupling[58] = 0.968; _maxweight[58] = 10.; 
  // initial size of the vectors for the database output
  _initsize=_incoming.size();
}
 
int VectorMeson2MesonDecayer::modeNumber(bool & cc,tcPDPtr parent,
					 const tPDVector & children) const {
  if(children.size()!=2) return -1;
  int id(parent->id());
  int idbar = parent->CC() ? parent->CC()->id() : id;
  int id1(children[0]->id());
  int id1bar = children[0]->CC() ? children[0]->CC()->id() : id1;
  int id2(children[1]->id());
  int id2bar = children[1]->CC() ? children[1]->CC()->id() : id2;
  int imode(-1);
  unsigned int ix(0);
  cc=false;
  do {
    if(id   ==_incoming[ix]) {
      if((id1   ==_outgoing1[ix]&&id2   ==_outgoing2[ix])||
	 (id2   ==_outgoing1[ix]&&id1   ==_outgoing2[ix])) imode=ix;
    }
    if(idbar==_incoming[ix]&&imode<0) {
      if((id1bar==_outgoing1[ix]&&id2bar==_outgoing2[ix])||
	 (id2bar==_outgoing1[ix]&&id1bar==_outgoing2[ix])) {
	imode=ix;
	cc=true;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  return imode;
}
  
void VectorMeson2MesonDecayer::persistentOutput(PersistentOStream & os) const {
  os << _incoming << _outgoing1 << _outgoing2 << _maxweight << _coupling;
}
  
void VectorMeson2MesonDecayer::persistentInput(PersistentIStream & is, int) {
  is >> _incoming >> _outgoing1 >> _outgoing2 >> _maxweight >> _coupling;
}
  
// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<VectorMeson2MesonDecayer,DecayIntegrator>
describeHerwigVectorMeson2MesonDecayer("Herwig::VectorMeson2MesonDecayer", "HwVMDecay.so");

void VectorMeson2MesonDecayer::Init() {
  
  static ClassDocumentation<VectorMeson2MesonDecayer> documentation
    ("The VectorMeson2MesonDecayer class is designed to implement "
     "the decay of vector mesons to 2 scalar mesons via a current which is the "
     "difference of the momenta of the two scalars. The order of the scalar meson "
     "momenta does not matter as it only changes the sign of the matrix element.");

  static ParVector<VectorMeson2MesonDecayer,int> interfaceIncoming
    ("Incoming",
     "The PDG code for the incoming particle",
     &VectorMeson2MesonDecayer::_incoming,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMeson2MesonDecayer,int> interfaceOutcoming1
    ("FirstOutgoing",
     "The PDG code for the first outgoing particle",
     &VectorMeson2MesonDecayer::_outgoing1,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMeson2MesonDecayer,int> interfaceOutcoming2
    ("SecondOutgoing",
     "The PDG code for the second outgoing particle",
     &VectorMeson2MesonDecayer::_outgoing2,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMeson2MesonDecayer,double> interfaceCoupling
    ("Coupling",
     "The coupling for the decay mode",
     &VectorMeson2MesonDecayer::_coupling,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMeson2MesonDecayer,double> interfaceMaxWeight
    ("MaxWeight",
     "The maximum weight for the decay mode",
     &VectorMeson2MesonDecayer::_maxweight,
     0, 0, 0, -10000000, 10000000, false, false, true);
  
}

double VectorMeson2MesonDecayer::me2(const int,
				     const Particle & inpart,
				     const ParticleVector & decay,
				     MEOption meopt) const {
  if(!ME())
    ME(new_ptr(TwoBodyDecayMatrixElement(PDT::Spin1,PDT::Spin0,PDT::Spin0)));
  if(meopt==Initialize) {
    VectorWaveFunction::calculateWaveFunctions(_vectors,_rho,
					       const_ptr_cast<tPPtr>(&inpart),
					       incoming,false);
  }
  if(meopt==Terminate) {
    VectorWaveFunction::constructSpinInfo(_vectors,const_ptr_cast<tPPtr>(&inpart),
					  incoming,true,false);
    for(unsigned int ix=0;ix<2;++ix)
      ScalarWaveFunction::constructSpinInfo(decay[ix],outgoing,true);
    return 0.;
  }
  // difference of the momenta
  Lorentz5Vector<double> pdiff
    = (decay[0]->momentum()-decay[1]->momentum()) 
    * _coupling[imode()]/inpart.mass();
  // compute the matrix element
  for(unsigned int ix=0;ix<3;++ix) 
    (*ME())(ix,0,0)=_vectors[ix].dot(pdiff);
  // test of the matrix element
//   double me = newME.contract(_rho).real();
//   Energy pcm=Kinematics::pstarTwoBodyDecay(inpart.mass(),decay[0]->mass(),
// 					   decay[1]->mass());
//   double test = 4.*sqr(_coupling[imode()]*pcm/inpart.mass())/3.;
//   cerr << "testing matrix element for " << inpart.PDGName() << " -> " 
//        << decay[0]->PDGName() << " " << decay[1]->PDGName() << " "
//        << me << " " << test << " " << (me-test)/(me+test) << "\n";
  // return the answer
  return ME()->contract(_rho).real();
}
 
bool VectorMeson2MesonDecayer::twoBodyMEcode(const DecayMode & dm,int & mecode,
					     double & coupling) const {
  int imode(-1);
  int id(dm.parent()->id()),idbar(id);
  if(dm.parent()->CC()) idbar=dm.parent()->CC()->id();
  ParticleMSet::const_iterator pit(dm.products().begin());
  int id1((**pit).id()),id1bar(id1);
  if((**pit).CC()) id1bar=(**pit).CC()->id();
  ++pit;
  int id2((**pit).id()),id2bar(id2);
  if((**pit).CC()) id2bar=(**pit).CC()->id();
  unsigned int ix(0); bool order(false);
  do {
    if(id   ==_incoming[ix]) {
      if(id1==_outgoing1[ix]&&id2==_outgoing2[ix]) {
	imode=ix;
	order=true;
      }
      if(id2==_outgoing1[ix]&&id1==_outgoing2[ix]) {
	imode=ix;
	order=false;
      }
    }
    if(idbar==_incoming[ix]&&imode<0) {
      if(id1bar==_outgoing1[ix]&&id2bar==_outgoing2[ix]) {
	imode=ix;
	order=true;
      }
      if(id2bar==_outgoing1[ix]&&id1bar==_outgoing2[ix]) {
	imode=ix;
	order=false;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  coupling=_coupling[imode];
  mecode=0;
  return order;
}

// output the setup information for the particle database
void VectorMeson2MesonDecayer::dataBaseOutput(ofstream & output,
					      bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  // the rest of the parameters
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":Incoming " << ix << " " 
	     << _incoming[ix] << "\n";
      output << "newdef " << name() << ":FirstOutgoing " << ix << " " 
	     << _outgoing1[ix] << "\n";
      output << "newdef " << name() << ":SecondOutgoing " << ix << " " 
	     << _outgoing2[ix] << "\n";
      output << "newdef " << name() << ":Coupling " << ix << " " 
	     << _coupling[ix] << "\n";
      output << "newdef " << name() << ":MaxWeight " << ix << " " 
	     << _maxweight[ix] << "\n";
    }
    else {
      output << "insert " << name() << ":Incoming " << ix << " " 
	     << _incoming[ix] << "\n";
      output << "insert " << name() << ":FirstOutgoing " << ix << " " 
	     << _outgoing1[ix] << "\n";
      output << "insert " << name() << ":SecondOutgoing " << ix << " " 
	     << _outgoing2[ix] << "\n";
      output << "insert " << name() << ":Coupling " << ix << " " 
	     << _coupling[ix] << "\n";
      output << "insert " << name() << ":MaxWeight " << ix << " " 
	     << _maxweight[ix] << "\n";
    }
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" << fullName() << "\";" << endl;
}
#line 1 "./VectorMeson3PionDecayer.cc"
// -*- C++ -*-
//
// VectorMeson3PionDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the VectorMeson3PionDecayer class.
//

#include "VectorMeson3PionDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "Herwig/Utilities/Kinematics.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Helicity/epsilon.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "Herwig/PDT/ThreeBodyAllOnCalculator.h"
#include "Herwig/Decay/GeneralDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void VectorMeson3PionDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    double temp;
    unsigned int iy;
    for(unsigned int ix=0;ix<_incoming.size();++ix) {
      _maxwgt[ix]=mode(ix)->maxWeight();
      for(iy=0;iy<3;++iy) {
	if(mode(ix)->numberChannels()>3*iy+1) {
	  temp=mode(ix)->channelWeight(3*iy)+mode(ix)->channelWeight(3*iy+1)+
	    mode(ix)->channelWeight(3*iy+2);
	  temp/=3.;
	  switch(iy) {
	  case 0: _rho1wgt[ix]=temp; break;
	  case 1: _rho2wgt[ix]=temp; break;
	  case 2: _rho3wgt[ix]=temp; break;
	  }
	}
      }
    }
  }
}

VectorMeson3PionDecayer::VectorMeson3PionDecayer() 
  : _incoming(2), _coupling(2), _directcoupling(2), _directphase(2),
    _rho2coupling(2), _rho2phase(2), _rho3coupling(2), _rho3phase(2),
    _maxwgt(2), _rho1wgt(2), _rho2wgt(2), _rho3wgt(2), _rho1mass(2),
    _rho2mass(2), _rho3mass(2), _rho1width(2), _rho2width(2), 
    _rho3width(2), _defaultmass(2), _mpic(ZERO), _mpi0(ZERO) {
  // matrix element storage
  // omega decay
  _incoming[0] = 223;
  _coupling[0] = 178.71/GeV;
  _directcoupling[0] = 0.;_directphase[0] = 0.;
  _rho2coupling[0] = 0.;_rho2phase[0] = 0.;
  _rho3coupling[0] = 0.;_rho3phase[0] = 0.;
  _maxwgt[0] = 6.64168;
  _rho1wgt[0] =  1.0;
  _rho2wgt[0] = -1.0; 
  _rho3wgt[0] = -1.0;
  _rho1mass[0] = 0.7758*GeV; 
  _rho2mass[0] = 1.4650*GeV;
  _rho3mass[0] = 1.7000*GeV; 
  _rho1width[0] = 0.1503*GeV; 
  _rho2width[0] = 0.3100*GeV;
  _rho3width[0] = 0.2400*GeV;
  _defaultmass[0] = true;
  // phi decay
  _incoming[1] = 333;
  _coupling[1] = 8.788/GeV;
  _directcoupling[1] = 0.78; _directphase[1] = -2.47;
  _rho2coupling[1] = 0.;_rho2phase[1] = 0.;
  _rho3coupling[1] = 0.;_rho3phase[1] = 0.;
  _maxwgt[1] = 5.62103;
  _rho1wgt[1] =  1.0;
  _rho2wgt[1] = -1.0; 
  _rho3wgt[1] = -1.0;
  _rho1mass[1] = 0.7758*GeV; 
  _rho2mass[1] = 1.4500*GeV;
  _rho3mass[1] = 1.7000*GeV; 
  _rho1width[1] = 0.1439*GeV; 
  _rho2width[1] = 0.3100*GeV;
  _rho3width[1] = 0.2400*GeV;
  _defaultmass[1] = false;
  // initial size of the arrays
  _initsize=_coupling.size();
  // generation of intermediates
  generateIntermediates(true);
}

void VectorMeson3PionDecayer::doinit() {
  DecayIntegrator::doinit();
  // check the consistence of the decay modes
  unsigned int isize=_incoming.size();
  if(isize!=_coupling.size()       || 
     isize!=_directcoupling.size() || isize!=_directphase.size()  ||
     isize!=_rho2coupling.size()   || isize!=_rho2phase.size()    ||
     isize!=_rho3coupling.size()   || isize!=_rho3phase.size()    ||
     isize!=_maxwgt.size()         || isize!=_rho1wgt.size()      ||
     isize!=_rho2wgt.size()        || isize!=_rho3wgt.size()      ||
     isize!=_rho1mass.size()       || isize!=_rho2mass.size()     ||
     isize!=_rho3mass.size()       || isize!=_rho1width.size()    ||
     isize!=_rho2width.size()      || isize!=_rho3width.size())
    throw InitException() << "Inconsistent parameters in " 
			  << "VectorMeson3PionDecayer::doinit()" 
			  << Exception::abortnow;
  // calculate the parameters 
  // set the external particles
  tPDVector extpart(4);
  extpart[1]=getParticleData(ParticleID::pi0);
  extpart[2]=getParticleData(ParticleID::piplus);
  extpart[3]=getParticleData(ParticleID::piminus);
  // pointer to the different rho resonances
  // the rho0 resonances
  tPDPtr rho0[3]={getParticleData(113),getParticleData(100113),getParticleData(30113)};
  // the charged rho resonance
  tPDPtr rhom[3]={getParticleData(-213),getParticleData(-100213),
		  getParticleData(-30213)};
  tPDPtr rhop[3]={getParticleData(213),getParticleData(100213),getParticleData(30213)};
  // create the integration channels for the decay
  DecayPhaseSpaceModePtr mode;
  DecayPhaseSpaceChannelPtr newchannel;
  unsigned int iy,iz;
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    extpart[0]=getParticleData(int(_incoming[ix]));
    mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
    // decide which rho resonances to add
    double  temp[3] = {_rho1wgt[ix]  ,_rho2wgt[ix]  ,_rho3wgt[ix]  };
    Energy  mass[3] = {_rho1mass[ix] ,_rho2mass[ix] ,_rho3mass[ix] };
    Energy width[3] = {_rho1width[ix],_rho2width[ix],_rho3width[ix]};
    vector<double> wgt;
    // set the mass parameters to the default if needed
    if(_defaultmass[ix]) {
      if(rhom[0]) {
	_rho1mass[ix] = rhom[0]->mass(); 
	_rho1width[ix] = rhom[0]->width();
      }
      if(rhom[1]) {
	_rho2mass[ix] = rhom[1]->mass(); 
	_rho2width[ix] = rhom[1]->width();
      }
      if(rhom[2]) {
	_rho3mass[ix] = rhom[2]->mass(); 
	_rho3width[ix] = rhom[2]->width();
      }
    }
    double sumwgt(0);
    for(iy=0;iy<3;++iy) {
      if(temp[iy]>0) sumwgt+=temp[iy];
    }
    for(iy=0;iy<3;++iy) {
      if(temp[iy]>0) {
	// set the weights for the channels
	for(iz=0;iz<3;++iz) wgt.push_back(temp[iy]/3./sumwgt);
	// rho0 channel
	newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
	newchannel->addIntermediate(extpart[0],0,0.0,-1,1);
	newchannel->addIntermediate(rho0[iy]  ,0,0.0, 2,3);
	mode->addChannel(newchannel);
	if(!_defaultmass[ix]) resetIntermediate(rho0[iy],mass[iy],width[iy]);
	// rho+ channel
	newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
	newchannel->addIntermediate(extpart[0],0,0.0,-1,3);
	newchannel->addIntermediate(rhop[iy]  ,0,0.0, 1,2);
	mode->addChannel(newchannel);
	if(!_defaultmass[ix]) resetIntermediate(rhop[iy],mass[iy],width[iy]);
	// rho- channel
	newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
	newchannel->addIntermediate(extpart[0],0,0.0,-1,2);
	newchannel->addIntermediate(rhom[iy]  ,0,0.0, 1,3);
	mode->addChannel(newchannel);
	if(!_defaultmass[ix]) mode->resetIntermediate(rhom[iy],mass[iy],width[iy]);
	addMode(mode,_maxwgt[ix],wgt);
      }
    }
  }
  // work out the masses and constants for the running widths
  Energy pcm;
  _mpi0=getParticleData(ParticleID::pi0)->mass();
  _mpic=getParticleData(ParticleID::piplus)->mass();
  Complex ii(0.,1.);
  _rhomass.resize(_incoming.size()); _rhomass2.resize(_incoming.size());
  _rho0const.resize(_incoming.size()); _rhocconst.resize( _incoming.size());
  _ccoupling.resize(_incoming.size());
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    // set the masses of the rho resonances
    _rhomass[ix].push_back(_rho1mass[ix]);
    _rhomass[ix].push_back(_rho2mass[ix]);
    _rhomass[ix].push_back(_rho3mass[ix]);
    _rhomass2[ix].push_back(_rho1mass[ix]*_rho1mass[ix]);
    _rhomass2[ix].push_back(_rho2mass[ix]*_rho2mass[ix]);
    _rhomass2[ix].push_back(_rho3mass[ix]*_rho3mass[ix]);
    // set up the constants for the running width
    pcm=Kinematics::pstarTwoBodyDecay(_rho1mass[ix],_mpic,_mpic);
    _rho0const[ix].push_back(_rho1mass[ix]*_rho1mass[ix]*_rho1width[ix]/(pcm*pcm*pcm));
    pcm=Kinematics::pstarTwoBodyDecay(_rho2mass[ix],_mpic,_mpic);
    _rho0const[ix].push_back(_rho2mass[ix]*_rho2mass[ix]*_rho2width[ix]/(pcm*pcm*pcm));
    pcm=Kinematics::pstarTwoBodyDecay(_rho3mass[ix],_mpic,_mpic);
    _rho0const[ix].push_back(_rho3mass[ix]*_rho3mass[ix]*_rho3width[ix]/(pcm*pcm*pcm));
    pcm=Kinematics::pstarTwoBodyDecay(_rho1mass[ix],_mpi0,_mpic);
    _rhocconst[ix].push_back(_rho1mass[ix]*_rho1mass[ix]*_rho1width[ix]/(pcm*pcm*pcm));
    pcm=Kinematics::pstarTwoBodyDecay(_rho2mass[ix],_mpi0,_mpic);
    _rhocconst[ix].push_back(_rho2mass[ix]*_rho2mass[ix]*_rho2width[ix]/(pcm*pcm*pcm));
    pcm=Kinematics::pstarTwoBodyDecay(_rho3mass[ix],_mpi0,_mpic);
    _rhocconst[ix].push_back(_rho3mass[ix]*_rho3mass[ix]*_rho3width[ix]/(pcm*pcm*pcm));
    // set the complex coupling constants
    _ccoupling[ix].push_back(1./_rhomass2[ix][0]);
    _ccoupling[ix].push_back( _rho2coupling[ix]/_rhomass2[ix][0]*
			      (cos(_rho2phase[ix])+ii*sin(_rho2phase[ix])));
    _ccoupling[ix].push_back( _rho3coupling[ix]/_rhomass2[ix][0]*
			      (cos(_rho3phase[ix])+ii*sin(_rho3phase[ix])));
    _ccoupling[ix].push_back(_directcoupling[ix]/_rhomass2[ix][0]*
			     (cos(_directphase[ix])+ii*sin(_directphase[ix])));
  }
}

int VectorMeson3PionDecayer::modeNumber(bool & cc,tcPDPtr parent,
					const tPDVector & children) const {
  cc=false;
  // must be three outgoing particles
  if(children.size()!=3) return -1;
  // check the id's of the outgoing particles
  int id;
  unsigned int npi0(0),npip(0),npim(0);
  tPDVector::const_iterator pit  = children.begin();
  for(;pit!=children.end();++pit) {
    id = (*pit)->id();
    if(id==ParticleID::pi0)          ++npi0;
    else if(id==ParticleID::piplus)  ++npip;
    else if(id==ParticleID::piminus) ++npim;
  }
  if(!(npi0==1&&npip==1&&npim==1)) return -1;
  unsigned int ix(0);
  id=parent->id();
  int imode(-1);
  do {
    if(_incoming[ix]==id) imode=ix;
    ++ix;
  }
  while(imode<0&&ix<_incoming.size());
  return imode;
}

void VectorMeson3PionDecayer::persistentOutput(PersistentOStream & os) const {
  os << _incoming << ounit(_coupling,1/MeV) << _directcoupling << _rho2coupling 
     << _rho3coupling << _directphase << _rho2phase << _rho3phase 
     << _maxwgt << _rho1wgt << _rho2wgt << _rho3wgt << ounit(_rho1mass,GeV) 
     << ounit(_rho2mass,GeV) << ounit(_rho3mass,GeV) << ounit(_rho1width,GeV) 
     << ounit(_rho2width,GeV) << ounit(_rho3width,GeV) << _defaultmass 
     << _rho0const << _rhocconst << ounit(_rhomass,GeV) << ounit(_rhomass2,GeV2) 
     << ounit(_ccoupling,1/MeV2) << ounit(_mpi0,MeV) << ounit(_mpic,MeV);
}

void VectorMeson3PionDecayer::persistentInput(PersistentIStream & is, int) {
  is >> _incoming >> iunit(_coupling,1/MeV) >> _directcoupling >> _rho2coupling 
     >> _rho3coupling >> _directphase >> _rho2phase >> _rho3phase 
     >> _maxwgt >> _rho1wgt >> _rho2wgt >> _rho3wgt >> iunit(_rho1mass,GeV) 
     >> iunit(_rho2mass,GeV) >> iunit(_rho3mass,GeV) >> iunit(_rho1width,GeV) 
     >> iunit(_rho2width,GeV) >> iunit(_rho3width,GeV) >> _defaultmass
     >> _rho0const >> _rhocconst >> iunit(_rhomass,GeV) >> iunit(_rhomass2,GeV2) 
     >> iunit(_ccoupling,1/MeV2) >> iunit(_mpi0,MeV) >> iunit(_mpic,MeV);
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<VectorMeson3PionDecayer,DecayIntegrator>
describeHerwigVectorMeson3PionDecayer("Herwig::VectorMeson3PionDecayer", "HwVMDecay.so");

void VectorMeson3PionDecayer::Init() {

  static ClassDocumentation<VectorMeson3PionDecayer> documentation
    ("The VectorMeson3PionDecayer class is designed for the decay "
     "of I=0 vector mesons to three pions via a current taking into account the "
     "rho and a possible direct term",
     "The decay of I=0 vector mesons to three pions via a current taking into account the "
     "rho and a possible direct term is taken from \\cite{Aloisio:2003ur}.",
     "%\\cite{Aloisio:2003ur}\n"
     "\\bibitem{Aloisio:2003ur}\n"
     "  A.~Aloisio {\\it et al.}  [KLOE Collaboration],\n"
     "  %``Study of the decay Phi --> pi+ pi- pi0 with the KLOE detector,''\n"
     "  Phys.\\ Lett.\\  B {\\bf 561}, 55 (2003)\n"
     "  [Erratum-ibid.\\  B {\\bf 609}, 449 (2005)]\n"
     "  [arXiv:hep-ex/0303016].\n"
     "  %%CITATION = PHLTA,B561,55;%%\n"
     );
  
  static ParVector<VectorMeson3PionDecayer,double> interfaceIncoming
    ("Incoming",
     "The PDG code for the incoming particle",
     &VectorMeson3PionDecayer::_incoming,
     0, 0, 0, 0, 1000000, false, false, true);
  
  static ParVector<VectorMeson3PionDecayer,InvEnergy> interfaceCoupling
    ("Coupling",
     "The overall coupling for the decay, this is the coupling of the decaying "
     "particle to the lowest lying rho multiplet.",
     &VectorMeson3PionDecayer::_coupling,
     1./GeV, -1, 1./GeV,-1000./GeV, 1000./GeV, false, false, true);
  
  static ParVector<VectorMeson3PionDecayer,double> interfaceDirectCoupling
    ("DirectCoupling",
     "The magnitude of the coupling of the direct term with respect to the "
     "coupling of the lowest lying rho multiplet",
     &VectorMeson3PionDecayer::_directcoupling,
     0, 0, 0, -100000., 100000., false, false, true);
  
  static ParVector<VectorMeson3PionDecayer,double> interfaceRho2Coupling
    ("Rho2Coupling",
     "The magnitude of the coupling of the second rho multiplet with respect to the "
     "lowest lying  multiplet",
     &VectorMeson3PionDecayer::_rho2coupling,
     0, 0, 0, -10., 10., false, false, true);
  
  static ParVector<VectorMeson3PionDecayer,double> interfaceRho3Coupling
    ("Rho3Coupling",
     "The magntiude of the coupling of the third rho multiplet with respect to the "
     "lowest lying multiplet",
     &VectorMeson3PionDecayer::_rho3coupling,
     0, 0, 0, -10., 10., false, false, true);

  static ParVector<VectorMeson3PionDecayer,double> interfaceDirectPhase
    ("DirectPhase",
     "The phase of the coupling of the direct term with respect to the "
     "coupling of the lowest lying rho multiplet",
     &VectorMeson3PionDecayer::_directphase,
     0, 0, 0, -Constants::twopi, Constants::twopi, false, false, true);
  
  static ParVector<VectorMeson3PionDecayer,double> interfaceRho2Phase
    ("Rho2Phase",
     "The phasee of the coupling of the second rho multiplet with respect to the "
     "lowest lying  multiplet",
     &VectorMeson3PionDecayer::_rho2phase,
     0, 0, 0, -Constants::twopi, Constants::twopi, false, false, true);
  
  static ParVector<VectorMeson3PionDecayer,double> interfaceRho3Phase
    ("Rho3Phase",
     "The phase of the coupling of the third rho multiplet with respect to the "
     "lowest lying multiplet",
     &VectorMeson3PionDecayer::_rho3phase,
     0, 0, 0, -Constants::twopi, Constants::twopi, false, false, true);
  
  static ParVector<VectorMeson3PionDecayer,double> interfaceMaxWgt
    ("MaxWeight",
     "The maximum weight for the integration of the channel",
     &VectorMeson3PionDecayer::_maxwgt,
     0, 0, 0, 0., 100., false, false, true);
  
  static ParVector<VectorMeson3PionDecayer,double> interfaceRho1Wgt
    ("Rho1Weight",
     "The weight for the lowest lying rho multiplet's in the integration",
     &VectorMeson3PionDecayer::_rho1wgt,
     0, 0, 0, 0, 1., false, false, true);

  static ParVector<VectorMeson3PionDecayer,double> interfaceRho2Wgt
    ("Rho2Weight",
     "The weight for the second rho multiplet's in the integration",
     &VectorMeson3PionDecayer::_rho2wgt,
     0, 0, 0, -2., 1., false, false, true);

  static ParVector<VectorMeson3PionDecayer,double> interfaceRho3Wgt
    ("Rho3Weight",
     "The weight for the third rho multiplet's in the integration",
     &VectorMeson3PionDecayer::_rho3wgt,
     0, 0, 0, -2., 1., false, false, true);

  static ParVector<VectorMeson3PionDecayer,Energy> interfaceRho1Mass
    ("Rho1Mass",
     "The mass of the lowest lying rho multiplet",
     &VectorMeson3PionDecayer::_rho1mass,
     GeV, -1, 0.77*GeV, ZERO, 5.*GeV, false, false, true);

  static ParVector<VectorMeson3PionDecayer,Energy> interfaceRho2Mass
    ("Rho2Mass",
     "The mass of the second rho multiplet",
     &VectorMeson3PionDecayer::_rho2mass,
     GeV, -1, 0.77*GeV, ZERO, 5.*GeV, false, false, true);

  static ParVector<VectorMeson3PionDecayer,Energy> interfaceRho3Mass
    ("Rho3Mass",
     "The mass of the third rho multiplet",
     &VectorMeson3PionDecayer::_rho3mass,
     GeV, -1, 0.77*GeV, ZERO, 5.*GeV, false, false, true);

  static ParVector<VectorMeson3PionDecayer,Energy> interfaceRho1Width
    ("Rho1Width",
     "The width of the lowest lying rho multiplet",
     &VectorMeson3PionDecayer::_rho1width,
     GeV, -1, 0.15*GeV, ZERO, 1.*GeV, false, false, true);

  static ParVector<VectorMeson3PionDecayer,Energy> interfaceRho2Width
    ("Rho2Width",
     "The width of the second rho multiplet",
     &VectorMeson3PionDecayer::_rho2width,
     GeV, -1, 0.15*GeV, ZERO, 1.*GeV, false, false, true);

  static ParVector<VectorMeson3PionDecayer,Energy> interfaceRho3Width
    ("Rho3Width",
     "The width of the third rho multiplet",
     &VectorMeson3PionDecayer::_rho3width,
     GeV, -1, 0.15*GeV, ZERO, 1.*GeV, false, false, true);

  static ParVector<VectorMeson3PionDecayer,bool> interfaceDefaultParam
    ("DefaultParameters",
     "If true the default rho masses and widths are used, otherwise "
     "the values specified in the rhomass and width arrays are used,",
     &VectorMeson3PionDecayer::_defaultmass,
     0, 0, 1, 0, 1, false, false, true);

}

double VectorMeson3PionDecayer::me2(const int ichan,
				    const Particle & inpart,
				    const ParticleVector & decay,
				    MEOption meopt) const {
  if(!ME())
    ME(new_ptr(GeneralDecayMatrixElement(PDT::Spin1,PDT::Spin0,PDT::Spin0,PDT::Spin0)));
  useMe();
  if(meopt==Initialize) {
    VectorWaveFunction::calculateWaveFunctions(_vectors,_rho,
						const_ptr_cast<tPPtr>(&inpart),
						incoming,false);
  }
  if(meopt==Terminate) {
    VectorWaveFunction::constructSpinInfo(_vectors,const_ptr_cast<tPPtr>(&inpart),
					  incoming,true,false);
    // set up the spin information for the decay products
    for(unsigned int ix=0;ix<3;++ix)
      ScalarWaveFunction::constructSpinInfo(decay[ix],outgoing,true);
    return 0.;
  }
  // compute the matrix element
  // work out the prefactor
  complex<InvEnergy2> pre(ZERO);
  Complex resfact,ii(0.,1.);
  if(ichan<0){pre=_ccoupling[imode()][3];}
  Energy pcm;
  // work out the direct invariant masses needed
  Energy mrho0(sqrt(decay[1]->momentum().m2(decay[2]->momentum())));
  Energy mrhop(sqrt(decay[1]->momentum().m2(decay[0]->momentum())));
  Energy mrhom(sqrt(decay[2]->momentum().m2(decay[0]->momentum())));
  // contribution of the resonances
  int ichannow(-3);
  for(unsigned int ix=0;ix<3;++ix) {
    ichannow+=3;
    if((ix==0 && _rho1wgt[imode()]>0.) || (ix==1 && _rho2wgt[imode()]>0.) ||
       (ix==2 && _rho3wgt[imode()]>0.)) {
      if(ichan<0) {
	// rho0 contribution
	pcm = Kinematics::pstarTwoBodyDecay(mrho0,_mpic,_mpic);
	resfact = _rhomass2[imode()][ix]/
	  (mrho0*mrho0-_rhomass2[imode()][ix]
	   +ii*pcm*pcm*pcm*_rho0const[imode()][ix]/mrho0);
	// rho+ contribution
	pcm = Kinematics::pstarTwoBodyDecay(mrhop,_mpic,_mpi0);
	resfact+= _rhomass2[imode()][ix]/
	  (mrhop*mrhop-_rhomass2[imode()][ix]
	   +ii*pcm*pcm*pcm*_rhocconst[imode()][ix]/mrhop);
	// rho- contribution
	pcm = Kinematics::pstarTwoBodyDecay(mrhom,_mpic,_mpi0);
	resfact+= _rhomass2[imode()][ix]/
	  (mrhom*mrhom-_rhomass2[imode()][ix]
	   +ii*pcm*pcm*pcm*_rhocconst[imode()][ix]/mrhom);
	// add the contribution
      }
      else if(ichan==ichannow) {
	pcm = Kinematics::pstarTwoBodyDecay(mrho0,_mpic,_mpic);
	resfact = _rhomass2[imode()][ix]/
	  (mrho0*mrho0-_rhomass2[imode()][ix]
	   +ii*pcm*pcm*pcm*_rho0const[imode()][ix]/mrho0);
      }
      else if(ichan==ichannow+1) {
	pcm = Kinematics::pstarTwoBodyDecay(mrhop,_mpic,_mpi0);
	resfact+= _rhomass2[imode()][ix]/
	  (mrhop*mrhop-_rhomass2[imode()][ix]
	   +ii*pcm*pcm*pcm*_rhocconst[imode()][ix]/mrhop);
      }
      else if(ichan==ichannow+2) {
	pcm = Kinematics::pstarTwoBodyDecay(mrhom,_mpic,_mpi0);
	resfact+= _rhomass2[imode()][ix]/
	  (mrhom*mrhom-_rhomass2[imode()][ix]
	   +ii*pcm*pcm*pcm*_rhocconst[imode()][ix]/mrhom);
      }
      pre += resfact * _ccoupling[imode()][ix];
      ichannow+=3;
    }
  }
  // polarization vector piece
  LorentzPolarizationVector 
    scalar=_coupling[imode()]*pre*epsilon(decay[0]->momentum(),
					  decay[1]->momentum(),
					  decay[2]->momentum());
  // compute the matrix element
  for(unsigned int ix=0;ix<3;++ix) 
    (*ME())(ix,0,0,0)=scalar.dot(_vectors[ix]);
  // return the answer
  return ME()->contract(_rho).real();
}

double VectorMeson3PionDecayer::
threeBodyMatrixElement(const int imode, const Energy2 q2,
		       const  Energy2 s3, const Energy2 s2, const Energy2 s1, const 
		       Energy , const Energy , const Energy ) const {
  Lorentz5Momentum p1,p2,p3; Energy2 ee1,ee2,ee3;Energy pp1,pp2,pp3;
  Energy q(sqrt(q2));
  Energy2 mpi2c(_mpic*_mpic),mpi20(_mpi0*_mpi0);
  p1.setE(0.5*(q2+mpi20-s1)/q); ee1=p1.e()*p1.e(); pp1=sqrt(ee1-mpi20);
  p2.setE(0.5*(q2+mpi2c-s2)/q); ee2=p2.e()*p2.e(); pp2=sqrt(ee2-mpi2c);
  p3.setE(0.5*(q2+mpi2c-s3)/q); ee3=p3.e()*p3.e(); pp3=sqrt(ee3-mpi2c);
  // take momentum of 1 parallel to z axis
  p1.setX(ZERO);p1.setY(ZERO);p1.setZ(pp1);
  // construct 2 
  double cos2(0.5*(ee1+ee2-ee3-mpi20)/pp1/pp2);
  p2.setX(pp2*sqrt(1.-cos2*cos2)); p2.setY(ZERO); p2.setZ(-pp2*cos2);
  // construct 3
  double cos3(0.5*(ee1-ee2+ee3-mpi20)/pp1/pp3);
  p3.setX(-pp3*sqrt(1.-cos3*cos3)); p3.setY(ZERO); p3.setZ(-pp3*cos3); 
  // compute the prefactor
  complex<InvEnergy2> pre(_ccoupling[imode][3]);
  Complex resfact,ii(0.,1.);
  // rho0 contribution
  Energy pcm,mrho1(sqrt(s1)),mrho2(sqrt(s2)),mrho3(sqrt(s3));
  for(unsigned int ix=0;ix<3;++ix) {
    // rho0 contribution
    pcm = Kinematics::pstarTwoBodyDecay(mrho1,_mpic,_mpic);
    resfact = _rhomass2[imode][ix]/(mrho1*mrho1-_rhomass2[imode][ix]
				    +ii*pcm*pcm*pcm*_rho0const[imode][ix]/mrho1);
    // rho+ contribution
    pcm = Kinematics::pstarTwoBodyDecay(mrho2,_mpic,_mpi0);
    resfact+= _rhomass2[imode][ix]/(mrho2*mrho3-_rhomass2[imode][ix]
				    +ii*pcm*pcm*pcm*_rhocconst[imode][ix]/mrho2);
    // rho- contribution
    pcm = Kinematics::pstarTwoBodyDecay(mrho3,_mpic,_mpi0);
    resfact+= _rhomass2[imode][ix]/(mrho3*mrho3-_rhomass2[imode][ix]
				    +ii*pcm*pcm*pcm*_rhocconst[imode][ix]/mrho3);
    // add the contribution
    pre+=resfact *_ccoupling[imode][ix];
  }
  LorentzPolarizationVector current =
    _coupling[imode]*(pre*epsilon(p1,p2,p3));
  Complex temp(current.dot(current.conjugate()));
  return -temp.real()/3.;
} 

WidthCalculatorBasePtr 
VectorMeson3PionDecayer::threeBodyMEIntegrator(const DecayMode & dm) const {
  // workout which mode we are doing
  int imode(-1),id(dm.parent()->id());
  unsigned int ix=0;
  do {
    if(_incoming[ix]==id) imode=ix; 
    ++ix;
  }
  while(imode<0&&ix<_incoming.size());
  // construct the integrator
  vector<double> inweights(3,1./3.);
  vector<int> intype;intype.push_back(1);intype.push_back(2);intype.push_back(3);
  Energy mrho(getParticleData(ParticleID::rhoplus)->mass());
  Energy wrho(getParticleData(ParticleID::rhoplus)->width());
  vector<Energy> inmass(3,mrho);
  vector<Energy> inwidth(3,wrho);
  vector<double> inpow(2,0.0);
  //tcDecayIntegratorPtr decayer(this);
  WidthCalculatorBasePtr output(
    new_ptr(ThreeBodyAllOnCalculator<VectorMeson3PionDecayer>
	    (inweights,intype,inmass,inwidth,inpow,
	     *this,imode,_mpi0,_mpic,_mpic)));
  return output;
}

void VectorMeson3PionDecayer::dataBaseOutput(ofstream & output,
					     bool header) const {
  if(header){output << "update decayers set parameters=\"";}
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":Incoming " 
	     << ix << " " << _incoming[ix] << endl;
      output << "newdef " << name() << ":Coupling " 
	     << ix << " " << _coupling[ix]*GeV << endl;
      output << "newdef " << name() << ":DirectCoupling " 
	     << ix << " " << _directcoupling[ix] << endl;
      output << "newdef " << name() << ":Rho2Coupling " 
	     << ix << " " << _rho2coupling[ix] << endl;
      output << "newdef " << name() << ":Rho3Coupling " 
	     << ix << " " << _rho3coupling[ix] << endl;
      output << "newdef " << name() << ":DirectPhase " 
	     << ix << " " << _directphase[ix] << endl;
      output << "newdef " << name() << ":Rho2Phase " 
	     << ix << " " << _rho2phase[ix] << endl;
      output << "newdef " << name() << ":Rho3Phase " 
	     << ix << " " << _rho3phase[ix] << endl;
      output << "newdef " << name() << ":MaxWeight " 
	     << ix << " " << _maxwgt[ix] << endl;
      output << "newdef " << name() << ":Rho1Weight " 
	     << ix << " " << _rho1wgt[ix] << endl;
      output << "newdef " << name() << ":Rho2Weight " 
	     << ix << " " << _rho2wgt[ix] << endl;
      output << "newdef " << name() << ":Rho3Weight " 
	     << ix << " " << _rho3wgt[ix] << endl;
      output << "newdef " << name() << ":Rho1Mass " 
	     << ix << " " << _rho1mass[ix]/GeV << endl;
      output << "newdef " << name() << ":Rho2Mass " 
	     << ix << " " << _rho2mass[ix]/GeV<< endl;
      output << "newdef " << name() << ":Rho3Mass " 
	     << ix << " " << _rho3mass[ix]/GeV<< endl;
      output << "newdef " << name() << ":Rho1Width " 
	     << ix << " " << _rho1width[ix]/GeV << endl;
      output << "newdef " << name() << ":Rho2Width " 
	     << ix << " " << _rho2width[ix]/GeV << endl;
      output << "newdef " << name() << ":Rho3Width " 
	     << ix << " " << _rho3width[ix]/GeV << endl;
      output << "newdef " << name() << ":DefaultParameters " 
	     << ix << " " << _defaultmass[ix] << endl;
    }
    else {
      output << "insert " << name() << ":Incoming " 
	     << ix << " " << _incoming[ix] << endl;
      output << "insert " << name() << ":Coupling " 
	     << ix << " " << _coupling[ix]*GeV << endl;
      output << "insert " << name() << ":DirectCoupling " 
	     << ix << " " << _directcoupling[ix] << endl;
      output << "insert " << name() << ":Rho2Coupling " 
	     << ix << " " << _rho2coupling[ix] << endl;
      output << "insert " << name() << ":Rho3Coupling " 
	     << ix << " " << _rho3coupling[ix] << endl;
      output << "insert " << name() << ":DirectPhase " 
	     << ix << " " << _directphase[ix] << endl;
      output << "insert " << name() << ":Rho2Phase " 
	     << ix << " " << _rho2phase[ix] << endl;
      output << "insert " << name() << ":Rho3Phase " 
	     << ix << " " << _rho3phase[ix] << endl;
      output << "insert " << name() << ":MaxWeight " 
	     << ix << " " << _maxwgt[ix] << endl;
      output << "insert " << name() << ":Rho1Weight " 
	     << ix << " " << _rho1wgt[ix] << endl;
      output << "insert " << name() << ":Rho2Weight " 
	     << ix << " " << _rho2wgt[ix] << endl;
      output << "insert " << name() << ":Rho3Weight " 
	     << ix << " " << _rho3wgt[ix] << endl;
      output << "insert " << name() << ":Rho1Mass " 
	     << ix << " " << _rho1mass[ix]/GeV << endl;
      output << "insert " << name() << ":Rho2Mass " 
	     << ix << " " << _rho2mass[ix]/GeV<< endl;
      output << "insert " << name() << ":Rho3Mass " 
	     << ix << " " << _rho3mass[ix]/GeV<< endl;
      output << "insert " << name() << ":Rho1Width " 
	     << ix << " " << _rho1width[ix]/GeV << endl;
      output << "insert " << name() << ":Rho2Width " 
	     << ix << " " << _rho2width[ix]/GeV << endl;
      output << "insert " << name() << ":Rho3Width " 
	     << ix << " " << _rho3width[ix]/GeV << endl;
      output << "insert " << name() << ":DefaultParameters " 
	     << ix << " " << _defaultmass[ix] << endl;
    }
  }
  if(header){output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;}
}
#line 1 "./VectorMesonPScalarFermionsDecayer.cc"
// -*- C++ -*-
//
// VectorMesonPScalarFermionsDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the VectorMesonPScalarFermionsDecayer class.
//
//  Author: Peter Richardson
//

#include "VectorMesonPScalarFermionsDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/SpinorWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/SpinorBarWaveFunction.h"
#include "ThePEG/Helicity/epsilon.h"
#include "Herwig/PDT/ThreeBodyAllOn1IntegralCalculator.h"
#include "Herwig/Decay/GeneralDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void VectorMesonPScalarFermionsDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    for(unsigned int ix=0;ix<_incoming.size();++ix) {
      _maxweight[ix] = mode(ix)->maxWeight();
      _weight[ix]    = mode(ix)->channelWeight(1);
    }
  }
}

VectorMesonPScalarFermionsDecayer::VectorMesonPScalarFermionsDecayer() 
  : _coupling(6), _incoming(6), _outgoingP(6), _outgoingf(6), _outgoinga(6), 
    _maxweight(6), _weight(6), _includeVMD(6), _VMDid(6), _VMDmass(6), 
    _VMDwidth(6) {
  // omega -> pi e+e- /mu+mu-
  _incoming[0] =  223; _outgoingP[0] =  111; 
  _outgoingf[0] = 11; _outgoinga[0] = -11; 
  _coupling[0] = 0.2179/GeV; _maxweight[0] = 4.0; _weight[0] = 0.; 
  _includeVMD[0] = 2; _VMDid[0] = 113; 
  _VMDmass[0] = 0.7758*GeV; _VMDwidth[0] = 0.1503*GeV; 
  _incoming[1] =  223; _outgoingP[1] =  111; 
  _outgoingf[1] = 13; _outgoinga[1] = -13; 
  _coupling[1] = 0.2179/GeV; _maxweight[1] = 2.8; _weight[1] = 0.; 
  _includeVMD[1] = 2; _VMDid[1] = 113; 
  _VMDmass[1] = 0.7758*GeV; _VMDwidth[1] = 0.1503*GeV; 
  // phi -> eta e+e-/mu+mu-
  _incoming[2] =  333; _outgoingP[2] =  221; 
  _outgoingf[2] = 11; _outgoinga[2] = -11; 
  _coupling[2] = 0.0643/GeV; _maxweight[2] = 3.7; _weight[2] = 0.; 
  _includeVMD[2] = 2; _VMDid[2] = 113; 
  _VMDmass[2] = 0.7758*GeV; _VMDwidth[2] = 0.1503*GeV; 
  _incoming[3] =  333; ;_outgoingP[3] =  221; 
  _outgoingf[3] = 13; _outgoinga[3] = -13; 
  _coupling[3] = 0.0643/GeV; _maxweight[3] = 2.8; _weight[3] = 0.; 
  _includeVMD[3] = 2; _VMDid[3] = 113; 
  _VMDmass[3] = 0.7758*GeV; _VMDwidth[3] = 0.1503*GeV; 
  // phi -> pi e+e-/mu+mu-
  _incoming[4] =  333; ;_outgoingP[4] =  111; 
  _outgoingf[4] = 11; _outgoinga[4] = -11; 
  _coupling[4] = 0.0120094/GeV; _maxweight[4] = 4.9; _weight[4] = 0.21; 
  _includeVMD[4] = 2; _VMDid[4] = 113; 
  _VMDmass[4] = 0.7758*GeV; _VMDwidth[4] = 0.1503*GeV; 
  _incoming[5] =  333; ;_outgoingP[5] =  111; 
  _outgoingf[5] = 13; _outgoinga[5] = -13; 
  _coupling[5] = 0.0120094/GeV; _maxweight[5] = 3.2; _weight[5] = 0.33; 
  _includeVMD[5] = 2; _VMDid[5] = 113; 
  _VMDmass[5] = 0.7758*GeV; _VMDwidth[5] = 0.1503*GeV; 
  // the initial size of the arrays
  _initsize=_coupling.size();
  // intermediates
  generateIntermediates(false);
}

void VectorMesonPScalarFermionsDecayer::doinit() {
  DecayIntegrator::doinit();
  // check the parameters are consistent
  unsigned int isize(_coupling.size());
  if(isize!=_incoming.size()  || isize!=_outgoingP.size()|| isize!=_outgoingf.size()||
     isize!=_outgoinga.size() || isize!=_maxweight.size()|| isize!=_includeVMD.size()||
     isize!=_VMDid.size()     || isize!=_VMDmass.size()  || isize!=_VMDwidth.size()||
     isize!=_weight.size())
    throw InitException() << "Inconsistent parameters in VectorMesonPScalar"
			  << "FermionsDecayer" << Exception::abortnow;
  // create the integration channel for each mode 
  tPDVector extpart(4);
  tPDPtr gamma(getParticleData(ParticleID::gamma)),rho;
  DecayPhaseSpaceChannelPtr newchannel;
  DecayPhaseSpaceModePtr newmode;
  vector<double> wgt(2,1.);
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    rho=getParticleData(_VMDid[ix]);
    extpart[0] = getParticleData(_incoming[ix]);
    extpart[1] = getParticleData(_outgoingP[ix]);
    extpart[2] = getParticleData(_outgoingf[ix]);
    extpart[3] = getParticleData(_outgoinga[ix]);
    newmode = new_ptr(DecayPhaseSpaceMode(extpart,this));
    // photon channel
    newchannel=new_ptr(DecayPhaseSpaceChannel(newmode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,1);
    newchannel->addIntermediate(gamma     ,1,-1.1, 2,3);
    newmode->addChannel(newchannel);
    wgt[0]=1.-_weight[ix];
    // vmd channel
    newchannel=new_ptr(DecayPhaseSpaceChannel(newmode));
    newchannel->addIntermediate(extpart[0],0, 0.0,-1,1);
    newchannel->addIntermediate(rho       ,0, 0.0, 2,3);
    newmode->addChannel(newchannel);
    wgt[1]=_weight[ix];
    addMode(newmode,_maxweight[ix],wgt);
  }
  // set up the values for the VMD factor if needed (copy the default mass and width 
  //                                                 into the array)
  for(unsigned ix=0;ix<isize;++ix) {
    if(_includeVMD[ix]==1) {
      _VMDmass[ix]=getParticleData(_VMDid[ix])->mass();
      _VMDwidth[ix]=getParticleData(_VMDid[ix])->width();
    }
  }
}

int VectorMesonPScalarFermionsDecayer::modeNumber(bool & cc,tcPDPtr parent,
					   const tPDVector & children) const {
  int imode(-1);
  // must be three outgoing particles
  if(children.size()!=3){return imode;}
  // ids of the particles
  int id0(parent->id()),idf[2]={0,0},ids(0);
  unsigned int nf(0);
  tPDVector::const_iterator pit = children.begin();
  for( ;pit!=children.end();++pit) {
    if((**pit).iSpin()==PDT::Spin0) ids=(**pit).id();
    else {
      idf[nf]=(**pit).id();
      ++nf;
    }
  }
  // loop over the modes and see if this is one of them
  unsigned int ix=0;
  do {
    if(_incoming[ix]==id0&&_outgoingP[ix]==ids) {
      if((idf[0]==_outgoingf[ix]&&idf[1]==_outgoinga[ix])||
	 (idf[1]==_outgoingf[ix]&&idf[0]==_outgoinga[ix])) imode=ix;
    }
    ++ix;
  }
  while(imode<0&&ix<_incoming.size());
  // perform the decay
  cc=false;
  return imode;
}

void VectorMesonPScalarFermionsDecayer::persistentOutput(PersistentOStream & os) const {
  os << ounit(_coupling,1/GeV) << _incoming << _outgoingP << _outgoingf << _outgoinga 
     << _maxweight << _weight << _includeVMD << _VMDid << ounit(_VMDmass,GeV) 
     << ounit(_VMDwidth,GeV);
}

void VectorMesonPScalarFermionsDecayer::persistentInput(PersistentIStream & is, int) {
  is >> iunit(_coupling,1/GeV) >> _incoming >> _outgoingP >> _outgoingf >> _outgoinga 
     >> _maxweight >> _weight >> _includeVMD >> _VMDid >> iunit(_VMDmass,GeV) 
     >> iunit(_VMDwidth,GeV);
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<VectorMesonPScalarFermionsDecayer,DecayIntegrator>
describeHerwigVectorMesonPScalarFermionsDecayer("Herwig::VectorMesonPScalarFermionsDecayer", "HwVMDecay.so");

void VectorMesonPScalarFermionsDecayer::Init() {

  static ClassDocumentation<VectorMesonPScalarFermionsDecayer> documentation
    ("The VectorMesonPScalarFermionsDecayer class is designed to "
     "perform the decay of a vector meson to a pseudoscalar meson and a "
     "fermion-antifermion pair.");

  static ParVector<VectorMesonPScalarFermionsDecayer,int> interfaceIncoming
    ("Incoming",
     "The PDG code for the incoming particle",
     &VectorMesonPScalarFermionsDecayer::_incoming,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMesonPScalarFermionsDecayer,int> interfaceOutcomingP
    ("OutgoingPseudoScalar",
     "The PDG code for the outgoing pseudoscalar",
     &VectorMesonPScalarFermionsDecayer::_outgoingP,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMesonPScalarFermionsDecayer,int> interfaceOutcomingF
    ("OutgoingFermion",
     "The PDG code for the outgoing fermion",
     &VectorMesonPScalarFermionsDecayer::_outgoingf,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMesonPScalarFermionsDecayer,int> interfaceOutcomingA
    ("OutgoingAntiFermion",
     "The PDG code for the outgoing antifermion",
     &VectorMesonPScalarFermionsDecayer::_outgoinga,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMesonPScalarFermionsDecayer,InvEnergy> interfaceCoupling
    ("Coupling",
     "The coupling for the decay mode",
     &VectorMesonPScalarFermionsDecayer::_coupling,
     1/MeV, 0, ZERO, -10000000/MeV, 10000000/MeV, false, false, true);

  static ParVector<VectorMesonPScalarFermionsDecayer,double> interfaceMaxWeight
    ("MaxWeight",
     "The maximum weight for the decay mode",
     &VectorMesonPScalarFermionsDecayer::_maxweight,
     0, 0, 0, 0., 200.0, false, false, true);

  static ParVector<VectorMesonPScalarFermionsDecayer,double> interfaceWeight
    ("Weight",
     "The weight for vector meson phasse space channel",
     &VectorMesonPScalarFermionsDecayer::_weight,
     0, 0, 0, 0., 200.0, false, false, true);

  static ParVector<VectorMesonPScalarFermionsDecayer,int> interfaceIncludeVMD
    ("IncludeVMD",
     "There are three options for 0 the VMD factor is not included, for 1 the factor "
     "is included using the default mass and width of the particle specified by"
     " VMDID, and for 2 the factor is included using the mass and width specified"
     " by VMDwidth and VMDmass.",
     &VectorMesonPScalarFermionsDecayer::_includeVMD,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMesonPScalarFermionsDecayer,int> interfaceVMDID
    ("VMDID",
     "The PDG code for the particle to be used for the VMD factor.",
     &VectorMesonPScalarFermionsDecayer::_VMDid,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMesonPScalarFermionsDecayer,Energy> interfaceVMDmass
    ("VMDmass",
     "The mass to use for the particle in the VMD factor",
     &VectorMesonPScalarFermionsDecayer::_VMDmass,
     MeV, 0, ZERO, -10000000*MeV, 10000000*MeV, false, false, true);

  static ParVector<VectorMesonPScalarFermionsDecayer,Energy> interfaceVMDwidth
    ("VMDwidth",
     "The width to use for the particle in the VMD factor",
     &VectorMesonPScalarFermionsDecayer::_VMDwidth,
     MeV, 0, ZERO, -10000000*MeV, 10000000*MeV, false, false, true);

}

double VectorMesonPScalarFermionsDecayer::me2(const int,
					      const Particle & inpart,
					      const ParticleVector & decay,
					      MEOption meopt) const {
  if(!ME())
    ME(new_ptr(GeneralDecayMatrixElement(PDT::Spin1,PDT::Spin0,
					 PDT::Spin1Half,PDT::Spin1Half)));
  // initialization
  if(meopt==Initialize) {
    VectorWaveFunction::calculateWaveFunctions(_vectors,_rho,
					       const_ptr_cast<tPPtr>(&inpart),
					       incoming,false);
  }
  if(meopt==Terminate) {
    VectorWaveFunction::constructSpinInfo(_vectors,const_ptr_cast<tPPtr>(&inpart),
					  incoming,true,false);
    ScalarWaveFunction::constructSpinInfo(decay[0],outgoing,true);
    SpinorBarWaveFunction::
      constructSpinInfo(_wavebar,decay[1],outgoing,true);
    SpinorWaveFunction::
      constructSpinInfo(_wave   ,decay[2],outgoing,true);
    return 0.;
  }
  SpinorBarWaveFunction::
    calculateWaveFunctions(_wavebar,decay[1],outgoing);
  SpinorWaveFunction::
    calculateWaveFunctions(_wave   ,decay[2],outgoing);
  // the factor for the off-shell photon
  Lorentz5Momentum pff(decay[1]->momentum()+decay[2]->momentum());
  pff.rescaleMass();
  Energy2 mff2(pff.mass2());
  // prefactor
  complex<InvEnergy3> pre(_coupling[imode()]/mff2);
  Complex ii(0.,1.);
  // the VMD factor
  if(_includeVMD[imode()]>0) {
    Energy2 mrho2(_VMDmass[imode()]*_VMDmass[imode()]);
    Energy2 mwrho(_VMDmass[imode()]*_VMDwidth[imode()]);
    pre = pre*(-mrho2+ii*mwrho)/(mff2-mrho2+ii*mwrho);
  }
  // calculate the matrix element
  LorentzPolarizationVector temp;
  unsigned int ix,iy,iz;
  for(ix=0;ix<2;++ix) {
    for(iy=0;iy<2;++iy) {
      temp=pre*epsilon(inpart.momentum(),pff,
				    _wave[ix].vectorCurrent(_wavebar[iy]));
      for(iz=0;iz<3;++iz) 
	(*ME())(iz,0,iy,ix)=temp.dot(_vectors[iz]); 
    }
  }
  /* code for the spin averaged me for testing only
  Energy  m[4]={inpart.mass(),decay[0]->mass(),decay[1]->mass(),decay[2]->mass()};
  Energy m2[4]={m[0]*m[0],m[1]*m[1],m[2]*m[2],m[3]*m[3]};
  Lorentz5Momentum p12=decay[0]->momentum()+decay[1]->momentum();p12.rescaleMass();
  Energy m122(p12.mass2());
  cout << "testing the matrix element " 
       << -1./3.*(pre*conj(pre)).real()*(-2*m122*m122*mff2 - mff2*mff2*mff2 + 
		   m2[1]*(2*m2[2]*m2[3] - 2*m2[3]*m2[3] + 
			  m2[1]*(m2[2] - 2*m[2]*m[3] - m2[3])) - 
		   2*m[2]*(m2[2]*m[2] - 2*m2[1]*m[3] - m[2]*m2[3])*
		   m2[0] - (m2[2] + 2*m[2]*m[3] - m2[3])*
		   m2[0]*m2[0] + mff2*mff2*
		   (2*m2[1] + (m[2] - m[3])*(m[2] - m[3]) + 2*m2[0]) - 
		   mff2*(m2[1]*m2[1] + 2*m2[1]*m[2]*(m[2] - 2*m[3]) + 
			 2*m2[2]*m2[3] - 2*(2*m[2] - m[3])*m[3]*m2[0] + 
			 m2[0]*m2[0]) + 2*m122*
		   (-mff2*mff2 - (m[2] - m[3])*(m[2] + m[3])*(m[1] - m[0])*
		    (m[1] + m[0]) + mff2*
		    (m2[1] + m2[2] + m2[3] + m2[0])))
       << endl;
   */
  // return the answer
  return ME()->contract(_rho).real();
}

WidthCalculatorBasePtr 
VectorMesonPScalarFermionsDecayer::threeBodyMEIntegrator(const DecayMode & dm) const {
  // workout which mode we are doing
  int imode(-1);
  // ids of the particles
  int id0(dm.parent()->id());
  int idf[2] = {0,0};
  int ids(0);
  unsigned int nf(0);
  ParticleMSet::const_iterator pit = dm.products().begin();
  for( ;pit!=dm.products().end();++pit) {
    if((**pit).iSpin()==PDT::Spin0){ids=(**pit).id();}
    else{idf[nf]=(**pit).id();++nf;}
  }
  // loop over the modes and see if this is one of them
  unsigned int ix=0;
  do {
    if(_incoming[ix]==id0&&_outgoingP[ix]==ids) {
      if((idf[0]==_outgoingf[ix]&&idf[1]==_outgoinga[ix])||
	 (idf[1]==_outgoingf[ix]&&idf[0]==_outgoinga[ix])) imode=ix;
    }
    ++ix;
  }
  while(imode<0&&ix<_incoming.size());
  // get the masses we need
  Energy m[3]={getParticleData(_outgoingP[imode])->mass(),
	       getParticleData(_outgoingf[imode])->mass(),
	       getParticleData(_outgoinga[imode])->mass()};
  return 
    new_ptr(ThreeBodyAllOn1IntegralCalculator<VectorMesonPScalarFermionsDecayer>
	    (3,-1000.*MeV,-0.8*MeV,-0.8,*this,imode,m[0],m[1],m[2]));
} 

InvEnergy VectorMesonPScalarFermionsDecayer::
threeBodydGammads(int imodeb, const Energy2 q2, const Energy2 mff2, 
		  const  Energy m1, const Energy m2, const  Energy m3) const {
  // the masses of the external particles
  Energy q(sqrt(q2));
  Energy2 m12(m1*m1),m22(m2*m2),m32(m3*m3);
  // prefactor
  complex<InvEnergy3> pre(_coupling[imodeb]/mff2);
  Complex ii(0.,1.);
  // the VMD factor
  if(_includeVMD[imodeb]>0) {
    Energy2 mrho2(_VMDmass[imodeb]*_VMDmass[imodeb]),
      mwrho(_VMDmass[imodeb]*_VMDwidth[imodeb]);
    pre = pre*(-mrho2+ii*mwrho)/(mff2-mrho2+ii*mwrho);
  }
  InvEnergy6 factor(real(pre*conj(pre)));
  // compute the pieces from the integration limits
  Energy mff(sqrt(mff2)),e2star(0.5*(mff2-m32+m22)/mff),e1star(0.5*(q2-mff2-m12)/mff),
    e1sm(sqrt(e1star*e1star-m12)),e2sm(sqrt(e2star*e2star-m22));
  Energy2 a(2*e1star*e2star+m12+m22),b(2*e1sm*e2sm);
  // term independent of s3
  Energy8 me = 2*b*(-mff2*mff2*mff2 +m12*m12*(m22 - 2*m2*m3 - m32) - 
		   2*m22*(m22 - m32)*q2 -(m22 + 2*m2*m3 - m32)*q2*q2 + 
		   mff2*mff2*(2*m12 +(m2-m3)*(m2-m3)+2*q2) + 2*m12*m3*
		   ((m22-m32)*m3 + 2*m2*q2) - 
		   mff2*(m12*m12 + 2*m12*m2*(m2 - 2*m3) + 2*m22*m32 - 
			 2*(2*m2 - m3)*m3*q2 + q2*q2));
  // linear term
  me+= 2.*a*b*(2*(-mff2*mff2-(m22-m32)*(m12-q2)+mff2*(m12+m22+m32+q2)));
  // quadratic term
  me+=2*b*(3.*a*a+b*b)/3.*(-2*mff2);
  // phase space factors
  using Constants::pi;
  return -factor * me/768./pi/pi/pi/q2/q;
}

// output the setup information for the particle database
void VectorMesonPScalarFermionsDecayer::dataBaseOutput(ofstream & output,
						       bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":Incoming   " << ix << "  " 
	     << _incoming[ix]   << "\n";
      output << "newdef " << name() << ":OutgoingPseudoScalar  " 
	     << ix << "  " << _outgoingP[ix]  << "\n";
      output << "newdef " << name() << ":OutgoingFermion  " 
	     << ix << "  " << _outgoingf[ix]  << "\n";
      output << "newdef " << name() << ":OutgoingAntiFermion " 
	     << ix << "  " << _outgoinga[ix]  << "\n";
      output << "newdef " << name() << ":Coupling   " << ix << "  " 
	     << _coupling[ix]*MeV   << "\n";
      output << "newdef " << name() << ":MaxWeight  " << ix << "  " 
	     << _maxweight[ix]  << "\n";
      output << "newdef " << name() << ":Weight  "    << ix << "  " 
	     << _weight[ix]  << "\n";
      output << "newdef " << name() << ":IncludeVMD " << ix << "  " 
	     << _includeVMD[ix] << "\n";
      output << "newdef " << name() << ":VMDID      " << ix << "  " 
	     << _VMDid[ix]      << "\n";
      output << "newdef " << name() << ":VMDmass    " << ix << "  " 
	     << _VMDmass[ix]/MeV    << "\n";
      output << "newdef " << name() << ":VMDwidth   " << ix << "  " 
	     << _VMDwidth[ix]/MeV   << "\n";
    }
    else {
      output << "insert " << name() << ":Incoming   " << ix << "  " 
	     << _incoming[ix]   << "\n";
      output << "insert " << name() << ":OutgoingPseudoScalar  " 
	     << ix << "  " << _outgoingP[ix]  << "\n";
      output << "insert " << name() << ":OutgoingFermion  " 
	     << ix << "  " << _outgoingf[ix]  << "\n";
      output << "insert " << name() << ":OutgoingAntiFermion " 
	     << ix << "  " << _outgoinga[ix]  << "\n";
      output << "insert " << name() << ":Coupling   " << ix << "  " 
	     << _coupling[ix]*MeV   << "\n";
      output << "insert " << name() << ":Weight  "    << ix << "  " 
	     << _weight[ix]  << "\n";
      output << "insert " << name() << ":IncludeVMD " << ix << "  " 
	     << _includeVMD[ix] << "\n";
      output << "insert " << name() << ":VMDID      " << ix << "  " 
	     << _VMDid[ix]      << "\n";
      output << "insert " << name() << ":VMDmass    " << ix << "  " 
	     << _VMDmass[ix]/MeV    << "\n";
      output << "insert " << name() << ":VMDwidth   " << ix << "  " 
	     << _VMDwidth[ix]/MeV   << "\n";
    }
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" << fullName() << "\";" << endl;
}
#line 1 "./VectorMesonPVectorPScalarDecayer.cc"
// -*- C++ -*-
//
// VectorMesonPVectorPScalarDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the VectorMesonPVectorPScalarDecayer class.
//

#include "VectorMesonPVectorPScalarDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "Herwig/Decay/TwoBodyDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void VectorMesonPVectorPScalarDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    for(unsigned int ix=0;ix<_incoming.size();++ix)
      if(mode(ix)) _maxweight[ix] = mode(ix)->maxWeight();
  }
}

void VectorMesonPVectorPScalarDecayer::doinit() {
  DecayIntegrator::doinit();
  // check consistence of the parameters
  unsigned int isize=_incoming.size();
  if(isize!=_outgoingA.size()||isize!=_outgoingP.size()||
     isize!=_maxweight.size()||isize!=_coupling.size())
    throw InitException() << "Inconsistent parameters in "
			  << "VectorMesonPVectorPScalarDecayer::doinit()" 
			  << Exception::abortnow;
  // set up the integration channels
  vector<double> wgt(0);
  tPDVector extpart(3);
  DecayPhaseSpaceModePtr mode;
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    extpart[0]=getParticleData(_incoming[ix]);
    extpart[1]=getParticleData(_outgoingA[ix]);
    extpart[2]=getParticleData(_outgoingP[ix]);
    if(extpart[0]&&extpart[1]&&extpart[2]) 
      mode=new_ptr(DecayPhaseSpaceMode(extpart,this));
    else
      mode=DecayPhaseSpaceModePtr();
    addMode(mode,_maxweight[ix],wgt);
  }
}

VectorMesonPVectorPScalarDecayer::VectorMesonPVectorPScalarDecayer()
  :  _coupling(21), _incoming(21), _outgoingA(21), _outgoingP(21), 
     _maxweight(21) {
  // Jpsi to K_1 K
  _incoming[0] = 443; _outgoingA[0] =  20313; _outgoingP[0] = -311; 
  _coupling[0] = 0.00127/GeV; _maxweight[0] = 12.; 
  _incoming[1] = 443; _outgoingA[1] =  20323; _outgoingP[1] = -321; 
  _coupling[1] = 0.00127/GeV; _maxweight[1] = 12.; 
  // Jpsi to b_1 pi
  _incoming[2] = 443; _outgoingA[2] =  10213; _outgoingP[2] = -211; 
  _coupling[2] = 0.00106/GeV; _maxweight[2] = 10.5; 
  _incoming[3] = 443; _outgoingA[3] =  10113; _outgoingP[3] =  111; 
  _coupling[3] = 0.00106/GeV; _maxweight[3] = 10.5; 
  // psi(2s) to K_1 K
  _incoming[4] = 100443; _outgoingA[4] =  10313; _outgoingP[4] = -311; 
  _coupling[4] = 0.00152/GeV; _maxweight[4] = 12.; 
  _incoming[5] = 100443; _outgoingA[5] =  10323; _outgoingP[5] = -321; 
  _coupling[5] = 0.00152/GeV; _maxweight[5] = 12.; 
  // psi(2s) to b_1 pi
  _incoming[6] = 100443; _outgoingA[6] =  10213; _outgoingP[6] = -211; 
  _coupling[6] = 0.000694/GeV; _maxweight[6] = 10.5; 
  _incoming[7] = 100443; _outgoingA[7] =  10113; _outgoingP[7] =  111; 
  _coupling[7] = 0.000694/GeV; _maxweight[7] = 10.5; 
  // rho'' decays
  // to h_1
  _incoming[8] =  30213; _outgoingA[8] =  10223; _outgoingP[8] = 211; 
  _coupling[8] = 1.45/GeV; _maxweight[8] = 5.5; 
  _incoming[9] =  30113; _outgoingA[9] =  10223; _outgoingP[9] = 111; 
  _coupling[9] = 1.45/GeV; _maxweight[9] = 5.5; 
  // to a_1
  _incoming[10] =  30213; _outgoingA[10] =  20213; _outgoingP[10] =  111; 
  _coupling[10] = 1.09/GeV; _maxweight[10] = 4.; 
  _incoming[11] =  30213; _outgoingA[11] =  20113; _outgoingP[11] =  211; 
  _coupling[11] = 1.09/GeV; _maxweight[11] = 4.; 
  _incoming[12] =  30113; _outgoingA[12] =  20213; _outgoingP[12] = -211; 
  _coupling[12] = 1.09/GeV; _maxweight[12] = 4.; 
  //  rho' decays
  // to h_1
  _incoming[13] =  100213; _outgoingA[13] =  10223; _outgoingP[13] = 211; 
  _coupling[13] = 1.20/GeV; _maxweight[13] = 5.; 
  _incoming[14] =  100113; _outgoingA[14] =  10223; _outgoingP[14] = 111; 
  _coupling[14] = 1.20/GeV; _maxweight[14] = 5.; 
  // to a_1
  _incoming[15] =  100213; _outgoingA[15] =  20213; _outgoingP[15] = 111; 
  _coupling[15] = 1.83/GeV; _maxweight[15] = 4.; 
  _incoming[16] =  100213; _outgoingA[16] =  20113; _outgoingP[16] = 211; 
  _coupling[16] = 1.83/GeV; _maxweight[16] = 4.; 
  _incoming[17] =  100113; _outgoingA[17] =  20213; _outgoingP[17] = -211; 
  _coupling[17] = 1.83/GeV; _maxweight[17] = 4.; 
  // omega' to b pi
  _incoming[18] = 100223; _outgoingA[18] =  10113; _outgoingP[18] =  111; 
  _coupling[18] = 1.659/GeV; _maxweight[18] = 7.; 
  _incoming[19] = 100223; _outgoingA[19] =  10213; _outgoingP[19] = -211; 
  _coupling[19] = 1.659/GeV; _maxweight[19] = 6.; 
  // psi(2s) -> h_c pi0
  _incoming[20] = 100443; _outgoingA[20] =  10443; _outgoingP[20] = 111; 
  _coupling[20] = 0.0029/GeV; _maxweight[20] = 6.; 
  // initial size of the arrays
  _initsize = _coupling.size();
  // intermediates
  generateIntermediates(false);
}

int VectorMesonPVectorPScalarDecayer::modeNumber(bool & cc,tcPDPtr parent,
						 const tPDVector & children) const {
  if(children.size()!=2) return -1;
  int id(parent->id());
  int idbar = parent->CC() ? parent->CC()->id() : id;
  int id1(children[0]->id());
  int id1bar = children[0]->CC() ? children[0]->CC()->id() : id1;
  int id2(children[1]->id());
  int id2bar = children[1]->CC() ? children[1]->CC()->id() : id2;
  int imode(-1);
  unsigned int ix(0);
  cc=false;
  do {
    if(id   ==_incoming[ix]) {
      if((id1   ==_outgoingA[ix]&&id2   ==_outgoingP[ix])||
	 (id2   ==_outgoingA[ix]&&id1   ==_outgoingP[ix])) imode=ix;
    }
    if(idbar==_incoming[ix]) {
      if((id1bar==_outgoingA[ix]&&id2bar==_outgoingP[ix])||
	 (id2bar==_outgoingA[ix]&&id1bar==_outgoingP[ix])) {
	imode=ix;
	cc=true;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  return imode;
}


void VectorMesonPVectorPScalarDecayer::persistentOutput(PersistentOStream & os) const {
  os << _incoming << _outgoingA << _outgoingP << _maxweight << ounit(_coupling,1/GeV);
}

void VectorMesonPVectorPScalarDecayer::persistentInput(PersistentIStream & is, int) {
  is >> _incoming >> _outgoingA >> _outgoingP >> _maxweight >> iunit(_coupling,1/GeV);
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<VectorMesonPVectorPScalarDecayer,DecayIntegrator>
describeHerwigVectorMesonPVectorPScalarDecayer("Herwig::VectorMesonPVectorPScalarDecayer", "HwVMDecay.so");

void VectorMesonPVectorPScalarDecayer::Init() {

  static ClassDocumentation<VectorMesonPVectorPScalarDecayer> documentation
    ("The VectorMesonPVectorPScalarDecayer class is designed for the "
     "decay of a vector meson to a pseudovector meson and a "
     "pseudoscalar meson.");

  static ParVector<VectorMesonPVectorPScalarDecayer,int> interfaceIncoming
    ("Incoming",
     "The PDG code for the incoming particle",
     &VectorMesonPVectorPScalarDecayer::_incoming,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMesonPVectorPScalarDecayer,int> interfaceOutcomingVector
    ("OutgoingPVector",
     "The PDG code for the outgoing spin-1 particle",
     &VectorMesonPVectorPScalarDecayer::_outgoingA,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMesonPVectorPScalarDecayer,int> interfaceOutcomingScalar
    ("OutgoingPScalar",
     "The PDG code for the outgoing spin-0 particle",
     &VectorMesonPVectorPScalarDecayer::_outgoingP,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMesonPVectorPScalarDecayer,InvEnergy> interfaceCoupling
    ("Coupling",
     "The coupling for the decay mode",
     &VectorMesonPVectorPScalarDecayer::_coupling,
     1/GeV, 0, ZERO, ZERO, 100./GeV, false, false, true);

  static ParVector<VectorMesonPVectorPScalarDecayer,double> interfaceMaxWeight
    ("MaxWeight",
     "The maximum weight for the decay mode",
     &VectorMesonPVectorPScalarDecayer::_maxweight,
     0, 0, 0, 0., 100., false, false, true);
}

double VectorMesonPVectorPScalarDecayer::me2(const int,
					     const Particle & inpart,
					     const ParticleVector & decay,
					     MEOption meopt) const {
  if(!ME())
    ME(new_ptr(TwoBodyDecayMatrixElement(PDT::Spin1,PDT::Spin1,PDT::Spin0)));
  // is the vector massless
  bool photon(_outgoingA[imode()]==ParticleID::gamma);
  if(meopt==Initialize) {
    VectorWaveFunction::calculateWaveFunctions(_vectors[0],_rho,
					       const_ptr_cast<tPPtr>(&inpart),
					       incoming,false);
  }
  if(meopt==Terminate) {
    VectorWaveFunction::constructSpinInfo(_vectors[0],const_ptr_cast<tPPtr>(&inpart),
					  incoming,true,false);
    VectorWaveFunction::constructSpinInfo(_vectors[1],decay[0],
					  outgoing,true,photon);
    ScalarWaveFunction::constructSpinInfo(decay[1],outgoing,true);
    return 0.;
  }
  VectorWaveFunction::calculateWaveFunctions(_vectors[1],decay[0],outgoing,photon);
  // compute the matrix element
  Energy2 p0dotpv(inpart.momentum()*decay[0]->momentum());
  complex<Energy> epsdot(ZERO);
  InvEnergy2 pre(_coupling[imode()]/inpart.mass());  
  for(unsigned int ix=0;ix<3;++ix) {
    epsdot=_vectors[1][ix]*inpart.momentum();
    for(unsigned int iy=0;iy<3;++iy) {
      (*ME())(iy,ix,0)=Complex(pre*(p0dotpv*(_vectors[1][ix].dot(_vectors[0][iy]))-
				    epsdot*(_vectors[0][iy]*decay[0]->momentum())));
    }
  }
  // test of the matrix element
//   double me = ME()->contract(_rho).real();
//   Energy pcm=Kinematics::pstarTwoBodyDecay(inpart.mass(),decay[0]->mass(),
// 					   decay[1]->mass());
//   double test = sqr(_coupling[imode()])/3.*(2.*sqr(pcm)+3.*sqr(decay[0]->mass()));
//   cerr << "testing matrix element for " << inpart.PDGName() << " -> " 
//        << decay[0]->PDGName() << " " << decay[1]->PDGName() << " "
//        << me << " " << test << " " << (me-test)/(me+test) << "\n";
  // return the answer
  return ME()->contract(_rho).real();
}

bool VectorMesonPVectorPScalarDecayer::twoBodyMEcode(const DecayMode & dm,
						     int & mecode,
						     double & coupling) const {
  int imode(-1);
  int id(dm.parent()->id());
  int idbar = dm.parent()->CC() ? dm.parent()->CC()->id() : id;
  ParticleMSet::const_iterator pit(dm.products().begin());
  int id1((**pit).id());
  int id1bar = (**pit).CC() ? (**pit).CC()->id() : id1;
  ++pit;
  int id2((**pit).id());
  int id2bar = (**pit).CC() ? (**pit).CC()->id() : id2;
  unsigned int ix(0); bool order(false);
  do {
    if(id==_incoming[ix]) {
      if(id1   ==_outgoingA[ix]&&id2   ==_outgoingP[ix]) {
	imode=ix;
	order=true;
      }
      if(id2   ==_outgoingA[ix]&&id1   ==_outgoingP[ix]) {
	imode=ix;
	order=false;
      }
    }
    if(idbar==_incoming[ix]&&imode<0) {
      if(id1bar==_outgoingA[ix]&&id2bar==_outgoingP[ix]) {
	imode=ix;
	order=true;
      }
      if(id2bar==_outgoingA[ix]&&id1bar==_outgoingP[ix]) {
	imode=ix;
	order=false;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  coupling = _coupling[imode]*dm.parent()->mass();  
  mecode = 4;
  return order;
}

void VectorMesonPVectorPScalarDecayer::dataBaseOutput(ofstream & output,
						      bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  // the rest of the parameters
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":Incoming " << ix << " "
	     << _incoming[ix] << "\n";
      output << "newdef " << name() << ":OutgoingPVector " << ix << " "
	     << _outgoingA[ix] << "\n";
      output << "newdef " << name() << ":OutgoingPScalar " << ix << " "
	     << _outgoingP[ix] << "\n";
      output << "newdef " << name() << ":Coupling " << ix << " "
	     << _coupling[ix]*GeV << "\n";
      output << "newdef " << name() << ":MaxWeight " << ix << " "
	     << _maxweight[ix] << "\n";
    }
    else {
      output << "insert " << name() << ":Incoming "  << ix << " "
	     << _incoming[ix] << "\n";
      output << "insert " << name() << ":OutgoingPVector " << ix << " "
	     << _outgoingA[ix] << "\n";
      output << "insert " << name() << ":OutgoingPScalar " << ix << " "
	     << _outgoingP[ix] << "\n";
      output << "insert " << name() << ":Coupling " << ix << " "
	     << _coupling[ix]*GeV << "\n";
      output << "insert " << name() << ":MaxWeight " << ix << " "
	     << _maxweight[ix] << "\n";
    }
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}
#line 1 "./VectorMesonVectorPScalarDecayer.cc"
// -*- C++ -*-
//
// VectorMesonVectorPScalarDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the VectorMesonVectorPScalarDecayer class.
//

#include "VectorMesonVectorPScalarDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Helicity/VectorSpinInfo.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "ThePEG/Helicity/epsilon.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "Herwig/Decay/TwoBodyDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void VectorMesonVectorPScalarDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    for(unsigned int ix=0;ix<_incoming.size();++ix) {
      if(mode(ix)) _maxweight[ix] = mode(ix)->maxWeight();
    }
  }
}

void VectorMesonVectorPScalarDecayer::doinit() {
  DecayIntegrator::doinit();
  // check consistency of the parameters
  unsigned int isize=_incoming.size();
  if(isize!=_outgoingV.size()||isize!=_outgoingP.size()||
     isize!=_maxweight.size()||isize!=_coupling.size())
    throw InitException() << "Inconsistent parameters in " 
			  << "VectorMesonVectorPScalarDecayer" << Exception::runerror;
  // set up the integration channels
  vector<double> wgt(0);
  tPDVector extpart(3);
  DecayPhaseSpaceModePtr mode;
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    extpart[0]=getParticleData(_incoming[ix]);
    extpart[1]=getParticleData(_outgoingV[ix]);
    extpart[2]=getParticleData(_outgoingP[ix]);
    if(extpart[0]&&extpart[1]&&extpart[2]) 
      mode=new_ptr(DecayPhaseSpaceMode(extpart,this));
    else
      mode=DecayPhaseSpaceModePtr();
    addMode(mode,_maxweight[ix],wgt);
  }
}

VectorMesonVectorPScalarDecayer::VectorMesonVectorPScalarDecayer() 
  : _coupling(73), _incoming(73), _outgoingV(73), _outgoingP(73), _maxweight(73) {
  // intermediates
  generateIntermediates(false);
  // rho -> gamma pi modes
  _incoming[0] =  113; _outgoingV[0] =  22; _outgoingP[0] =  111; 
  _coupling[0] = 0.2527/GeV; _maxweight[0] = 1.7; 
  _incoming[1] =  213; _outgoingV[1] =  22; _outgoingP[1] =  211; 
  _coupling[1] = 0.2210/GeV; _maxweight[1] = 1.7; 
  // rho  -> gamma eta mode
  _incoming[2] =  113; _outgoingV[2] =  22; _outgoingP[2] =  221; 
  _coupling[2] = 0.492/GeV; _maxweight[2] = 1.7; 
  // omega -> gamma pi 
  _incoming[3] =  223; _outgoingV[3] =  22; _outgoingP[3] =  111; 
  _coupling[3] = 0.7279465/GeV; _maxweight[3] = 1.7; 
  // omega -> gamma eta
  _incoming[4] =  223; _outgoingV[4] =  22; _outgoingP[4] =  221; 
  _coupling[4] = 0.143/GeV; _maxweight[4] = 1.7; 
  // phi -> gamma pi 
  _incoming[5] =  333; _outgoingV[5] =  22; _outgoingP[5] =  111; 
  _coupling[5] = 0.0397/GeV; _maxweight[5] = 1.7; 
  // phi -> gamma eta
  _incoming[6] =  333; _outgoingV[6] =  22; _outgoingP[6] =  221; 
  _coupling[6] = 0.212/GeV; _maxweight[6] = 1.7; 
  // phi -> gamma eta'
  _incoming[7] =  333; _outgoingV[7] =  22; _outgoingP[7] =  331; 
  _coupling[7] = 0.219/GeV; _maxweight[7] = 1.8; 
  // phi -> omega pi
  _incoming[8] =  333; _outgoingV[8] = 223; _outgoingP[8] =  111; 
  _coupling[8] = 0.0417/GeV; _maxweight[8] = 1.9; 
  // phi' -> K* K
  _incoming[9] =  100333; _outgoingV[9] =  323; _outgoingP[9] = -321; 
  _coupling[9] = 3.934/GeV; _maxweight[9] = 5.; 
  _incoming[10] =  100333; _outgoingV[10] =  313; _outgoingP[10] = -311; 
  _coupling[10] = 4.011/GeV; _maxweight[10] = 5.; 
  // K* -> gamma K 
  _incoming[11] =  313; _outgoingV[11] =  22; _outgoingP[11] =  311; 
  _coupling[11] = 0.384/GeV; _maxweight[11] = 1.7; 
  _incoming[12] =  323; _outgoingV[12] =  22; _outgoingP[12] =  321; 
  _coupling[12] = 0.253/GeV; _maxweight[12] = 1.7; 
  // d* decay
  _incoming[13] =  423; _outgoingV[13] = 22; _outgoingP[13] =  421; 
  _coupling[13] = 0.616/GeV; _maxweight[13] = 1.7; 
  _incoming[14] =  413; _outgoingV[14] = 22; _outgoingP[14] =  411; 
  _coupling[14] = 0.152/GeV; _maxweight[14] = 1.7; 
  // D_s* decays
  _incoming[15] =  433; _outgoingV[15] = 22; _outgoingP[15] =  431; 
  _coupling[15] = 0.764/GeV; _maxweight[15] = 1.7; 
  // B_s* decays
  _incoming[16] =  533; _outgoingV[16] = 22; _outgoingP[16] =  531; 
  _coupling[16] = 0.248/GeV; _maxweight[16] = 1.7; 
  // B_c* decays
  _incoming[17] =  543; _outgoingV[17] = 22; _outgoingP[17] =  541; 
  _coupling[17] = 0.266/GeV; _maxweight[17] = 1.7; 
  // B* decay
  _incoming[18] =  523; _outgoingV[18] = 22; _outgoingP[18] =  521; 
  _coupling[18] = 0.553/GeV; _maxweight[18] = 1.7; 
  _incoming[19] =  513; _outgoingV[19] = 22; _outgoingP[19] =  511; 
  _coupling[19] = 0.310/GeV; _maxweight[19] = 1.7; 
  // rho'' eta rho
  _incoming[20] =  30113; _outgoingV[20] =  113; _outgoingP[20] = 221; 
  _coupling[20] = 2.663/GeV; _maxweight[20] = 4.; 
  _incoming[21] =  30213; _outgoingV[21] =  213; _outgoingP[21] = 221; 
  _coupling[21] = 2.663/GeV; _maxweight[21] = 4.; 
  // rho '' K* K
  _incoming[22] =  30113; _outgoingV[22] =  323; _outgoingP[22] = -321; 
  _coupling[22] = 0.894/GeV; _maxweight[22] = 5.; 
  _incoming[23] =  30113; _outgoingV[23] =  313; _outgoingP[23] = -311; 
  _coupling[23] = 0.908/GeV; _maxweight[23] = 5.; 
  _incoming[24] =  30213; _outgoingV[24] =  323; _outgoingP[24] = -311; 
  _coupling[24] = 1.265/GeV; _maxweight[24] = 5.; 
  _incoming[25] =  30213; _outgoingV[25] = -313; _outgoingP[25] =  321; 
  _coupling[25] = 1.273/GeV; _maxweight[25] = 5.; 
  // omega'' rho pi
  _incoming[26] =  30223; _outgoingV[26] =  213; _outgoingP[26] = -211; 
  _coupling[26] = 2.996/GeV; _maxweight[26] = 3.5; 
  _incoming[27] =  30223; _outgoingV[27] =  113; _outgoingP[27] =  111; 
  _coupling[27] = 2.996/GeV; _maxweight[27] = 3.5; 
  // omega' rho pi
  _incoming[28] =  100223; _outgoingV[28] =  213; _outgoingP[28] = -211; 
  _coupling[28] = 4.507/GeV; _maxweight[28] = 4.; 
  _incoming[29] =  100223; _outgoingV[29] =  113; _outgoingP[29] =  111; 
  _coupling[29] = 4.507/GeV; _maxweight[29] = 4.; 
  // K*''->K* pi decays
  _incoming[30] =  30313; _outgoingV[30] =  323; _outgoingP[30] = -211; 
  _coupling[30] = 3.36/GeV;  _maxweight[30] = 4.; 
  _incoming[31] =  30313; _outgoingV[31] =  313; _outgoingP[31] =  111; 
  _coupling[31] = 2.38/GeV;  _maxweight[31] = 4.; 
  _incoming[32] =  30323; _outgoingV[32] =  313; _outgoingP[32] =  211; 
  _coupling[32] = 3.36/GeV;  _maxweight[32] = 4.; 
  _incoming[33] =  30323; _outgoingV[33] =  323; _outgoingP[33] =  111; 
  _coupling[33] = 2.38/GeV;  _maxweight[33] = 4.; 
  // K*''->K rho decays
  _incoming[34] =  30313; _outgoingP[34] =  321; _outgoingV[34] = -213; 
  _coupling[34] = 4.159/GeV;  _maxweight[34] = 3.; 
  _incoming[35] =  30313; _outgoingP[35] =  311; _outgoingV[35] =  113; 
  _coupling[35] = 2.939/GeV;  _maxweight[35] = 3.; 
  _incoming[36] =  30323; _outgoingP[36] =  311; _outgoingV[36] =  213; 
  _coupling[36] = 4.159/GeV;  _maxweight[36] = 3.; 
  _incoming[37] =  30323; _outgoingP[37] =  321; _outgoingV[37] =  113; 
  _coupling[37] = 2.939/GeV;  _maxweight[37] = 3.; 
  // K*' decays
  _incoming[38] =  100313; _outgoingV[38] =  323; _outgoingP[38] = -211; 
  _coupling[38] = 9.469/GeV;  _maxweight[38] = 6.; 
  _incoming[39] =  100313; _outgoingV[39] =  313; _outgoingP[39] =  111; 
  _coupling[39] = 6.781/GeV;  _maxweight[39] = 6.; 
  _incoming[40] =  100323; _outgoingV[40] =  313; _outgoingP[40] =  211; 
  _coupling[40] = 9.469/GeV;  _maxweight[40] = 6.; 
  _incoming[41] =  100323; _outgoingV[41] =  323; _outgoingP[41] =  111; 
  _coupling[41] = 6.781/GeV;  _maxweight[41] = 6.; 
  // J/psi -> gamma eta_c decay
  _incoming[42] = 443; _outgoingV[42] = 22; _outgoingP[42] = 441; 
  _coupling[42] = 0.149/GeV; _maxweight[42] = 30.; 
  // J/psi -> gamma eta' decay
  _incoming[43] = 443; _outgoingV[43] = 22; _outgoingP[43] = 331; 
  _coupling[43] = 0.00250/GeV; _maxweight[43] = 1.7; 
  // J/psi -> rho pi decay
  _incoming[44] = 443; _outgoingV[44] = 213; _outgoingP[44] = -211; 
  _coupling[44] = 0.00274/GeV; _maxweight[44] = 3.3; 
  _incoming[45] = 443; _outgoingV[45] = 113; _outgoingP[45] = 111; 
  _coupling[45] = 0.00274/GeV; _maxweight[45] = 3.3; 
  // J/psi -> K* K decay
  _incoming[46] = 443; _outgoingV[46] = 323; _outgoingP[46] = -321; 
  _coupling[46] = 0.00180/GeV; _maxweight[46] = 6.; 
  _incoming[47] = 443; _outgoingV[47] = 313; _outgoingP[47] = -311; 
  _coupling[47] = 0.00180/GeV; _maxweight[47] = 6.; 
  // J/psi -> omega eta decay
  _incoming[48] = 443; _outgoingV[48] = 223; _outgoingP[48] = 221; 
  _coupling[48] = 0.00154/GeV; _maxweight[48] = 6.; 
  // J/psi -> gamma eta decay
  _incoming[49] = 443; _outgoingV[49] = 22; _outgoingP[49] = 221; 
  _coupling[49] = 0.00103/GeV; _maxweight[49] = 1.7; 
  // J/psi -> phi eta decay
  _incoming[50] = 443; _outgoingV[50] = 333; _outgoingP[50] = 221; 
  _coupling[50] = 0.00110/GeV; _maxweight[50] = 5.5; 
  // J/psi -> phi eta' decay
  _incoming[51] = 443; _outgoingV[51] = 333; _outgoingP[51] = 331; 
  _coupling[51] = 0.00085/GeV; _maxweight[51] = 5.5; 
  // J/psi -> omega pi0 decay
  _incoming[52] = 443; _outgoingV[52] = 223; _outgoingP[52] = 111; 
  _coupling[52] = 0.00073/GeV; _maxweight[52] = 6.; 
  // J/psi -> rho0 eta decay
  _incoming[53] = 443; _outgoingV[53] = 113; _outgoingP[53] = 221; 
  _coupling[53] = 0.00054/GeV; _maxweight[53] = 3.; 
  // J/psi -> rho0 eta' decay
  _incoming[54] = 443; _outgoingV[54] = 113; _outgoingP[54] = 331; 
  _coupling[54] = 0.00045/GeV; _maxweight[54] = 3.; 
  // J/psi -> omega eta' decay
  _incoming[55] = 443; _outgoingV[55] = 223; _outgoingP[55] = 331; 
  _coupling[55] = 0.00058/GeV; _maxweight[55] = 6.; 
  // J/psi -> gamma pi0 decay
  _incoming[56] = 443; _outgoingV[56] = 22; _outgoingP[56] = 111; 
  _coupling[56] = 0.000177/GeV; _maxweight[56] = 1.7; 
  // psi(2s)-> j/psi eta decay
  _incoming[57] = 100443; _outgoingV[57] = 443; _outgoingP[57] = 221; 
  _coupling[57] = 0.230/GeV; _maxweight[57] = 1.7; 
  // psi(2s)-> gamma eta_c decay
  _incoming[58] = 100443; _outgoingV[58] = 22; _outgoingP[58] = 441; 
  _coupling[58] = 0.0114/GeV; _maxweight[58] = 3.4; 
  // psi(2s)-> gamma eta' decay
  _incoming[59] = 100443; _outgoingV[59] = 22; _outgoingP[59] = 331; 
  _coupling[59] = 0.00062/GeV; _maxweight[59] = 1.7; 
  // psi(2s)-> J/psi pi0 decay
  _incoming[60] = 100443; _outgoingV[60] = 443; _outgoingP[60] = 111; 
  _coupling[60] = 0.0106/GeV; _maxweight[60] = 1.7; 
  // psi(1d)-> gamma eta_c decay
  _incoming[61] = 30443; _outgoingV[61] = 443; _outgoingP[61] = 221; 
  _coupling[61] = 0.135/GeV; _maxweight[61] = 1.7;
  _incoming[62] = 30443; _outgoingV[62] = 333; _outgoingP[62] = 221; 
  _coupling[62] = 0.0076/GeV; _maxweight[62] = 5.5; 
  // psi(2s) K* K
  _incoming[63] = 100443; _outgoingV[63] = 323; _outgoingP[63] = -321; 
  _coupling[63] = 0.00021/GeV; _maxweight[63] = 6.5; 
  _incoming[64] = 100443; _outgoingV[64] = 313; _outgoingP[64] = -311; 
  _coupling[64] = 0.00054/GeV; _maxweight[64] = 6.5; 
  // psi(2s) -> phi eta decay
  _incoming[65] = 100443; _outgoingV[65] = 333; _outgoingP[65] = 221; 
  _coupling[65] = 0.00029/GeV; _maxweight[65] = 5.5; 
  // psi(2s) -> phi eta' decay
  _incoming[66] = 100443; _outgoingV[66] = 333; _outgoingP[66] = 331; 
  _coupling[66] = 0.00033/GeV; _maxweight[66] = 5.5; 
  // psi(2s) -> rho pi decay
  _incoming[67] = 100443; _outgoingV[67] = 213; _outgoingP[67] = -211; 
  _coupling[67] = 0.00017/GeV; _maxweight[67] = 3.5; 
  _incoming[68] = 100443; _outgoingV[68] = 113; _outgoingP[68] = 111; 
  _coupling[68] = 0.00017/GeV; _maxweight[68] = 3.5; 
  // psi(2s) -> omega eta' decay
  _incoming[69] = 100443; _outgoingV[69] = 223; _outgoingP[69] = 331; 
  _coupling[69] = 0.00032/GeV; _maxweight[69] = 6.;
  // psi(2s) -> rho0 eta decay
  _incoming[70] = 100443; _outgoingV[70] = 113; _outgoingP[70] = 221; 
  _coupling[70] = 0.00025/GeV; _maxweight[70] = 3.5; 
  // psi(2s) -> omega pi0 decay
  _incoming[71] = 100443; _outgoingV[71] = 223; _outgoingP[71] = 111; 
  _coupling[71] = 0.00022/GeV; _maxweight[71] = 6.; 
  // psi(2s) -> rho0 eta' decay
  _incoming[72] = 100443; _outgoingV[72] = 113; _outgoingP[72] = 331; 
  _coupling[72] = 0.00026/GeV; _maxweight[72] = 3.5; 
  // initial size of the vectors for the database output
  _initsize=_incoming.size();
}

int VectorMesonVectorPScalarDecayer::modeNumber(bool & cc,tcPDPtr parent,
					   const tPDVector & children) const {
  if(children.size()!=2) return -1;
  int id(parent->id());
  int idbar = parent->CC() ? parent->CC()->id() : id;
  int id1(children[0]->id());
  int id1bar = children[0]->CC() ? children[0]->CC()->id() : id1;
  int id2(children[1]->id());
  int id2bar = children[1]->CC() ? children[1]->CC()->id() : id2;
  int imode(-1);
  unsigned int ix(0);
  cc=false;
  do {
    if(id   ==_incoming[ix]) {
      if((id1   ==_outgoingV[ix]&&id2   ==_outgoingP[ix])||
	 (id2   ==_outgoingV[ix]&&id1   ==_outgoingP[ix])) imode=ix;
    }
    if(idbar==_incoming[ix]) {
      if((id1bar==_outgoingV[ix]&&id2bar==_outgoingP[ix])||
	 (id2bar==_outgoingV[ix]&&id1bar==_outgoingP[ix])) {
	imode=ix;
	cc=true;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  return imode;
}


void VectorMesonVectorPScalarDecayer::persistentOutput(PersistentOStream & os) const  {
  os << _incoming << _outgoingV << _outgoingP << _maxweight << ounit(_coupling,1/MeV);
}

void VectorMesonVectorPScalarDecayer::persistentInput(PersistentIStream & is, int)  {
  is >> _incoming >> _outgoingV >> _outgoingP >> _maxweight >> iunit(_coupling,1/MeV);
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<VectorMesonVectorPScalarDecayer,DecayIntegrator>
describeHerwigVectorMesonVectorPScalarDecayer("Herwig::VectorMesonVectorPScalarDecayer", "HwVMDecay.so");

void VectorMesonVectorPScalarDecayer::Init() {

  static ClassDocumentation<VectorMesonVectorPScalarDecayer> documentation
    ("The VectorMesonVectorPScalarDecayer class is designed for the "
     "decay of a vector meson to another vector meson, or the photon, and a "
     "pseudoscalar meson.");

  static ParVector<VectorMesonVectorPScalarDecayer,int> interfaceIncoming
    ("Incoming",
     "The PDG code for the incoming particle",
     &VectorMesonVectorPScalarDecayer::_incoming,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMesonVectorPScalarDecayer,int> interfaceOutcomingVector
    ("OutgoingVector",
     "The PDG code for the outgoing spin-1 particle",
     &VectorMesonVectorPScalarDecayer::_outgoingV,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMesonVectorPScalarDecayer,int> interfaceOutcomingPScalar
    ("OutgoingPScalar",
     "The PDG code for the outgoing spin-0 particle",
     &VectorMesonVectorPScalarDecayer::_outgoingP,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMesonVectorPScalarDecayer,InvEnergy> interfaceCoupling
    ("Coupling",
     "The coupling for the decay mode",
     &VectorMesonVectorPScalarDecayer::_coupling,
     1/MeV, 0, ZERO, -10000000/MeV, 10000000/MeV, false, false, true);

  static ParVector<VectorMesonVectorPScalarDecayer,double> interfaceMaxWeight
    ("MaxWeight",
     "The maximum weight for the decay mode",
     &VectorMesonVectorPScalarDecayer::_maxweight,
     0, 0, 0, -10000000, 10000000, false, false, true);

}

double VectorMesonVectorPScalarDecayer::me2(const int,
					    const Particle & inpart,
					    const ParticleVector & decay,
					    MEOption meopt) const {
  if(!ME())
    ME(new_ptr(TwoBodyDecayMatrixElement(PDT::Spin1,PDT::Spin1,PDT::Spin0)));
  // is the vector massless
  bool photon(_outgoingV[imode()]==ParticleID::gamma);
  if(meopt==Initialize) {
    VectorWaveFunction::calculateWaveFunctions(_vectors[0],_rho,
					       const_ptr_cast<tPPtr>(&inpart),
					       incoming,false);
  }
  if(meopt==Terminate) {
    VectorWaveFunction::constructSpinInfo(_vectors[0],const_ptr_cast<tPPtr>(&inpart),
					  incoming,true,false);
    VectorWaveFunction::constructSpinInfo(_vectors[1],decay[0],
					  outgoing,true,photon);
    ScalarWaveFunction::constructSpinInfo(decay[1],outgoing,true);
    return 0.;
  }
  VectorWaveFunction::calculateWaveFunctions(_vectors[1],decay[0],outgoing,photon);
  // compute the matrix element
  LorentzPolarizationVector vtemp;
  for(unsigned int ix=0;ix<3;++ix) {
    if(ix==1&&photon) {
      for(unsigned int iy=0;iy<3;++iy) (*ME())(iy,ix,0)=0.;
    }
    else {
      vtemp=_coupling[imode()]/inpart.mass()*
	epsilon(inpart.momentum(),_vectors[1][ix],decay[0]->momentum());
      for(unsigned int iy=0;iy<3;++iy) (*ME())(iy,ix,0)=_vectors[0][iy].dot(vtemp);
    }
  }
  // test of the matrix element
//   double me = newME.contract(rhoin).real();
//   Energy pcm=Kinematics::pstarTwoBodyDecay(inpart.mass(),decay[0]->mass(),
// 					   decay[1]->mass());
//   double test = sqr(_coupling[imode()]*pcm)*2./3.;
//   cerr << "testing matrix element for " << inpart.PDGName() << " -> " 
//        << decay[0]->PDGName() << " " << decay[1]->PDGName() << " "
//        << me << " " << test << " " << (me-test)/(me+test) << "\n";
  // return the answer
  return ME()->contract(_rho).real();
}

bool VectorMesonVectorPScalarDecayer::twoBodyMEcode(const DecayMode & dm,int & mecode,
						    double & coupling) const {
  int imode(-1);
  int id(dm.parent()->id());
  int idbar = dm.parent()->CC() ? dm.parent()->CC()->id() : id;
  ParticleMSet::const_iterator pit(dm.products().begin());
  int id1((**pit).id());
  int id1bar = (**pit).CC() ? (**pit).CC()->id() : id1;
  ++pit;
  int id2((**pit).id());
  int id2bar = (**pit).CC() ? (**pit).CC()->id() : id2;
  unsigned int ix(0); bool order(false);
  do {
    if(id==_incoming[ix]) {
      if(id1   ==_outgoingV[ix]&&id2   ==_outgoingP[ix]) {
	imode=ix;
	order=true;
      }
      if(id2   ==_outgoingV[ix]&&id1   ==_outgoingP[ix]) {
	imode=ix;
	order=false;
      }
    }
    if(idbar==_incoming[ix]&&imode<0) {
      if(id1bar==_outgoingV[ix]&&id2bar==_outgoingP[ix]) {
	imode=ix;
	order=true;
      }
      if(id2bar==_outgoingV[ix]&&id1bar==_outgoingP[ix]) {
	imode=ix;
	order=false;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  coupling = _coupling[imode]*dm.parent()->mass();  
  mecode = 1;
  return order;
}

// output the setup info for the particle database
void VectorMesonVectorPScalarDecayer::dataBaseOutput(ofstream & output,
						     bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  // the rest of the parameters
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":Incoming " << ix << " "
	     << _incoming[ix] << "\n";
      output << "newdef " << name() << ":OutgoingVector " << ix << " "
	     << _outgoingV[ix] << "\n";
      output << "newdef " << name() << ":OutgoingPScalar " << ix << " "
	     << _outgoingP[ix] << "\n";
      output << "newdef " << name() << ":Coupling " << ix << " "
	     << _coupling[ix]*MeV << "\n";
      output << "newdef " << name() << ":MaxWeight " << ix << " "
	     << _maxweight[ix] << "\n";
    }
    else {
      output << "insert " << name() << ":Incoming "  << ix << " "
	     << _incoming[ix] << "\n";
      output << "insert " << name() << ":OutgoingVector " << ix << " "
	     << _outgoingV[ix] << "\n";
      output << "insert " << name() << ":OutgoingPScalar " << ix << " "
	     << _outgoingP[ix] << "\n";
      output << "insert " << name() << ":Coupling " << ix << " "
	     << _coupling[ix]*MeV << "\n";
      output << "insert " << name() << ":MaxWeight " << ix << " "
	     << _maxweight[ix] << "\n";
    }
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}
#line 1 "./VectorMesonVectorScalarDecayer.cc"
// -*- C++ -*-
//
// VectorMesonVectorScalarDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the VectorMesonVectorScalarDecayer class.
//
#include "VectorMesonVectorScalarDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "Herwig/Decay/TwoBodyDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void VectorMesonVectorScalarDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    for(unsigned int ix=0;ix<_incoming.size();++ix) 
      if(mode(ix)) _maxweight[ix] = mode(ix)->maxWeight();
  }
}

VectorMesonVectorScalarDecayer::VectorMesonVectorScalarDecayer() 
  : _coupling(16), _incoming(16), _outgoingV(16), 
    _outgoingS(16), _maxweight(16) {
  // decay of the phi to the a_0 and f_0 and a photon
  _incoming[0] = 333; _outgoingV[0] = 22; _outgoingS[0] = 9000111; 
  _coupling[0] = 0.238/GeV; _maxweight[0] = 10.; 
  _incoming[1] = 333; _outgoingV[1] = 22; _outgoingS[1] = 9010221; 
  _coupling[1] = 0.267/GeV; _maxweight[1] = 15.; 
  // Jpsi decays
  _incoming[2] = 443; _outgoingV[2] = 22; _outgoingS[2] = 10331; 
  _coupling[2] = 0.00207/GeV; _maxweight[2] = 3.; 
  _incoming[3] = 443; _outgoingV[3] = 223; _outgoingS[3] = 10331; 
  _coupling[3] = 0.00144/GeV; _maxweight[3] = 9.; 
  _incoming[4] = 443; _outgoingV[4] = 333; _outgoingS[4] = 10331; 
  _coupling[4] = 0.00127/GeV; _maxweight[4] = 9.; 
  _incoming[5] = 443; _outgoingV[5] = 333; _outgoingS[5] = 9010221; 
  _coupling[5] = 0.00064/GeV; _maxweight[5] = 6.; 
  _incoming[6] = 443; _outgoingV[6] = 223; _outgoingS[6] = 9010221; 
  _coupling[6] = 0.00044/GeV; _maxweight[6] = 7.;
  _incoming[15] = 443; _outgoingV[15] = 22; _outgoingS[15] = 9030221; 
  _coupling[15] = 0.00114/GeV; _maxweight[15] = 3.; 
  // upsilon(2s)
  _incoming[7] = 100553; _outgoingV[7] = 22; _outgoingS[7] = 10551; 
  _coupling[7] = 0.105/GeV; _maxweight[7] = 2.; 
  // upsilon(3s)
  _incoming[8] = 200553; _outgoingV[8] = 22; _outgoingS[8] = 110551; 
  _coupling[8] = 0.160/GeV; _maxweight[8] = 2.; 
  // psi2s decays
  _incoming[9] = 100443; _outgoingV[9] = 22; _outgoingS[9] = 10441; 
  _coupling[9] = 0.258/GeV; _maxweight[9] = 4.; 
  _incoming[10] = 100443; _outgoingV[10] = 22; _outgoingS[10] = 331; 
  _coupling[10] = 0.0508/GeV; _maxweight[10] = 2.; 
  _incoming[11] = 100443; _outgoingV[11] = 22; _outgoingS[11] = 10331; 
  _coupling[11] = 0.000680/GeV; _maxweight[11] = 2.; 
  _incoming[12] = 100443; _outgoingV[12] = 333; _outgoingS[12] = 9010221; 
  _coupling[12] = 0.000509/GeV; _maxweight[12] = 6.; 
  // rho' to sigma rho
  _incoming[13] = 100213; _outgoingV[13] = 213; _outgoingS[13] = 9000221; 
  _coupling[13] = 5.056/GeV; _maxweight[13] = 5.5; 
  _incoming[14] = 100113; _outgoingV[14] = 113; _outgoingS[14] = 9000221; 
  _coupling[14] = 5.056/GeV; _maxweight[14] = 5.5; 
  // initial size of the arrays
  _initsize = _coupling.size();
  // intermediates
  generateIntermediates(false);
}

void VectorMesonVectorScalarDecayer::doinit() {
  DecayIntegrator::doinit();
  // check consistence of the parameters
  unsigned int isize=_incoming.size();
  if(isize!=_outgoingV.size()||isize!=_outgoingS.size()||
     isize!=_maxweight.size()||isize!=_coupling.size())
    throw InitException() << "Inconsistent parameters in "
			  << "VectorMesonVectorScalarDecayer::doinit()" 
			  << Exception::abortnow;
  // set up the integration channels
  vector<double> wgt(0);
  tPDVector extpart(3);
  DecayPhaseSpaceModePtr mode;
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    extpart[0]=getParticleData(_incoming[ix]);
    extpart[1]=getParticleData(_outgoingV[ix]);
    extpart[2]=getParticleData(_outgoingS[ix]);
    if(extpart[0]&&extpart[1]&&extpart[2]) 
      mode=new_ptr(DecayPhaseSpaceMode(extpart,this));
    else
      mode=DecayPhaseSpaceModePtr();
    addMode(mode,_maxweight[ix],wgt);
  }
}

int VectorMesonVectorScalarDecayer::modeNumber(bool & cc,tcPDPtr parent,
					   const tPDVector & children) const {
  if(children.size()!=2) return -1;
  int id(parent->id());
  int idbar = parent->CC() ? parent->CC()->id() : id;
  int id1(children[0]->id());
  int id1bar = children[0]->CC() ? children[0]->CC()->id() : id1;
  int id2(children[1]->id());
  int id2bar = children[1]->CC() ? children[1]->CC()->id() : id2;
  int imode(-1);
  unsigned int ix(0);
  cc=false;
  do {
    if(id   ==_incoming[ix]) {
      if((id1   ==_outgoingV[ix]&&id2   ==_outgoingS[ix])||
	 (id2   ==_outgoingV[ix]&&id1   ==_outgoingS[ix])) imode=ix;
    }
    if(idbar==_incoming[ix]) {
      if((id1bar==_outgoingV[ix]&&id2bar==_outgoingS[ix])||
	 (id2bar==_outgoingV[ix]&&id1bar==_outgoingS[ix])) {
	imode=ix;
	cc=true;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  return imode;
}

void VectorMesonVectorScalarDecayer::persistentOutput(PersistentOStream & os) const {
  os << _incoming << _outgoingV << _outgoingS << _maxweight << ounit(_coupling,1/GeV);
}

void VectorMesonVectorScalarDecayer::persistentInput(PersistentIStream & is, int) {
  is >> _incoming >> _outgoingV >> _outgoingS >> _maxweight >> iunit(_coupling,1/GeV);
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<VectorMesonVectorScalarDecayer,DecayIntegrator>
describeHerwigVectorMesonVectorScalarDecayer("Herwig::VectorMesonVectorScalarDecayer", "HwVMDecay.so");

void VectorMesonVectorScalarDecayer::Init() {

  static ClassDocumentation<VectorMesonVectorScalarDecayer> documentation
    ("The VectorMesonVectorScalarDecayer class is designed for the "
     "decay of a vector meson to a vector meson, or the photon, and a "
     "scalar meson.");

  static ParVector<VectorMesonVectorScalarDecayer,int> interfaceIncoming
    ("Incoming",
     "The PDG code for the incoming particle",
     &VectorMesonVectorScalarDecayer::_incoming,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMesonVectorScalarDecayer,int> interfaceOutcomingVector
    ("OutgoingVector",
     "The PDG code for the outgoing spin-1 particle",
     &VectorMesonVectorScalarDecayer::_outgoingV,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMesonVectorScalarDecayer,int> interfaceOutcomingScalar
    ("OutgoingScalar",
     "The PDG code for the outgoing spin-0 particle",
     &VectorMesonVectorScalarDecayer::_outgoingS,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMesonVectorScalarDecayer,InvEnergy> interfaceCoupling
    ("Coupling",
     "The coupling for the decay mode",
     &VectorMesonVectorScalarDecayer::_coupling,
     1/GeV, 0, ZERO, ZERO, 100./GeV, false, false, true);

  static ParVector<VectorMesonVectorScalarDecayer,double> interfaceMaxWeight
    ("MaxWeight",
     "The maximum weight for the decay mode",
     &VectorMesonVectorScalarDecayer::_maxweight,
     0, 0, 0, 0., 100., false, false, true);
}

double VectorMesonVectorScalarDecayer::me2(const int,
					   const Particle & inpart,
					   const ParticleVector & decay,
					   MEOption meopt) const {
  if(!ME())
    ME(new_ptr(TwoBodyDecayMatrixElement(PDT::Spin1,PDT::Spin1,PDT::Spin0)));
  // is the vector massless
  bool photon(_outgoingV[imode()]==ParticleID::gamma);
  if(meopt==Initialize) {
    VectorWaveFunction::calculateWaveFunctions(_vectors[0],_rho,
						const_ptr_cast<tPPtr>(&inpart),
						incoming,false);
  }
  if(meopt==Terminate) {
    VectorWaveFunction::constructSpinInfo(_vectors[0],const_ptr_cast<tPPtr>(&inpart),
					  incoming,true,false);
    VectorWaveFunction::constructSpinInfo(_vectors[1],decay[0],
					  outgoing,true,photon);
    ScalarWaveFunction::constructSpinInfo(decay[1],outgoing,true);
    return 0.;
  }
  VectorWaveFunction::calculateWaveFunctions(_vectors[1],decay[0],outgoing,photon);
  // compute the matrix element
  Energy2 p0dotpv(inpart.momentum()*decay[0]->momentum());
  complex<Energy> epsdot(ZERO);
  InvEnergy2 pre(_coupling[imode()]/inpart.mass());
  for(unsigned int ix=0;ix<3;++ix) {
    if(ix==1&&photon) {
      for(unsigned int iy=0;iy<3;++iy) (*ME())(iy,ix,0)=0.;
    }
    else {
      epsdot=_vectors[1][ix]*inpart.momentum();
      for(unsigned int iy=0;iy<3;++iy) {
	(*ME())(iy,ix,0)=Complex(pre*_vectors[0][iy].dot(p0dotpv*_vectors[1][ix]-
							 epsdot*decay[0]->momentum()));
      }
    }
  }
  // test of the matrix element
//   double me = newME.contract(rhoin).real();
//   Energy pcm=Kinematics::pstarTwoBodyDecay(inpart.mass(),decay[0]->mass(),
// 					   decay[1]->mass());
//   double test = sqr(_coupling[imode()])/3.*(2.*sqr(pcm)+3.*sqr(decay[0]->mass()));
//   cerr << "testing matrix element for " << inpart.PDGName() << " -> " 
//        << decay[0]->PDGName() << " " << decay[1]->PDGName() << " "
//        << me << " " << test << " " << (me-test)/(me+test) << "\n";
  // return the answer
  return ME()->contract(_rho).real();
}

bool VectorMesonVectorScalarDecayer::twoBodyMEcode(const DecayMode & dm,
						     int & mecode,
						     double & coupling) const {
  int id(dm.parent()->id());
  int idbar = dm.parent()->CC() ? dm.parent()->CC()->id() : id;
  ParticleMSet::const_iterator pit(dm.products().begin());
  int id1((**pit).id());
  int id1bar = (**pit).CC() ? (**pit).CC()->id() : id1;
  ++pit;
  int id2((**pit).id());
  int id2bar = (**pit).CC() ? (**pit).CC()->id() : id2;
  unsigned int ix(0); 
  bool order(false);
  int imode(-1);
  do {
    if(id==_incoming[ix]) {
      if(id1   ==_outgoingV[ix]&&id2   ==_outgoingS[ix]) {
	imode=ix;
	order=true;
      }
      if(id2   ==_outgoingV[ix]&&id1   ==_outgoingS[ix]) {
	imode=ix;
	order=false;
      }
    }
    if(idbar==_incoming[ix]&&imode<0) {
      if(id1bar==_outgoingV[ix]&&id2bar==_outgoingS[ix]) {
	imode=ix;
	order=true;
      }
      if(id2bar==_outgoingV[ix]&&id1bar==_outgoingS[ix]) {
	imode=ix;
	order=false;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  coupling = _coupling[imode]*dm.parent()->mass();  
  mecode = 4;
  return order;
}

void VectorMesonVectorScalarDecayer::dataBaseOutput(ofstream & output,
						    bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  // the rest of the parameters
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":Incoming " << ix << " "
	     << _incoming[ix] << "\n";
      output << "newdef " << name() << ":OutgoingVector " << ix << " "
	     << _outgoingV[ix] << "\n";
      output << "newdef " << name() << ":OutgoingScalar " << ix << " "
	     << _outgoingS[ix] << "\n";
      output << "newdef " << name() << ":Coupling " << ix << " "
	     << _coupling[ix]*GeV << "\n";
      output << "newdef " << name() << ":MaxWeight " << ix << " "
	     << _maxweight[ix] << "\n";
    }
    else {
      output << "insert " << name() << ":Incoming "  << ix << " "
	     << _incoming[ix] << "\n";
      output << "insert " << name() << ":OutgoingVector " << ix << " "
	     << _outgoingV[ix] << "\n";
      output << "insert " << name() << ":OutgoingScalar " << ix << " "
	     << _outgoingS[ix] << "\n";
      output << "insert " << name() << ":Coupling " << ix << " "
	     << _coupling[ix]*GeV << "\n";
      output << "insert " << name() << ":MaxWeight " << ix << " "
	     << _maxweight[ix] << "\n";
    }
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}
#line 1 "./VectorMesonVectorVectorDecayer.cc"
// -*- C++ -*-
//
// VectorMesonVectorVectorDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the VectorMesonVectorVectorDecayer class.
//

#include "VectorMesonVectorVectorDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "Herwig/Decay/TwoBodyDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void VectorMesonVectorVectorDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  if(initialize()) {
    for(unsigned int ix=0;ix<_incoming.size();++ix)
      if(mode(ix)) _maxweight[ix]=mode(ix)->maxWeight();
  }
}

void VectorMesonVectorVectorDecayer::doinit() {
  DecayIntegrator::doinit();
  unsigned int isize(_incoming.size());
  if(isize!=_outgoing1.size()||isize!=_outgoing2.size()||
     isize!=_maxweight.size()||isize!=_coupling.size())
    throw InitException() << "Inconsistent parameters in " 
			  << "VectorMesonVectorVectorDecayer" << Exception::runerror;
  // set up the integration channels
  vector<double> wgt(0);
  tPDVector extpart(3);
  DecayPhaseSpaceModePtr mode;
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    extpart[0]=getParticleData(_incoming[ix]);
    extpart[1]=getParticleData(_outgoing1[ix]);
    extpart[2]=getParticleData(_outgoing2[ix]);
    if(extpart[0]&&extpart[1]&&extpart[2]) 
      mode=new_ptr(DecayPhaseSpaceMode(extpart,this));
    else
      mode=DecayPhaseSpaceModePtr();
    addMode(mode,_maxweight[ix],wgt);
  }
}

VectorMesonVectorVectorDecayer::VectorMesonVectorVectorDecayer() 
  : _coupling(4), _incoming(4), _outgoing1(4), _outgoing2(4), 
    _maxweight(4) {
  // decay of rho'' to rho rho
  _incoming[0] = 30213; _outgoing1[0] =  213; _outgoing2[0] = 113; 
  _coupling[0] = 3.21; _maxweight[0] = 35.; 
  _incoming[1] = 30113; _outgoing1[1] = -213; _outgoing2[1] = 213; 
  _coupling[1] = 3.21; _maxweight[1] = 22.; 
  // decay of rho' to rho rho
  _incoming[2] =  100213; _outgoing1[2] =  213; _outgoing2[2] = 113; 
  _coupling[2] = 9.59; _maxweight[2] = 55.; 
  _incoming[3] =  100113; _outgoing1[3] = -213; _outgoing2[3] = 213; 
  _coupling[3] = 9.59; _maxweight[3] = 50.; 
  // initial size of the arrays
  _initsize=_incoming.size();
  // intermediates
  generateIntermediates(false);
}

int VectorMesonVectorVectorDecayer::modeNumber(bool & cc,tcPDPtr parent,
						 const tPDVector & children) const {
  if(children.size()!=2) return -1;
  int id(parent->id());
  int idbar = parent->CC() ? parent->CC()->id() : id;
  int id1(children[0]->id());
  int id1bar = children[0]->CC() ? children[0]->CC()->id() : id1;
  int id2(children[1]->id());
  int id2bar = children[1]->CC() ? children[1]->CC()->id() : id2;
  int imode(-1);
  unsigned int ix(0);
  cc=false;
  do {
    if(id   ==_incoming[ix]) {
      if((id1   ==_outgoing1[ix]&&id2   ==_outgoing2[ix])||
	 (id2   ==_outgoing1[ix]&&id1   ==_outgoing2[ix])) imode=ix;
    }
    if(idbar==_incoming[ix]) {
      if((id1bar==_outgoing1[ix]&&id2bar==_outgoing2[ix])||
	 (id2bar==_outgoing1[ix]&&id1bar==_outgoing2[ix])) {
	imode=ix;
	cc=true;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  return imode;
}

void VectorMesonVectorVectorDecayer::persistentOutput(PersistentOStream & os) const {
  os << _incoming << _outgoing1 << _outgoing2 << _maxweight << _coupling;
}

void VectorMesonVectorVectorDecayer::persistentInput(PersistentIStream & is, int) {
  is >> _incoming >> _outgoing1 >> _outgoing2 >> _maxweight >> _coupling;
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<VectorMesonVectorVectorDecayer,DecayIntegrator>
describeHerwigVectorMesonVectorVectorDecayer("Herwig::VectorMesonVectorVectorDecayer", "HwVMDecay.so");

void VectorMesonVectorVectorDecayer::Init() {

  static ClassDocumentation<VectorMesonVectorVectorDecayer> documentation
    ("The VectorMesonVectorVectorDecayer class is designed for the "
     "decay of a vector meson to two vector particles, either photons or other "
     "vector mesons.");

  static ParVector<VectorMesonVectorVectorDecayer,int> interfaceIncoming
    ("Incoming",
     "The PDG code for the incoming particle",
     &VectorMesonVectorVectorDecayer::_incoming,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMesonVectorVectorDecayer,int> interfaceOutgoing1
    ("Outgoing1",
     "The PDG code for the first outgoing particle",
     &VectorMesonVectorVectorDecayer::_outgoing1,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMesonVectorVectorDecayer,int> interfaceOutgoing2
    ("Outgoing2",
     "The PDG code for the second outgoing particle",
     &VectorMesonVectorVectorDecayer::_outgoing2,
     0, 0, 0, -10000000, 10000000, false, false, true);

  static ParVector<VectorMesonVectorVectorDecayer,double> interfaceCoupling
    ("Coupling",
     "The coupling for the decay mode",
     &VectorMesonVectorVectorDecayer::_coupling,
     0, 0, 0, 0., 100., false, false, true);

  static ParVector<VectorMesonVectorVectorDecayer,double> interfaceMaxWeight
    ("MaxWeight",
     "The maximum weight for the decay mode",
     &VectorMesonVectorVectorDecayer::_maxweight,
     0, 0, 0, 0., 1000., false, false, true);

}

double VectorMesonVectorVectorDecayer::me2(const int,
					   const Particle & inpart,
					   const ParticleVector & decay,
					   MEOption meopt) const {
  if(!ME())
    ME(new_ptr(TwoBodyDecayMatrixElement(PDT::Spin1,PDT::Spin1,PDT::Spin1)));
  bool photon[2];
  for(unsigned int ix=0;ix<2;++ix) 
    photon[ix] = decay[ix]->id()==ParticleID::gamma;
  if(meopt==Initialize) {
    VectorWaveFunction::calculateWaveFunctions(_vectors[0],_rho,
					       const_ptr_cast<tPPtr>(&inpart),
					       incoming,false);
  }
  if(meopt==Terminate) {
    VectorWaveFunction::constructSpinInfo(_vectors[0],const_ptr_cast<tPPtr>(&inpart),
					  incoming,true,false);
    for(unsigned int ix=0;ix<2;++ix)
      VectorWaveFunction::constructSpinInfo(_vectors[ix+1],decay[ix],
					    outgoing,true,photon[ix]);
    return 0.;
  }
  for(unsigned int ix=0;ix<2;++ix)
    VectorWaveFunction::
      calculateWaveFunctions(_vectors[ix+1],decay[ix],outgoing,photon[ix]);
  // work out the dot products we need for the matrix element
  Energy2 p1p2((decay[0]->momentum())*(decay[1]->momentum()));
  complex<Energy> p1eps2[3],p2eps1[3];
  for(unsigned int ix=0;ix<3;++ix) {
    p1eps2[ix]=_vectors[2][ix]*(decay[0]->momentum());
    p2eps1[ix]=_vectors[1][ix]*(decay[1]->momentum());
  }
  // compute the matrix element
  Lorentz5Momentum pdiff(decay[0]->momentum()-decay[1]->momentum());
  Energy2 m12(decay[0]->mass()*decay[0]->mass()),m22(decay[1]->mass()*decay[1]->mass());
  InvEnergy3 fact(2.*_coupling[imode()]/(inpart.mass()*inpart.mass()*inpart.mass()));
  LorentzPolarizationVector vtemp;
  for(unsigned int ipol1=0;ipol1<3;++ipol1) {
    for(unsigned int ipol2=0;ipol2<3;++ipol2) {
      Complex eps1eps2=_vectors[1][ipol1].dot(_vectors[2][ipol2]);
      vtemp=fact*(p1eps2[ipol2]*p2eps1[ipol1]*pdiff
		  +p1eps2[ipol2]*m22*_vectors[1][ipol1]
		  -p2eps1[ipol1]*m12*_vectors[2][ipol2]
		  +eps1eps2*(-p1p2*pdiff+m12*decay[1]->momentum()
			     -m22*decay[0]->momentum()));
      for(unsigned int inpol=0;inpol<3;++inpol) 
	(*ME())(inpol,ipol1,ipol2)=_vectors[0][inpol].dot(vtemp);
    }
  }
  // test of the matrix element
//   double me = newME.contract(rhoin).real();
//   Energy pcm=Kinematics::pstarTwoBodyDecay(inpart.mass(),decay[0]->mass(),
// 					   decay[1]->mass());
//   double test = 8./3.*sqr(_coupling[imode()]*pcm/inpart.mass())*
//     (1.+sqr(decay[0]->mass()/inpart.mass())+sqr(decay[1]->mass()/inpart.mass()));
//   cerr << "testing matrix element for " << inpart.PDGName() << " -> " 
//        << decay[0]->PDGName() << " " << decay[1]->PDGName() << " "
//        << me << " " << test << " " << (me-test)/(me+test) << "\n";
  // return the answer
  return ME()->contract(_rho).real();
}

bool VectorMesonVectorVectorDecayer::twoBodyMEcode(const DecayMode & dm,int & mecode,
						    double & coupling) const {
  int imode(-1);
  int id(dm.parent()->id()),idbar(id);
  if(dm.parent()->CC()){idbar=dm.parent()->CC()->id();}
  ParticleMSet::const_iterator pit(dm.products().begin());
  int id1((**pit).id()),id1bar(id1);
  if((**pit).CC()){id1bar=(**pit).CC()->id();}
  ++pit;
  int id2((**pit).id()),id2bar(id2);
  if((**pit).CC()){id2bar=(**pit).CC()->id();}
  unsigned int ix(0); bool order(false);
  do {
    if(id   ==_incoming[ix]) {
      if(id1==_outgoing1[ix]&&id2==_outgoing2[ix]) {
	imode=ix;
	order=true;
      }
      if(id2==_outgoing1[ix]&&id1==_outgoing2[ix]) {
	imode=ix;
	order=false;
      }
    }
    if(idbar==_incoming[ix]&&imode<0) {
      if(id1bar==_outgoing1[ix]&&id2bar==_outgoing2[ix]) {
	imode=ix;
	order=true;
      }
      if(id2bar==_outgoing1[ix]&&id1bar==_outgoing2[ix]) {
	imode=ix;
	order=false;
      }
    }
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  coupling = _coupling[imode]; 
  mecode = 5;
  return order; 
}

// output the setup information for the particle database
void VectorMesonVectorVectorDecayer::dataBaseOutput(ofstream & output,
						    bool header) const {
  if(header){output << "update decayers set parameters=\"";}
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  // the rest of the parameters
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":Incoming " << ix << " " 
	     << _incoming[ix] << "\n";
      output << "newdef " << name() << ":Outgoing1 " << ix << " " 
	     << _outgoing1[ix] << "\n";
      output << "newdef " << name() << ":Outgoing2 " << ix << " " 
	     << _outgoing2[ix] << "\n";
      output << "newdef " << name() << ":Coupling " << ix << " " 
	     << _coupling[ix] << "\n";
      output << "newdef " << name() << ":MaxWeight " << ix << " " 
	     << _maxweight[ix] << "\n";
    }
    else {
      output << "insert " << name() << ":Incoming " << ix << " " 
	     << _incoming[ix] << "\n";
      output << "insert " << name() << ":Outgoing1 " << ix << " " 
	     << _outgoing1[ix] << "\n";
      output << "insert " << name() << ":Outgoing2 " << ix << " " 
	     << _outgoing2[ix] << "\n";
      output << "insert " << name() << ":Coupling " << ix << " " 
	     << _coupling[ix] << "\n";
      output << "insert " << name() << ":MaxWeight " << ix << " " 
	     << _maxweight[ix] << "\n";
    }
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}
#line 1 "./OniumToOniumPiPiDecayer.cc"
// -*- C++ -*-
//
// OniumToOniumPiPiDecayer.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the OniumToOniumPiPiDecayer class.
//

#include "OniumToOniumPiPiDecayer.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/PDT/EnumParticles.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "Herwig/PDT/ThreeBodyAllOnCalculator.h"
#include "Herwig/Decay/GeneralDecayMatrixElement.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void OniumToOniumPiPiDecayer::doinitrun() {
  DecayIntegrator::doinitrun();
  for(unsigned int ix=0;ix<_maxweight.size();++ix) {
    if(initialize()) _maxweight[ix] = mode(ix)->maxWeight();
  }
}

OniumToOniumPiPiDecayer::OniumToOniumPiPiDecayer() {
  // Upsilon(3S)->Upsilon(1S) pi pi
  _incoming.push_back(200553);
  _outgoing.push_back(   553);
  _maxweight.push_back(1.);
  _maxweight.push_back(1.);
  _coupling.push_back(3.92e-6);
  _reA.push_back( 1.   /MeV2);_imA.push_back( ZERO);
  _reB.push_back(-2.523/MeV2);_imB.push_back( 1.189/MeV2);
  _reC.push_back( ZERO);_imC.push_back( ZERO);
  // Upsilon(3S)->Upsilon(2S) pi pi
  _incoming.push_back(200553);
  _outgoing.push_back(100553);
  _maxweight.push_back(1.);
  _maxweight.push_back(1.);
  _coupling.push_back(311e-6);
  _reA.push_back( 1.   /MeV2);_imA.push_back( ZERO);
  _reB.push_back(-0.395/MeV2);_imB.push_back( 0.001/MeV2);
  _reC.push_back( ZERO);_imC.push_back( ZERO);
  // Upsilon(2S)->Upsilon(1S) pi pi
  _incoming.push_back(100553);
  _outgoing.push_back(   553);
  _maxweight.push_back(1.);
  _maxweight.push_back(1.);
  _coupling.push_back(61.4e-6);
  _reA.push_back( 1.   /MeV2);_imA.push_back( ZERO);
  _reB.push_back(-0.753/MeV2);_imB.push_back( ZERO);
  _reC.push_back( ZERO);_imC.push_back( ZERO);
  // Upsilon(4S)->Upsilon(1S) pi pi
  _incoming.push_back(300553);
  _outgoing.push_back(   553);
  _maxweight.push_back(1.);
  _maxweight.push_back(1.);
  _coupling.push_back(1.77e-6);
  _reA.push_back( 1.   /MeV2);_imA.push_back( ZERO);
  _reB.push_back( ZERO);_imB.push_back( ZERO);
  _reC.push_back( ZERO);_imC.push_back( ZERO);
  // Upsilon(4S)->Upsilon(2S) pi pi
  _incoming.push_back(300553);
  _outgoing.push_back(100553);
  _maxweight.push_back(1.);
  _maxweight.push_back(1.);
  _coupling.push_back(68.8e-6);
  _reA.push_back( 1.   /MeV2);_imA.push_back( ZERO);
  _reB.push_back(-2.35   /MeV2);_imB.push_back( 0.55/MeV2);
  _reC.push_back( ZERO);_imC.push_back( ZERO);
  // psi(2s)->psi(1S) pi pi
  _incoming.push_back(100443);
  _outgoing.push_back(   443);
  _maxweight.push_back(1.);
  _maxweight.push_back(1.);
  _coupling.push_back(66.2e-6);
  _reA.push_back( 1.   /MeV2);_imA.push_back( ZERO);
  _reB.push_back(-0.336/MeV2);_imB.push_back( ZERO);
  _reC.push_back( ZERO);_imC.push_back( ZERO);
  // psi(3770)->psi(1S) pi pi
  _incoming.push_back(30443);
  _outgoing.push_back(   443);
  _maxweight.push_back(1.);
  _maxweight.push_back(1.);
  _coupling.push_back(20.6e-6);
  _reA.push_back( 1.   /MeV2);_imA.push_back( ZERO);
  _reB.push_back( ZERO);_imB.push_back( ZERO);
  _reC.push_back( ZERO);_imC.push_back( ZERO);
  // Initial size of the vectors
  _initsize=_incoming.size();
  // don'y generate the intermediates in the phase-space
  generateIntermediates(false);
}

void OniumToOniumPiPiDecayer::doinit() {
  DecayIntegrator::doinit();
  // check consistency of the vectors
  unsigned int isize=_incoming.size();
  if(_outgoing.size()!=isize||_maxweight.size()!=2*isize||
     _coupling.size()!=isize||
     _reA     .size()!=isize||_imA.size()      !=isize||
     _reB     .size()!=isize||_imB.size()      !=isize||
     _reC     .size()!=isize||_imC.size()      !=isize)
    throw InitException() << "Inconsistent size of the parameter vectors in "
			  << "OniumToOniumPiPiDecayer"
			  << Exception::runerror;
  // construct the complex couplings
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    _cA.push_back(complex<InvEnergy2>(_reA[ix],_imA[ix]));
    _cB.push_back(complex<InvEnergy2>(_reB[ix],_imB[ix]));
    _cC.push_back(complex<InvEnergy2>(_reC[ix],_imC[ix]));
  }
  // construct the decay channels
  tPDVector extpart(4);
  tPDPtr pip(getParticleData(ParticleID::piplus ));
  tPDPtr pim(getParticleData(ParticleID::piminus));
  tPDPtr pi0(getParticleData(ParticleID::pi0    ));
  tPDPtr rho0(getParticleData(113)); 
  DecayPhaseSpaceModePtr mode;
  DecayPhaseSpaceChannelPtr newchannel;
  vector<double> dummyweights(1,1.);
  for(unsigned int ix=0;ix<isize;++ix) {
    extpart[0]=getParticleData(_incoming[ix]);
    extpart[1]=getParticleData(_outgoing[ix]);
    for(unsigned int iy=0;iy<2;++iy) {
      // pi+ pi-
      if(iy==0) {
	extpart[2]=pip;
	extpart[3]=pim;
      }
      // pi0 pi0
      else {
	extpart[2]=pi0;
	extpart[3]=pi0;
      }
      // construct the phase-space mode
      mode = new_ptr(DecayPhaseSpaceMode(extpart,this));
      newchannel=new_ptr(DecayPhaseSpaceChannel(mode));
      newchannel->addIntermediate(extpart[0],0, 0.0,1,-1);
      newchannel->addIntermediate(rho0,1,0.0, 2,3);
      mode->addChannel(newchannel);
      // reset the resonance parameters
      mode->resetIntermediate(rho0,2*extpart[0]->mass(),2*extpart[0]->mass());
      // add the mode
      addMode(mode,_maxweight[2*ix+iy],dummyweights);
    }
  }
}


void OniumToOniumPiPiDecayer::persistentOutput(PersistentOStream & os) const {
  os << _incoming << _outgoing << _maxweight << _initsize << ounit(_reA,1./GeV2) 
     << ounit(_imA,1./GeV2) << ounit(_cA,1./GeV2) << ounit(_reB,1./GeV2) 
     << ounit(_imB,1./GeV2) << ounit(_cB,1./GeV2) << ounit(_reC,1./GeV2) 
     << ounit(_imC,1./GeV2) << ounit(_cC,1./GeV2);
}

void OniumToOniumPiPiDecayer::persistentInput(PersistentIStream & is, int) {
  is >> _incoming >> _outgoing >> _maxweight >> _initsize >> iunit(_reA,1./GeV2) 
     >> iunit(_imA,1./GeV2) >> iunit(_cA,1./GeV2) >> iunit(_reB,1./GeV2) 
     >> iunit(_imB,1./GeV2) >> iunit(_cB,1./GeV2) >> iunit(_reC,1./GeV2) 
     >> iunit(_imC,1./GeV2) >> iunit(_cC,1./GeV2);
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<OniumToOniumPiPiDecayer,DecayIntegrator>
describeHerwigOniumToOniumPiPiDecayer("Herwig::OniumToOniumPiPiDecayer", "HwVMDecay.so");

void OniumToOniumPiPiDecayer::Init() {

  static ClassDocumentation<OniumToOniumPiPiDecayer> documentation
    ("The OniumToOniumPiPiDecayer class uses the matrix element of "
     "Brown and Cahn, PRL35, 1 (1975), for"
     " the decay of onium resonaces to lighter states and pion pairs."
     " The results of hep-ex/9909038 are used for psi'->psi and "
     " arXiv:0706.2317 for Upsilon(3S) and Upsilon(2S) decays."
     " The remaining parameters are choosen to approximately reproduce"
     " the distributions from hep-ex/0604031 and hep-ex/0508023.",
     "The decays of onium resonances to lighter states and pion pairs were modelled"
     " using the matrix element of \\cite{Brown:1975dz}. The results of "
     "\\cite{Bai:1999mj} are used for $\\psi'\\to\\psi$ and "
     "\\cite{Cronin-Hennessy:2007sj} for $\\Upsilon(3S)$ and $\\Upsilon(2S)$ decays."
     " The remaining parameters are choosen to approximately reproduce"
     " the distributions from \\cite{Aubert:2006bm} and \\cite{Adam:2005mr}.",
     "\\bibitem{Brown:1975dz} L.~S.~Brown and R.~N.~Cahn,"
     "Phys.\\ Rev.\\ Lett.\\  {\\bf 35} (1975) 1."
     "%%CITATION = PRLTA,35,1;%%\n"
     "\\bibitem{Bai:1999mj} J.~Z.~Bai {\\it et al.}  [BES Collaboration],"
     "Phys.\\ Rev.\\  D {\\bf 62} (2000) 032002 [arXiv:hep-ex/9909038]."
     "%%CITATION = PHRVA,D62,032002;%%\n"
     "\\bibitem{Cronin-Hennessy:2007sj} D.~Cronin-Hennessy{\\it et al.} "
     "[CLEO Collaboration], arXiv:0706.2317 [hep-ex]."
     "%%CITATION = ARXIV:0706.2317;%%\n"
     "\\bibitem{Aubert:2006bm} B.~Aubert {\\it et al.}  [BABAR Collaboration],"
     "Phys.\\ Rev.\\ Lett.\\  {\\bf 96} (2006) 232001 [arXiv:hep-ex/0604031]."
     "%%CITATION = PRLTA,96,232001;%%\n"
     "\\bibitem{Adam:2005mr} N.~E.~Adam {\\it et al.}  [CLEO Collaboration],"
     "Phys.\\ Rev.\\ Lett.\\  {\\bf 96} (2006) 082004 [arXiv:hep-ex/0508023]."
     "%%CITATION = PRLTA,96,082004;%%");

  static ParVector<OniumToOniumPiPiDecayer,long> interfaceIncoming
    ("Incoming",
     "The PDG code for the incoming onium state",
     &OniumToOniumPiPiDecayer::_incoming, -1, long(0), -10000000, 10000000,
     false, false, Interface::limited);

  static ParVector<OniumToOniumPiPiDecayer,long> interfaceOutgoing
    ("Outgoing",
     "The PDG code for the outgoing onium state",
     &OniumToOniumPiPiDecayer::_outgoing, -1, long(0), -10000000, 10000000,
     false, false, Interface::limited);

  static ParVector<OniumToOniumPiPiDecayer,double> interfaceMaxWeight
    ("MaxWeight",
     "The maximum weight for the decay mode, there should be two "
     "for each mode as we have pi+ pi- and pi0 pi0",
     &OniumToOniumPiPiDecayer::_maxweight, -1, 1.0, 0.0, 10000.0,
     false, false, Interface::limited);

  static ParVector<OniumToOniumPiPiDecayer,double> interfaceCoupling
    ("Coupling",
     "The overall coupling for the decay",
     &OniumToOniumPiPiDecayer::_coupling, -1, 1.0, 0.0, 10.0,
     false, false, Interface::limited);

  static ParVector<OniumToOniumPiPiDecayer,InvEnergy2> interfaceReA
    ("ReA",
     "The real part of the A coupling",
     &OniumToOniumPiPiDecayer::_reA, 1./MeV2, -1, 1.0/MeV2, -1000.0/MeV2, 1000.0/MeV2,
     false, false, Interface::limited);

  static ParVector<OniumToOniumPiPiDecayer,InvEnergy2> interfaceImA
    ("ImA",
     "The imaginary part of the A coupling",
     &OniumToOniumPiPiDecayer::_imA, 1./MeV2, -1, 1.0/MeV2, -1000.0/MeV2, 1000.0/MeV2,
     false, false, Interface::limited);

  static ParVector<OniumToOniumPiPiDecayer,InvEnergy2> interfaceReB
    ("ReB",
     "The real part of the B coupling",
     &OniumToOniumPiPiDecayer::_reB, 1./MeV2, -1, 1.0/MeV2, -1000.0/MeV2, 1000.0/MeV2,
     false, false, Interface::limited);

  static ParVector<OniumToOniumPiPiDecayer,InvEnergy2> interfaceImB
    ("ImB",
     "The imaginary part of the B coupling",
     &OniumToOniumPiPiDecayer::_imB, 1./MeV2, -1, 1.0/MeV2, -1000.0/MeV2, 1000.0/MeV2,
     false, false, Interface::limited);

  static ParVector<OniumToOniumPiPiDecayer,InvEnergy2> interfaceReC
    ("ReC",
     "The real part of the C coupling",
     &OniumToOniumPiPiDecayer::_reC, 1./MeV2, -1, 1.0/MeV2, -1000.0/MeV2, 1000.0/MeV2,
     false, false, Interface::limited);

  static ParVector<OniumToOniumPiPiDecayer,InvEnergy2> interfaceImC
    ("ImC",
     "The imaginary part of the C coupling",
     &OniumToOniumPiPiDecayer::_imC, 1./MeV2, -1, 1.0/MeV2, -1000.0/MeV2, 1000.0/MeV2,
     false, false, Interface::limited);
}

int OniumToOniumPiPiDecayer::modeNumber(bool & cc,tcPDPtr parent,
					const tPDVector & children) const {
  cc=false;
  int imode(-1);
  long idin(parent->id());
  if(children.size()!=3) return -1;
  unsigned int npip(0),npim(0),npi0(0);
  long idother(0),id;
  for(tPDVector::const_iterator pit=children.begin();
      pit!=children.end();++pit) {
    id=(**pit).id();
    if(id==ParticleID::piplus)       ++npip;
    else if(id==ParticleID::piminus) ++npim;
    else if(id==ParticleID::pi0)     ++npi0;
    else idother=id;
  }
  // check pi+ pi- or pi0 pi0 and outgoing state
  if(!((npip==1&&npim==1)||npi0==2)||idother==0) return -1;
  unsigned int ix=0;
  do {
    if(idin==_incoming[ix]&&idother==_outgoing[ix]) imode=ix;
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  return npi0==2 ? 2*imode+1 : 2*imode;
}

double OniumToOniumPiPiDecayer::me2(const int,
				    const Particle & inpart,
				    const ParticleVector & decay,
				    MEOption meopt) const {
  if(!ME())
    ME(new_ptr(GeneralDecayMatrixElement(PDT::Spin1,PDT::Spin1,PDT::Spin0,PDT::Spin0)));
  useMe();
  if(meopt==Initialize) {
    VectorWaveFunction::calculateWaveFunctions(_vectors[0],_rho,
						const_ptr_cast<tPPtr>(&inpart),
						incoming,false);
  }
  if(meopt==Terminate) {
    VectorWaveFunction::constructSpinInfo(_vectors[0],const_ptr_cast<tPPtr>(&inpart),
					  incoming,true,false);
    VectorWaveFunction::constructSpinInfo(_vectors[1],decay[0],
					  outgoing,true,false);
    for(unsigned int ix=1;ix<3;++ix)
      ScalarWaveFunction::constructSpinInfo(decay[ix],outgoing,true);
    return 0.;
  }
  VectorWaveFunction::calculateWaveFunctions(_vectors[1],decay[0],outgoing,false);
  // compute the matrix element
  complex<InvEnergy2> A(_cA[imode()/2]),B(_cB[imode()/2]),C(_cC[imode()/2]);
  Energy2 q2  =(decay[1]->momentum()+decay[2]->momentum()).m2();
  Energy2 mpi2=sqr(decay[1]->mass());
  for(unsigned int ix=0;ix<3;++ix) {
    for(unsigned int iy=0;iy<3;++iy) {
      Complex dota = _vectors[0][ix].dot(_vectors[1][iy]);
      complex<Energy2> dotb = 
	(_vectors[0][ix]*decay[1]->momentum())*(_vectors[1][iy]*decay[2]->momentum())+
	(_vectors[0][ix]*decay[2]->momentum())*(_vectors[1][iy]*decay[1]->momentum());
      (*ME())(ix,iy,0,0)= _coupling[imode()/2]*
	Complex(A*dota*(q2-2.*mpi2)+B*dota*decay[1]->momentum().e()*decay[2]->momentum().e()
	 +C*dotb);
    }
  }
  // matrix element
  double output=ME()->contract(_rho).real();
  if(imode()%2==1) output*=0.5;
  // test of the matrix element
//   Energy2 s1=(decay[1]->momentum()+decay[2]->momentum()).m2();
//   Energy2 s2=(decay[0]->momentum()+decay[2]->momentum()).m2();
//   Energy2 s3=(decay[1]->momentum()+decay[0]->momentum()).m2();
//   double test=threeBodyMatrixElement(imode(),sqr(inpart.mass()),
// 				     s3,s2,s1,decay[0]->mass(),
// 				     decay[1]->mass(),decay[2]->mass());
  // return the answer
  return output;
}

// output the setup information for the particle database
void OniumToOniumPiPiDecayer::dataBaseOutput(ofstream & output,
					     bool header) const {
  if(header) output << "update decayers set parameters=\"";
  // parameters for the DecayIntegrator base class
  DecayIntegrator::dataBaseOutput(output,false);
  // the rest of the parameters
  for(unsigned int ix=0;ix<_incoming.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":Incoming " << ix << " " 
	     << _incoming[ix] << "\n";
      output << "newdef " << name() << ":Outgoing " << ix << " " 
	     << _outgoing[ix] << "\n";
      output << "newdef " << name() << ":Coupling " << ix << " " 
	     << _coupling[ix] << "\n";
      output << "newdef " << name() << ":ReA " << ix << " " 
	     << _reA[ix]*MeV2 << "\n";
      output << "newdef " << name() << ":ImA " << ix << " " 
	     << _imA[ix]*MeV2 << "\n";
      output << "newdef " << name() << ":ReB " << ix << " " 
	     << _reB[ix]*MeV2 << "\n";
      output << "newdef " << name() << ":ImB " << ix << " " 
	     << _imB[ix]*MeV2 << "\n";
      output << "newdef " << name() << ":ReC " << ix << " " 
	     << _reC[ix]*MeV2 << "\n";
      output << "newdef " << name() << ":ImC " << ix << " " 
	     << _imC[ix]*MeV2 << "\n";
    }
    else {
      output << "insert " << name() << ":Incoming " << ix << " " 
	     << _incoming[ix] << "\n";
      output << "insert " << name() << ":Outgoing " << ix << " " 
	     << _outgoing[ix] << "\n";
      output << "insert " << name() << ":Coupling " << ix << " " 
	     << _coupling[ix] << "\n";
      output << "insert " << name() << ":ReA " << ix << " " 
	     << _reA[ix]*MeV2 << "\n";
      output << "insert " << name() << ":ImA " << ix << " " 
	     << _imA[ix]*MeV2 << "\n";
      output << "insert " << name() << ":ReB " << ix << " " 
	     << _reB[ix]*MeV2 << "\n";
      output << "insert " << name() << ":ImB " << ix << " " 
	     << _imB[ix]*MeV2 << "\n";
      output << "insert " << name() << ":ReC " << ix << " " 
	     << _reC[ix]*MeV2 << "\n";
      output << "insert " << name() << ":ImC " << ix << " " 
	     << _imC[ix]*MeV2 << "\n";
    }
  }
  for(unsigned int ix=0;ix<_maxweight.size();++ix) {
    if(ix<2*_initsize) {
      output << "newdef " << name() << ":MaxWeight " << ix << " " 
	     << _maxweight[ix] << "\n";
    }
    else {
      output << "insert " << name() << ":MaxWeight " << ix << " " 
	     << _maxweight[ix] << "\n";
    }
  }
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}

WidthCalculatorBasePtr OniumToOniumPiPiDecayer::
threeBodyMEIntegrator(const DecayMode & dm) const {
  int imode(-1);
  long idin(dm.parent()->id());
  unsigned int npip(0),npim(0),npi0(0);
  long idother(0),id;
  for(ParticleMSet::const_iterator pit=dm.products().begin();
      pit!=dm.products().end();++pit) {
    id=(**pit).id();
    if(id==ParticleID::piplus)       ++npip;
    else if(id==ParticleID::piminus) ++npim;
    else if(id==ParticleID::pi0)     ++npi0;
    else idother=id;
  }
  unsigned int ix=0;
  do {
    if(idin==_incoming[ix]&&idother==_outgoing[ix]) imode=ix;
    ++ix;
  }
  while(ix<_incoming.size()&&imode<0);
  imode = npi0==2 ? 2*imode+1 : 2*imode;
  // construct the integrator
  vector<double> inweights(1,1.);
  Energy scale=getParticleData(_incoming[ix-1])->mass();
  Energy m1=getParticleData(_outgoing[ix-1])->mass();
  Energy mpi = npi0==2 ? getParticleData(ParticleID::pi0)->mass() :
    getParticleData(ParticleID::piplus)->mass();
  vector<int> intype(1,3);
  vector<Energy> inmass (1,scale);
  vector<Energy> inwidth(1,scale);
  vector<double> inpow(1,0.0);
  return new_ptr(ThreeBodyAllOnCalculator<OniumToOniumPiPiDecayer>
		 (inweights,intype,inmass,inwidth,inpow,
		  *this,imode,m1,mpi,mpi));
}

double OniumToOniumPiPiDecayer::
threeBodyMatrixElement(const int imode, const Energy2 q2,
		       const  Energy2 s3, const Energy2 s2, const Energy2 s1, 
		       const Energy m1, const Energy m2, const Energy m3) const {
  Energy q=sqrt(q2);
  Energy e2 = 0.5*(q2+sqr(m2)-s2)/q;
  Energy e3 = 0.5*(q2+sqr(m3)-s3)/q;
  Complex amp = _cA[imode/2]*(s1-sqr(m2)-sqr(m3))+_cB[imode/2]*e2*e3;
  Energy2 dot = 0.5*(q2+sqr(m1)-s1);
  double output=(2.+sqr(dot/q/m1))*real(amp*conj(amp))*sqr(_coupling[imode/2])/3.;
  if(imode%2==1) output*=0.5;
  return output;
}
