#line 1 "./FourPionNovosibirskCurrent.cc"
// -*- C++ -*-
//
// FourPionNovosibirskCurrent.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the FourPionNovosibirskCurrent class.
//

#include "FourPionNovosibirskCurrent.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Interface/Switch.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/Helicity/ScalarSpinInfo.h"
#include "Herwig/PDT/ThreeBodyAllOnCalculator.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "ThePEG/Utilities/DescribeClass.h"

#include <functional>

using namespace Herwig;
using namespace ThePEG;
using namespace ThePEG::Helicity;

DescribeClass<FourPionNovosibirskCurrent,WeakDecayCurrent>
describeHerwigFourPionNovosibirskCurrent("Herwig::FourPionNovosibirskCurrent",
					 "HwWeakCurrents.so");

HERWIG_INTERPOLATOR_CLASSDESC(FourPionNovosibirskCurrent1,Energy,Energy2)

HERWIG_INTERPOLATOR_CLASSDESC(FourPionNovosibirskCurrent2,double,Energy)

HERWIG_INTERPOLATOR_CLASSDESC(FourPionNovosibirskCurrent3,double,Energy2)

IBPtr FourPionNovosibirskCurrent::clone() const {
  return new_ptr(*this);
}

IBPtr FourPionNovosibirskCurrent::fullclone() const {
  return new_ptr(*this);
}

void FourPionNovosibirskCurrent::doupdate() {
  WeakDecayCurrent::doupdate();
  // update running width if needed
  if ( !touched() ) return;
  if(_maxmass!=_maxcalc) inita1width(-1);
}

FourPionNovosibirskCurrent::FourPionNovosibirskCurrent() : _mpic(), _mpi0(),
							   _mpic2(), _mpi02(), _prho()
{
  // set the number of modes
  addDecayMode(2,-1);
  addDecayMode(2,-1);
  setInitialModes(2);
  // masses of the particles used in the current
  _rhomass    = 0.7761*GeV;
  _a1mass     = 1.2300*GeV;
  _omegamass  = 0.7820*GeV;
  _sigmamass  = 0.8000*GeV;
  // widths of the particles used in the current
  _rhowidth   = 0.14450*GeV;
  _a1width    = 0.45000*GeV;
  _omegawidth = 0.00841*GeV;
  _sigmawidth = 0.80000*GeV;
  // parameters for the resonance used in the integration
  _intmass = 1.4*GeV;
  _intwidth = 0.5*GeV;
  // relative coupling of the sigma and rho in the a_1 decay
  _zmag   = 1.3998721;
  _zphase = 0.43585036;
  _zsigma=0.;
  // parameter for f_a1
  _lambda2 = 1.2*GeV2;
  _onedlam2 = 1./_lambda2;
  _a1massolam2 = _a1mass*_a1mass*_onedlam2;
  _hm2=ZERO; 
  _rhoD=ZERO;
  _dhdq2m2=0.;
  // use local values of the parameters
  _localparameters=true;
  // magic numbers from TAUOLA (N.B. conversion from GeV to MeV)
  _athreec = 76.565/GeV;
  _bthreec = 0.71709;
  _cthreec = 0.27505;
  _aomega  = 886.84/GeV;
  _bomega  = 0.70983;
  _comega  = 0.26689;
  _aonec   = 96.867/GeV;
  _bonec   = 0.70907;
  _conec   = 0.26413;
  //parameters for the running omega width
  _omegaparam.resize(10);
  _omegaparam[0] = 17.560;
  _omegaparam[1] = 141.110;
  _omegaparam[2] = 894.884;
  _omegaparam[3] = 4977.35;
  _omegaparam[4] = 7610.66;
  _omegaparam[5] =-42524.4;
  _omegaparam[6] =-1333.26;
  _omegaparam[7] = 4860.19;
  _omegaparam[8] =-6000.81;
  _omegaparam[9] = 2504.97;
  // values of the g omega function from hep-ph/0201149
  double Fomegainit[98]={ 0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
			  0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
			  0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
			  0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
			  0.0000000, 0.0000000, 0.0000000, 0.0000000, 2.2867811,
			  2.9710648, 2.9344304, 2.6913538, 2.5471206, 2.3557470,
			  2.2448280, 2.1074708, 2.0504866, 1.9270257, 1.8669430,
			  1.7907301, 1.7184515, 1.6535717, 1.6039416, 1.5535343,
			  1.5065620, 1.4608675, 1.4215596, 1.3849826, 1.3480113,
			  1.3147917, 1.2793381, 1.2487282, 1.2184237, 1.1952927,
			  1.1683835, 1.1458827, 1.1145806, 1.0935820, 1.0608720,
			  1.0390474, 1.0164336, 0.9908721, 0.9585276, 0.9307971,
			  0.9017274, 0.8731154, 0.8452763, 0.8145532, 0.7817339,
			  0.7493086, 0.7199919, 0.6887290, 0.6568120, 0.6255773,
			  0.5944664, 0.5661956, 0.5391204, 0.5102391, 0.4786543,
			  0.4546428, 0.4316779, 0.4063754, 0.3769831, 0.3561141,
			  0.3333555, 0.3139160, 0.2949214, 0.2814728, 0.2602444,
			  0.2349602, 0.2269845, 0.2192318, 0.2286938, 0.2839763,
			  0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
			  0.0000000, 0.0000000, 0.0000000};
  // values of the three charged pion G function from hep-ph/0201149
  double Fthreeinit[98]={ 0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
			  0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
			  0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
			  13.1664906,10.7234087, 8.8219614,10.7989664, 9.1883001,
			  7.8526378, 7.7481031, 8.2633696, 5.5042820, 4.9029269,
			  4.4794345, 3.9654009, 4.5254011, 3.6509495, 3.5005512,
			  3.2274280, 3.1808922, 2.9925177, 2.6886659, 2.5195024,
			  2.4678771, 2.3540580, 2.2123868, 2.1103525, 2.0106986,
			  1.8792295, 1.8250662, 1.7068460, 1.6442842, 1.5503920,
			  1.4814349, 1.4225838, 1.3627135, 1.3205355, 1.2784383,
			  1.2387408, 1.1975995, 1.1633024, 1.1318133, 1.1114354,
			  1.0951439, 1.0691465, 1.0602311, 1.0392803, 1.0220672,
			  1.0154786, 1.0010130, 0.9908018, 0.9710845, 0.9602382,
			  0.9488459, 0.9316537, 0.9118049, 0.8920435, 0.8719332,
			  0.8520256, 0.8280582, 0.8064085, 0.7767881, 0.7570597,
			  0.7382626, 0.7100251, 0.6846500, 0.6666913, 0.6372250,
			  0.6162248, 0.6007728, 0.5799103, 0.5674670, 0.5446148,
			  0.5352115, 0.5128809, 0.4932536, 0.5310397, 0.8566489,
			  0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
			  0.0000000, 0.0000000, 0.0000000};
  double   Foneinit[98]={ 0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
			  0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
			  0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
			  1.4819183, 1.7086354, 1.6958492, 1.6172935, 1.6301320,
			  1.5719706, 1.5459771, 1.5377471, 1.5008864, 1.4924121,
			  1.4720882, 1.4371741, 1.3990080, 1.3879193, 1.4030601,
			  1.3768673, 1.3493533, 1.3547127, 1.3275831, 1.3167892,
			  1.3035913, 1.2968298, 1.2801558, 1.2650299, 1.2557997,
			  1.2325822, 1.2210644, 1.1935984, 1.1746194, 1.1510350,
			  1.1358515, 1.1205584, 1.1010553, 1.0903869, 1.0731295,
			  1.0578678, 1.0438409, 1.0377911, 1.0253277, 1.0103551,
			  1.0042409, 0.9937978, 0.9858117, 0.9770346, 0.9724492,
			  0.9656686, 0.9606671, 0.9525813, 0.9488522, 0.9417335,
			  0.9399430, 0.9323438, 0.9281269, 0.9244171, 0.9237418,
			  0.9174354, 0.9177181, 0.9120840, 0.9047825, 0.9065579,
			  0.9034142, 0.8992961, 0.9011586, 0.9036470, 0.8954964,
			  0.8898208, 0.8911991, 0.8854824, 0.8888282, 0.8868449,
			  0.9004632, 0.8981572, 0.9096183, 0.9046990, 1.7454215,
			  0.0000000, 0.0000000, 0.0000000, 0.0000000, 0.0000000,
			  0.0000000, 0.0000000, 0.0000000};
  // eninit in GeV
  double     eninit[98]={ 0.6000000, 0.6131313, 0.6262626, 0.6393939, 0.6525252,
			  0.6656566, 0.6787879, 0.6919192, 0.7050505, 0.7181818,
			  0.7313131, 0.7444444, 0.7575758, 0.7707071, 0.7838384,
			  0.7969697, 0.8101010, 0.8232324, 0.8363636, 0.8494949,
			  0.8626263, 0.8757576, 0.8888889, 0.9020202, 0.9151515,
			  0.9282829, 0.9414141, 0.9545454, 0.9676768, 0.9808081,
			  0.9939394, 1.0070707, 1.0202020, 1.0333333, 1.0464647,
			  1.0595959, 1.0727273, 1.0858586, 1.0989898, 1.1121212,
			  1.1252525, 1.1383839, 1.1515151, 1.1646465, 1.1777778,
			  1.1909091, 1.2040404, 1.2171717, 1.2303030, 1.2434343,
			  1.2565657, 1.2696970, 1.2828283, 1.2959596, 1.3090909,
			  1.3222222, 1.3353535, 1.3484849, 1.3616161, 1.3747475,
			  1.3878788, 1.4010102, 1.4141414, 1.4272727, 1.4404041,
			  1.4535353, 1.4666667, 1.4797980, 1.4929293, 1.5060606,
			  1.5191919, 1.5323232, 1.5454545, 1.5585859, 1.5717171,
			  1.5848485, 1.5979798, 1.6111112, 1.6242424, 1.6373737,
			  1.6505051, 1.6636363, 1.6767677, 1.6898990, 1.7030303,
			  1.7161616, 1.7292930, 1.7424242, 1.7555555, 1.7686869,
			  1.7818182, 1.7949495, 1.8080808, 1.8212122, 1.8343434,
			  1.8474747, 1.8606061, 1.8737373};          
  // ensigma in GeV2
  double   ensigma[100]={ 0.2916000, 0.3206586, 0.3497172, 0.3787757, 0.4078344,
			  0.4368929, 0.4659515, 0.4950101, 0.5240687, 0.5531273,
			  0.5821859, 0.6112444, 0.6403030, 0.6693616, 0.6984202,
			  0.7274788, 0.7565374, 0.7855960, 0.8146545, 0.8437131,
			  0.8727717, 0.9018303, 0.9308889, 0.9599475, 0.9890060,
			  1.0180646, 1.0471232, 1.0761818, 1.1052403, 1.1342990,
			  1.1633576, 1.1924162, 1.2214748, 1.2505333, 1.2795919,
			  1.3086505, 1.3377091, 1.3667676, 1.3958262, 1.4248848,
			  1.4539435, 1.4830021, 1.5120606, 1.5411192, 1.5701778,
			  1.5992364, 1.6282949, 1.6573535, 1.6864121, 1.7154707,
			  1.7445292, 1.7735878, 1.8026465, 1.8317051, 1.8607637,
			  1.8898222, 1.9188808, 1.9479394, 1.9769980, 2.0060565,
			  2.0351152, 2.0641737, 2.0932324, 2.1222908, 2.1513495,
			  2.1804080, 2.2094667, 2.2385252, 2.2675838, 2.2966425,
			  2.3257010, 2.3547597, 2.3838181, 2.4128768, 2.4419353,
			  2.4709940, 2.5000525, 2.5291111, 2.5581696, 2.5872283,
			  2.6162868, 2.6453454, 2.6744041, 2.7034626, 2.7325213,
			  2.7615798, 2.7906384, 2.8196969, 2.8487556, 2.8778141,
			  2.9068727, 2.9359312, 2.9649899, 2.9940486, 3.0231071,
			  3.0521657, 3.0812242, 3.1102829, 3.1393414, 3.1684000};
  double    Fsigma[100]={ 2.0261996, 2.2349865, 2.4839740, 2.7840748, 3.1488798,
			  3.5936222, 4.1301847, 4.7517977, 5.3984838, 5.9147439,
			  6.0864558, 5.8283591, 5.2841811, 4.6615186, 4.0839195,
			  3.5914702, 3.1841860, 2.8494759, 2.5732665, 2.3434010,
			  2.1502059, 1.9862038, 1.8456544, 1.7241427, 1.6182493,
			  1.5253036, 1.4432002, 1.3702650, 1.3051554, 1.2467849,
			  1.1942677, 1.1468738, 1.1039963, 1.0651271, 1.0298390,
			  0.9977714, 0.9686196, 0.9421255, 0.9180685, 0.8962603,
			  0.8765374, 0.8587573, 0.8427927, 0.8285285, 0.8158574,
			  0.8046767, 0.7948853, 0.7863811, 0.7790571, 0.7728010,
			  0.7674922, 0.7630011, 0.7591889, 0.7559078, 0.7530031,
			  0.7503147, 0.7476809, 0.7449428, 0.7419487, 0.7385587,
			  0.7346500, 0.7301207, 0.7248930, 0.7189151, 0.7121620,
			  0.7046344, 0.6963565, 0.6873729, 0.6777444, 0.6675445,
			  0.6568548, 0.6457604, 0.6343476, 0.6227004, 0.6108983,
			  0.5990148, 0.5871165, 0.5752623, 0.5635037, 0.5518846,
			  0.5404415, 0.5292045, 0.5181981, 0.5074410, 0.4969472,
			  0.4867267, 0.4767860, 0.4671288, 0.4577557, 0.4486661,
			  0.4398569, 0.4313242, 0.4230627, 0.4150662, 0.4073282,
			  0.3998415, 0.3925985, 0.3855914, 0.3788125, 0.3722538};
  // set up the interpolators
  _Fomega  = make_InterpolatorPtr( 98,Fomegainit,1.0,eninit, GeV, 1);
  _Fthreec = make_InterpolatorPtr( 98,Fthreeinit,1.0,eninit, GeV, 1);
  _Fonec   = make_InterpolatorPtr( 98,Foneinit  ,1.0,eninit, GeV, 1);
  _Fsigma  = make_InterpolatorPtr(100,Fsigma    ,1.0,ensigma,GeV2,1);
  // initialise the calculation of the a_1 width
  _initializea1=false;
  // in GeV2
  double  a1q2in[200]={0,15788.6,31577.3,47365.9,63154.6,78943.2,94731.9,110521,
		       126309,142098,157886,173675,189464,205252,221041,236830,252618,
		       268407,284196,299984,315773,331562,347350,363139,378927,394716,
		       410505,426293,442082,457871,473659,489448,505237,521025,536814,
		       552603,568391,584180,599969,615757,631546,647334,663123,678912,
		       694700,710489,726278,742066,757855,773644,789432,805221,821010,
		       836798,852587,868375,884164,899953,915741,931530,947319,963107,
		       978896,994685,1.01047e+06,1.02626e+06,1.04205e+06,1.05784e+06,
		       1.07363e+06,1.08942e+06,1.10521e+06,1.12099e+06,1.13678e+06,
		       1.15257e+06,1.16836e+06,1.18415e+06,1.19994e+06,1.21573e+06,
		       1.23151e+06,1.2473e+06,1.26309e+06,1.27888e+06,1.29467e+06,
		       1.31046e+06,1.32625e+06,1.34203e+06,1.35782e+06,1.37361e+06,
		       1.3894e+06,1.40519e+06,1.42098e+06,1.43677e+06,1.45256e+06,
		       1.46834e+06,1.48413e+06,1.49992e+06,1.51571e+06,1.5315e+06,
		       1.54729e+06,1.56308e+06,1.57886e+06,1.59465e+06,1.61044e+06,
		       1.62623e+06,1.64202e+06,1.65781e+06,1.6736e+06,1.68939e+06,
		       1.70517e+06,1.72096e+06,1.73675e+06,1.75254e+06,1.76833e+06,
		       1.78412e+06,1.79991e+06,1.81569e+06,1.83148e+06,1.84727e+06,
		       1.86306e+06,1.87885e+06,1.89464e+06,1.91043e+06,1.92621e+06,
		       1.942e+06,1.95779e+06,1.97358e+06,1.98937e+06,2.00516e+06,
		       2.02095e+06,2.03674e+06,2.05252e+06,2.06831e+06,2.0841e+06,
		       2.09989e+06,2.11568e+06,2.13147e+06,2.14726e+06,2.16304e+06,
		       2.17883e+06,2.19462e+06,2.21041e+06,2.2262e+06,2.24199e+06,
		       2.25778e+06,2.27356e+06,2.28935e+06,2.30514e+06,2.32093e+06,
		       2.33672e+06,2.35251e+06,2.3683e+06,2.38409e+06,2.39987e+06,
		       2.41566e+06,2.43145e+06,2.44724e+06,2.46303e+06,2.47882e+06,
		       2.49461e+06,2.51039e+06,2.52618e+06,2.54197e+06,2.55776e+06,
		       2.57355e+06,2.58934e+06,2.60513e+06,2.62092e+06,2.6367e+06,
		       2.65249e+06,2.66828e+06,2.68407e+06,2.69986e+06,2.71565e+06,
		       2.73144e+06,2.74722e+06,2.76301e+06,2.7788e+06,2.79459e+06,
		       2.81038e+06,2.82617e+06,2.84196e+06,2.85774e+06,2.87353e+06,
		       2.88932e+06,2.90511e+06,2.9209e+06,2.93669e+06,2.95248e+06,
		       2.96827e+06,2.98405e+06,2.99984e+06,3.01563e+06,3.03142e+06,
		       3.04721e+06,3.063e+06,3.07879e+06,3.09457e+06,3.11036e+06,
		       3.12615e+06,3.14194e+06};
  // in GeV
  double a1widthin[200]={0,0,0,0,0,0,0,0,
			 0,0,0,0,0.000634625,0.00686721,0.026178,0.066329,
			 0.134996,0.239698,0.387813,0.586641,0.843471,1.16567,
			 1.56076,2.03654,2.60115,3.26324,4.03202,4.91749,
			 5.93053,7.08313,8.38858,9.86176,11.5194,13.3805,
			 15.4667,17.8029,20.4175,23.3438,26.6202,30.2917,
			 34.4108,39.0384,44.2457,50.1143,56.7369,64.2147,
			 72.6566,82.1666,92.8329,104.708,117.786,131.981,
			 147.124,162.974,179.244,195.64,211.904,227.818,
			 243.223,257.991,272.06,285.392,297.971,309.8,
			 320.894,331.278,340.979,350.03,358.463,366.31,
			 373.608,380.386,386.677,392.511,397.945,402.935,
			 407.563,411.841,415.79,419.433,422.766,425.853,
			 428.695,431.302,433.715,435.883,437.887,439.716,
			 441.426,442.947,444.326,445.575,446.65,447.666,
			 448.578,449.395,450.123,450.821,451.343,451.847,
			 452.283,452.859,452.987,453.266,453.496,453.686,
			 453.839,453.958,454.05,454.113,454.149,454.16,
			 454.154,454.13,454.091,454.037,453.966,453.9,
			 453.814,453.724,453.628,453.528,453.417,453.314,
			 453.206,453.1,452.995,452.891,452.79,452.697,
			 452.598,452.509,452.423,452.343,452.269,452.201,
			 452.141,452.086,452.039,452.004,451.966,451.941,
			 451.926,451.888,451.919,451.928,451.945,451.971,
			 452.006,452.05,452.102,452.163,452.234,452.421,
			 452.401,452.498,452.605,452.718,452.84,452.971,
			 453.111,453.261,453.417,453.583,453.756,453.937,
			 454.126,454.324,454.529,455.023,454.964,455.719,
			 455.428,455.671,455.921,456.179,456.444,456.695,
			 456.996,457.276,457.57,457.867,458.171,458.478,
			 458.793,459.115,459.442,459.777,460.115,460.458,
			 460.809,461.161,461.52,461.884,462.253,462.626,
			 463.004,463.832,463.778,464.166};
  vector<double> tmp1(a1widthin,a1widthin+200);
  _a1runwidth.clear();
  std::transform(tmp1.begin(), tmp1.end(),
		 back_inserter(_a1runwidth),
		 [](double x){return x*GeV;});
  
  vector<double> tmp2(a1q2in,a1q2in+200);
  _a1runq2.clear();
  std::transform(tmp2.begin(), tmp2.end(),
		 back_inserter(_a1runq2),
		 [](double x){return x*GeV2;});

  _maxmass=ZERO;
  _maxcalc=ZERO;
}

void FourPionNovosibirskCurrent::doinit() {
  WeakDecayCurrent::doinit();
  // pion masses
  _mpic=getParticleData(ParticleID::piplus)->mass();
  _mpic2=sqr(_mpic);
  _mpi0=getParticleData(ParticleID::pi0)->mass();
  _mpi02=sqr(_mpi0);
  if(!_localparameters) {
    _rhomass    = getParticleData(ParticleID::rhominus)->mass();
    _rhowidth   = getParticleData(ParticleID::rhominus)->width();
    _omegamass  = getParticleData(ParticleID::omega)->mass();
    _omegawidth = getParticleData(ParticleID::omega)->width();
    _sigmamass  = getParticleData(9000221)->mass();
    _sigmawidth = getParticleData(9000221)->width();
    _a1mass    = getParticleData(ParticleID::a_1minus)->mass();
    _a1width   = getParticleData(ParticleID::a_1minus)->width();
  }
  // calculate the constants for the a_1 form factor
  _onedlam2 = 1./_lambda2;
  _a1massolam2 = _a1mass*_a1mass*_onedlam2;
  // parameter for the sigma breit-wigner
  _psigma.push_back(Kinematics::pstarTwoBodyDecay(_sigmamass,_mpi0,_mpi0));
  _psigma.push_back(Kinematics::pstarTwoBodyDecay(_sigmamass,_mpic,_mpic));
  // parameters for the rho breit wigner
  _prho=Kinematics::pstarTwoBodyDecay(_rhomass,_mpic,_mpic);
  _hm2 = hFunction(_rhomass);
  _dhdq2m2=dhdq2Parameter();
  _rhoD=DParameter();
  // convert the magnitude and phase of z into a phase
  _zsigma = _zmag*(cos(_zphase)+Complex(0.,1.)*sin(_zphase));
  // initialize the a_1 width
  inita1width(-1);
}

void FourPionNovosibirskCurrent::doinitrun() {
  // set up the running a_1 width
  inita1width(0);
  WeakDecayCurrent::doinitrun();
}

void FourPionNovosibirskCurrent::persistentOutput(PersistentOStream & os) const {
  os << _a1runinter << _Fomega << _Fthreec << _Fonec << _Fsigma
     << ounit(_rhomass,GeV) << ounit(_a1mass,GeV) << ounit(_omegamass,GeV) 
     << ounit(_sigmamass,GeV) << ounit(_rhowidth,GeV) << ounit(_a1width,GeV)
     << ounit(_omegawidth,GeV) << ounit(_sigmawidth,GeV) 
     << _zsigma << ounit(_lambda2,GeV2)
     << _initializea1 << _localparameters 
     << ounit(_a1runwidth,GeV) << ounit(_a1runq2,GeV2) << ounit(_onedlam2,1/GeV2) 
     << _a1massolam2 << ounit(_psigma,GeV) << ounit(_mpic,GeV) << ounit(_mpi0,GeV)
     << ounit(_aomega,1/GeV) << ounit(_athreec,1/GeV) << ounit(_aonec,1/GeV) 
     << _bomega << _bthreec << _bonec 
     << _comega << _cthreec <<_conec << _omegaparam 
     << ounit(_intwidth,GeV) << ounit(_intmass,GeV)
     << ounit(_mpic2,GeV2) << ounit(_mpi02,GeV2) << ounit(_hm2,GeV2) << _dhdq2m2 
     << ounit(_prho,GeV) << ounit(_rhoD,GeV2) << _zmag << _zphase
     << ounit(_maxmass,GeV) << ounit(_maxcalc,GeV);
}

void FourPionNovosibirskCurrent::persistentInput(PersistentIStream & is, int) {
  is >> _a1runinter >> _Fomega >> _Fthreec >> _Fonec >> _Fsigma
     >> iunit(_rhomass,GeV) >> iunit(_a1mass,GeV) >> iunit(_omegamass,GeV) 
     >> iunit(_sigmamass,GeV) >> iunit(_rhowidth,GeV) >> iunit(_a1width,GeV)
     >> iunit(_omegawidth,GeV) >> iunit(_sigmawidth,GeV) 
     >> _zsigma >> iunit(_lambda2,GeV2)
     >> _initializea1 >> _localparameters 
     >> iunit(_a1runwidth,GeV) >> iunit(_a1runq2,GeV2) >> iunit(_onedlam2,1/GeV2)
     >> _a1massolam2 >> iunit(_psigma,GeV) >> iunit(_mpic,GeV) >> iunit(_mpi0,GeV)
     >> iunit(_aomega,1/GeV) >> iunit(_athreec,1/GeV) >> iunit(_aonec,1/GeV) 
     >> _bomega >> _bthreec >> _bonec 
     >> _comega >> _cthreec >>_conec >> _omegaparam 
     >> iunit(_intwidth,GeV) >> iunit(_intmass,GeV)
     >> iunit(_mpic2,GeV2) >> iunit(_mpi02,GeV2)>> iunit(_hm2,GeV2) >> _dhdq2m2
     >> iunit(_prho,GeV) >> iunit(_rhoD,GeV2) >> _zmag >> _zphase
     >> iunit(_maxmass,GeV) >> iunit(_maxcalc,GeV);
}

// Definition of the static class description member.

void FourPionNovosibirskCurrent::Init() {

  static ClassDocumentation<FourPionNovosibirskCurrent> documentation
    ("The FourPionNovosibirskCurrent class performs the decay"
     " of the tau to four pions using currents based on the the"
     " Novosibirsk e+e- data",
     "The decay of the tau to four pions uses currents based on \\cite{Bondar:2002mw}.",
     "%\\cite{Bondar:2002mw}\n"
     "\\bibitem{Bondar:2002mw}\n"
     "  A.~E.~Bondar, S.~I.~Eidelman, A.~I.~Milstein, T.~Pierzchala, N.~I.~Root, Z.~Was and M.~Worek,\n"
     "   ``Novosibirsk hadronic currents for tau --> 4pi channels of tau decay\n"
     "  %library TAUOLA,''\n"
     "  Comput.\\ Phys.\\ Commun.\\  {\\bf 146}, 139 (2002)\n"
     "  [arXiv:hep-ph/0201149].\n"
     "  %%CITATION = CPHCB,146,139;%%\n"
     );

  static Parameter<FourPionNovosibirskCurrent,Energy> interfacerhoMass
    ("rhoMass",
     "The local value of the rho mass",
     &FourPionNovosibirskCurrent::_rhomass, GeV,0.7761*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static Parameter<FourPionNovosibirskCurrent,Energy> interfacea1mass
    ("a1Mass",
     "The local value of the square of the a_1 mass",
     &FourPionNovosibirskCurrent::_a1mass, GeV, 1.2300*GeV, 0.5*GeV, 10.0*GeV,
     false, false, true);

  static Parameter<FourPionNovosibirskCurrent,Energy> interfaceSigmaMass
    ("sigmaMass",
     "The local value of the sigma mass",
     &FourPionNovosibirskCurrent::_sigmamass, GeV, 0.8*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static Parameter<FourPionNovosibirskCurrent,Energy> interfaceOmegaMass
    ("omegaMass",
     "The local value of the omega mass",
     &FourPionNovosibirskCurrent::_omegamass, GeV, 0.7820*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static Parameter<FourPionNovosibirskCurrent,Energy> interfacerhoWidth
    ("rhoWidth",
     "The local value of the rho width",
     &FourPionNovosibirskCurrent::_rhowidth, GeV,0.1445*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static Parameter<FourPionNovosibirskCurrent,Energy> interfacea1width
    ("a1Width",
     "The local value of the square of the a_1 width",
     &FourPionNovosibirskCurrent::_a1width, GeV, 0.45*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static Parameter<FourPionNovosibirskCurrent,Energy> interfaceSigmaWidth
    ("sigmaWidth",
     "The local value of the sigma width",
     &FourPionNovosibirskCurrent::_sigmawidth, GeV, 0.8*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static Parameter<FourPionNovosibirskCurrent,Energy> interfaceOmegaWidth
    ("omegaWidth",
     "The local value of the omega width",
     &FourPionNovosibirskCurrent::_omegawidth, GeV, 0.00841*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static Parameter<FourPionNovosibirskCurrent,Energy> interfaceIntegrationMass
    ("IntegrationMass",
     "Mass of the pseudoresonance used to improve integration effciency",
     &FourPionNovosibirskCurrent::_intmass, GeV, 1.4*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static Parameter<FourPionNovosibirskCurrent,Energy> interfaceIntegrationWidth
    ("IntegrationWidth",
     "Width of the pseudoresonance used to improve integration effciency",
     &FourPionNovosibirskCurrent::_intwidth, GeV, 0.5*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static Parameter<FourPionNovosibirskCurrent,double> interfaceSigmaMagnitude
    ("SigmaMagnitude",
     "magnitude of the relative sigma coupling",
     &FourPionNovosibirskCurrent::_zmag, 1.3998721, 0.0, 10.0e20,
     false, false, true);

  static Parameter<FourPionNovosibirskCurrent,double> interfaceSigmaPhase
    ("SigmaPhase",
     "phase of the relative sigma coupling",
     &FourPionNovosibirskCurrent::_zphase, 0.43585036, 0.0, Constants::twopi,
     false, false, true);

  static Parameter<FourPionNovosibirskCurrent,Energy2> interfaceLambda2
    ("Lambda2",
     "The value of the mass scale squared to use in the form-factor",
     &FourPionNovosibirskCurrent::_lambda2, GeV2, 1.2*GeV2, 0.0001*GeV2, 10.0*GeV2,
     false, false, true);

  static Switch<FourPionNovosibirskCurrent,bool> interfaceLocalParameters
    ("LocalParameters",
     "Use local values of the intermediate resonances masses and widths",
     &FourPionNovosibirskCurrent::_localparameters, true, false, false);
  static SwitchOption interfaceLocalParametersLocal
    (interfaceLocalParameters,
     "Local",
     "Use the local values",
     true);
  static SwitchOption interfaceLocalParametersDefault
    (interfaceLocalParameters,
     "ParticleData",
     "Use the values from the particleData objects",
     false);

  static Switch<FourPionNovosibirskCurrent,bool> interfaceInitializea1
    ("Initializea1",
     "Initialise the calculation of the a_1 running width",
     &FourPionNovosibirskCurrent::_initializea1, false, false, false);
  static SwitchOption interfaceInitializea1Initialization
    (interfaceInitializea1,
     "Yes",
     "Initialize the calculation",
     true);
  static SwitchOption interfaceInitializea1NoInitialization
    (interfaceInitializea1,
     "No",
     "Use the default values",
     false);
  
  static ParVector<FourPionNovosibirskCurrent,Energy> interfacea1RunningWidth
    ("a1RunningWidth",
     "The values of the a_1 width for interpolation to giving the running width.",
     &FourPionNovosibirskCurrent::_a1runwidth, GeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static ParVector<FourPionNovosibirskCurrent,Energy2> interfacea1RunningQ2
    ("a1RunningQ2",
     "The values of the q^2 for interpolation to giving the running width.",
     &FourPionNovosibirskCurrent::_a1runq2, GeV2, -1, 1.0*GeV2, ZERO, 10.0*GeV2,
     false, false, true);

}

// initialisation of the a_1 running width 
void FourPionNovosibirskCurrent::inita1width(int iopt) {
  // set up the interpolator
  if(iopt==0||!_initializea1) {
    _a1runinter = make_InterpolatorPtr(_a1runwidth,_a1runq2,3);
    return;
  }
  _maxcalc=_maxmass;
  if(_maxmass==ZERO) return;
  // parameters for the table of values
  Energy2 step(sqr(_maxmass)/200.);
  // function to be integrated to give the matrix element
  // integrator to perform the integral
  // weights for the integration channels
  vector<double> inweights;
  inweights.push_back(0.3);inweights.push_back(0.3);inweights.push_back(0.3);
  vector<double> inpower(3, 0.0);
  // types of integration channels
  vector<int> intype;
  intype.push_back(2);intype.push_back(3);intype.push_back(1);
  // masses for the integration channels
  vector<Energy> inmass(2,_rhomass);inmass.push_back(_sigmamass);
  // widths for the integration channels
  vector<Energy> inwidth(2,_rhowidth);inwidth.push_back(_sigmawidth);
  ThreeBodyAllOnCalculator<FourPionNovosibirskCurrent> 
    widthgen1(inweights,intype,inmass,inwidth,inpower,*this,0,_mpi0,_mpic,_mpic); 
  ThreeBodyAllOnCalculator<FourPionNovosibirskCurrent>
    widthgen2(inweights,intype,inmass,inwidth,inpower,*this,1,_mpi0,_mpi0,_mpi0); 
  // normalisation constant to give physical width if on shell
  double a1const(_a1width/(widthgen1.partialWidth(sqr(_a1mass))+
			   widthgen2.partialWidth(sqr(_a1mass))));
  // loop to give the values
  Energy2 moff2(ZERO);
  _a1runwidth.clear();_a1runq2.clear();
  for(;moff2<=sqr(_maxmass);moff2+=step) {
    Energy total = a1const*(widthgen1.partialWidth(moff2)+widthgen2.partialWidth(moff2));
    _a1runwidth.push_back(total);
    _a1runq2.push_back(moff2);
  }
}

// complete the construction of the decay mode for integration
bool FourPionNovosibirskCurrent::createMode(int icharge, unsigned int imode,
					    DecayPhaseSpaceModePtr mode,
					    unsigned int iloc,unsigned int ires,
					    DecayPhaseSpaceChannelPtr phase,Energy upp)
{
  // check the charge
  if(abs(icharge)!=3) return false;
  // check that the modes are kinematical allowed
  Energy min(ZERO);
  if(imode==0) {
    min=   getParticleData(ParticleID::piplus)->mass()
      +3.*getParticleData(ParticleID::pi0)->mass();
  }
  else {
    min=3.*getParticleData(ParticleID::piplus)->mass()
      +getParticleData(ParticleID::pi0)->mass();
  }
  if(min>upp) return false;
  _maxmass=max(upp,_maxmass);
  // intermediates for the channels
  tPDPtr omega(getParticleData(ParticleID::omega)),rhop,rhom,
    rho0(getParticleData(ParticleID::rho0)),a1m,a10(getParticleData(ParticleID::a_10)),
    sigma(getParticleData(9000221)),rhot;
  if(icharge==3) {
    rhop = getParticleData(ParticleID::rhominus);
    rhom = getParticleData(ParticleID::rhoplus);
    a1m  = getParticleData(ParticleID::a_1plus);
    rhot = getParticleData(24);
  }
  else {
    rhop = getParticleData(ParticleID::rhoplus);
    rhom = getParticleData(ParticleID::rhominus);
    a1m  = getParticleData(ParticleID::a_1minus);
    rhot = getParticleData(-24);
  }
  DecayPhaseSpaceChannelPtr newchannel;
  // the omega channels for the three charged pion mode
  // first  channel two channels with rho0
  if(imode==1) {
    newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc);
    newchannel->addIntermediate(omega   ,0,0.0,-ires-2,iloc+3);
    newchannel->addIntermediate(rho0    ,0,0.0, iloc+1,iloc+2);
    mode->addChannel(newchannel);
    newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc+1);
    newchannel->addIntermediate(omega   ,0,0.0,-ires-2,iloc+3);
    newchannel->addIntermediate(rho0    ,0,0.0, iloc,iloc+2);
    mode->addChannel(newchannel);
    // second two channels with rho -
    newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc);
    newchannel->addIntermediate(omega   ,0,0.0,-ires-2,iloc+2);
    newchannel->addIntermediate(rhom    ,0,0.0, iloc+1,iloc+3);
    mode->addChannel(newchannel);
    newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc+1);
    newchannel->addIntermediate(omega   ,0,0.0,-ires-2,iloc+2);
    newchannel->addIntermediate(rhom    ,0,0.0, iloc,iloc+3);
    mode->addChannel(newchannel);
    // third two channels with rho +
    newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc);
    newchannel->addIntermediate(omega   ,0,0.0,-ires-2,iloc+1);
    newchannel->addIntermediate(rhop    ,0,0.0, iloc+2,iloc+3);
    mode->addChannel(newchannel);
    newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc+1);
    newchannel->addIntermediate(omega   ,0,0.0,-ires-2,iloc);
    newchannel->addIntermediate(rhop    ,0,0.0, iloc+2,iloc+3);
    mode->addChannel(newchannel);
    //  a_1 channels with rhos
    newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc+3);
    newchannel->addIntermediate(a1m     ,0,0.0,-ires-2,iloc);
    newchannel->addIntermediate(rho0    ,0,0.0, iloc+1,iloc+2);
    mode->addChannel(newchannel);
    newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc+3);
    newchannel->addIntermediate(a1m     ,0,0.0,-ires-2,iloc+1);
    newchannel->addIntermediate(rho0    ,0,0.0, iloc,iloc+2);
    mode->addChannel(newchannel);
    // neutral a_1 channels with rhos
    newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc);
    newchannel->addIntermediate(a10     ,0,0.0,-ires-2,iloc+2);
    newchannel->addIntermediate(rhom    ,0,0.0, iloc+1,iloc+3);
    mode->addChannel(newchannel);
    newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc+1);
    newchannel->addIntermediate(a10     ,0,0.0,-ires-2,iloc+2);
    newchannel->addIntermediate(rhom    ,0,0.0, iloc,iloc+3);
    mode->addChannel(newchannel);
    newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc);
    newchannel->addIntermediate(a10     ,0,0.0,-ires-2,iloc+1);
    newchannel->addIntermediate(rhop    ,0,0.0, iloc+2,iloc+3);
    mode->addChannel(newchannel);
    newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc+1);
    newchannel->addIntermediate(a10     ,0,0.0,-ires-2,iloc);
    newchannel->addIntermediate(rhop    ,0,0.0, iloc+2,iloc+3);
    mode->addChannel(newchannel);
    //  a_1 channels with sigmas
    if(sigma) {
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc+3);
      newchannel->addIntermediate(a1m     ,0,0.0,-ires-2,iloc);
      newchannel->addIntermediate(sigma   ,0,0.0, iloc+1,iloc+2);
      mode->addChannel(newchannel);
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc+3);
      newchannel->addIntermediate(a1m     ,0,0.0,-ires-2,iloc+1);
      newchannel->addIntermediate(sigma   ,0,0.0, iloc,iloc+2);
      mode->addChannel(newchannel);
      // neutral a_1 channels with sigma
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc);
      newchannel->addIntermediate(a10     ,0,0.0,-ires-2,iloc+3);
      newchannel->addIntermediate(sigma   ,0,0.0, iloc+1,iloc+2);
      mode->addChannel(newchannel);
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc+1);
      newchannel->addIntermediate(a10     ,0,0.0,-ires-2,iloc+3);
      newchannel->addIntermediate(sigma   ,0,0.0, iloc,iloc+2);
      mode->addChannel(newchannel);
    }
  }
  else {
    // channels with an a1- and a rho -
    newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc+1);
    newchannel->addIntermediate(a1m     ,0,0.0,-ires-2,iloc+2);
    newchannel->addIntermediate(rhom    ,0,0.0, iloc+3,iloc);
    mode->addChannel(newchannel);
    newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc+1);
    newchannel->addIntermediate(a1m     ,0,0.0,-ires-2,iloc+3);
    newchannel->addIntermediate(rhom    ,0,0.0, iloc+2,iloc);
    mode->addChannel(newchannel);
    newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc+2);
    newchannel->addIntermediate(a1m     ,0,0.0,-ires-2,iloc+1);
    newchannel->addIntermediate(rhom    ,0,0.0, iloc+3,iloc);
    mode->addChannel(newchannel);
    newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc+2);
    newchannel->addIntermediate(a1m     ,0,0.0,-ires-2,iloc+3);
    newchannel->addIntermediate(rhom    ,0,0.0, iloc+1,iloc);
    mode->addChannel(newchannel);
    newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc+3);
    newchannel->addIntermediate(a1m     ,0,0.0,-ires-2,iloc+1);
    newchannel->addIntermediate(rhom    ,0,0.0, iloc+2,iloc);
    mode->addChannel(newchannel);
    newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc+3);
    newchannel->addIntermediate(a1m     ,0,0.0,-ires-2,iloc+2);
    newchannel->addIntermediate(rhom    ,0,0.0, iloc+1,iloc);
    mode->addChannel(newchannel);
    // channels with a sigma and a10
    if(sigma ) {
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc);
      newchannel->addIntermediate(a10     ,0,0.0,-ires-2,iloc+1);
      newchannel->addIntermediate(sigma   ,0,0.0, iloc+2,iloc+3);
      mode->addChannel(newchannel);
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc);
      newchannel->addIntermediate(a10     ,0,0.0,-ires-2,iloc+2);
      newchannel->addIntermediate(sigma   ,0,0.0, iloc+1,iloc+3);
      mode->addChannel(newchannel);
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc);
      newchannel->addIntermediate(a10     ,0,0.0,-ires-2,iloc+3);
      newchannel->addIntermediate(sigma   ,0,0.0, iloc+1,iloc+2);
      mode->addChannel(newchannel);
      // channels with a1- and sigma
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc+1);
      newchannel->addIntermediate(a1m     ,0,0.0,-ires-2,iloc);
      newchannel->addIntermediate(sigma   ,0,0.0, iloc+2,iloc+3);
      mode->addChannel(newchannel);
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc+2);
      newchannel->addIntermediate(a1m     ,0,0.0,-ires-2,iloc);
      newchannel->addIntermediate(sigma   ,0,0.0, iloc+1,iloc+3);
      mode->addChannel(newchannel);
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(rhot    ,0,0.0,-ires-1,iloc+3);
      newchannel->addIntermediate(a1m     ,0,0.0,-ires-2,iloc);
      newchannel->addIntermediate(sigma   ,0,0.0, iloc+1,iloc+2);
      mode->addChannel(newchannel);
    }
  }
  // reset the parameters of the dummy resonance used for integration
  mode->resetIntermediate(rhot,_intmass,_intwidth);
  // reset the parameters of the resonances if using local values
  if(_localparameters) {
    mode->resetIntermediate(rhom,_rhomass,_rhowidth);
    mode->resetIntermediate(rhop,_rhomass,_rhowidth);
    mode->resetIntermediate(rho0,_rhomass,_rhowidth);
    mode->resetIntermediate(omega,_omegamass,_omegawidth);
    if(sigma) mode->resetIntermediate(sigma,_sigmamass,_sigmawidth);
  }
  // return if successful
  return true;
}

// the particles produced by the current
tPDVector FourPionNovosibirskCurrent::particles(int icharge, unsigned int imode,int,int) {
  if(abs(icharge)!=3) return tPDVector();
  tPDVector output(4);
  if(imode==1) {
    output[0]=getParticleData(ParticleID::piplus);
    output[1]=getParticleData(ParticleID::piplus);
    output[2]=getParticleData(ParticleID::piminus);
  }
  else {
    output[0]=getParticleData(ParticleID::piplus);
    output[1]=getParticleData(ParticleID::pi0);
    output[2]=getParticleData(ParticleID::pi0);
  }
  output[3]=getParticleData(ParticleID::pi0);
  if(icharge==-3) {
    for(unsigned int ix=0;ix<output.size();++ix) {
      if(output[ix]->CC()) output[ix]=output[ix]->CC();
    }
  }
  return output;
}

 
// the hadronic currents    
vector<LorentzPolarizationVectorE> 
FourPionNovosibirskCurrent::current(const int imode, const int ichan,
				    Energy & scale,const ParticleVector & decay,
				    DecayIntegrator::MEOption meopt) const {
  useMe();
  if(meopt==DecayIntegrator::Terminate) {
    for(unsigned int ix=0;ix<4;++ix)
      ScalarWaveFunction::constructSpinInfo(decay[ix],outgoing,true);
    return vector<LorentzPolarizationVectorE>(1,LorentzPolarizationVectorE());
  }
  LorentzVector<complex<InvEnergy> > output;
  double fact(1.);
  // the momenta of the particles
  Lorentz5Momentum q1(decay[0]->momentum()),q2(decay[2]->momentum()),
    q3(decay[1]->momentum()),q4(decay[3]->momentum());
  Lorentz5Momentum Q(q1+q2+q3+q4);Q.rescaleMass();
  scale = Q.mass();
  // decide which decay mode
  // three charged pions
  if(imode==1) {
    // momenta of the particles
    LorentzVector<complex<Energy5> > veca1rho,vecomega,veca1sig;
    if(ichan<0) {
      // a_1 rho current
      veca1rho = 
	t1(q1,q2,q3,q4)+t1(q3,q2,q1,q4)+t1(q1,q3,q2,q4)
	+t1(q3,q1,q2,q4)+t1(q4,q3,q1,q2)+t1(q4,q1,q3,q2);
      // a_1 sigma current
      veca1sig = 
	t2(q4,q3,q1,q2,1)+t2(q4,q1,q3,q2,1)
	-t2(q1,q4,q3,q2,1)-t2(q3,q4,q1,q2,1);
      // omega current
      vecomega = 
	t3(q1,q2,q3,q4)+t3(q3,q2,q1,q4)-t3(q1,q3,q2,q4)
	-t3(q3,q1,q2,q4)-t3(q1,q4,q3,q2)-t3(q3,q4,q1,q2);
    }
    else if(ichan== 0) vecomega = t3(q1,q4,q3,q2);
    else if(ichan== 1) vecomega = t3(q3,q4,q1,q2);
    else if(ichan== 2) vecomega = t3(q1,q2,q3,q4);
    else if(ichan== 3) vecomega = t3(q3,q2,q1,q4);
    else if(ichan== 4) vecomega = t3(q1,q3,q2,q4);
    else if(ichan== 5) vecomega = t3(q3,q1,q2,q4);
    else if(ichan== 6) veca1rho = t1(q4,q1,q3,q2);
    else if(ichan== 7) veca1rho = t1(q4,q3,q1,q2);
    else if(ichan== 8) veca1rho = t1(q1,q2,q3,q4);
    else if(ichan== 9) veca1rho = t1(q3,q2,q1,q4);
    else if(ichan==10) veca1rho = t1(q1,q3,q2,q4);
    else if(ichan==11) veca1rho = t1(q3,q1,q2,q4);
    else if(ichan==12) veca1sig = t2(q4,q1,q3,q2,1);
    else if(ichan==13) veca1sig = t2(q4,q3,q1,q2,1);
    else if(ichan==14) veca1sig = t2(q1,q4,q3,q2,1);
    else if(ichan==15) veca1sig = t2(q3,q4,q1,q2,1);
    // final manipulations
    veca1rho += veca1sig;
    LorentzVector<complex<InvEnergy> > 
      veca1rho1 = veca1rho * gFunction(Q.mass2(),1);
    LorentzVector<complex<InvEnergy> > 
      vecomega1 = vecomega * gFunction(Q.mass2(),2);
    output = vecomega1 + veca1rho1;
    // this is 1/sqrt(2) for identical particles
    fact *= 1./sqrt(2.);
  }
  else if(imode==0) {
    // momenta of the particles
    LorentzVector<complex<Energy5> > veca1rho,veca1sig;
    if(ichan<0) {
      // a_1 rho current
      veca1rho= t1(q2,q3,q1,q4)+t1(q2,q4,q1,q3)+t1(q3,q2,q1,q4)
	+t1(q3,q4,q1,q2)+t1(q4,q2,q1,q3)+t1(q4,q3,q1,q2);
      // a_1 sigma current
      veca1sig=
 	t2(q2,q1,q3,q4,0)+t2(q3,q1,q2,q4,0)+t2(q4,q1,q3,q2,0)
	-t2(q1,q2,q3,q4,0)-t2(q1,q3,q2,q4,0)-t2(q1,q4,q3,q2,0);
    }
    else if(ichan== 0) veca1rho = t1(q2,q3,q1,q4);
    else if(ichan== 1) veca1rho = t1(q2,q4,q1,q3);
    else if(ichan== 2) veca1rho = t1(q3,q2,q1,q4);
    else if(ichan== 3) veca1rho = t1(q3,q4,q1,q2);
    else if(ichan== 4) veca1rho = t1(q4,q2,q1,q3);
    else if(ichan== 5) veca1rho = t1(q4,q3,q1,q2);
    else if(ichan== 6) veca1sig = t2(q2,q1,q3,q4,0);
    else if(ichan== 7) veca1sig = t2(q3,q1,q2,q4,0);
    else if(ichan== 8) veca1sig = t2(q4,q1,q3,q2,0);
    else if(ichan== 9) veca1sig = t2(q1,q2,q3,q4,0);
    else if(ichan==10) veca1sig = t2(q1,q3,q2,q4,0);
    else if(ichan==11) veca1sig = t2(q1,q4,q3,q2,0);
    // add them up 
    output = (veca1rho + veca1sig) * gFunction(Q.mass2(),0);
    // this is sqrt(1/3!) for identical particles
    fact *= 1./sqrt(6.);
  }     
  else {
    throw DecayIntegratorError() << "Unknown decay mode in the " 
				 << "FourPionNovosibirskCurrent::"
				 << "hadronCurrent()" << Exception::abortnow;
  }  
  vector<LorentzPolarizationVectorE> temp(1, output * fact * Q.mass2()); 
  return temp;
}

bool FourPionNovosibirskCurrent::accept(vector<int> id) {
  bool allowed(false);
  // check four products
  if(id.size()!=4){return false;}
  int npiminus=0,npiplus=0,npi0=0;
  for(unsigned int ix=0;ix<id.size();++ix) {
    if(id[ix]==ParticleID:: piplus)      ++npiplus;
    else if(id[ix]==ParticleID::piminus) ++npiminus;
    else if(id[ix]==ParticleID::pi0)     ++npi0;
  }
  if(npiminus==2&&npiplus==1&&npi0==1)      allowed=true;
  else if(npiminus==1&&npi0==3)             allowed=true;
  else if(npiplus==2&&npiminus==1&&npi0==1) allowed=true;
  else if(npiplus==1&&npi0==3)              allowed=true;
  return allowed;
}

// the decay mode
unsigned int FourPionNovosibirskCurrent::decayMode(vector<int> idout) {
  unsigned int npi(0);
  for(unsigned int ix=0;ix<idout.size();++ix) {
    if(abs(idout[ix])==ParticleID::piplus) ++npi;
  }
  if(npi==3) return 1;
  return 0;
}


// output the information for the database
void FourPionNovosibirskCurrent::dataBaseOutput(ofstream & output,bool header,
						bool create) const {
  if(header) output << "update decayers set parameters=\"";
  if(create) output << "create Herwig::FourPionNovosibirskCurrent " 
		    << name() << " HwWeakCurrents.so\n";
  output << "newdef " << name() << ":rhoMass "    << _rhomass/GeV << "\n";
  output << "newdef " << name() << ":a1Mass  "    << _a1mass/GeV  << "\n";
  output << "newdef " << name() << ":sigmaMass  " << _sigmamass/GeV  << "\n";
  output << "newdef " << name() << ":omegaMass  " << _omegamass/GeV  << "\n";
  output << "newdef " << name() << ":rhoWidth "    << _rhowidth/GeV << "\n";
  output << "newdef " << name() << ":a1Width  "    << _a1width/GeV  << "\n";
  output << "newdef " << name() << ":sigmaWidth  " << _sigmawidth/GeV  << "\n";
  output << "newdef " << name() << ":omegaWidth  " << _omegawidth/GeV  << "\n";
  output << "newdef " << name() << ":IntegrationMass "  << _intmass/GeV  << "\n";
  output << "newdef " << name() << ":IntegrationWidth " << _intwidth/GeV  << "\n";
  output << "newdef " << name() << ":SigmaMagnitude "  <<  _zmag << "\n";
  output << "newdef " << name() << ":SigmaPhase " << _zphase  << "\n";
  output << "newdef " << name() << ":Lambda2 "  <<  _lambda2/GeV2 << "\n";
  output << "newdef " << name() << ":LocalParameters " <<  _localparameters << "\n";
  output << "newdef " << name() << ":Initializea1 " <<  _initializea1 << "\n";
  for(unsigned int ix=0;ix<_a1runwidth.size();++ix) {
    if(ix<200) output << "newdef ";
    else       output << "insert ";
    output << name() << ":a1RunningWidth " << ix << " " 
	   << _a1runwidth[ix]/GeV << "\n";
  }
  for(unsigned int ix=0;ix<_a1runq2.size();++ix) {
    if(ix<200) output << "newdef ";
    else       output << "insert ";
    output << name() << ":a1RunningQ2 " << ix << " " << _a1runq2[ix]/GeV2 << "\n";
  }
  WeakDecayCurrent::dataBaseOutput(output,false,false);
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}

double FourPionNovosibirskCurrent::
threeBodyMatrixElement(const int iopt, const Energy2 q2,
		       const Energy2 s3, const Energy2 s2, 
		       const Energy2 s1, const Energy,
		       const Energy, const Energy) const {
  unsigned int ix;
  // construct the momenta of the decay products
  Energy p1[5],p2[5],p3[5];
  Energy2 p1sq, p2sq, p3sq;
  Energy q(sqrt(q2));
  if(iopt==0) {
    p1[0] = 0.5*(q2+_mpi02-s1)/q; p1sq=p1[0]*p1[0]; p1[4]=sqrt(p1sq-_mpi02);
    p2[0] = 0.5*(q2+_mpic2-s2)/q; p2sq=p2[0]*p2[0]; p2[4]=sqrt(p2sq-_mpic2);
    p3[0] = 0.5*(q2+_mpic2-s3)/q; p3sq=p3[0]*p3[0]; p3[4]=sqrt(p3sq-_mpic2);
  }
  else {
    p1[0] = 0.5*(q2+_mpi02-s1)/q; p1sq=p1[0]*p1[0]; p1[4]=sqrt(p1sq-_mpi02);
    p2[0] = 0.5*(q2+_mpi02-s2)/q; p2sq=p2[0]*p2[0]; p2[4]=sqrt(p2sq-_mpi02);
    p3[0] = 0.5*(q2+_mpi02-s3)/q; p3sq=p3[0]*p3[0]; p3[4]=sqrt(p3sq-_mpi02);
  }
  // take momentum of 1 parallel to z axis
  p1[1]=ZERO;p1[2]=ZERO;p1[3]=p1[4];
  // construct 2 
  double cos2(0.5*(sqr(p1[4])+sqr(p2[4])-sqr(p3[4]))/p1[4]/p2[4]);
  p2[1] = p2[4]*sqrt(1.-sqr(cos2)); p2[2]=ZERO; p2[3]=-p2[4]*cos2;
  // construct 3
  double cos3(0.5*(sqr(p1[4])-sqr(p2[4])+sqr(p3[4]))/p1[4]/p3[4]);
  p3[1] =-p3[4]*sqrt(1.-sqr(cos3)); p3[2]=ZERO; p3[3]=-p3[4]*cos3;
  // pi+pi-pi0 term
  complex<Energy4> output(0.*sqr(MeV2));
  if(iopt==0) {
    // values for the different Breit-Wigner terms
    Complex rho1(2.365*rhoBreitWigner(s2)),
      rho2(2.365*rhoBreitWigner(s3)),
      sig1(sigmaBreitWigner(s1,1));
    // compute the vector
    complex<Energy2> term;
    for(ix=1;ix<4;++ix) { 
      term = (p1[0]*p2[ix]-p2[0]*p1[ix])*rho2+(p1[0]*p3[ix]-p3[0]*p1[ix])*rho1
	+_zsigma*q*p1[ix]*sig1;
      output+=term*conj(term);
    }
  }
  // pi0pi0pi0 term
  else if(iopt==1) {
    // values for the different Breit-Wigner terms
    Complex sig1(sigmaBreitWigner(s1,0)),
      sig2(sigmaBreitWigner(s2,0)),
      sig3(sigmaBreitWigner(s3,0));
    // compute the vector
    complex<Energy2> term;
    for(ix=1;ix<4;++ix) {
      term = _zsigma * q * (p1[ix]*sig1 + p2[ix]*sig2 + p3[ix]*sig3);
      output += term*conj(term);
    }
    output/=6.;
  }
  output *= a1FormFactor(q2);
  return output.real() / pow<4,1>(_rhomass);
}

Complex FourPionNovosibirskCurrent::sigmaBreitWigner(Energy2 q2,
						     unsigned int iopt) const {
  Energy q(sqrt(q2));
  Energy pcm = iopt==0 ? 
    Kinematics::pstarTwoBodyDecay(q,_mpi0,_mpi0) :
    Kinematics::pstarTwoBodyDecay(q,_mpic,_mpic);
  if(pcm<ZERO) pcm=ZERO;
  Energy  width(_sigmawidth*pcm/_psigma[iopt]);
  Energy2 msigma2 = sqr(_sigmamass);
  return msigma2/(q2-msigma2+Complex(0.,1.)*msigma2*width/q);
}

Complex FourPionNovosibirskCurrent::a1BreitWigner(Energy2 q2) const {
  Complex ii(0.,1.);
  Energy2 m2 = sqr(_a1mass);
  Energy q = sqrt(q2);
  return (m2/complex<Energy2>(q2 - m2 + ii*q*a1width(q2)));
}

Complex FourPionNovosibirskCurrent::omegaBreitWigner(Energy2 q2) const {
  Energy q(sqrt(q2));
  // calcluate the running width
  double diff((q-_omegamass)/GeV),temp(diff);
  double gomega(1.);
  Complex ii(0.,1.);
  if(q<=1.*GeV) {
    for(unsigned int ix=0;ix<6;++ix) {
      gomega +=temp*_omegaparam[ix];
      temp*=diff;
    }
  }
  else {
    gomega=_omegaparam[6]+q/GeV*(_omegaparam[7]+q/GeV*_omegaparam[8]
				 +q2/GeV2*_omegaparam[9]);
  }
  if(gomega<0.){gomega=0.;}
  Energy2 numer=_omegamass*_omegamass;
  complex<Energy2> denom=q2-_omegamass*_omegamass+ii*_omegamass*_omegawidth*gomega;
  return numer/denom;
}

Complex FourPionNovosibirskCurrent::rhoBreitWigner(Energy2 q2) const {
  Energy q(sqrt(q2));
  Energy2 grhom(8.*_prho*_prho*_prho/_rhomass);
  complex<Energy2> denom;
  Complex ii(0.,1.);
  if(q2<4.*_mpic2) {
    denom=q2-_rhomass*_rhomass
      -_rhowidth*_rhomass*(hFunction(q)-_hm2-(q2-_rhomass*_rhomass)*_dhdq2m2)/grhom;
  }
  else {
    Energy pcm(2.*Kinematics::pstarTwoBodyDecay(q,_mpic,_mpic));
    Energy2 grho(pcm*pcm*pcm/q);
    denom=q2-_rhomass*_rhomass
      -_rhowidth*_rhomass*(hFunction(q)-_hm2-(q2-_rhomass*_rhomass)*_dhdq2m2)/grhom
      +ii*_rhomass*_rhowidth*grho/grhom;
  }
  return _rhoD/denom;
}

LorentzVector<complex<Energy5> > 
FourPionNovosibirskCurrent::t1(Lorentz5Momentum & q1,Lorentz5Momentum & q2,
			       Lorentz5Momentum & q3,Lorentz5Momentum & q4) const {
  // momentum of the whole sysytem
  Lorentz5Momentum Q(q1+q2+q3+q4);Q.rescaleMass();
  // compute the virtuality of the a_1
  Lorentz5Momentum a1(q2+q3+q4);a1.rescaleMass();
  // compute the virtuality of the  rho
  Lorentz5Momentum rho(q3+q4);rho.rescaleMass();
  // compute the prefactor    
  Complex pre(-a1FormFactor(a1.mass2())*a1BreitWigner(a1.mass2())*
	      rhoBreitWigner(rho.mass2()));
  // dot products we need
  Energy2 QdQmq1(Q*a1);
  complex<Energy4> consta(QdQmq1*(a1*q3)), constb(QdQmq1*(a1*q4)),
    constc(((Q*q4)*(q1*q3)-(Q*q3)*(q1*q4)));
  // compute the current
  return pre*(consta*q4-constb*q3+constc*a1);
}

LorentzVector<complex<Energy5> > 
FourPionNovosibirskCurrent::t2(Lorentz5Momentum & q1,Lorentz5Momentum & q2,
			       Lorentz5Momentum & q3,Lorentz5Momentum & q4,
			       unsigned int iopt) const {
  // momentum of the whole system
  Lorentz5Momentum Q(q1+q2+q3+q4);Q.rescaleMass();
  // compute the virtuality of the a_1
  Lorentz5Momentum a1(q2+q3+q4);a1.rescaleMass();
  // compute the virtuality of the  sigma
  Lorentz5Momentum sigma(q3+q4);sigma.rescaleMass();
  // compute the prefactor
  Complex pre(_zsigma*a1FormFactor(a1.mass2())
	      *a1BreitWigner(a1.mass2())*
	      sigmaBreitWigner(sigma.mass2(),iopt));
  // dot products we need
  complex<Energy4> consta((Q*a1)*a1.mass2()),constb((Q*q2)*a1.mass2());
  // compute the current
  return pre*(consta*q2-constb*a1);
}

LorentzVector<complex<Energy5> > 
FourPionNovosibirskCurrent::t3(Lorentz5Momentum & q1,Lorentz5Momentum & q2,
			       Lorentz5Momentum & q3,Lorentz5Momentum & q4) const {
  // momentum of the whole sysytem
  Lorentz5Momentum Q(q1+q2+q3+q4);Q.rescaleMass();
  // compute the virtuality of the omega
  Lorentz5Momentum omega(q2+q3+q4);omega.rescaleMass();
  // compute the virtuality of the  rho
  Lorentz5Momentum rho(q3+q4);rho.rescaleMass();
  // compute the prefactor
  Complex pre(omegaBreitWigner(omega.mass2())*rhoBreitWigner(rho.mass2()));
  // dot products we need
  complex<Energy4> consta((Q*q3)*(q1*q4)-(Q*q4)*(q1*q3)),
    constb(-(Q*q2)*(q1*q4)+(q1*q2)*(Q*q4)),
    constc((Q*q2)*(q1*q3)-(q1*q2)*(Q*q3));
  // compute the current
  return pre*(consta*q2+constb*q3+constc*q4);
}

InvEnergy6 FourPionNovosibirskCurrent::gFunction(Energy2 q2, int ichan) const {
  Energy q(sqrt(q2));
  InvEnergy4 invmrho4 = 1/sqr(sqr(_rhomass));
  // the one charged pion G function
  if(ichan==0) {
    return (*_Fonec)(q) * _aonec * (*_Fsigma)(q2) * sqrt(_bonec*q/GeV-_conec) *
      invmrho4/q;
  }
  // the three charged pion G function
  else if(ichan==1) {
    return (*_Fthreec)(q)*_athreec*sqrt(_bthreec*q/GeV-_cthreec)*invmrho4/q; 
  }
  // the omega G function
  else if(ichan==2) {
    return(*_Fomega)(q)*_aomega*sqrt(_bomega*q/GeV-_comega)*invmrho4/q;
  }
  assert(false);
  return InvEnergy6();
}

Energy2 FourPionNovosibirskCurrent::DParameter() const {
  Energy2 grhom(8.*_prho*_prho*_prho/_rhomass);
  return _rhomass*_rhomass+_rhowidth*_rhomass*
    (hFunction(ZERO)-_hm2+_rhomass*_rhomass*_dhdq2m2)/grhom;
}

double FourPionNovosibirskCurrent::dhdq2Parameter() const {
  Energy2 mrho2(_rhomass*_rhomass);
  double root(sqrt(1.-4.*_mpic2/mrho2));
  return root/Constants::pi*(root+(1.+2*_mpic2/mrho2)*log((1+root)/(1-root)));
}

Energy2 FourPionNovosibirskCurrent::hFunction(const Energy q) const {
  using Constants::pi;
  static const Energy2 eps(0.01*MeV2);

  Energy2 q2(q*q), output;
  if (q2 > 4*_mpic2) {
    double root = sqrt(1.-4.*_mpic2/q2);
    output = root*log((1.+root)/(1.-root))*(q2-4*_mpic2)/pi;
  }
  else if (q2 > eps) output = ZERO;
  else               output = -8.*_mpic2/pi;
  return output;
}

#line 1 "./ScalarMesonCurrent.cc"
// -*- C++ -*-
//
// ScalarMesonCurrent.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the ScalarMesonCurrent class.
//

#include "ScalarMesonCurrent.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/StandardModel/StandardModelBase.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void ScalarMesonCurrent::doinit() {
  unsigned int isize=numberOfModes();
  if(_id.size()!=isize||_decay_constant.size()!=isize)
    {throw InitException() << "Inconsistent parameters in ScalarMesonCurrent::doinit()"
			   << Exception::abortnow;}
  WeakDecayCurrent::doinit();
}

ScalarMesonCurrent::ScalarMesonCurrent() {
  // the eta/eta' mixing angle
  _thetaeta=-0.194;
  // the decay constants for the different modes
  _id.push_back(211);_decay_constant.push_back(130.7*MeV);
  addDecayMode(2,-1);
  _id.push_back(111);_decay_constant.push_back(130.7*MeV);
  addDecayMode(1,-1);
  _id.push_back(111);_decay_constant.push_back(130.7*MeV);
  addDecayMode(2,-2);
  _id.push_back(221);_decay_constant.push_back(130.7*MeV);
  addDecayMode(1,-1);
  _id.push_back(221);_decay_constant.push_back(130.7*MeV);
  addDecayMode(2,-2);
  _id.push_back(221);_decay_constant.push_back(130.7*MeV);
  addDecayMode(3,-3);
  _id.push_back(331);_decay_constant.push_back(130.7*MeV);
  addDecayMode(1,-1);
  _id.push_back(331);_decay_constant.push_back(130.7*MeV);
  addDecayMode(2,-2);
  _id.push_back(331);_decay_constant.push_back(130.7*MeV);
  addDecayMode(3,-3);
  _id.push_back(311);_decay_constant.push_back(159.8*MeV);
  addDecayMode(1,-3);
  _id.push_back(321);_decay_constant.push_back(159.8*MeV);
  addDecayMode(2,-3);
  _id.push_back(411);_decay_constant.push_back(200.0*MeV);
  addDecayMode(4,-1);
  _id.push_back(421);_decay_constant.push_back(200.0*MeV);
  addDecayMode(4,-2);
  _id.push_back(431);_decay_constant.push_back(241.0*MeV);
  addDecayMode(4,-3);
  _id.push_back(10431);_decay_constant.push_back(73.7*MeV);
  addDecayMode(4,-3);
  // initial size of the arrays
  _initsize = _id.size();
  setInitialModes(_initsize);
}

void ScalarMesonCurrent::persistentOutput(PersistentOStream & os) const {
  os << _id << ounit(_decay_constant,GeV) << _thetaeta;
}

void ScalarMesonCurrent::persistentInput(PersistentIStream & is, int) {
  is >> _id >> iunit(_decay_constant,GeV) >> _thetaeta;
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<ScalarMesonCurrent,WeakDecayCurrent>
describeHerwigScalarMesonCurrent("Herwig::ScalarMesonCurrent", "HwWeakCurrents.so");

void ScalarMesonCurrent::Init() {

  static ClassDocumentation<ScalarMesonCurrent> documentation
    ("The ScalarMesonCurrent class implements the current"
     " for the decay of the weak current into a pseudoscalar meson.");

  static ParVector<ScalarMesonCurrent,long> interfaceID
    ("ID",
     "The PDG code for the outgoing meson.",
     &ScalarMesonCurrent::_id,
     0, 0, 0, -1000000, 1000000, false, false, true);

  static ParVector<ScalarMesonCurrent,Energy> interfaceDecay_Constant
    ("Decay_Constant",
     "The decay constant for the meson.",
     &ScalarMesonCurrent::_decay_constant, MeV, -1, 100.*MeV,-1000.0*MeV, 1000.0*MeV,
     false, false, true);

  static Parameter<ScalarMesonCurrent,double> interfaceThetaEtaEtaPrime
    ("ThetaEtaEtaPrime",
     "The eta-eta' mixing angle",
     &ScalarMesonCurrent::_thetaeta, -0.194, -Constants::pi, Constants::pi,
     false, false, true);

}

// create the decay phase space mode
bool ScalarMesonCurrent::createMode(int icharge,unsigned int imode,
				    DecayPhaseSpaceModePtr mode,
				    unsigned int iloc,unsigned int ires,
				    DecayPhaseSpaceChannelPtr phase,Energy upp) {
  // check the mode has the correct charge
  if(abs(icharge)!=abs(int(getParticleData(_id[imode])->iCharge()))) return false;
  // check if the particle is kinematically allowed
  tPDPtr part(getParticleData(_id[imode]));
  Energy min=part->massMin();
  if(min>upp) return false;
  // construct the mode
  DecayPhaseSpaceChannelPtr newchannel(new_ptr(DecayPhaseSpaceChannel(*phase)));
  newchannel->resetDaughter(-ires,iloc);
  mode->addChannel(newchannel);
  return true;
}

// outgoing particles 
tPDVector ScalarMesonCurrent::particles(int icharge, unsigned int imode, int iq, int ia) {
  tPDPtr part(getParticleData(_id[imode]));
  tPDVector output;
  if(icharge==int(part->iCharge())) {
    if(icharge==0) {
      int iqb,iab; 
      decayModeInfo(imode,iqb,iab);
      if(iq==iqb&&ia==iab) {
	output.push_back(part);
      }
      else {
	output.push_back(part->CC());
      }
    }
    else {
      output.push_back(part);
    }
  }
  else if(icharge==-int(part->iCharge())) {
    output.push_back(part->CC());
  }
  return output;
}

vector<LorentzPolarizationVectorE> 
ScalarMesonCurrent::current(const int imode, const int, 
			    Energy & scale,const ParticleVector & decay,
			    DecayIntegrator::MEOption meopt) const {
  static const Complex ii(0.,1.);
  if(meopt==DecayIntegrator::Terminate) {
    ScalarWaveFunction::constructSpinInfo(decay[0],outgoing,true);
    return vector<LorentzPolarizationVectorE>(1,LorentzPolarizationVectorE());
  }
  scale = decay[0]->mass();
  Complex pre(-ii*_decay_constant[imode]/scale);
  // quarks in the current
  int iq,ia;
  decayModeInfo(imode,iq,ia);
  if(abs(iq)==abs(ia)) {
    int id(decay[0]->id());
    if(id==ParticleID::eta) {
      if(abs(iq)==3) pre*=-2.*cos(_thetaeta)/sqrt(6.)-sin(_thetaeta)/sqrt(3.);
      else           pre*=cos(_thetaeta)/sqrt(6.)-sin(_thetaeta)/sqrt(3.);
    }
    else if(id==ParticleID::etaprime) {
      if(abs(iq)==3) pre*=-2.*sin(_thetaeta)/sqrt(6.)+cos(_thetaeta)/sqrt(3.);
      else           pre*=sin(_thetaeta)/sqrt(6.)+cos(_thetaeta)/sqrt(3.);
    }
    else if(id==ParticleID::pi0&&abs(iq)==1) {
      pre*=-sqrt(0.5);
    }
    else {
      pre*= sqrt(0.5);
    }
  }
  // return the answer
  return vector<LorentzPolarizationVectorE>(1,pre*decay[0]->momentum());
}
  
bool ScalarMesonCurrent::accept(vector<int> id) {
  if(id.size()!=1){return false;}
  int idtemp(abs(id[0]));
  for(unsigned int ix=0;ix<_id.size();++ix) {
    if(abs(_id[ix])==idtemp) return true;
  }
  return false;
}

unsigned int ScalarMesonCurrent::decayMode(vector<int> idout)
{
  int idtemp(abs(idout[0])); unsigned int ix(0);
  bool found(false);
  do {
    if(idtemp==abs(_id[ix])) found=true;
    else                     ++ix;
  }
  while(!found);
  return ix;
}

void ScalarMesonCurrent::dataBaseOutput(ofstream & output,
					bool header,bool create) const {
  if(header) {
    output << "update decayers set parameters=\"";
  }
  if(create) {
    output << "create Herwig::ScalarMesonCurrent " << name() 
	   << " HwWeakCurrents.so\n";
  }
  output << "newdef " << name() << ":ThetaEtaEtaPrime " << _thetaeta  << "\n";
  unsigned int ix;
  for(ix=0;ix<_id.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":ID " << ix 
	     << " " << _id[ix] << "\n";
      output << "newdef " << name() << ":Decay_Constant " << ix 
	     << " " << _decay_constant[ix]/MeV << "\n";
    }
    else {
      output << "insert " << name() << ":ID " << ix 
	     << " " << _id[ix] << "\n";
      output << "insert " << name() << ":Decay_Constant " << ix 
	     << " " << _decay_constant[ix]/MeV << "\n";
    }
  }
  WeakDecayCurrent::dataBaseOutput(output,false,false);
  if(header) {
    output << "\n\" where BINARY ThePEGName=\"" << fullName() << "\";\n";
  }
}
#line 1 "./ThreeMesonCurrentBase.cc"
// -*- C++ -*-
//
// ThreeMesonCurrentBase.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the ThreeMesonCurrentBase class.
//

#include "ThreeMesonCurrentBase.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/Helicity/epsilon.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"
#include "ThePEG/Utilities/DescribeClass.h"

using namespace Herwig;
using namespace ThePEG;
using namespace ThePEG::Helicity;

ThreeMesonCurrentBase::ThreeMesonCurrentBase() {
  // the quarks for the different modes
  addDecayMode(2,-1);
  addDecayMode(2,-1);
  addDecayMode(2,-1);
  addDecayMode(2,-1);
  addDecayMode(2,-1);
  addDecayMode(2,-3);
  addDecayMode(2,-3);
  addDecayMode(2,-3);
  addDecayMode(2,-1);
  addDecayMode(2,-1);
  addDecayMode(2,-1);
  addDecayMode(2,-1);
  setInitialModes(12);
}

DescribeAbstractNoPIOClass<ThreeMesonCurrentBase,WeakDecayCurrent>
describeHerwigThreeMesonCurrentBase("Herwig::ThreeMesonCurrentBase",
				    "HwWeakCurrents.so");



void ThreeMesonCurrentBase::Init() {
    
  static ClassDocumentation< ThreeMesonCurrentBase> documentation
    ("The ThreeMesonCurrentBase class is designed to be the "
     "base class for "
     "the three meson decays of the tau, ie pi- pi- pi+, pi0 pi0 pi-, " 
     "K- pi- K+, K0 pi- Kbar0, K- pi0 K0,pi0 pi0 K-, K- pi- pi+, "
     "pi- Kbar0 pi0, pi- pi0 eta, K0S pi- K0S, K0L pi- K0L, K0S pi- K0L");

}

// the hadronic currents    
vector<LorentzPolarizationVectorE> 
ThreeMesonCurrentBase::current(const int imode, const int ichan, 
			       Energy & scale,const ParticleVector & decay,
			       DecayIntegrator::MEOption meopt) const {
  if(meopt==DecayIntegrator::Terminate) {
    for(unsigned int ix=0;ix<3;++ix)
      ScalarWaveFunction::constructSpinInfo(decay[ix],outgoing,true);
    return vector<LorentzPolarizationVectorE>(1,LorentzPolarizationVectorE());
  }
  // calculate q2,s1,s2,s3
  Lorentz5Momentum q;
  for(unsigned int ix=0;ix<decay.size();++ix){q+=decay[ix]->momentum();}
  q.rescaleMass();
  scale=q.mass();
  Energy2 q2=q.mass2();
  Energy2 s1 = (decay[1]->momentum()+decay[2]->momentum()).m2();
  Energy2 s2 = (decay[0]->momentum()+decay[2]->momentum()).m2();
  Energy2 s3 = (decay[0]->momentum()+decay[1]->momentum()).m2();
  FormFactors F = calculateFormFactors(ichan,imode,q2,s1,s2,s3);
  //if(inpart.id()==ParticleID::tauplus){F.F5=conj(F.F5);}
  // the first three form-factors
  LorentzPolarizationVector vect;
  vect = LorentzPolarizationVector((F.F2-F.F1)*decay[2]->momentum())
    +LorentzPolarizationVector((F.F1-F.F3)*decay[1]->momentum())
    +LorentzPolarizationVector((F.F3-F.F2)*decay[0]->momentum());
  // multiply by the transverse projection operator
  complex<InvEnergy> dot=(vect*q)/q2;
  // scalar and parity violating terms
  vect += LorentzPolarizationVector((F.F4-dot)*q);
  if(F.F5!=complex<InvEnergy3>()) 
    vect += LorentzPolarizationVector(Complex(0.,1.)*F.F5*Helicity::epsilon(decay[0]->momentum(),
									    decay[1]->momentum(),
									    decay[2]->momentum()));
  // factor to get dimensions correct
  return vector<LorentzPolarizationVectorE>(1,q.mass()*vect);
}

bool ThreeMesonCurrentBase::accept(vector<int> id) {
  int npip(0),npim(0),nkp(0),nkm(0),
    npi0(0),nk0(0),nk0bar(0),neta(0),nks(0),nkl(0);
  for(unsigned int ix=0;ix<id.size();++ix) {
    if(id[ix]==ParticleID::piplus)       ++npip;
    else if(id[ix]==ParticleID::piminus) ++npim;
    else if(id[ix]==ParticleID::Kplus)   ++nkp;
    else if(id[ix]==ParticleID::Kminus)  ++nkm;
    else if(id[ix]==ParticleID::pi0)     ++npi0;
    else if(id[ix]==ParticleID::K0)      ++nk0;
    else if(id[ix]==ParticleID::Kbar0)   ++nk0bar;
    else if(id[ix]==ParticleID::eta)     ++neta;
    else if(id[ix]==ParticleID::K_S0)    ++nks;
    else if(id[ix]==ParticleID::K_L0)    ++nkl;
  }
  int imode(-1);
  if(      (npip==2&&npim==1) || (npim==2&&npip==1) ) imode= 0;
  else if( (npip==1&&npi0==2) || (npim==1&&npi0==2) ) imode= 1;
  else if( (nkp==1&&nkm==1&&npip==1) ||
	   (nkp==1&&nkm==1&&npim==1))                 imode= 2;
  else if( (nk0==1&&nk0bar==1&&npip==1) ||
	   (nk0==1&&nk0bar==1&&npim==1))              imode= 3;
  else if( (nkp==1&&nk0bar==1&&npi0==1) ||
	   (nkm==1&&npi0==1&&nk0==1))                 imode= 4;
  else if( (nkp==1&&npi0==2) || (npi0==2&&nkm==1) )   imode= 5;
  else if( (npip==1&&npim==1&&nkp==1) ||
	   (nkm==1&&npim==1&&npip==1) )               imode= 6;
  else if( (nk0==1&&npip==1&&npi0==1)  ||
	   (npim==1&&nk0bar==1&&npi0==1))             imode= 7;
  else if( (npip==1&&npi0==1&&neta==1) ||
	   (npim==1&&npi0==1&&neta==1))               imode= 8;
  else if( nks==2 && (npip==1||npim==1) )             imode= 9;
  else if( nkl==2 && (npip==1||npim==1) )             imode=10;
  else if( nks==1&&nkl==1 && (npip==1||npim==1) )     imode=11;
  return imode==-1 ? false : acceptMode(imode);
}

unsigned int ThreeMesonCurrentBase::decayMode(vector<int> id) {
  int npip(0),npim(0),nkp(0),nkm(0),
    npi0(0),nk0(0),nk0bar(0),neta(0),nks(0),nkl(0);
  for(unsigned int ix=0;ix<id.size();++ix) {
    if(id[ix]==ParticleID::piplus)       ++npip;
    else if(id[ix]==ParticleID::piminus) ++npim;
    else if(id[ix]==ParticleID::Kplus)   ++nkp;
    else if(id[ix]==ParticleID::Kminus)  ++nkm;
    else if(id[ix]==ParticleID::pi0)     ++npi0;
    else if(id[ix]==ParticleID::K0)      ++nk0;
    else if(id[ix]==ParticleID::Kbar0)   ++nk0bar;
    else if(id[ix]==ParticleID::eta)     ++neta;
    else if(id[ix]==ParticleID::K_S0)    ++nks;
    else if(id[ix]==ParticleID::K_L0)    ++nkl;
  }
  int imode(-1);
  if(      (npip==2&&npim==1) || (npim==2&&npip==1) ) imode= 0;
  else if( (npip==1&&npi0==2) || (npim==1&&npi0==2) ) imode= 1;
  else if( (nkp==1&&nkm==1&&npip==1) ||
	   (nkp==1&&nkm==1&&npim==1))                 imode= 2;
  else if( (nk0==1&&nk0bar==1&&npip==1) ||
	   (nk0==1&&nk0bar==1&&npim==1))              imode= 3;
  else if( (nkp==1&&nk0bar==1&&npi0==1) ||
	   (nkm==1&&npi0==1&&nk0==1))                 imode= 4;
  else if( (nkp==1&&npi0==2) || (npi0==2&&nkm==1) )   imode= 5;
  else if( (npip==1&&npim==1&&nkp==1) ||
	   (nkm==1&&npim==1&&npip==1) )               imode= 6;
  else if( (nk0==1&&npip==1&&npi0==1)  ||
	   (npim==1&&nk0bar==1&&npi0==1))             imode= 7;
  else if( (npip==1&&npi0==1&&neta==1) ||
	   (npim==1&&npi0==1&&neta==1))               imode= 8;
  else if( nks==2 && (npip==1||npim==1) )             imode= 9;
  else if( nkl==2 && (npip==1||npim==1) )             imode=10;
  else if( nks==1&&nkl==1 && (npip==1||npim==1) )     imode=11;
  return imode;
}

void ThreeMesonCurrentBase::dataBaseOutput(ofstream & output,bool header,
					   bool create) const {
  WeakDecayCurrent::dataBaseOutput(output,header,create);
}

tPDVector ThreeMesonCurrentBase::particles(int icharge, unsigned int imode,int,int) {
  tPDVector extpart(3);
  if(imode==0) {
    extpart[0]=getParticleData(ParticleID::piminus);
    extpart[1]=getParticleData(ParticleID::piminus);
    extpart[2]=getParticleData(ParticleID::piplus);
  }
  else if(imode==1) {
    extpart[0]=getParticleData(ParticleID::pi0);
    extpart[1]=getParticleData(ParticleID::pi0);
    extpart[2]=getParticleData(ParticleID::piminus);
  }
  else if(imode==2) {
    extpart[0]=getParticleData(ParticleID::Kminus);
    extpart[1]=getParticleData(ParticleID::piminus);
    extpart[2]=getParticleData(ParticleID::Kplus);
  }
  else if(imode==3) {
    extpart[0]=getParticleData(ParticleID::K0);
    extpart[1]=getParticleData(ParticleID::piminus);
    extpart[2]=getParticleData(ParticleID::Kbar0);
  }
  else if(imode==4) {
    extpart[0]=getParticleData(ParticleID::Kminus);
    extpart[1]=getParticleData(ParticleID::pi0);
    extpart[2]=getParticleData(ParticleID::K0);
  }
  else if(imode==5) {
    extpart[0]=getParticleData(ParticleID::pi0);
    extpart[1]=getParticleData(ParticleID::pi0);
    extpart[2]=getParticleData(ParticleID::Kminus);
  }
  else if(imode==6) {
    extpart[0]=getParticleData(ParticleID::Kminus);
    extpart[1]=getParticleData(ParticleID::piminus);
    extpart[2]=getParticleData(ParticleID::piplus);
  }
  else if(imode==7) {
    extpart[0]=getParticleData(ParticleID::piminus);
    extpart[1]=getParticleData(ParticleID::Kbar0);
    extpart[2]=getParticleData(ParticleID::pi0);
  }
  else if(imode==8) {
    extpart[0]=getParticleData(ParticleID::piminus);
    extpart[1]=getParticleData(ParticleID::pi0);
    extpart[2]=getParticleData(ParticleID::eta);
  }
  else if(imode==9) {
    extpart[0]=getParticleData(ParticleID::K_S0);
    extpart[1]=getParticleData(ParticleID::piminus);
    extpart[2]=getParticleData(ParticleID::K_S0);
  }
  else if(imode==10) {
    extpart[0]=getParticleData(ParticleID::K_L0);
    extpart[1]=getParticleData(ParticleID::piminus);
    extpart[2]=getParticleData(ParticleID::K_L0);
  }
  else if(imode==11) {
    extpart[0]=getParticleData(ParticleID::K_S0);
    extpart[1]=getParticleData(ParticleID::piminus);
    extpart[2]=getParticleData(ParticleID::K_L0);
  }
  // conjugate the particles if needed
  if(icharge==3) {
    for(unsigned int ix=0;ix<3;++ix) {
      if(extpart[ix]->CC()) extpart[ix]=extpart[ix]->CC();
    }
  }
  // return the answer
  return extpart;
}

#line 1 "./ThreeMesonDefaultCurrent.cc"
// -*- C++ -*-
//
// ThreeMesonDefaultCurrent.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the ThreeMesonDefaultCurrent class.
//

#include "ThreeMesonDefaultCurrent.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/Switch.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "Herwig/PDT/ThreeBodyAllOnCalculator.h"
#include "ThePEG/Utilities/DescribeClass.h"

using namespace Herwig;
using namespace ThePEG;

DescribeClass<ThreeMesonDefaultCurrent,ThreeMesonCurrentBase>
describeHerwigThreeMesonDefaultCurrent("Herwig::ThreeMesonDefaultCurrent",
				       "HwWeakCurrents.so");
HERWIG_INTERPOLATOR_CLASSDESC(ThreeMesonDefaultCurrent,Energy,Energy2)


ThreeMesonDefaultCurrent::ThreeMesonDefaultCurrent() {
  // the pion decay constant
  _fpi=130.7*MeV/sqrt(2.);
  _mpi=ZERO;_mK=ZERO;
  // set the initial weights for the resonances
  // the rho weights
  _rhoF123wgts.push_back(1.0);_rhoF123wgts.push_back(-0.145);
  _rhoF123wgts.push_back(0.);
  _rhoF5wgts.push_back(-26.);_rhoF5wgts.push_back(6.5);
  _rhoF5wgts.push_back(1.);
  // the Kstar weights
  _kstarF123wgts.push_back(1.);
  _kstarF5wgts.push_back(1.);
  // relative rho/Kstar weights
  _rhoKstarwgt=-0.2;
  // local values of the a_1 parameters
  _a1parameters=true;_a1mass=1.251*GeV;_a1width=0.599*GeV;
  _a1opt=true;
  // local values of the K_1 parameters
  _k1parameters=true;_k1mass=1.402*GeV,_k1width=0.174*GeV;
  // local values of the rho parameters
  _rhoparameters=true;
  _rhoF123masses.push_back(0.773*GeV);_rhoF123masses.push_back(1.370*GeV);
  _rhoF123masses.push_back(1.750*GeV);
  _rhoF123widths.push_back(0.145*GeV);_rhoF123widths.push_back(0.510*GeV);
  _rhoF123widths.push_back(0.120*GeV);
  _rhoF5masses.push_back(0.773*GeV);_rhoF5masses.push_back(1.500*GeV);
  _rhoF5masses.push_back(1.750*GeV);
  _rhoF5widths.push_back(0.145*GeV);_rhoF5widths.push_back(0.220*GeV);
  _rhoF5widths.push_back(0.120*GeV);
  // local values for the Kstar parameters
  _kstarparameters=true;
  _kstarF123masses.push_back(0.8921*GeV);
  _kstarF123widths.push_back(0.0513*GeV);
  _kstarF5masses.push_back(0.8921*GeV);
  _kstarF5widths.push_back(0.0513*GeV);
  // initialization of the a_1 running width
  _initializea1=false;
  double a1q2in[200]={0,15788.6,31577.3,47365.9,63154.6,78943.2,94731.9,110521,
		       126309,142098,157886,173675,189464,205252,221041,236830,
		       252618,268407,284196,299984,315773,331562,347350,363139,
		       378927,394716,410505,426293,442082,457871,473659,489448,
		       505237,521025,536814,552603,568391,584180,599969,615757,
		       631546,647334,663123,678912,694700,710489,726278,742066,
		       757855,773644,789432,805221,821010,836798,852587,868375,
		       884164,899953,915741,931530,947319,963107,978896,994685,
		       1.01047e+06,1.02626e+06,1.04205e+06,1.05784e+06,1.07363e+06,
		       1.08942e+06,1.10521e+06,1.12099e+06,1.13678e+06,1.15257e+06,
		       1.16836e+06,1.18415e+06,1.19994e+06,1.21573e+06,1.23151e+06,
		       1.2473e+06,1.26309e+06,1.27888e+06,1.29467e+06,1.31046e+06,
		       1.32625e+06,1.34203e+06,1.35782e+06,1.37361e+06,1.3894e+06,
		       1.40519e+06,1.42098e+06,1.43677e+06,1.45256e+06,1.46834e+06
		       ,1.48413e+06,1.49992e+06,1.51571e+06,1.5315e+06,1.54729e+06,
		       1.56308e+06,1.57886e+06,1.59465e+06,1.61044e+06,1.62623e+06,
		       1.64202e+06,1.65781e+06,1.6736e+06,1.68939e+06,1.70517e+06,
		       1.72096e+06,1.73675e+06,1.75254e+06,1.76833e+06,1.78412e+06,
		       1.79991e+06,1.81569e+06,1.83148e+06,1.84727e+06,1.86306e+06,
		       1.87885e+06,1.89464e+06,1.91043e+06,1.92621e+06,1.942e+06,
		       1.95779e+06,1.97358e+06,1.98937e+06,2.00516e+06,2.02095e+06,
		       2.03674e+06,2.05252e+06,2.06831e+06,2.0841e+06,2.09989e+06,
		       2.11568e+06,2.13147e+06,2.14726e+06,2.16304e+06,2.17883e+06,
		       2.19462e+06,2.21041e+06,2.2262e+06,2.24199e+06,2.25778e+06,
		       2.27356e+06,2.28935e+06,2.30514e+06,2.32093e+06,2.33672e+06,
		       2.35251e+06,2.3683e+06,2.38409e+06,2.39987e+06,2.41566e+06,
		       2.43145e+06,2.44724e+06,2.46303e+06,2.47882e+06,2.49461e+06,
		       2.51039e+06,2.52618e+06,2.54197e+06,2.55776e+06,2.57355e+06,
		       2.58934e+06,2.60513e+06,2.62092e+06,2.6367e+06,2.65249e+06,
		       2.66828e+06,2.68407e+06,2.69986e+06,2.71565e+06,2.73144e+06,
		       2.74722e+06,2.76301e+06,2.7788e+06,2.79459e+06,2.81038e+06,
		       2.82617e+06,2.84196e+06,2.85774e+06,2.87353e+06,2.88932e+06,
		       2.90511e+06,2.9209e+06,2.93669e+06,2.95248e+06,2.96827e+06,
		       2.98405e+06,2.99984e+06,3.01563e+06,3.03142e+06,3.04721e+06,
		       3.063e+06,3.07879e+06,3.09457e+06,3.11036e+06,3.12615e+06,
		       3.14194e+06};
  double a1widthin[200]={0,0,0,0,0,0,0,0,0,0,0,0,0.00153933,0.0136382,0.0457614,
			 0.105567,0.199612,0.333825,0.513831,0.745192,1.0336,1.38501,
			 1.80581,2.30295,2.88403,3.5575,4.33278,5.22045,6.23243,
			 7.38223,8.68521,10.1589,11.8234,13.7018,15.8206,18.2107,
			 20.9078,23.9533,27.3954,31.2905,35.7038,40.7106,46.3984,
			 52.8654,60.2207,68.581,78.0637,88.7754,100.794,114.145,
			 128.783,144.574,161.299,178.683,196.426,214.248,231.908,
			 249.221,266.059,282.336,298.006,313.048,327.46,341.254,
			 354.448,367.066,379.133,390.677,401.726,412.304,422.439,
			   432.155,441.474,450.419,459.01,467.267,475.207,482.847,
			 490.203,497.29,504.121,510.71,517.068,523.207,529.138,
			 534.869,540.411,545.776,550.961,556.663,560.851,565.566,
			 570.137,574.569,578.869,583.041,587.091,591.023,594.843,
			 598.553,602.16,605.664,609.072,612.396,615.626,618.754,
			 621.796,624.766,627.656,630.47,633.21,635.878,638.5,
			 641.006,643.471,645.873,648.213,650.493,652.715,654.88,
			 656.99,659.047,661.052,663.007,664.963,666.771,668.6,
			 670.351,672.075,673.828,675.397,676.996,678.567,680.083,
			 681.589,683.023,684.457,685.825,687.18,688.499,689.789,
			 691.058,692.284,693.501,694.667,695.82,696.947,698.05,
			 699.129,700.186,701.221,702.234,703.226,704.198,705.158,
			 706.085,707.001,707.899,708.78,709.644,710.474,711.334,
			 712.145,712.943,713.727,714.505,715.266,716.015,716.751,
			 717.474,718.183,718.88,719.645,720.243,720.91,721.565,
			 722.211,722.851,723.473,724.094,724.697,725.296,725.886,
			 726.468,727.041,727.608,728.166,728.718,729.262,729.808,
			 730.337,730.856,731.374,731.883,732.386,732.884,733.373,
			 733.859,734.339,734.813};

  vector<double> tmp1(a1widthin,a1widthin+200);
  _a1runwidth.clear();
  std::transform(tmp1.begin(), tmp1.end(),
		 back_inserter(_a1runwidth),
		 [](double x){return x*GeV;});
  
  vector<double> tmp2(a1q2in,a1q2in+200);
  _a1runq2.clear();
  std::transform(tmp2.begin(), tmp2.end(),
		 back_inserter(_a1runq2),
		 [](double x){return x*GeV2;});

  _maxmass=ZERO;
  _maxcalc=ZERO;
}

void ThreeMesonDefaultCurrent::doinit() {
  ThreeMesonCurrentBase::doinit();
  // the particles we will use a lot
  tPDPtr a1(getParticleData(ParticleID::a_1minus)),
    k1(getParticleData(ParticleID::Kstar_1minus)),pi0(getParticleData(ParticleID::pi0)),
    piplus(getParticleData(ParticleID::piplus)),
    piminus(getParticleData(ParticleID::piminus));
  // masses for the running widths
  _mpi=piplus->mass();
  _mK=getParticleData(ParticleID::Kminus)->mass();
  // the charged rho resonances
  tPDPtr rhoc[3]={getParticleData(-213),getParticleData(-100213),
		  getParticleData(-30213)};
  // the charged K* resonances
  tPDPtr Kstarc[3]={getParticleData(-323),getParticleData(-100323),
		    getParticleData(-30323)};
  if(!_a1parameters) {
    _a1mass=a1->mass();
    _a1width=a1->width();
  }
  // mass and width of the k_1
  if(!_k1parameters) {
    _k1mass=k1->mass();
    _k1width=k1->width();
  }
  // initialise the a_1 running width calculation
  inita1Width(-1);
  // rho parameters in the base classs
  tcPDPtr temp;
  unsigned int ix;
  if(_rhoparameters&&_rhoF123masses.size()<3) {
    ix = _rhoF123masses.size();
    _rhoF123masses.resize(3);_rhoF123widths.resize(3);
    for(;ix<3;++ix) {
      if(rhoc[ix]) {
	_rhoF123masses[ix]=rhoc[ix]->mass();
	_rhoF123widths[ix]=rhoc[ix]->width();
      }
    }
  }
  else if(!_rhoparameters) {
    _rhoF123masses.resize(3);_rhoF123widths.resize(3);
    for(ix=0;ix<3;++ix) {
      if(rhoc[ix]) {
	_rhoF123masses[ix]=rhoc[ix]->mass();
	_rhoF123widths[ix]=rhoc[ix]->width();
      }
    }
  }
  // K star parameters in the base class
  if(_kstarparameters&&_kstarF123masses.size()<3) {
    ix = _kstarF123masses.size();
    _kstarF123masses.resize(3);_kstarF123widths.resize(3);
    for(;ix<3;++ix) {
      if(Kstarc[ix]) {
	_kstarF123masses[ix]=Kstarc[ix]->mass();
	_kstarF123widths[ix]=Kstarc[ix]->width();
      }
    }
  }
  else if(!_kstarparameters) {
    _kstarF123masses.resize(3);_kstarF123widths.resize(3);
    for(ix=0;ix<3;++ix) {
      if(Kstarc[ix]) {
	_kstarF123masses[ix]=Kstarc[ix]->mass();
	_kstarF123widths[ix]=Kstarc[ix]->width();
      }
    }
  }
  // rho parameters here
  if(_rhoparameters&&_rhoF5masses.size()<3) {
    ix = _rhoF5masses.size();
    _rhoF5masses.resize(3);_rhoF5widths.resize(3);
    for(;ix<3;++ix) {
      if(rhoc[ix]) {
	_rhoF5masses[ix]=rhoc[ix]->mass();
	_rhoF5widths[ix]=rhoc[ix]->width();
      }
    }
  }
  else if(!_rhoparameters) {
    _rhoF5masses.resize(3);_rhoF5widths.resize(3);
    for(ix=0;ix<3;++ix) {
      if(rhoc[ix]) {
	_rhoF5masses[ix]=rhoc[ix]->mass();
	_rhoF5widths[ix]=rhoc[ix]->width();
      }
    }
  }
  // Kstar parameters here
  if(_kstarparameters&&_kstarF5widths.size()<3) {
    ix = _kstarF5masses.size();
    _kstarF5masses.resize(3);_kstarF5widths.resize(3);
    for(;ix<3;++ix) {
      if(Kstarc[ix]) {
	_kstarF5masses[ix]=Kstarc[ix]->mass();
	_kstarF5widths[ix]=Kstarc[ix]->width();
      }
    }
  }
  else if(!_kstarparameters) {
    _kstarF5masses.resize(3);_kstarF5widths.resize(3);
    for(ix=0;ix<3;++ix) {
      if(Kstarc[ix]) {
	_kstarF5masses[ix]=Kstarc[ix]->mass();
	_kstarF5widths[ix]=Kstarc[ix]->width();
      }
    }
  }
}

void ThreeMesonDefaultCurrent::persistentOutput(PersistentOStream & os) const {
  os << _rhoF123wgts << _kstarF123wgts << _rhoF5wgts << _kstarF5wgts
     << _rhoKstarwgt <<  ounit(_a1runwidth,GeV)<< ounit(_a1runq2,GeV2)
     <<  _initializea1
     << ounit(_a1mass,GeV)<< ounit(_a1width,GeV)<< ounit(_k1mass,GeV)
     << ounit(_k1width,GeV)<< ounit(_fpi,GeV) << ounit(_mpi,GeV)<< ounit(_mK,GeV)
     <<_rhoparameters << ounit(_rhoF123masses,GeV) << ounit(_rhoF5masses,GeV) 
     << ounit(_rhoF123widths,GeV) 
     << ounit(_rhoF5widths,GeV) << _kstarparameters << ounit(_kstarF123masses,GeV) 
     <<ounit(_kstarF5masses,GeV)
     << ounit(_kstarF123widths,GeV) << ounit(_kstarF5widths,GeV) << _a1parameters 
     << _k1parameters
     << _a1opt << ounit(_maxmass,GeV) << ounit(_maxcalc,GeV) << _a1runinter;
}

void ThreeMesonDefaultCurrent::persistentInput(PersistentIStream & is, int) {
  is >> _rhoF123wgts >> _kstarF123wgts >> _rhoF5wgts >> _kstarF5wgts
     >> _rhoKstarwgt >>  iunit(_a1runwidth,GeV) >> iunit(_a1runq2,GeV2) 
     >>  _initializea1
     >> iunit(_a1mass,GeV) >> iunit(_a1width,GeV) >> iunit(_k1mass,GeV) 
     >> iunit(_k1width,GeV) >> iunit(_fpi,GeV) >> iunit(_mpi,GeV) >> iunit(_mK,GeV)
     >>_rhoparameters >> iunit(_rhoF123masses,GeV) >> iunit(_rhoF5masses,GeV) 
     >> iunit(_rhoF123widths,GeV) 
     >> iunit(_rhoF5widths,GeV) >> _kstarparameters >> iunit(_kstarF123masses,GeV) 
     >>iunit(_kstarF5masses,GeV)
     >> iunit(_kstarF123widths,GeV) >> iunit(_kstarF5widths,GeV) >> _a1parameters 
     >> _k1parameters
     >> _a1opt >> iunit(_maxmass,GeV) >> iunit(_maxcalc,GeV) >> _a1runinter;
}

void ThreeMesonDefaultCurrent::Init() {
        
  static ClassDocumentation<ThreeMesonDefaultCurrent> documentation
    ("The ThreeMesonDefaultCurrent class is designed to implement "
     "the three meson decays of the tau, ie pi- pi- pi+, pi0 pi0 pi-, " 
     "K- pi- K+, K0 pi- Kbar0, K- pi0 K0,pi0 pi0 K-, K- pi- pi+, "
     "pi- Kbar0 pi0, pi- pi0 eta. It uses the same currents as those in TAUOLA.",
     "The three meson decays of the tau, ie pi- pi- pi+, pi0 pi0 pi-, "
     "K- pi- K+, K0 pi- Kbar0, K- pi0 K0,pi0 pi0 K-, K- pi- pi+, "
     "and pi- Kbar0 pi0, pi- pi0 eta "
     "use the same currents as \\cite{Jadach:1993hs,Kuhn:1990ad,Decker:1992kj}.",
     "%\\cite{Jadach:1993hs}\n"
     "\\bibitem{Jadach:1993hs}\n"
     "  S.~Jadach, Z.~Was, R.~Decker and J.~H.~Kuhn,\n"
     "  %``The Tau Decay Library Tauola: Version 2.4,''\n"
     "  Comput.\\ Phys.\\ Commun.\\  {\\bf 76}, 361 (1993).\n"
     "  %%CITATION = CPHCB,76,361;%%\n"
     "%\\cite{Kuhn:1990ad}\n"
     "\\bibitem{Kuhn:1990ad}\n"
     "  J.~H.~Kuhn and A.~Santamaria,\n"
     "  %``Tau decays to pions,''\n"
     "  Z.\\ Phys.\\  C {\\bf 48}, 445 (1990).\n"
     "  %%CITATION = ZEPYA,C48,445;%%\n"
     "%\\cite{Decker:1992kj}\n"
     "\\bibitem{Decker:1992kj}\n"
     "  R.~Decker, E.~Mirkes, R.~Sauer and Z.~Was,\n"
     "  %``Tau decays into three pseudoscalar mesons,''\n"
     "  Z.\\ Phys.\\  C {\\bf 58}, 445 (1993).\n"
     "  %%CITATION = ZEPYA,C58,445;%%\n"
     );

  
  static ParVector<ThreeMesonDefaultCurrent,double> interfaceF123RhoWgt
    ("F123RhoWeight",
     "The weights of the different rho resonances in the F1,2,3 form factor",
     &ThreeMesonDefaultCurrent::_rhoF123wgts,
     0, 0, 0, -1000, 1000, false, false, true);
  
  static ParVector<ThreeMesonDefaultCurrent,double> interfaceF123KstarWgt
    ("F123KstarWeight",
     "The weights of the different Kstar resonances in the F1,2,3 form factor",
     &ThreeMesonDefaultCurrent::_kstarF123wgts,
     0, 0, 0, -1000, 1000, false, false, true);
  
  static ParVector<ThreeMesonDefaultCurrent,double> interfaceF5RhoWgt
    ("F5RhoWeight",
     "The weights of the different rho resonances in the F1,2,3 form factor",
     &ThreeMesonDefaultCurrent::_rhoF5wgts,
     0, 0, 0, -1000, 1000, false, false, true);
  
  static ParVector<ThreeMesonDefaultCurrent,double> interfaceF5KstarWgt
    ("F5KstarWeight",
     "The weights of the different Kstar resonances in the F1,2,3 form factor",
     &ThreeMesonDefaultCurrent::_kstarF5wgts,
     0, 0, 0, -1000, 1000, false, false, true);
  
  static Parameter<ThreeMesonDefaultCurrent,double> interfaceRhoKstarWgt
    ("RhoKstarWgt",
     "The relative weights of the rho and K* in the F5 form factor",
     &ThreeMesonDefaultCurrent::_rhoKstarwgt, -0.2, -10., 10.,
     false, false, false);
  
  static Switch<ThreeMesonDefaultCurrent,bool> interfaceInitializea1
    ("Initializea1",
     "Initialise the calculation of the a_1 running width",
     &ThreeMesonDefaultCurrent::_initializea1, false, false, false);
  static SwitchOption interfaceInitializea1Initialization
    (interfaceInitializea1,
     "Yes",
     "Initialize the calculation",
     true);
  static SwitchOption interfaceInitializea1NoInitialization
    (interfaceInitializea1,
     "No",
     "Use the default values",
     false);
  
  static Switch<ThreeMesonDefaultCurrent,bool> interfaceRhoParameters
    ("RhoParameters",
     "Use local values of the rho meson masses and widths",
     &ThreeMesonDefaultCurrent::_rhoparameters, true, false, false);
  static SwitchOption interfaceRhoParameterstrue
    (interfaceRhoParameters,
     "Local",
     "Use local values of the parameters",
     true);
  static SwitchOption interfaceRhoParametersParticleData
    (interfaceRhoParameters,
     "ParticleData",
     "Use the masses and wdiths from the particle data objects",
     false);
  
  static Switch<ThreeMesonDefaultCurrent,bool> interfaceKstarParameters
    ("KstarParameters",
     "Use local values of the rho meson masses and widths",
     &ThreeMesonDefaultCurrent::_kstarparameters, true, false, false);
  static SwitchOption interfaceKstarParameterstrue
    (interfaceKstarParameters,
       "Local",
     "Use local values of the parameters",
     true);
  static SwitchOption interfaceKstarParametersParticleData
    (interfaceKstarParameters,
     "ParticleData",
     "Use the masses and wdiths from the particle data objects",
     false);
  
  static Switch<ThreeMesonDefaultCurrent,bool> interfacea1Parameters
    ("a1Parameters",
     "Use local values of the rho meson masses and widths",
     &ThreeMesonDefaultCurrent::_a1parameters, true, false, false);
  static SwitchOption interfacea1Parameterstrue
    (interfacea1Parameters,
     "Local",
     "Use local values of the parameters",
     true);
  static SwitchOption interfacea1ParametersParticleData
    (interfacea1Parameters,
     "ParticleData",
     "Use the masses and wdiths from the particle data objects",
     false);
  
  static Switch<ThreeMesonDefaultCurrent,bool> interfaceK1Parameters
    ("K1Parameters",
     "Use local values of the rho meson masses and widths",
     &ThreeMesonDefaultCurrent::_k1parameters, true, false, false);
  static SwitchOption interfaceK1Parameterstrue
    (interfaceK1Parameters,
     "Local",
     "Use local values of the parameters",
     true);
  static SwitchOption interfaceK1ParametersParticleData
    (interfaceK1Parameters,
     "ParticleData",
     "Use the masses and wdiths from the particle data objects",
     false);
  
  static Switch<ThreeMesonDefaultCurrent,bool> interfacea1WidthOption
    ("a1WidthOption",
     "Option for the treatment of the a1 width",
     &ThreeMesonDefaultCurrent::_a1opt, true, false, false);
  static SwitchOption interfacea1WidthOptionLocal
    (interfacea1WidthOption,
     "Local",
     "Use a calculation of the running width based on the parameters as"
     " interpolation table.",
     true);
  static SwitchOption interfacea1WidthOptionParam
    (interfacea1WidthOption,
     "Kuhn",
     "Use the parameterization of Kuhn and Santamaria for default parameters."
     " This should only be used for testing vs TAUOLA",
     false);

  static ParVector<ThreeMesonDefaultCurrent,Energy> interfacea1RunningWidth
    ("a1RunningWidth",
     "The values of the a_1 width for interpolation to giving the running width.",
     &ThreeMesonDefaultCurrent::_a1runwidth, GeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static ParVector<ThreeMesonDefaultCurrent,Energy2> interfacea1RunningQ2
    ("a1RunningQ2",
     "The values of the q^2 for interpolation to giving the running width.",
     &ThreeMesonDefaultCurrent::_a1runq2, GeV2, -1, 1.0*GeV2, ZERO, 10.0*GeV2,
     false, false, true);
    
  static Parameter<ThreeMesonDefaultCurrent,Energy> interfaceA1Width
    ("A1Width",
     "The a_1 width if using local values.",
     &ThreeMesonDefaultCurrent::_a1width, GeV, 0.599*GeV, ZERO, 10.0*GeV,
     false, false, false);
  
  static Parameter<ThreeMesonDefaultCurrent,Energy> interfaceA1Mass
    ("A1Mass",
     "The a_1 mass if using local values.",
     &ThreeMesonDefaultCurrent::_a1mass, GeV, 1.251*GeV, ZERO, 10.0*GeV,
     false, false, false);
  
  static Parameter<ThreeMesonDefaultCurrent,Energy> interfaceK1Width
    ("K1Width",
     "The K_1 width if using local values.",
     &ThreeMesonDefaultCurrent::_k1width, GeV, 0.174*GeV, ZERO, 10.0*GeV,
     false, false, false);
  
  static Parameter<ThreeMesonDefaultCurrent,Energy> interfaceK1Mass
    ("K1Mass",
     "The K_1 mass if using local values.",
     &ThreeMesonDefaultCurrent::_k1mass, GeV, 1.402*GeV, ZERO, 10.0*GeV,
     false, false, false);
  
  static ParVector<ThreeMesonDefaultCurrent,Energy> interfacerhoF123masses
    ("rhoF123masses",
     "The masses for the rho resonances if used local values",
     &ThreeMesonDefaultCurrent::_rhoF123masses, GeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, true);
  
  static ParVector<ThreeMesonDefaultCurrent,Energy> interfacerhoF123widths
    ("rhoF123widths",
     "The widths for the rho resonances if used local values",
     &ThreeMesonDefaultCurrent::_rhoF123widths, GeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, true);
  
  static ParVector<ThreeMesonDefaultCurrent,Energy> interfacerhoF5masses
    ("rhoF5masses",
     "The masses for the rho resonances if used local values",
     &ThreeMesonDefaultCurrent::_rhoF5masses, GeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, true);
  
  static ParVector<ThreeMesonDefaultCurrent,Energy> interfacerhoF5widths
    ("rhoF5widths",
     "The widths for the rho resonances if used local values",
     &ThreeMesonDefaultCurrent::_rhoF5widths, GeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static ParVector<ThreeMesonDefaultCurrent,Energy> interfaceKstarF123masses
    ("KstarF123masses",
     "The masses for the Kstar resonances if used local values",
     &ThreeMesonDefaultCurrent::_kstarF123masses, GeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, true);
  
  static ParVector<ThreeMesonDefaultCurrent,Energy> interfaceKstarF123widths
    ("KstarF123widths",
     "The widths for the Kstar resonances if used local values",
     &ThreeMesonDefaultCurrent::_kstarF123widths, GeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, true);
  
  static ParVector<ThreeMesonDefaultCurrent,Energy> interfaceKstarF5masses
    ("KstarF5masses",
     "The masses for the Kstar resonances if used local values",
     &ThreeMesonDefaultCurrent::_kstarF5masses, GeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, true);
  
  static ParVector<ThreeMesonDefaultCurrent,Energy> interfaceKstarF5widths
    ("KstarF5widths",
     "The widths for the Kstar resonances if used local values",
     &ThreeMesonDefaultCurrent::_kstarF5widths, GeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static Parameter<ThreeMesonDefaultCurrent,Energy> interfaceFPi
    ("FPi",
     "The pion decay constant",
     &ThreeMesonDefaultCurrent::_fpi, MeV, 92.4*MeV, ZERO, 200.0*MeV,
     false, false, true);
}
  
// modes handled by this class
bool ThreeMesonDefaultCurrent::acceptMode(int imode) const { 
  return imode>=0&&imode<=8;
}

// calculate the form-factors
ThreeMesonDefaultCurrent::FormFactors 
ThreeMesonDefaultCurrent::calculateFormFactors(const int ichan, const int imode,
					       Energy2 q2, Energy2 s1, 
					       Energy2 s2, Energy2 s3) const {
  useMe();
  Complex F1, F2, F3, F4, F5;
  F1 = F2 = F3 = F4 = F5 = 0.0;
  // calculate the pi- pi- pi+ factor
  if(imode==0) {
    Complex a1fact(a1BreitWigner(q2)*2./3.);
    if(ichan<0) {
      F1= a1fact*BrhoF123(s1,-1);
      F2 =-a1fact*BrhoF123(s2,-1);
    }
    else if(ichan%2==0) F1 = a1fact*BrhoF123(s1,    ichan/2);
    else if(ichan%2==1) F2 =-a1fact*BrhoF123(s2,(ichan-1)/2);
  }
  // calculate the pi0 pi0 pi- factor
  else if(imode==1) {
    Complex a1fact(a1BreitWigner(q2)*2./3.);
    if(ichan<0) {
      F1 = a1fact*BrhoF123(s1,-1);
      F2 =-a1fact*BrhoF123(s2,-1);
    }
    else if(ichan%2==0) F1 = a1fact*BrhoF123(s1,    ichan/2);
    else if(ichan%2==1) F2 =-a1fact*BrhoF123(s2,(ichan-1)/2);
  }
  // calculate the K- pi - K+ factor
  else if(imode==2) {
    Complex a1fact(a1BreitWigner(q2)*sqrt(2.)/3.);
    if(ichan<0) {
      F1 =-a1fact*BKstarF123(s1,-1); 
      F2 = a1fact*BrhoF123(s2,-1);
      F5 = BrhoF5(q2,-1)*FKrho(s1,s2,-1)*sqrt(2.);
    }
    else if(ichan%8==0) F1 =-a1fact*BKstarF123(s1,ichan/8);
    else if(ichan%8==1) F2 = a1fact*BrhoF123(s2,(ichan-1)/8);
    else if(ichan%8>=2) F5 = BrhoF5(q2,ichan/8)*FKrho(s1,s2,(ichan-2)%8)*sqrt(2.);
  }
  // calculate the K0 pi- K0bar
  else if(imode==3) {
    Complex a1fact(a1BreitWigner(q2)*sqrt(2.)/3.);
    if(ichan<0) {
      F1 =-a1fact*BKstarF123(s1,-1);
      F2 = a1fact*BrhoF123(s2,-1);
      F5 =-BrhoF5(q2,-1)*FKrho(s1,s2,-1)*sqrt(2.);
    }
    else if(ichan%8==0) F1 = -a1fact*BKstarF123(s1,ichan/8);
    else if(ichan%8==1) F2 = a1fact*BrhoF123(s2,(ichan-1)/8);
    else if(ichan%8>=2) F5 = -BrhoF5(q2,ichan/8)*FKrho(s1,s2,(ichan-2)%8)*sqrt(2.);
  }
  // calculate the K- pi0 k0
  else if(imode==4) {
    Complex a1fact(a1BreitWigner(q2));
    if(ichan<0){F2 =-a1fact*BrhoF123(s2,-1);}
    else{F2 =-a1fact*BrhoF123(s2,ichan);}
  }
  // calculate the pi0 pi0 K-
  else if(imode==5) {
    Complex K1fact(K1BreitWigner(q2)/6.);
    if(ichan<0) {
      F1 = K1fact*BKstarF123(s1,-1);
      F2 =-K1fact*BKstarF123(s2,-1);
    }
    else if(ichan%2==0) F1 = K1fact*BKstarF123(s1,ichan/2);
    else                F2 =-K1fact*BKstarF123(s2,(ichan-1)/2);
  }
  // calculate the K- pi- pi+
  else if(imode==6) {
    Complex K1fact(K1BreitWigner(q2)*sqrt(2.)/3.);
    if(ichan<0) {
      F1 =-K1fact*BrhoF123(s1,-1);
      F2 = K1fact*BKstarF123(s2,-1);
      F5 =-BKstarF123(q2,-1)*FKrho(s2,s1,-1)*sqrt(2.);
    }
    else if(ichan%8==0) F1 =-K1fact*BrhoF123(s1,ichan/8);
    else if(ichan%8==1) F2 = K1fact*BKstarF123(s2,(ichan-1)/8);
    else                F5 = -BKstarF123(q2,ichan/8)*FKrho(s2,s1,(ichan-2)%8)*sqrt(2.);
  }
  // calculate the pi- K0bar pi0
  else if(imode==7) {
    Complex K1fact(K1BreitWigner(q2));
    if(ichan<0) {
      F2 =-K1fact*BrhoF123(s2,-1);
      F5 =-2.*BKstarF123(q2,-1)*FKrho(s1,s2,-1);
    }
    else if(ichan%7==0) F2 =-K1fact*BrhoF123(s2,ichan/7);
    else                F5 =-2.*BKstarF123(q2,ichan/7)*FKrho(s1,s2,(ichan-1)%7);
  }
  // calculate the pi- pi0 eta
  else if(imode==8) {
    if(ichan<0) F5 = BrhoF5(q2,     -1)*BrhoF123(s3,     -1)*sqrt(2./3.);
    else        F5 = BrhoF5(q2,ichan/3)*BrhoF123(s3,ichan%3)*sqrt(2./3.);
  }
  // multiply by the prefactors
  using Constants::twopi;
  return FormFactors(F1/_fpi,
		     F2/_fpi,
		     F3/_fpi,
		     F4/_fpi,
		     -F5/sqr(twopi)/pow<3,1>(_fpi)
		     );
}

// complete the construction of the decay mode for integration
bool ThreeMesonDefaultCurrent::createMode(int icharge, unsigned int imode,
					  DecayPhaseSpaceModePtr mode,
					  unsigned int iloc,unsigned int ires,
					  DecayPhaseSpaceChannelPtr phase,Energy upp) {
  int iq(0),ia(0);
  if(!acceptMode(imode)) return false;
  tPDVector extpart(particles(1,imode,iq,ia));
  Energy min(ZERO);
  for(unsigned int ix=0;ix<extpart.size();++ix) min+=extpart[ix]->massMin();
  if(min>upp) return false;
  // the particles we will use a lot
  tPDPtr a1,k1;
  if(icharge==-3) {
    a1=getParticleData(ParticleID::a_1minus);
    k1=getParticleData(ParticleID::Kstar_1minus);
  }
  else if(icharge==3) {
    a1=getParticleData(ParticleID::a_1plus);
    k1=getParticleData(ParticleID::Kstar_1plus);
  }
  else {
    return false;
  }
  _maxmass=max(_maxmass,upp);
  // the rho0 resonances
  tPDPtr rho0[3]={getParticleData(113),getParticleData(100113),getParticleData(30113)};
  tPDPtr rhoc[3],Kstar0[3],Kstarc[3];
  if(icharge==-3) {
    // the charged rho resonances
    rhoc[0] = getParticleData(-213);
    rhoc[1] = getParticleData(-100213);
    rhoc[2] = getParticleData(-30213);
    // the K*0 resonances
    Kstar0[0] = getParticleData(313);
    Kstar0[1] = getParticleData(100313);
    Kstar0[2] = getParticleData(30313);
    // the charged K* resonances
    Kstarc[0] = getParticleData(-323);
    Kstarc[1] = getParticleData(-100323);
    Kstarc[2] = getParticleData(-30323);
  }
  else {
    // the charged rho resonances
    rhoc[0] = getParticleData(213);
    rhoc[1] = getParticleData(100213);
    rhoc[2] = getParticleData(30213);
    // the K*0 resonances
    Kstar0[0] = getParticleData(-313);
    Kstar0[1] = getParticleData(-100313);
    Kstar0[2] = getParticleData(-30313);
    // the charged K* resonances
    Kstarc[0] = getParticleData(323);
    Kstarc[1] = getParticleData(100323);
    Kstarc[2] = getParticleData(30323);
  }
  DecayPhaseSpaceChannelPtr newchannel;
  if(imode==0) {
    // channels for pi- pi- pi+
    for(unsigned int ix=0;ix<3;++ix) {
      if(!rho0[ix]) continue;
      newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1      ,0,0.0,-ires-1,iloc);
      newchannel->addIntermediate(rho0[ix],0,0.0, iloc+1,iloc+2);
      mode->addChannel(newchannel);
      newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1      ,0,0.0,-ires-1,iloc+1);
      newchannel->addIntermediate(rho0[ix],0,0.0, iloc,iloc+2);
      mode->addChannel(newchannel);
    }
  }
  else if(imode==1) {
    // channels for pi0 pi0 pi-
    for(unsigned int ix=0;ix<3;++ix) {
      if(!rhoc[ix]) continue;
      newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1      ,0,0.0,-ires-1,iloc);
      newchannel->addIntermediate(rhoc[ix],0,0.0, iloc+1,iloc+2);
      mode->addChannel(newchannel);
      newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1      ,0,0.0,-ires-1,iloc+1);
      newchannel->addIntermediate(rhoc[ix],0,0.0, iloc,iloc+2);
      mode->addChannel(newchannel);
    }
  }
  else if(imode==2) {
    // channels for K- pi- K+
    for(unsigned int ix=0;ix<3;++ix) {
      if(Kstar0[ix]) {
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(a1        ,0,0.0,-ires-1,iloc);
	newchannel->addIntermediate(Kstar0[ix],0,0.0, iloc+1,iloc+2);
	mode->addChannel(newchannel);
      }
      if(rho0[ix]) {
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(a1      ,0,0.0,-ires-1,iloc+1);
	newchannel->addIntermediate(rho0[ix],0,0.0,iloc,iloc+2);
	mode->addChannel(newchannel);
      }
      for(unsigned int iy=0;iy<3;++iy) {
	if(!rhoc[ix]) continue;
	if(Kstar0[iy]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(rhoc[ix]  ,0,0.0,-ires-1,iloc);
	  newchannel->addIntermediate(Kstar0[iy],0,0.0, iloc+1,iloc+2);
	  mode->addChannel(newchannel);
	}
	if(rho0[iy]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(rhoc[ix],0,0.0,-ires-1,iloc+1);
	  newchannel->addIntermediate(rho0[iy],0,0.0,iloc,iloc+2);
	  mode->addChannel(newchannel);
	}
      }
    }
  }
  else if(imode==3) {
    // channels for K0 pi- K0bar
    for(unsigned int ix=0;ix<3;++ix) {
      if(Kstarc[ix]) {
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(a1        ,0,0.0,-ires-1,iloc);
	newchannel->addIntermediate(Kstarc[ix],0,0.0, iloc+1,iloc+2);
	mode->addChannel(newchannel);
      }
      if(rho0[ix]) {
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(a1      ,0,0.0,-ires-1,iloc+1);
	newchannel->addIntermediate(rho0[ix],0,0.0, iloc,iloc+2);
	mode->addChannel(newchannel);
      }
      for(unsigned int iy=0;iy<3;++iy) {
	if(!rhoc[ix]) continue;
	if(Kstarc[iy]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(rhoc[ix]  ,0,0.0,-ires-1,iloc);
	  newchannel->addIntermediate(Kstarc[iy],0,0.0, iloc+1,iloc+2);
	  mode->addChannel(newchannel);
	}
	if(rho0[iy]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(rhoc[ix],0,0.0,-ires-1,iloc+1);
	  newchannel->addIntermediate(rho0[iy],0,0.0, iloc,iloc+2);
	  mode->addChannel(newchannel);
	}
      }
    }
  }
  else if(imode==4) {
    // channels for K- pi0 K0
    for(unsigned int ix=0;ix<3;++ix) {
      if(!rhoc[ix]) continue;
      newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1      ,0,0.0,-ires-1,iloc+1);
      newchannel->addIntermediate(rhoc[ix],0,0.0, iloc,iloc+2);
      mode->addChannel(newchannel);
    }
  }
  else if(imode==5) {  
    // channels for pi0 pi0 K-
    for(unsigned int ix=0;ix<3;++ix) {
      if(!Kstarc[ix]) continue;
      newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(k1        ,0,0.0,-ires-1,iloc);
      newchannel->addIntermediate(Kstarc[ix],0,0.0, iloc+1,iloc+2);
      mode->addChannel(newchannel);
      newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(k1        ,0,0.0,-ires-1,iloc+1);
      newchannel->addIntermediate(Kstarc[ix],0,0.0, iloc,iloc+2);
      mode->addChannel(newchannel);
    }
  }
  else if(imode==6) {
    // channels for K- pi- pi+
    for(unsigned int ix=0;ix<3;++ix) {
      if(rho0[ix]) {
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(k1      ,0,0.0,-ires-1,iloc);
	newchannel->addIntermediate(rho0[ix],0,0.0, iloc+1,iloc+2);
	mode->addChannel(newchannel);
      }
      if(Kstar0[ix]) {
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(k1        ,0,0.0,-ires-1,iloc+1);
	newchannel->addIntermediate(Kstar0[ix],0,0.0, iloc,iloc+2);
	mode->addChannel(newchannel);
      }
      for(unsigned int iy=0;iy<3;++iy) {
	if(!Kstarc[ix]) continue;
	if(rho0[iy]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(Kstarc[ix],0,0.0,-ires-1,iloc);
	  newchannel->addIntermediate(rho0[iy]  ,0,0.0, iloc+1,iloc+2);
	  mode->addChannel(newchannel);
	}
	if(Kstar0[iy]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(Kstarc[ix],0,0.0,-ires-1,iloc+1);
	  newchannel->addIntermediate(Kstar0[iy],0,0.0, iloc,iloc+2);
	  mode->addChannel(newchannel);
	}
      }
    }
  }
  else if(imode==7) {
    // channels for pi- kbar0 pi0
    for(unsigned int ix=0;ix<3;++ix) {
      if(rhoc[ix]) {
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(k1      ,0,0.0,-ires-1,iloc+1);
	newchannel->addIntermediate(rhoc[ix],0,0.0, iloc,iloc+2);
	mode->addChannel(newchannel);
      }
      for(unsigned int iy=0;iy<3;++iy) {
	if(!Kstarc[ix]) continue;
	if(Kstar0[iy]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(Kstarc[ix],0,0.0,-ires-1,iloc);
	  newchannel->addIntermediate(Kstar0[iy],0,0.0, iloc+1,iloc+2);
	  mode->addChannel(newchannel);
	}
	if(rhoc[iy]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(Kstarc[ix],0,0.0,-ires-1,iloc+1);
	  newchannel->addIntermediate(rhoc[iy]  ,0,0.0, iloc,iloc+2);
	  mode->addChannel(newchannel);
	}
      }
    }
  }
  else if(imode==8) {
    // channels for pi- pi0 eta
    for(unsigned int ix=0;ix<3;++ix) {
      if(!rhoc[ix]) continue;
      for(unsigned int iy=0;iy<3;++iy) {
	if(!rho0[iy]) continue;
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(rhoc[ix],0,0.0,-ires-1,iloc);
	newchannel->addIntermediate(rho0[iy],0,0.0, iloc+1,iloc+2);
	mode->addChannel(newchannel);
      }
    }
  }
  if(_rhoparameters) {
    if(imode!=8) {
      for(unsigned int ix=0;ix<_rhoF123masses.size();++ix) {
	if(rhoc[ix]) mode->resetIntermediate(rhoc[ix],_rhoF123masses[ix],
					     _rhoF123widths[ix]);
	if(rho0[ix]) mode->resetIntermediate(rho0[ix],_rhoF123masses[ix],
					     _rhoF123widths[ix]);
      }
    }
    else {
      for(unsigned int ix=0;ix<_rhoF5masses.size();++ix) {
	if(rhoc[ix]) mode->resetIntermediate(rhoc[ix],_rhoF5masses[ix],
					     _rhoF5widths[ix]);
	if(rho0[ix]) mode->resetIntermediate(rho0[ix],_rhoF5masses[ix],
					     _rhoF5widths[ix]);
      }
    }
  }
  // K star parameters in the base class
  if(_kstarparameters) {
    for(unsigned int ix=0;ix<_kstarF123masses.size();++ix) {
      if(Kstarc[ix]) mode->resetIntermediate(Kstarc[ix],_kstarF123masses[ix],
					     _kstarF123widths[ix]);
      if(Kstar0[ix]) mode->resetIntermediate(Kstar0[ix],_kstarF123masses[ix],
					     _kstarF123widths[ix]);
    }
  }
  return true;
}

// initialisation of the a_1 width
// (iopt=-1 initialises, iopt=0 starts the interpolation)
void ThreeMesonDefaultCurrent::inita1Width(int iopt) {
  if(iopt==-1) {
    _maxcalc=_maxmass;
    if(!_initializea1||_maxmass==ZERO) return;
    // parameters for the table of values
    Energy2 step(sqr(_maxcalc)/199.);
    // integrator to perform the integral
    vector<double> inweights;inweights.push_back(0.5);inweights.push_back(0.5);
    vector<int> intype;intype.push_back(2);intype.push_back(3);
    Energy mrho(getParticleData(ParticleID::rhoplus)->mass()),
      wrho(getParticleData(ParticleID::rhoplus)->width());
    vector<Energy> inmass(2,mrho),inwidth(2,wrho);
    vector<double> inpow(2,0.0);
    ThreeBodyAllOnCalculator<ThreeMesonDefaultCurrent> 
      widthgen(inweights,intype,inmass,inwidth,inpow,*this,0,_mpi,_mpi,_mpi);
    // normalisation constant to give physical width if on shell
    double a1const(_a1width/(widthgen.partialWidth(sqr(_a1mass))));
    // loop to give the values
    _a1runq2.clear(); _a1runwidth.clear();
    for(Energy2 moff2(ZERO); moff2<=sqr(_maxcalc); moff2+=step) {
      _a1runwidth.push_back(widthgen.partialWidth(moff2)*a1const);
      _a1runq2.push_back(moff2);
    }
  }
  // set up the interpolator
  else if(iopt==0) {
    _a1runinter = make_InterpolatorPtr(_a1runwidth,_a1runq2,3);
  }
}

void ThreeMesonDefaultCurrent::dataBaseOutput(ofstream & output,bool header,
					      bool create) const {
  if(header) output << "update decayers set parameters=\"";
  if(create) output << "create Herwig::ThreeMesonDefaultCurrent " 
		    << name() << " HwWeakCurrents.so\n";
  for(unsigned int ix=0;ix<_rhoF123wgts.size();++ix) {
    if(ix<3) output << "newdef ";
    else     output << "insert ";
    output << name() << ":F123RhoWeight " << ix << " " << _rhoF123wgts[ix] << "\n";
  }
  for(unsigned int ix=0;ix<_kstarF123wgts.size();++ix) {
    if(ix<1) output << "newdef ";
    else     output << "insert ";
    output << name() << ":F123KstarWeight " << ix << " " 
	   << _kstarF123wgts[ix] << "\n";
  }
  for(unsigned int ix=0;ix<_rhoF5wgts.size();++ix) {
    if(ix<3) output << "newdef ";
    else     output << "insert ";
    output << name() << ":F5RhoWeight " << ix << " " << _rhoF5wgts[ix] << "\n";
  }
  for(unsigned int ix=0;ix<_kstarF5wgts.size();++ix) {
    if(ix<1) output << "newdef ";
    else     output << "insert ";
    output << name() << ":F5KstarWeight " << ix << " " << _kstarF5wgts[ix] << "\n";
  }
  output << "newdef " << name() << ":RhoKstarWgt "     << _rhoKstarwgt     << "\n";
  output << "newdef " << name() << ":Initializea1 "    << _initializea1    << "\n";
  output << "newdef " << name() << ":RhoParameters "   << _rhoparameters   << "\n";
  output << "newdef " << name() << ":KstarParameters " << _kstarparameters << "\n";
  output << "newdef " << name() << ":a1Parameters "    << _a1parameters    << "\n";
  output << "newdef " << name() << ":K1Parameters "    << _k1parameters    << "\n";
  output << "newdef " << name() << ":a1WidthOption "   << _a1opt           << "\n";
  for(unsigned int ix=0;ix<_a1runwidth.size();++ix) {
    output << "newdef " << name() << ":a1RunningWidth " << ix 
	   << " " << _a1runwidth[ix]/GeV << "\n";
  }
  for(unsigned int ix=0;ix<_a1runq2.size();++ix) {
    output << "newdef " << name() << ":a1RunningQ2 " << ix 
	   << " " << _a1runq2[ix]/GeV2 << "\n";
  }
  output << "newdef " << name() << ":A1Width " << _a1width/GeV << "\n";
  output << "newdef " << name() << ":A1Mass "  << _a1mass/GeV  << "\n";
  output << "newdef " << name() << ":K1Width " << _k1width/GeV << "\n";
  output << "newdef " << name() << ":K1Mass "  << _k1mass/GeV  << "\n";
  output << "newdef " << name() << ":FPi "     << _fpi/MeV     << "\n";
  for(unsigned int ix=0;ix<_rhoF123masses.size();++ix) {
    if(ix<3) output << "newdef ";
    else     output << "insert ";
    output << name() << ":rhoF123masses " << ix 
	   << " " << _rhoF123masses[ix]/GeV << "\n";
  }
  for(unsigned int ix=0;ix<_rhoF123widths.size();++ix) {
    if(ix<3) output << "newdef ";
    else     output << "insert ";
    output << name() << ":rhoF123widths " << ix << " " 
	   << _rhoF123widths[ix]/GeV << "\n";
  }
  for(unsigned int ix=0;ix<_rhoF5masses.size();++ix) {
    if(ix<3) output << "newdef ";
    else     output << "insert ";
    output << name() << ":rhoF5masses " << ix << " " 
	   << _rhoF5masses[ix]/GeV << "\n";
  }
  for(unsigned int ix=0;ix<_rhoF5widths.size();++ix) {
    if(ix<3) output << "newdef ";
    else     output << "insert ";
    output << name() << ":rhoF5widths " << ix << " " 
	   << _rhoF5widths[ix]/GeV << "\n";
  }
  for(unsigned int ix=0;ix<_kstarF123masses.size();++ix) {
    if(ix<1) output << "newdef ";
    else     output << "insert ";
    output << name() << ":KstarF123masses " << ix << " " 
	   << _kstarF123masses[ix]/GeV << "\n";
  }
  for(unsigned int ix=0;ix<_kstarF123widths.size();++ix) {
    if(ix<1) output << "newdef ";
    else     output << "insert ";
    output << name() << ":KstarF123widths " << ix << " " 
	   << _kstarF123widths[ix]/GeV << "\n";
  }
  for(unsigned int ix=0;ix<_kstarF5masses.size();++ix) {
    if(ix<1) output << "newdef ";
    else     output << "insert ";
    output << name() << ":KstarF5masses " << ix << " " 
	   << _kstarF5masses[ix]/GeV << "\n";
  }
  for(unsigned int ix=0;ix<_kstarF5widths.size();++ix) {
    if(ix<1) output << "newdef ";
    else     output << "insert ";
    output << name() << ":KstarF5widths " << ix << " " 
	   << _kstarF5widths[ix]/GeV << "\n";
  }
  ThreeMesonCurrentBase::dataBaseOutput(output,false,false);
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}

void ThreeMesonDefaultCurrent::doinitrun() {
  // set up the running a_1 width
  inita1Width(0);
  ThreeMesonCurrentBase::doinitrun();
}

void ThreeMesonDefaultCurrent::doupdate() {
  ThreeMesonCurrentBase::doupdate();
  // update running width if needed
  if ( !touched() ) return;
  if(_maxmass!=_maxcalc) inita1Width(-1);
}

Complex ThreeMesonDefaultCurrent::rhoKBreitWigner(Energy2 q2,unsigned int itype,
							 unsigned int ires) const {
  Energy q(sqrt(q2)),mass,width,mout[2]={_mpi,_mpi};
  // get the mass and width of the requested resonance
  if(itype==0) {
    mass=_rhoF123masses[ires];
    width=_rhoF123widths[ires];
  }
  else if(itype==1) {
    mass=_rhoF5masses[ires];
    width=_rhoF5widths[ires];
  }
  else if(itype==2) {
    mass=_kstarF123masses[ires];
    width=_kstarF123widths[ires];
  }
  else if(itype==3) {
    mass=_kstarF5masses[ires];
    width=_kstarF5widths[ires];
  }
  else {
    return 0.;
  }
  // calculate the momenta for the running widths
  if(itype>1) mout[0]=_mK;
  Energy pcm0(Kinematics::pstarTwoBodyDecay(mass,mout[0],mout[1]));
  Energy pcm(ZERO);
  if(mout[0]+mout[1]<q){pcm=Kinematics::pstarTwoBodyDecay(q,mout[0],mout[1]);}
  double ratio = Math::Pow<3>(pcm/pcm0);
  Energy gamrun(width*mass*ratio/q);
  Complex ii(0.,1.);
  complex<Energy2> denom(q2-mass*mass+ii*mass*gamrun), numer(-mass*mass);
  return numer/denom;
}

double ThreeMesonDefaultCurrent::
threeBodyMatrixElement(const int       , const Energy2 q2,
		       const Energy2 s3, const Energy2 s2, 
		       const Energy2 s1, const Energy    , 
		       const Energy    , const Energy    ) const {
  Energy2 mpi2(sqr(_mpi));
  Complex propb(BrhoF123(s1,-1)),propa(BrhoF123(s2,-1)); 
  // the matrix element
  Energy2 output(ZERO); 
  // first resonance
  output += ((s1-4.*mpi2) + 0.25*(s3-s2)*(s3-s2)/q2) * real(propb*conj(propb)); 
  // second resonance
  output += ((s2-4.*mpi2) + 0.25*(s3-s1)*(s3-s1)/q2) * real(propa*conj(propa)); 
  // the interference term 
  output += (0.5*q2-s3-0.5*mpi2+0.25*(s3-s2)*(s3-s1)/q2)*real(propa*conj(propb)+
							      propb*conj(propa)); 
  return output/sqr(_rhoF123masses[0]);
}
#line 1 "./ThreePionCLEOCurrent.cc"
// -*- C++ -*-
//
// ThreePionCLEOCurrent.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the ThreePionCLEOCurrent class.
//

#include "ThreePionCLEOCurrent.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Interface/Switch.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "Herwig/PDT/ThreeBodyAllOnCalculator.h"
#include "ThePEG/Utilities/DescribeClass.h"

using namespace Herwig;

DescribeClass<ThreePionCLEOCurrent,ThreeMesonCurrentBase>
describeHerwigThreePionCLEOCurrent("Herwig::ThreePionCLEOCurrent",
				   "HwWeakCurrents.so");
HERWIG_INTERPOLATOR_CLASSDESC(ThreePionCLEOCurrent,Energy,Energy2)

ThreePionCLEOCurrent::ThreePionCLEOCurrent() {
  // local particle properties
  _localparameters=true;
  // rho masses and widths
  if(_rhomass.size()==0) {
    _rhomass.push_back(0.7743*GeV);_rhowidth.push_back(0.1491*GeV);
    _rhomass.push_back(1.370*GeV);_rhowidth.push_back(0.386*GeV);
  }
  // f_2 mass and width
  _f2mass=1.275*GeV;_f2width=0.185*GeV;
  // f_0(1370) mass and width
  _f0mass=1.186*GeV;_f0width=0.350*GeV;
  // sigma mass and width
  _sigmamass = 0.860*GeV;_sigmawidth =0.880*GeV;
  // a1 mass and width
  _a1mass = 1.331*GeV;_a1width=0.814*GeV;
  // parameters for the K K* contribution to the a_1 running width
  _mKstar = 894*MeV;
  _mK=496*MeV;
  _gammk=3.32;
  // pion decay constant
  _fpi = 130.7*MeV/sqrt(2.);
  // couplings and phases for the different channels
  // p-wave rho and rho prime
  using Constants::pi;
  if(_rhomagP.size()==0) {
    _rhomagP.push_back(1.)  ;_rhophaseP.push_back(0.);
    _rhomagP.push_back(0.12);_rhophaseP.push_back(0.99*pi);
  }
  // d-wave rho and rho prime
  if(_rhomagD.size()==0) {
    _rhomagD.push_back(0.37/GeV2);_rhophaseD.push_back(-0.15*pi);
    _rhomagD.push_back(0.87/GeV2);_rhophaseD.push_back( 0.53*pi);
  }
  // f_2
  _f2mag=0.71/GeV2;_f2phase=0.56*pi;_f2coup=ZERO;
  // sigma
  _sigmamag=2.10;_sigmaphase=0.23*pi;_sigmacoup=0.;
  // f_0
  _f0mag=0.77;_f0phase=-0.54*pi;_f0coup=0.;
  // initialize the a_1 width
  _initializea1=false;
  _a1opt=true;
  double  a1q2in[200]={0      ,15788.6,31577.3,47365.9,63154.6,78943.2,
		       94731.9,110521 ,126309 ,142098 ,157886 ,173675 ,
		       189464 ,205252 ,221041 ,236830 ,252618 ,268407 ,     
		       284196 ,299984 ,315773 ,331562 ,347350 ,363139 ,     
		       378927 ,394716 ,410505 ,426293 ,442082 ,457871 ,     
		       473659 ,489448 ,505237 ,521025 ,536814 ,552603 ,      
		       568391 ,584180 ,599969 ,615757 ,631546 ,647334 ,      
		       663123 ,678912 ,694700 ,710489 ,726278 ,742066 ,      
		       757855 ,773644 ,789432 ,805221 ,821010 ,836798 ,      
		       852587 ,868375 ,884164 ,899953 ,915741 ,931530 ,     
		       947319 ,963107 ,978896 ,994685 ,     
		       1.01047e+06,1.02626e+06,1.04205e+06,1.05784e+06, 
		       1.07363e+06,1.08942e+06,1.10521e+06,1.12099e+06, 
		       1.13678e+06,1.15257e+06,1.16836e+06,1.18415e+06, 
		       1.19994e+06,1.21573e+06,1.23151e+06,1.24730e+06, 
		       1.26309e+06,1.27888e+06,1.29467e+06,1.31046e+06, 
		       1.32625e+06,1.34203e+06,1.35782e+06,1.37361e+06, 
		       1.38940e+06,1.40519e+06,1.42098e+06,1.43677e+06, 
		       1.45256e+06,1.46834e+06,1.48413e+06,1.49992e+06, 
		       1.51571e+06,1.53150e+06,1.54729e+06,1.56308e+06, 
		       1.57886e+06,1.59465e+06,1.61044e+06,1.62623e+06, 
		       1.64202e+06,1.65781e+06,1.67360e+06,1.68939e+06, 
		       1.70517e+06,1.72096e+06,1.73675e+06,1.75254e+06, 
		       1.76833e+06,1.78412e+06,1.79991e+06,1.81569e+06, 
		       1.83148e+06,1.84727e+06,1.86306e+06,1.87885e+06, 
		       1.89464e+06,1.91043e+06,1.92621e+06,1.94200e+06, 
		       1.95779e+06,1.97358e+06,1.98937e+06,2.00516e+06, 
		       2.02095e+06,2.03674e+06,2.05252e+06,2.06831e+06, 
		       2.08410e+06,2.09989e+06,2.11568e+06,2.13147e+06, 
		       2.14726e+06,2.16304e+06,2.17883e+06,2.19462e+06, 
		       2.21041e+06,2.22620e+06,2.24199e+06,2.25778e+06, 
		       2.27356e+06,2.28935e+06,2.30514e+06,2.32093e+06, 
		       2.33672e+06,2.35251e+06,2.36830e+06,2.38409e+06, 
		       2.39987e+06,2.41566e+06,2.43145e+06,2.44724e+06, 
		       2.46303e+06,2.47882e+06,2.49461e+06,2.51039e+06, 
		       2.52618e+06,2.54197e+06,2.55776e+06,2.57355e+06, 
		       2.58934e+06,2.60513e+06,2.62092e+06,2.63670e+06, 
		       2.65249e+06,2.66828e+06,2.68407e+06,2.69986e+06, 
		       2.71565e+06,2.73144e+06,2.74722e+06,2.76301e+06, 
		       2.77880e+06,2.79459e+06,2.81038e+06,2.82617e+06, 
		       2.84196e+06,2.85774e+06,2.87353e+06,2.88932e+06, 
		       2.90511e+06,2.92090e+06,2.93669e+06,2.95248e+06, 
		       2.96827e+06,2.98405e+06,2.99984e+06,3.01563e+06, 
		       3.03142e+06,3.04721e+06,3.06300e+06,3.07879e+06, 
		       3.09457e+06,3.11036e+06,3.12615e+06,3.14194e+06};
  double a1widthin[200]={0,0,0,0,0,0,0,0,
			 0,0,0,0.00021256,0.0107225,0.0554708,0.150142,0.303848,
			 0.522655,0.81121,1.1736,1.61381,2.13606,2.74499,3.44583,4.24454,
			 5.14795,6.16391,7.3014,8.57079,9.98398,11.5547,13.2987,15.2344,
			 17.3827,19.7683,22.4195,25.3695,28.6568,32.3264,36.4311,41.0322,
			 46.201,52.0203,58.5847,66.0011,74.3871,83.8666,94.5615,106.578,
			 119.989,134.807,150.968,168.315,186.615,205.576,224.893,244.28,
			 263.499,282.364,300.748,318.569,335.781,352.367,368.327,383.677,
			 398.438,412.638,426.306,439.472,452.167,464.421,476.263,487.719,
			 498.815,509.576,520.024,530.179,540.063,549.693,559.621,568.26,
			 577.229,586.005,594.604,603.035,611.314,619.447,627.446,635.321,
			 643.082,650.736,658.288,665.75,673.127,680.427,687.659,694.82,
			 701.926,708.977,715.983,722.944,729.862,736.752,743.619,750.452,
			 757.271,764.076,770.874,777.658,784.444,791.233,798.027,804.838,
			 811.649,818.485,825.342,832.224,839.139,846.082,853.059,860.079,
			 867.143,874.248,881.409,919.527,945.28,965.514,983.228,999.471,
			 1014.69,1029.15,1043.05,1056.49,1069.57,1082.36,1094.88,1107.2,
			 1120.89,1131.4,1143.33,1155.15,1166.92,1178.61,1190.27,1201.92,
			 1213.55,1225.18,1236.81,1250.06,1260.16,1271.86,1283.64,1295.46,
			 1307.36,1319.3,1331.34,1343.45,1355.64,1367.93,1380.31,1392.77,
			 1405.35,1418.03,1430.83,1443.75,1457.17,1469.94,1483.22,1496.64,
			 1510.18,1523.86,1537.67,1551.64,1565.72,1579.99,1594.38,1608.92,
			 1623.63,1642.08,1653.51,1668.69,1684.03,1699.53,1715.21,1731.04,
			 1747.05,1763.23,1779.59,1796.12,1812.83,1829.72,1846.79,1864.04,
			 1881.49,1899.11,1916.93,1934.93,1953.13,1971.52,1990.12,2008.89};
  if(_a1runwidth.empty()) {
    vector<double> tmp1(a1widthin,a1widthin+200);
    std::transform(tmp1.begin(), tmp1.end(),
		   back_inserter(_a1runwidth),
		   [](double x){return x*GeV;});
    
    vector<double> tmp2(a1q2in,a1q2in+200);
    _a1runq2.clear();
    std::transform(tmp2.begin(), tmp2.end(),
		   back_inserter(_a1runq2),
		   [](double x){return x*GeV2;});
  }
  // zero parameters which will be calculated later to avoid problems
  _pf2cc=ZERO; 
  _pf200=ZERO;
  _pf0cc=ZERO;
  _pf000=ZERO;
  _psigmacc=ZERO;
  _psigma00=ZERO; 
  _mpi0=ZERO;
  _mpic=ZERO;
  _fact=ZERO;
  _maxmass=ZERO;
  _maxcalc=ZERO;
}

void ThreePionCLEOCurrent::doinit() {
  ThreeMesonCurrentBase::doinit();
  // pointers to the particles we need
  tPDPtr a1m = getParticleData(ParticleID::a_1minus);
  // the different rho resonances
  tPDPtr rhom[3];
  rhom[0] = getParticleData(-213);
  rhom[1] = getParticleData(-100213);
  rhom[2] = getParticleData(-30213);
  // the sigma
  tPDPtr sigma = getParticleData(9000221);
  // the f_2
  tPDPtr f2=getParticleData(225);
  // the f_0
  tPDPtr f0=getParticleData(10221);
  if(_localparameters) {
    // make sure the rho array has enough masses
    if(_rhomass.size()<3) {
      for(unsigned int ix=_rhomass.size();ix<3;++ix) {
	_rhomass.push_back(rhom[ix]->mass());
	_rhowidth.push_back(rhom[ix]->width());
      }
    }
  }
  // set the local variables if needed
  else {
    // masses and widths for the particles
    _rhomass.resize(3);_rhowidth.resize(3);
    for(unsigned int ix=0;ix<3;++ix)
      {_rhomass[ix]=rhom[ix]->mass();_rhowidth[ix]=rhom[ix]->width();}
    _f2mass=f2->mass();_f2width=f2->width();
    _f0mass=f0->mass();_f0width=f0->width();
    _sigmamass=sigma->mass();_sigmawidth=sigma->width();
    _a1mass=a1m->mass();_a1width=a1m->width();
    _mKstar=getParticleData(ParticleID::Kstarplus)->mass();
    _mK =getParticleData(ParticleID::Kplus)->mass();
  }
  // parameters for the breit-wigners
  _mpic=getParticleData(ParticleID::piplus)->mass();
  _mpi0=getParticleData(ParticleID::pi0)->mass();
  // momenta of the decay products for on-shell particles
  _psigmacc=Kinematics::pstarTwoBodyDecay(_sigmamass,_mpic,_mpic);
  _psigma00=Kinematics::pstarTwoBodyDecay(_sigmamass,_mpi0,_mpi0);
  _pf2cc=Kinematics::pstarTwoBodyDecay(_f2mass,_mpic,_mpic);
  _pf200=Kinematics::pstarTwoBodyDecay(_f2mass,_mpi0,_mpi0); 
  _pf0cc=Kinematics::pstarTwoBodyDecay(_f0mass,_mpic,_mpic);
  _pf000=Kinematics::pstarTwoBodyDecay(_f0mass,_mpi0,_mpi0); 
  _prhocc.resize(3);_prhoc0.resize(3);
  for(unsigned int ix=0;ix<3;++ix) {
    _prhocc[ix]=Kinematics::pstarTwoBodyDecay(_rhomass[ix],_mpic,_mpic);
    _prhoc0[ix]=Kinematics::pstarTwoBodyDecay(_rhomass[ix],_mpic,_mpi0);
  }
  // couplings for the different modes
  Complex ii(0.,1.);
  _rhocoupP.resize(_rhomagP.size());
  for(unsigned int ix=0;ix<_rhomagP.size();++ix)
    {_rhocoupP[ix]=_rhomagP[ix]*(cos(_rhophaseP[ix])+ii*sin(_rhophaseP[ix]));}
  _rhocoupD.resize(_rhomagD.size());
  for(unsigned int ix=0;ix<_rhomagD.size();++ix)
    {_rhocoupD[ix]=_rhomagD[ix]*(cos(_rhophaseD[ix])+ii*sin(_rhophaseD[ix]));}
  _f0coup=_f0mag*(cos(_f0phase)+ii*sin(_f0phase));
  _f2coup=_f2mag*(cos(_f2phase)+ii*sin(_f2phase));
  _sigmacoup=_sigmamag*(cos(_sigmaphase)+ii*sin(_sigmaphase));
  // overall coupling
  _fact = 2.*sqrt(2.)/_fpi/3.;
  // initialise the a_1 running width calculation
  inita1Width(-1);
}

void ThreePionCLEOCurrent::persistentOutput(PersistentOStream & os) const {
  os << ounit(_rhomass,GeV) << ounit(_rhowidth,GeV) << ounit(_prhocc,GeV) << ounit(_prhoc0,GeV) 
     << ounit(_f2mass,GeV) << ounit(_f2width,GeV) << ounit(_pf2cc,GeV) 
     << ounit(_pf200,GeV) << ounit(_f0mass,GeV) << ounit(_f0width,GeV) 
     << ounit(_pf0cc,GeV) << ounit(_pf000,GeV) 
     << ounit(_sigmamass,GeV) << ounit(_sigmawidth,GeV)
     << ounit(_psigmacc,GeV) << ounit(_psigma00,GeV) << ounit(_mpi0,GeV) << ounit(_mpic,GeV) 
     << ounit(_fpi,GeV) << ounit(_fact,1/GeV) 
     << _rhomagP << _rhophaseP 
     << _rhocoupP << ounit(_rhomagD,1/GeV2) << _rhophaseD 
     << ounit(_rhocoupD,1/GeV2) <<ounit(_f2mag,1/GeV2) << _f2phase << ounit(_f2coup ,1/GeV2)
     << _f0mag << _f0phase << _f0coup << _sigmamag << _sigmaphase << _sigmacoup 
     << _localparameters 
     << ounit(_a1mass,GeV) << ounit(_a1width,GeV) << ounit(_a1runwidth,GeV) 
     << ounit(_a1runq2,GeV2) <<  _initializea1
     << ounit(_mKstar,GeV) << ounit(_mK,GeV) << _gammk << _a1opt 
     << ounit(_maxmass,GeV) << ounit(_maxcalc,GeV) << _a1runinter;
}

void ThreePionCLEOCurrent::persistentInput(PersistentIStream & is, int) {
  is >> iunit(_rhomass,GeV) >> iunit(_rhowidth,GeV) >> iunit(_prhocc,GeV) >> iunit(_prhoc0,GeV) 
     >> iunit(_f2mass,GeV) >> iunit(_f2width,GeV) >> iunit(_pf2cc,GeV)
     >> iunit(_pf200,GeV) >> iunit(_f0mass,GeV) >> iunit(_f0width,GeV) 
     >> iunit(_pf0cc,GeV) >> iunit(_pf000,GeV) 
     >> iunit(_sigmamass,GeV) >> iunit(_sigmawidth,GeV)
     >> iunit(_psigmacc,GeV) >> iunit(_psigma00,GeV) >> iunit(_mpi0,GeV) >> iunit(_mpic,GeV) 
     >> iunit(_fpi,GeV) >> iunit(_fact,1/GeV) 
     >> _rhomagP >> _rhophaseP 
     >> _rhocoupP >> iunit(_rhomagD,1/GeV2) >> _rhophaseD >> iunit(_rhocoupD,1/GeV2) 
     >> iunit(_f2mag,1/GeV2) >> _f2phase >> iunit(_f2coup,1/GeV2) 
     >> _f0mag >> _f0phase >> _f0coup >> _sigmamag >> _sigmaphase >> _sigmacoup 
     >> _localparameters 
     >> iunit(_a1mass,GeV) >> iunit(_a1width,GeV) >>  iunit(_a1runwidth,GeV) 
     >> iunit(_a1runq2,GeV2) >>  _initializea1
     >> iunit(_mKstar,GeV) >> iunit(_mK,GeV) >> _gammk >> _a1opt 
     >> iunit(_maxmass,GeV) >> iunit(_maxcalc,GeV) >> _a1runinter;
}

void ThreePionCLEOCurrent::Init() {

  static ClassDocumentation<ThreePionCLEOCurrent> documentation
    ("The ThreePionCLEOCurrent class performs the decay of the"
     " tau to three pions using the currents from CLEO",
     "The decay of tau to three pions is modelled using the currents from "
     "\\cite{Asner:1999kj}.",
     "  %\\cite{Asner:1999kj}\n"
     "\\bibitem{Asner:1999kj}\n"
     "  D.~M.~Asner {\\it et al.}  [CLEO Collaboration],\n"
     "   ``Hadronic structure in the decay tau- --> nu/tau pi- pi0 pi0 and the  sign\n"
     "  %of the tau neutrino helicity,''\n"
     "  Phys.\\ Rev.\\  D {\\bf 61}, 012002 (2000)\n"
     "  [arXiv:hep-ex/9902022].\n"
     "  %%CITATION = PHRVA,D61,012002;%%\n"
     );

  static ParVector<ThreePionCLEOCurrent,Energy> interfacerhomass
    ("RhoMasses",
     "The masses of the different rho resonnaces",
     &ThreePionCLEOCurrent::_rhomass,
     MeV, 0, ZERO, -10000*MeV, 10000*MeV, false, false, true);

  static ParVector<ThreePionCLEOCurrent,Energy> interfacerhowidth
    ("RhoWidths",
     "The widths of the different rho resonnaces",
     &ThreePionCLEOCurrent::_rhowidth,
     MeV, 0, ZERO, -10000*MeV, 10000*MeV, false, false, true);

  static Parameter<ThreePionCLEOCurrent,Energy> interfacef_2Mass
    ("f_2Mass",
     "The mass of the f_2 meson",
     &ThreePionCLEOCurrent::_f2mass, GeV, 1.275*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static Parameter<ThreePionCLEOCurrent,Energy> interfacef_2Width
    ("f_2Width",
     "The width of the f_2 meson",
     &ThreePionCLEOCurrent::_f2width, GeV, 0.185*GeV, ZERO, 1.0*GeV,
     false, false, true);

  static Parameter<ThreePionCLEOCurrent,Energy> interfacef_0Mass
    ("f_0Mass",
     "The mass of the f_0 meson",
     &ThreePionCLEOCurrent::_f0mass, GeV, 1.186*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static Parameter<ThreePionCLEOCurrent,Energy> interfacef_0Width
    ("f_0Width",
     "The width of the f_0 meson",
     &ThreePionCLEOCurrent::_f0width, GeV, 0.350*GeV, ZERO, 1.0*GeV,
     false, false, true);

  static Parameter<ThreePionCLEOCurrent,Energy> interfacesigmaMass
    ("sigmaMass",
     "The mass of the sigma meson",
     &ThreePionCLEOCurrent::_sigmamass, GeV, 0.860*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static Parameter<ThreePionCLEOCurrent,Energy> interfacesigmaWidth
    ("sigmaWidth",
     "The width of the sigma meson",
     &ThreePionCLEOCurrent::_sigmawidth, GeV, 0.880*GeV, ZERO, 2.0*GeV,
     false, false, true);

  static Parameter<ThreePionCLEOCurrent,Energy> interfacea1Mass
    ("a1Mass",
     "The mass of the a_1 meson",
     &ThreePionCLEOCurrent::_a1mass, GeV, 1.331*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static Parameter<ThreePionCLEOCurrent,Energy> interfacea1Width
    ("a1Width",
     "The width of the a_1 meson",
     &ThreePionCLEOCurrent::_a1width, GeV, 0.814*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static Parameter<ThreePionCLEOCurrent,Energy> interfaceKaonMass
    ("KaonMass",
     "The mass of the kaon",
     &ThreePionCLEOCurrent::_mK, GeV, 0.496*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static Parameter<ThreePionCLEOCurrent,Energy> interfaceKStarMass
    ("KStarMass",
     "The mass of the k* meson",
     &ThreePionCLEOCurrent::_mKstar, GeV, 0.894*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static Parameter<ThreePionCLEOCurrent,double> interfaceKaonCoupling
    ("KaonCoupling",
     "The relative coupling for the kaon in the a_1 running width",
     &ThreePionCLEOCurrent::_gammk, 3.32, 0.0, 10.0,
     false, false, true);

  static Parameter<ThreePionCLEOCurrent,Energy> interfaceFpi
    ("Fpi",
     "The pion decay constant",
     &ThreePionCLEOCurrent::_fpi, MeV, 130.7*MeV/sqrt(2.), ZERO, 500.0*MeV,
     false, false, true);

  static ParVector<ThreePionCLEOCurrent,double> interfacerhomagP
    ("RhoPWaveMagnitude",
     "The magnitude of the couplings for the p-wave rho currents",
     &ThreePionCLEOCurrent::_rhomagP,
     0, 0, 0, 0, 10000, false, false, true);

  static ParVector<ThreePionCLEOCurrent,double> interfacerhophaseP
    ("RhoPWavePhase",
     "The phase of the couplings for the p-wave rho currents",
     &ThreePionCLEOCurrent::_rhophaseP,
     0, 0, 0, -Constants::twopi, Constants::twopi, false, false, true);

  static ParVector<ThreePionCLEOCurrent,InvEnergy2> interfacerhomagD
    ("RhoDWaveMagnitude",
     "The magnitude of the couplings for the d-wave rho currents",
     &ThreePionCLEOCurrent::_rhomagD,
     1/MeV2, 0, ZERO, ZERO, 10000/MeV2, false, false, true);

  static ParVector<ThreePionCLEOCurrent,double> interfacerhophaseD
    ("RhoDWavePhase",
     "The phase of the couplings for the d-wave rho currents",
     &ThreePionCLEOCurrent::_rhophaseD,
     0, 0, 0, -Constants::twopi, Constants::twopi, false, false, true);

  static Parameter<ThreePionCLEOCurrent,double> interfacef0Phase
    ("f0Phase",
     "The phase of the f_0 scalar current",
     &ThreePionCLEOCurrent::_f0phase, 0.54*Constants::pi, -Constants::twopi, Constants::twopi,
     false, false, true);

  static Parameter<ThreePionCLEOCurrent,double> interfacef2Phase
    ("f2Phase",
     "The phase of the f_2 tensor current",
     &ThreePionCLEOCurrent::_f2phase, 0.56*Constants::pi,-Constants::twopi, Constants::twopi,
     false, false, true);

  static Parameter<ThreePionCLEOCurrent,double> interfacesigmaPhase
    ("sigmaPhase",
     "The phase of the sigma scalar current",
     &ThreePionCLEOCurrent::_sigmaphase, 0.23*Constants::pi, -Constants::twopi, Constants::twopi,
     false, false, true);

  static Parameter<ThreePionCLEOCurrent,double> interfacef0Magnitude
    ("f0Magnitude",
     "The magnitude of the f_0 scalar current",
     &ThreePionCLEOCurrent::_f0mag, 0.77, 0.0, 10,
     false, false, true);


  static Parameter<ThreePionCLEOCurrent,InvEnergy2> interfacef2Magnitude
    ("f2Magnitude",
     "The magnitude of the f_2 tensor current",
     &ThreePionCLEOCurrent::_f2mag, 1./GeV2, 0.71/GeV2, ZERO, 10./GeV2,
     false, false, true);

  static Parameter<ThreePionCLEOCurrent,double> interfacesigmaMagnitude
    ("sigmaMagnitude",
     "The magnitude of the sigma scalar current",
     &ThreePionCLEOCurrent::_sigmamag, 2.1, 0.0, 10,
     false, false, true);

  static Switch<ThreePionCLEOCurrent,bool> interfaceLocalParameters
    ("LocalParameters",
     "Use local values of the intermediate resonances masses and widths",
     &ThreePionCLEOCurrent::_localparameters, true, false, false);
  static SwitchOption interfaceLocalParametersLocal
    (interfaceLocalParameters,
     "Local",
     "Use the local values",
     true);
  static SwitchOption interfaceLocalParametersDefault
    (interfaceLocalParameters,
     "ParticleData",
     "Use the values from the particleData objects",
     false);

  static ParVector<ThreePionCLEOCurrent,Energy> interfacea1RunningWidth
    ("a1RunningWidth",
     "The values of the a_1 width for interpolation to giving the running width.",
     &ThreePionCLEOCurrent::_a1runwidth,
     MeV, 0, ZERO, ZERO, 10000000*MeV, false, false, true);
  
  static ParVector<ThreePionCLEOCurrent,Energy2> interfacea1RunningQ2
    ("a1RunningQ2",
     "The values of the q^2 for interpolation to giving the running width.",
     &ThreePionCLEOCurrent::_a1runq2,
     MeV2, 0, ZERO, ZERO, 10000000*MeV2, false, false, true);

  static Switch<ThreePionCLEOCurrent,bool> interfaceInitializea1
    ("Initializea1",
     "Initialise the calculation of the a_1 running width",
     &ThreePionCLEOCurrent::_initializea1, false, false, false);
  static SwitchOption interfaceInitializea1Initialization
    (interfaceInitializea1,
     "Yes",
     "Initialize the calculation",
     true);
  static SwitchOption interfaceInitializea1NoInitialization
    (interfaceInitializea1,
     "No",
     "Use the default values",
     false);
  
  static Switch<ThreePionCLEOCurrent,bool> interfacea1WidthOption
    ("a1WidthOption",
     "Option for the treatment of the a1 width",
     &ThreePionCLEOCurrent::_a1opt, true, false, false);
  static SwitchOption interfacea1WidthOptionLocal
    (interfacea1WidthOption,
     "Local",
     "Use a calculation of the running width based on the parameters as"
     " interpolation table.",
     true);
  static SwitchOption interfacea1WidthOptionParam
    (interfacea1WidthOption,
     "Kuhn",
     "Use the parameterization of Kuhn and Santamaria for default parameters."
     " This should only be used for testing vs TAUOLA",
     false);
}

// initialisation of the a_1 width
// (iopt=-1 initialises, iopt=0 starts the interpolation)
void ThreePionCLEOCurrent::inita1Width(int iopt) {
  if(iopt==-1) {
    _maxcalc=_maxmass;
    if(!_initializea1||_maxmass==ZERO) return;
    // parameters for the table of values
    Energy2 step=sqr(_maxmass)/200.;
    // function to be integrated to give the matrix element
    // integrator to perform the integral
    vector<double> inweights;inweights.push_back(0.5);inweights.push_back(0.5);
    vector<int> intype;intype.push_back(2);intype.push_back(3);
    Energy mrho=getParticleData(ParticleID::rhoplus)->mass();
    Energy wrho=getParticleData(ParticleID::rhoplus)->width();
    vector<Energy> inmass;inmass.push_back(mrho);inmass.push_back(mrho);
    vector<Energy> inwidth;inwidth.push_back(wrho);inwidth.push_back(wrho);
    vector<double> inpow(2,0.0);
    ThreeBodyAllOnCalculator<ThreePionCLEOCurrent>
      widthgenN(inweights,intype,inmass,inwidth,inpow,*this,0,_mpi0,_mpi0,_mpic);
    ThreeBodyAllOnCalculator<ThreePionCLEOCurrent>
      widthgenC(inweights,intype,inmass,inwidth,inpow,*this,1,_mpic,_mpic,_mpic);
    // normalisation constant to give physical width if on shell
    double a1const = _a1width/(widthgenN.partialWidth(sqr(_a1mass))+
			       widthgenC.partialWidth(sqr(_a1mass)));
    // loop to give the values
    _a1runq2.clear();_a1runwidth.clear();
    for(Energy2 moff2=ZERO; moff2<=sqr(_maxmass); moff2+=step) {
      Energy moff=sqrt(moff2);
      _a1runq2.push_back(moff2);
      Energy charged=a1const*widthgenC.partialWidth(moff2);
      Energy neutral=a1const*widthgenN.partialWidth(moff2);
      Energy kaon = moff<=_mK+_mKstar ? ZERO : 2.870*_gammk*_gammk/8./Constants::pi*
	Kinematics::pstarTwoBodyDecay(moff,_mK,_mKstar)/moff2*GeV2;
      Energy total = charged + neutral + kaon;
      _a1runwidth.push_back(total);
    }
  }
  // set up the interpolator
  else if(iopt==0) {
    _a1runinter = make_InterpolatorPtr(_a1runwidth,_a1runq2,3);
  }
}

// modes handled by this class
bool ThreePionCLEOCurrent::acceptMode(int imode) const {
  return imode>=0&&imode<=1;
}

ThreePionCLEOCurrent::FormFactors ThreePionCLEOCurrent::
  calculateFormFactors(const int ichan, const int imode,
		      Energy2 q2, Energy2 s1, Energy2 s2, Energy2 s3) const {
  Complex F1=0., F2=0., F3=0.;//F4=0.;F5=0.;
  // calculate the form factors without the a_1 piece
  CLEOFormFactor(imode,ichan,q2,s1,s2,s3,F1,F2,F3);
  // change sign of the f_2 term
  F2=-F2;
  // multiply by the a_1 factor
  complex<InvEnergy> a1fact = a1BreitWigner(q2) * _fact;
  return FormFactors(F1*a1fact, F2*a1fact, F3*a1fact);
}

void ThreePionCLEOCurrent::CLEOFormFactor(int imode,int ichan,
					  Energy2 q2,Energy2 s1, Energy2 s2, Energy2 s3,
					  Complex & F1, Complex & F2, 
					  Complex & F3) const {
  useMe();
  if(imode==0) {
    // compute the breit wigners we need
    Complex rhos1bw[3],rhos2bw[3],f0bws1,sigbws1,f2bws1,f0bws2,sigbws2,f2bws2;
    for(unsigned int ix=0,N=max(_rhocoupP.size(),_rhocoupD.size());ix<N;++ix) {
      rhos1bw[ix]=rhoBreitWigner(ix,s1,0);
      rhos2bw[ix]=rhoBreitWigner(ix,s2,0);
    }
    f0bws1  =f0BreitWigner(s1,0);
    sigbws1 =sigmaBreitWigner(s1,0);
    f2bws1  =f2BreitWigner(s1,0);
    f0bws2  =f0BreitWigner(s2,0);
    sigbws2 =sigmaBreitWigner(s2,0);
    f2bws2  =f2BreitWigner(s2,0);
    if(ichan<0) {
      // the p-wave rho terms
      for(unsigned int ix=0;ix<_rhocoupP.size();++ix) {
	F1-=_rhocoupP[ix]*rhos1bw[ix];
	F2-=_rhocoupP[ix]*rhos2bw[ix];
      }
      // the D-wave rho terms
      Energy2 Dfact1=1./3.*(s1-s3);
      Energy2 Dfact2=1./3.*(s2-s3);
      for(unsigned int ix=0;ix<_rhocoupD.size();++ix) {
	F1-=Complex(Dfact1*_rhocoupD[ix]*rhos2bw[ix]);
	F2-=Complex(Dfact2*_rhocoupD[ix]*rhos1bw[ix]);
	F3-=Complex(_rhocoupD[ix]*(Dfact2*rhos1bw[ix]-Dfact1*rhos2bw[ix]));
      }
      // the scalar terms
      F1-=2./3.*(_sigmacoup*sigbws2+_f0coup*f0bws2);
      F2-=2./3.*(_sigmacoup*sigbws1+_f0coup*f0bws1);
      F3+=-2./3.*(_sigmacoup*sigbws1+_f0coup*f0bws1)
	  +2./3.*(_sigmacoup*sigbws2+_f0coup*f0bws2);
      // the tensor terms
      complex<Energy2> sfact1 = 1./18.*(4.*_mpic*_mpic-s1)*(q2+s1-_mpic*_mpic)/s1*f2bws1;
      complex<Energy2> sfact2 = 1./18.*(4.*_mpic*_mpic-s2)*(q2+s2-_mpic*_mpic)/s2*f2bws2;
      F1+=Complex(_f2coup*(0.5*(s3-s2)*f2bws1-sfact2));
      F2+=Complex(_f2coup*(0.5*(s3-s1)*f2bws2-sfact1));
      F3+=Complex(_f2coup*(-sfact1+sfact2));
    }
    else if(ichan%2==0&&ichan<=4) {
      unsigned int ires=ichan/2;
      Energy2 Dfact2=1./3.*(s2-s3);
      if(ires<_rhocoupP.size()) F1-=_rhocoupP[ires]*rhos1bw[ires];
      if(ires<_rhocoupD.size()) {
	F2-=Complex(Dfact2*_rhocoupD[ires]*rhos1bw[ires]);
	F3-=Complex(_rhocoupD[ires]*Dfact2*rhos1bw[ires]);
      }
    }
    else if(ichan%2==1&&ichan<=5) {
      unsigned int ires=(ichan-1)/2;
      Energy2 Dfact1=1./3.*(s1-s3);
      if(ires<_rhocoupP.size()) {
	F2-=_rhocoupP[ires]*rhos2bw[ires];
      }
      if(ires<_rhocoupD.size()) {
	F1-=Complex(Dfact1*_rhocoupD[ires]*rhos2bw[ires]);
	F3+=Complex(_rhocoupD[ires]*Dfact1*rhos2bw[ires]);
      }
    }
    else if(ichan==6) {
      F2-=2./3.*_sigmacoup*sigbws1;
      F3-=2./3.*_sigmacoup*sigbws1;
    }
    else if(ichan==7) {
      F1-=2./3.*_sigmacoup*sigbws2;
      F3+=2./3.*_sigmacoup*sigbws2;
    }
    else if(ichan==8) {
      complex<Energy2> sfact1 = 1./18.*(4.*_mpic*_mpic-s1)*(q2+s1-_mpic*_mpic)/s1*f2bws1;
      F1+=Complex(_f2coup*0.5*(s3-s2)*f2bws1);
      F2-=Complex(_f2coup*sfact1);
      F3-=Complex(_f2coup*sfact1);
    }
    else if(ichan==9) {
      complex<Energy2> sfact2 = 1./18.*(4.*_mpic*_mpic-s2)*(q2+s2-_mpic*_mpic)/s2*f2bws2;
      F1-=Complex(_f2coup*sfact2);
      F2+=Complex(_f2coup*0.5*(s3-s1)*f2bws2);
      F3+=Complex(_f2coup*sfact2);
    }
    else if(ichan==10) {
      F2-=2./3.*_f0coup*f0bws1;
      F3-=2./3.*_f0coup*f0bws1;
    }
    else if(ichan==11) {
      F1-=2./3.*_f0coup*f0bws2;
      F3+=2./3.*_f0coup*f0bws2;
    }
  }
  // calculate the pi0 pi0 pi+ factor
  else if(imode==1) {
    // compute the breit wigners we need
    Complex rhos1bw[3],rhos2bw[3],f0bw,sigbw,f2bw;
    for(unsigned int ix=0,N=max(_rhocoupP.size(),_rhocoupD.size());ix<N;++ix) {
      rhos1bw[ix]=rhoBreitWigner(ix,s1,1);
      rhos2bw[ix]=rhoBreitWigner(ix,s2,1);
    }
    f0bw  =f0BreitWigner(s3,1);
    sigbw =sigmaBreitWigner(s3,1);
    f2bw  =f2BreitWigner(s3,1);
    if(ichan<0) {
      // the p-wave rho terms
      for(unsigned int ix=0;ix<_rhocoupP.size();++ix) {
	F1+=_rhocoupP[ix]*rhos1bw[ix];
	F2+=_rhocoupP[ix]*rhos2bw[ix];
      }
      // the D-wave rho terms
      Energy2 Dfact1=-1./3.*((s3-_mpic*_mpic)-(s1-_mpi0*_mpi0));
      Energy2 Dfact2=-1./3.*((s3-_mpic*_mpic)-(s2-_mpi0*_mpi0));
      for(unsigned int ix=0;ix<_rhocoupD.size();++ix) {
	F1+=Complex(Dfact1*_rhocoupD[ix]*rhos2bw[ix]);
	F2+=Complex(Dfact2*_rhocoupD[ix]*rhos1bw[ix]);
	F3+=Complex(_rhocoupD[ix]*(Dfact2*rhos1bw[ix]-Dfact1*rhos2bw[ix]));
      }
      // the scalar terms
      Complex scalar=2./3.*(_sigmacoup*sigbw+_f0coup*f0bw);
      F1+=scalar;F2+=scalar;
      // the tensor terms
      Complex Dfact3=1./18./s3*_f2coup*(q2-_mpic*_mpic+s3)*(4.*_mpi0*_mpi0-s3)*f2bw;
      F1+=Dfact3;F2+=Dfact3;
      F3-=Complex(0.5*_f2coup*(s1-s2)*f2bw);
    }
    else if(ichan%2==0&&ichan<=4) {
      unsigned int ires=ichan/2;
      if(ires<_rhocoupP.size()){F1+=_rhocoupP[ires]*rhos1bw[ires];}
      Energy2 Dfact2=-1./3.*((s3-_mpic*_mpic)-(s2-_mpi0*_mpi0));
      if(ires<_rhocoupD.size()) {
	F2+=Complex(Dfact2*_rhocoupD[ires]*rhos1bw[ires]);
	F3+=Complex(_rhocoupD[ires]*Dfact2*rhos1bw[ires]);
      }
    }
    else if(ichan%2==1&&ichan<=5) {
      unsigned int ires=(ichan-1)/2;
      if(ires<_rhocoupP.size()){F2+=_rhocoupP[ires]*rhos2bw[ires];}
      Energy2 Dfact1=-1./3.*((s3-_mpic*_mpic)-(s1-_mpi0*_mpi0));
      if(ires<_rhocoupD.size()) {
	F1+=Complex(Dfact1*_rhocoupD[ires]*rhos2bw[ires]);
	F3-=Complex(_rhocoupD[ires]*Dfact1*rhos2bw[ires]);
      }
    }
    else if(ichan==6) {
      F1+=2./3.*_sigmacoup*sigbw;
      F2+=2./3.*_sigmacoup*sigbw;
    }
    else if(ichan==7) {
      Complex Dfact3=1./18./s3*_f2coup*(q2-_mpic*_mpic+s3)*(4.*_mpi0*_mpi0-s3)*f2bw;
      F1+=Dfact3;F2+=Dfact3;
      F3-=Complex(0.5*_f2coup*(s1-s2)*f2bw);
    }
    else if(ichan==8) {
      F1+=2./3.*_f0coup*f0bw;
      F2+=2./3.*_f0coup*f0bw;
    }
  }
  else {
    throw DecayIntegratorError() << "ThreePionCLEOCurrent Unknown Decay" << imode
				 << Exception::abortnow;
  }
  // identical particle factors
  double fact = 1./sqrt(2.);
  F1*=fact;F2*=fact;F3*=fact;
}

// complete the construction of the decay mode for integration
bool ThreePionCLEOCurrent::createMode(int icharge, unsigned int imode,
				      DecayPhaseSpaceModePtr mode,
				      unsigned int iloc,unsigned int ires,
				      DecayPhaseSpaceChannelPtr phase,Energy upp) {
  if(!acceptMode(imode)){return false;}
  int iq(0),ia(0);
  tPDVector extpart=particles(1,imode,iq,ia);
  Energy min(ZERO);
  for(unsigned int ix=0;ix<extpart.size();++ix) min+=extpart[ix]->massMin();
  if(min>upp) return false;
  _maxmass=max(_maxmass,upp);
  // pointers to the particles we need
  tPDPtr a1m = getParticleData(ParticleID::a_1minus);
  // the different rho resonances
  tPDPtr rhom[3];
  if(icharge==-3) {
    rhom[0] = getParticleData(-213);
    rhom[1] = getParticleData(-100213);
    rhom[2] = getParticleData(-30213);
  }
  else if(icharge==3) {
    rhom[0] = getParticleData(213);
    rhom[1] = getParticleData(100213);
    rhom[2] = getParticleData(30213);
  }
  else {
    return false;
  }
  tPDPtr rho0[3] = {getParticleData(113),getParticleData(100113),
		    getParticleData(30113)};
  // the sigma
  tPDPtr sigma = getParticleData(9000221);
  // the f_2
  tPDPtr f2=getParticleData(225);
  // the f_0
  tPDPtr f0=getParticleData(10221);
  // set up the integration channels
  DecayPhaseSpaceChannelPtr newchannel;
  if(imode==0) {
    for(unsigned int ix=0;ix<3;++ix) {
      if(!rho0[ix]) continue;
      // the neutral rho channels
      // first channel
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m,0,0.0,-ires-1,iloc);
      newchannel->addIntermediate(rho0[ix],0,0.0,iloc+1,iloc+2);
      mode->addChannel(newchannel);
      // interchanged channel
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m,0,0.0,-ires-1,iloc+1);
      newchannel->addIntermediate(rho0[ix],0,0.0,iloc,iloc+2);
      mode->addChannel(newchannel);      
    }
    // the sigma channels
    if(sigma) {
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m,0,0.0,-ires-1,iloc);
      newchannel->addIntermediate(sigma,0,0.0,iloc+1,iloc+2);
      mode->addChannel(newchannel);
      // interchanged channel
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m,0,0.0,-ires-1,iloc+1);
      newchannel->addIntermediate(sigma,0,0.0,iloc,iloc+2);
      mode->addChannel(newchannel);
    }
    // the f_2 channels
    if(f2) {
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m,0,0.0,-ires-1,iloc);
      newchannel->addIntermediate(f2,0,0.0,iloc+1,iloc+2);
      mode->addChannel(newchannel);
      // interchanged channel
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m,0,0.0,-ires-1,iloc+1);
      newchannel->addIntermediate(f2,0,0.0,iloc,iloc+2);
      mode->addChannel(newchannel);
    }
    if(f0) {
      // the f_0 channel
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m,0,0.0,-ires-1,iloc);
      newchannel->addIntermediate(f0,0,0.0,iloc+1,iloc+2);
      mode->addChannel(newchannel);
      // interchanged channel
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m,0,0.0,-ires-1,iloc+1);
      newchannel->addIntermediate(f0,0,0.0,iloc,iloc+2);
      mode->addChannel(newchannel);
    }
  }
  else {
    for(unsigned int ix=0;ix<3;++ix) {
      if(!rhom[ix]) continue;
      // first rho+ channel
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m,0,0.0,-ires-1,iloc);
      newchannel->addIntermediate(rhom[ix],0,0.0,iloc+2,iloc+1);
      mode->addChannel(newchannel);
      // second rho+ channel
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m,0,0.0,-ires-1,iloc+1);
      newchannel->addIntermediate(rhom[ix],0,0.0,iloc+2,iloc);
      mode->addChannel(newchannel);
    }
    // the sigma channel
    if(sigma) {
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m,0,0.0,-ires-1,iloc+2);
      newchannel->addIntermediate(sigma,0,0.0,iloc,iloc+1);
      mode->addChannel(newchannel);
    }
    //  the f_2  channel
    if(f2) {
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m,0,0.0,-ires-1,iloc+2);
      newchannel->addIntermediate(f2,0,0.0,iloc,iloc+1);
      mode->addChannel(newchannel);
    }
    // the f_0 channel
    if(f0) {
      newchannel = new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m,0,0.0,-ires-1,iloc+2);
      newchannel->addIntermediate(f0,0,0.0,iloc,iloc+1);
      mode->addChannel(newchannel);
    }
  }
  if(_localparameters) {
    for(unsigned int iy=0;iy<_rhomass.size();++iy) {
      if(rho0[iy]) mode->resetIntermediate(rho0[iy],_rhomass[iy],_rhowidth[iy]);
      if(rhom[iy]) mode->resetIntermediate(rhom[iy],_rhomass[iy],_rhowidth[iy]);
    }
    if(sigma) mode->resetIntermediate(sigma,_sigmamass,_sigmawidth);
    if(f2)    mode->resetIntermediate(f2,_f2mass,_f2width);
    if(f0)    mode->resetIntermediate(f0,_f0mass,_f0width);
    mode->resetIntermediate(a1m,_a1mass,_a1width);
  }
  return true;
}

void ThreePionCLEOCurrent::dataBaseOutput(ofstream & output,bool header,
					  bool create) const {
  if(header){output << "update decayers set parameters=\"";}
  if(create) {
    output << "create Herwig::ThreePionCLEOCurrent " << name() 
	   << " HwWeakCurrents.so\n";
  }
  for(unsigned int ix=0;ix<_rhomass.size();++ix) {
    if(ix<2) {
      output << "newdef    " << name() << ":RhoMasses " << ix 
	     << " " << _rhomass[ix]/MeV << "\n";
    }
    else {
      output << "insert " << name() << ":RhoMasses " << ix 
	     << " " << _rhomass[ix]/MeV << "\n";
    }
  }
  for(unsigned int ix=0;ix<_rhowidth.size();++ix) {
    if(ix<2) {
      output << "newdef    " << name() << ":RhoWidths " << ix 
	     << " " << _rhowidth[ix]/MeV << "\n";
    }
    else {
      output << "insert " << name() << ":RhoWidths " << ix 
	     << " " << _rhowidth[ix]/MeV << "\n";
    }
  }
  output << "newdef " << name() << ":f_2Mass " << _f2mass/GeV << "\n";
  output << "newdef " << name() << ":f_2Width " << _f2width/GeV << "\n";
  output << "newdef " << name() << ":f_0Mass " << _f0mass/GeV << "\n";
  output << "newdef " << name() << ":f_0Width " << _f0width/GeV << "\n";
  output << "newdef " << name() << ":sigmaMass " << _sigmamass/GeV << "\n";
  output << "newdef " << name() << ":sigmaWidth " << _sigmawidth/GeV << "\n";
  output << "newdef " << name() << ":a1Mass " << _a1mass/GeV << "\n";
  output << "newdef " << name() << ":a1Width " <<_a1width /GeV << "\n";
  output << "newdef " << name() << ":KaonMass " << _mK/GeV << "\n";
  output << "newdef " << name() << ":KStarMass " << _mKstar/GeV << "\n";
  output << "newdef " << name() << ":KaonCoupling " << _gammk << "\n";
  output << "newdef " << name() << ":Fpi " << _fpi/MeV << "\n";
  output << "newdef " << name() << ":a1WidthOption " << _a1opt << "\n";
  for(unsigned int ix=0;ix<_rhomagP.size();++ix) {
      if(ix<2) {
	output << "newdef    " << name() << ":RhoPWaveMagnitude " << ix 
	       << " " << _rhomagP[ix] << "\n";
      }
      else {
	output << "insert " << name() << ":RhoPWaveMagnitude " << ix 
	       << " " << _rhomagP[ix] << "\n";
      }
  }
  for(unsigned int ix=0;ix<_rhophaseP.size();++ix) {
    if(ix<2) {
      output << "newdef    " << name() << ":RhoPWavePhase " << ix 
	     << " " << _rhophaseP[ix] << "\n";
    }
    else {
      output << "insert " << name() << ":RhoPWavePhase " << ix 
	     << " " << _rhophaseP[ix] << "\n";
    }
  }
  for(unsigned int ix=0;ix<_rhomagD.size();++ix) {
    if(ix<2) {
      output << "newdef    " << name() << ":RhoDWaveMagnitude " << ix 
	     << " " << _rhomagD[ix]*MeV2 << "\n";
    }
    else {
      output << "insert " << name() << ":RhoDWaveMagnitude " << ix 
	     << " " << _rhomagD[ix]*MeV2 << "\n";
    }
  }
  for(unsigned int ix=0;ix<_rhophaseD.size();++ix) {
    if(ix<2) {
      output << "newdef    " << name() << ":RhoDWavePhase " << ix 
	     << " " << _rhophaseD[ix] << "\n";
    }
    else {
      output << "insert " << name() << ":RhoDWavePhase " << ix 
	     << " " << _rhophaseD[ix] << "\n";
    }
  }
  output << "newdef " << name() << ":f0Phase " << _f0phase << "\n";
  output << "newdef " << name() << ":f2Phase " <<_f2phase  << "\n";
  output << "newdef " << name() << ":sigmaPhase " <<_sigmaphase  << "\n";
  output << "newdef " << name() << ":f0Magnitude " << _f0mag << "\n";
  output << "newdef " << name() << ":f2Magnitude " << _f2mag*GeV2 << "\n";
  output << "newdef " << name() << ":sigmaMagnitude " <<_sigmamag  << "\n";
  output << "newdef " << name() << ":LocalParameters " << _localparameters << "\n";
  output << "newdef " << name() << ":Initializea1 " <<_initializea1  << "\n";
  for(unsigned int ix=0;ix<_a1runwidth.size();++ix) {
    if(ix<200) {
      output << "newdef    " << name() << ":a1RunningWidth " << ix 
	     << " " << _a1runwidth[ix]/MeV << "\n";
    }
    else {
      output << "insert " << name() << ":a1RunningWidth " << ix 
	     << " " << _a1runwidth[ix]/MeV << "\n";
    }
  }
  for(unsigned int ix=0;ix<_a1runq2.size();++ix) {
    if(ix<200) {
      output << "newdef    " << name() << ":a1RunningQ2 " << ix 
	     << " " << _a1runq2[ix]/MeV2 << "\n";
    }
    else {
      output << "insert " << name() << ":a1RunningQ2 " << ix 
	     << " " << _a1runq2[ix]/MeV2 << "\n";
    }
  }
  ThreeMesonCurrentBase::dataBaseOutput(output,false,false);
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}

void ThreePionCLEOCurrent::doinitrun() {
  // set up the running a_1 width
  inita1Width(0);
  ThreeMesonCurrentBase::doinitrun();
}

void ThreePionCLEOCurrent::doupdate() {
  ThreeMesonCurrentBase::doupdate();
  // update running width if needed
  if ( !touched() ) return;
  if(_maxmass!=_maxcalc) inita1Width(-1);
}

Energy ThreePionCLEOCurrent::a1width(Energy2 q2) const {
  Energy output;
  if(_a1opt) output=(*_a1runinter)(q2);
  else {
    double gam(0.);
    if(q2<0.1753*GeV2) {
      gam =0.;
    }
    else if(q2<0.823*GeV2) {
      double p=q2/GeV2-0.1753;
      gam = 5.80900*p*sqr(p)*(1.-3.00980*p+4.57920*sqr(p));
    }
    else {
      double p=q2/GeV2;
      gam = -13.91400+27.67900*p-13.39300*sqr(p)
	+3.19240*sqr(p)*p-0.10487*sqr(sqr(p));
    }
    if(q2<0.1676*GeV2) {
      gam+=0.;
    }
    else if(q2<0.823*GeV2) {
      double p=q2/GeV2-0.1676;
      gam+= 6.28450*p*sqr(p)*(1.-2.95950*p+4.33550*sqr(p));
    }
    else {
      double p=q2/GeV2;
      gam+= -15.41100+32.08800*p-17.66600*sqr(p)
	+4.93550*sqr(p)*p-0.37498*sqr(sqr(p));
    }
    Energy mkst=0.894*GeV,mk=0.496*GeV;
    Energy2 mk1sq=sqr(mkst+mk), mk2sq=sqr(mkst-mk);
    double c3pi=sqr(0.2384),ckst=sqr(4.7621)*c3pi;
    gam*=c3pi;
    if(q2>mk1sq) gam+=0.5*ckst*sqrt((q2-mk1sq)*(q2-mk2sq))/q2;
    gam = gam*_a1width*_a1mass/GeV2/1.331/0.814/1.0252088;
    output = gam*GeV2/sqrt(q2);
  }
  return output;
}

double 
ThreePionCLEOCurrent::threeBodyMatrixElement(const int iopt, const Energy2 q2,
					     const Energy2 s3, const Energy2 s2, 
					     const Energy2 s1, const Energy,
					     const Energy, const Energy) const {
  Energy p1[5],p2[5],p3[5];
  Energy2 p1sq, p2sq, p3sq;
  Energy q=sqrt(q2);
  Energy2 mpi2c=_mpic*_mpic;
  Energy2 mpi20=_mpi0*_mpi0;
  // construct the momenta for the 2 neutral 1 charged mode
  Complex F1,F2,F3;
  if(iopt==0) {
    // construct the momenta of the decay products
    p1[0] = 0.5*(q2+mpi20-s1)/q; p1sq=p1[0]*p1[0]; p1[4]=sqrt(p1sq-mpi20);
    p2[0] = 0.5*(q2+mpi20-s2)/q; p2sq=p2[0]*p2[0]; p2[4]=sqrt(p2sq-mpi20);
    p3[0] = 0.5*(q2+mpi2c-s3)/q; p3sq=p3[0]*p3[0]; p3[4]=sqrt(p3sq-mpi2c);
    // take momentum of 1 parallel to z axis
    p1[1]=ZERO;p1[2]=ZERO;p1[3]=p1[4];
    // construct 2 
    double cos2 = 0.5*(p1sq+p2sq-p3sq-2.*mpi20+mpi2c)/p1[4]/p2[4];
    p2[1] = p2[4]*sqrt(1.-cos2*cos2); p2[2]=ZERO; p2[3]=-p2[4]*cos2;
    // construct 3
    double cos3 = 0.5*(p1sq-p2sq+p3sq-mpi2c)/p1[4]/p3[4];
    p3[1] =-p3[4]*sqrt(1.-cos3*cos3); p3[2]=ZERO; p3[3]=-p3[4]*cos3; 
    // calculate the form factors
    CLEOFormFactor(1,-1,q2,s1,s2,s3,F1,F2,F3);
  }
  // construct the momenta for the 3 charged mode 
  else {
    // construct the momenta of the decay products
    p1[0] = 0.5*(q2+mpi2c-s1)/q; p1sq=p1[0]*p1[0]; p1[4]=sqrt(p1sq-mpi2c);
    p2[0] = 0.5*(q2+mpi2c-s2)/q; p2sq=p2[0]*p2[0]; p2[4]=sqrt(p2sq-mpi2c);
    p3[0] = 0.5*(q2+mpi2c-s3)/q; p3sq=p3[0]*p3[0]; p3[4]=sqrt(p3sq-mpi2c);
    // take momentum of 1 parallel to z axis
    p1[1]=ZERO;p1[2]=ZERO;p1[3]=p1[4];
    // construct 2 
    double cos2 = 0.5*(p1sq+p2sq-p3sq-mpi2c)/p1[4]/p2[4];
    p2[1] = p2[4]*sqrt(1.-cos2*cos2); p2[2]=ZERO; p2[3]=-p2[4]*cos2;
    // construct 3
    double cos3 = 0.5*(p1sq-p2sq+p3sq-mpi2c)/p1[4]/p3[4];
    p3[1] =-p3[4]*sqrt(1.-cos3*cos3); p3[2]=ZERO; p3[3]=-p3[4]*cos3; 
    // calculate the form factors
    CLEOFormFactor(0,-1,q2,s1,s2,s3,F1,F2,F3);
  }
  // construct a vector with the current
  complex<Energy> current[4];
  for(unsigned int ix=0;ix<4;++ix)
    current[ix] = F1*(p2[ix]-p3[ix])-F2*(p3[ix]-p1[ix])+F3*(p1[ix]-p2[ix]);
  complex<Energy2> dot1=current[0]*conj(current[0]);
  for(unsigned int ix=1;ix<4;++ix) dot1-=current[ix]*conj(current[ix]);
  complex<Energy2> dot2=current[0]*q;
  return(-dot1+dot2*conj(dot2)/q2).real() / sqr(_rhomass[0]);
}
#line 1 "./TwoMesonRhoKStarCurrent.cc"
// -*- C++ -*-
//
// TwoMesonRhoKStarCurrent.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the TwoMesonRhoKStarCurrent class.
//
//  Author: Peter Richardson
//

#include "TwoMesonRhoKStarCurrent.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/PDT/DecayMode.h"
#include "ThePEG/PDT/EnumParticles.h"
#include "ThePEG/Interface/Switch.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

TwoMesonRhoKStarCurrent::TwoMesonRhoKStarCurrent() {
  // set up for the modes in the base class
  addDecayMode(2,-1);
  addDecayMode(2,-3);
  addDecayMode(2,-3);
  addDecayMode(2,-1);
  addDecayMode(2,-3);
  setInitialModes(5);
  // the weights of the different resonances in the matrix elements
  _pimag.push_back(  1.0);_pimag.push_back(  0.167);_pimag.push_back(  0.05);
  _piphase.push_back(0.0);_piphase.push_back(  180);_piphase.push_back(0.0);
  _kmag.push_back(  1.0);_kmag.push_back(  0.038);_kmag.push_back(  0.00);
  _kphase.push_back(0.0);_kphase.push_back(  180);_kphase.push_back(0.0);
  // models to use
  _pimodel = 0;_kmodel=0;
  // parameter for the masses (use the parameters freom the CLEO fit 
  // rather than the PDG masses etc)
  _rhoparameters=true;
  _rhomasses.push_back(774.6*MeV);_rhomasses.push_back(1408*MeV);
  _rhomasses.push_back(1700*MeV);
  _rhowidths.push_back(149*MeV);_rhowidths.push_back(502*MeV);
  _rhowidths.push_back(235*MeV);
  _kstarparameters=true;
  _kstarmasses.push_back(0.8921*GeV);_kstarmasses.push_back(1.700*GeV);
  _kstarwidths.push_back(0.0513*GeV);_kstarwidths.push_back(0.235*GeV);
}

void TwoMesonRhoKStarCurrent::doinit() {
  WeakDecayCurrent::doinit();
  // check consistency of parametrers
  if(_rhomasses.size()!=_rhowidths.size()||
     _kstarmasses.size()!=_kstarwidths.size()) {
    throw InitException() << "Inconsistent parameters in TwoMesonRhoKStarCurrent"
			  << "::doinit()" << Exception::abortnow;
  }
  // the resonances
  tPDPtr res[6]={getParticleData(-213   ),getParticleData(-100213),
		 getParticleData(-30213 ),getParticleData(-323   ),
		 getParticleData(-100323),getParticleData(-30323 )};
  // reset the masses in the form-factors if needed
  if(_rhoparameters&&_rhomasses.size()<3) {
    for(unsigned int ix=_rhomasses.size();ix<3;++ix) {
      if(res[ix]) _rhomasses.push_back(res[ix]->mass() );
      if(res[ix]) _rhowidths.push_back(res[ix]->width());
    }
  }
  else if(!_rhoparameters) {
    _rhomasses.clear();_rhowidths.clear();
    for(unsigned int ix=0;ix<3;++ix) {
      if(res[ix]) _rhomasses.push_back(res[ix]->mass() );
      if(res[ix]) _rhowidths.push_back(res[ix]->width());
    }
  }
  // then the Kstar resonances
  if(_kstarparameters&&_kstarmasses.size()<3) {
    for(unsigned int ix=_kstarmasses.size();ix<3;++ix) {
      if(res[ix+3]) _kstarmasses.push_back(res[ix+3]->mass());
      if(res[ix+3]) _kstarwidths.push_back(res[ix+3]->width());
    }
  }
  else if(!_kstarparameters) {
    _kstarmasses.clear();_kstarwidths.clear();
    for(unsigned int ix=0;ix<3;++ix) {
      if(res[ix+3]) _kstarmasses.push_back(res[ix+3]->mass());
      if(res[ix+3]) _kstarwidths.push_back(res[ix+3]->width());
    }
  }
  // set up for the Breit Wigners
  Energy mpi0(   getParticleData(ParticleID::pi0   )->mass());
  Energy mpiplus(getParticleData(ParticleID::piplus)->mass());
  Energy mk0(    getParticleData(ParticleID::K0    )->mass());
  // rho resonances
  for(unsigned int ix=0;ix<3;++ix) {
    _mass.push_back(_rhomasses[ix]);
    _width.push_back(_rhowidths[ix]);
    _mass2.push_back(_mass[ix]*_mass[ix]);
    _massw.push_back(_mass[ix]*_width[ix]);
    _massa.push_back(mpi0);
    _massb.push_back(mpiplus);
    _mom.push_back(pcm(ix,_mass[ix]));
    _hm2.push_back(GSModelhFunction(ix,_mass[ix]));
    _dparam.push_back(GSModelDParameter(ix));
    _dhdq2.push_back(GSModeldhdq2Parameter(ix));
  }
  // Kstar resonances
  for(unsigned int ix=0;ix<3;++ix) {
    _mass.push_back(_kstarmasses[ix]);
    _width.push_back(_kstarwidths[ix]);
    _mass2.push_back(_mass[ix+3]*_mass[ix+3]);
    _massw.push_back(_mass[ix+3]*_width[ix+3]);
    _massa.push_back(mk0);
    _massb.push_back(mpiplus);
    _mom.push_back(pcm(ix+3,_mass[ix+3]));
    _hm2.push_back(GSModelhFunction(ix+3,_mass[ix+3]));
    _dparam.push_back(GSModelDParameter(ix+3));
    _dhdq2.push_back(GSModeldhdq2Parameter(ix+3));
  }
  // weights for the rho channels
  if(_pimag.size()!=_piphase.size()) 
    throw InitException() << "The vectors containing the weights and phase for the"
			  << " rho channel must be the same size in "
			  << "TwoMesonRhoKStarCurrent::doinit()" << Exception::runerror;
  _piwgt.resize(_pimag.size());
  for(unsigned int ix=0;ix<_pimag.size();++ix) {
    double angle = _piphase[ix]/180.*Constants::pi;
    _piwgt[ix] = _pimag[ix]*(cos(angle)+Complex(0.,1.)*sin(angle));
  }
  // weights for the K* channels
  if(_kmag.size()!=_kphase.size()) 
    throw InitException() << "The vectors containing the weights and phase for the"
			  << " K* channel must be the same size in "
			  << "TwoMesonRhoKStarCurrent::doinit()" << Exception::runerror;
  _kwgt.resize(_kmag.size());
  for(unsigned int ix=0;ix<_kmag.size();++ix) {
    double angle = _kphase[ix]/180.*Constants::pi;
    _kwgt[ix] = _kmag[ix]*(cos(angle)+Complex(0.,1.)*sin(angle));
  }
}

void TwoMesonRhoKStarCurrent::persistentOutput(PersistentOStream & os) const {
  os << _pimodel << _kmodel << _piwgt << _pimag << _piphase << _kwgt << _kmag 
     << _kphase << _rhoparameters << _kstarparameters << ounit(_rhomasses,GeV) << ounit(_rhowidths,GeV) 
     << ounit(_kstarmasses,GeV) << ounit(_kstarwidths,GeV) 
     << ounit(_mass,GeV) << ounit(_width,GeV) << ounit(_mass2,GeV2) << ounit(_massw,GeV2) 
     << ounit(_massa,GeV) <<ounit(_massb,GeV) << ounit(_mom,GeV) << ounit(_dhdq2,1/GeV2) 
     << _hm2 << _dparam;
}

void TwoMesonRhoKStarCurrent::persistentInput(PersistentIStream & is, int) {
  is >> _pimodel >> _kmodel >> _piwgt >> _pimag >> _piphase >> _kwgt >> _kmag 
     >> _kphase >> _rhoparameters >> _kstarparameters >> iunit(_rhomasses,GeV) >> iunit(_rhowidths,GeV) 
     >> iunit(_kstarmasses,GeV) >> iunit(_kstarwidths,GeV) 
     >> iunit(_mass,GeV) >> iunit(_width,GeV) >> iunit(_mass2,GeV2) >> iunit(_massw,GeV2) 
     >> iunit(_massa,GeV) >> iunit(_massb,GeV) >> iunit(_mom,GeV) >> iunit(_dhdq2,1/GeV2) 
     >> _hm2 >> _dparam;
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<TwoMesonRhoKStarCurrent,WeakDecayCurrent>
describeHerwigTwoMesonRhoKStarCurrent("Herwig::TwoMesonRhoKStarCurrent", "HwWeakCurrents.so");

void TwoMesonRhoKStarCurrent::Init() {

  static ParVector<TwoMesonRhoKStarCurrent,Energy> interfaceRhoMasses
    ("RhoMasses",
     "The masses of the different rho resonances for the pi pi channel",
     &TwoMesonRhoKStarCurrent::_rhomasses, MeV, -1, 775.8*MeV, ZERO, 10000.*MeV,
     false, false, true);

  static ParVector<TwoMesonRhoKStarCurrent,Energy> interfaceRhoWidths
    ("RhoWidths",
     "The widths of the different rho resonances for the pi pi channel",
     &TwoMesonRhoKStarCurrent::_rhowidths, MeV, -1, 150.3*MeV, ZERO, 1000.*MeV,
     false, false, true);
  
  static ParVector<TwoMesonRhoKStarCurrent,Energy> interfaceKstarMasses
    ("KstarMasses",
     "The masses of the different K* resonances for the pi pi channel",
     &TwoMesonRhoKStarCurrent::_kstarmasses, MeV, -1, 891.66*MeV, ZERO, 10000.*MeV,
     false, false, true);

  static ParVector<TwoMesonRhoKStarCurrent,Energy> interfaceKstarWidths
    ("KstarWidths",
     "The widths of the different K* resonances for the pi pi channel",
     &TwoMesonRhoKStarCurrent::_kstarwidths, MeV, -1, 50.8*MeV, ZERO, 1000.*MeV,
     false, false, true);
  
  static Switch<TwoMesonRhoKStarCurrent,bool> interfaceRhoParameters
    ("RhoParameters",
     "Use local values for the rho meson masses and widths",
     &TwoMesonRhoKStarCurrent::_rhoparameters, true, false, false);
  static SwitchOption interfaceRhoParameterstrue
    (interfaceRhoParameters,
     "Local",
     "Use local values",
     true);
  static SwitchOption interfaceRhoParametersParticleData
    (interfaceRhoParameters,
     "ParticleData",
     "Use the value from the particle data objects",
     false);

  static Switch<TwoMesonRhoKStarCurrent,bool> interfaceKstarParameters
    ("KstarParameters",
     "Use local values for the Kstar meson masses and widths",
     &TwoMesonRhoKStarCurrent::_kstarparameters, true, false, false);
  static SwitchOption interfaceKstarParameterstrue
    (interfaceKstarParameters,
     "Local",
     "Use local values",
     true);
  static SwitchOption interfaceKstarParametersParticleData
    (interfaceKstarParameters,
     "ParticleData",
     "Use the value from the particle data objects",
     false);
  
  static ParVector<TwoMesonRhoKStarCurrent,double> interfacePiMagnitude
    ("PiMagnitude",
     "Magnitude of the weight of the different resonances for the pi pi channel",
     &TwoMesonRhoKStarCurrent::_pimag, -1, 0., 0, 0,
     false, false, Interface::nolimits);

  static ParVector<TwoMesonRhoKStarCurrent,double> interfacePiPhase
    ("PiPhase",
     "Phase of the weight of the different resonances for the pi pi channel",
     &TwoMesonRhoKStarCurrent::_piphase, -1, 0., 0, 0,
     false, false, Interface::nolimits);

  static ParVector<TwoMesonRhoKStarCurrent,double> interfaceKMagnitude
    ("KMagnitude",
     "Magnitude of the weight of the different resonances for the K pi channel",
     &TwoMesonRhoKStarCurrent::_kmag, -1, 0., 0, 0,
     false, false, Interface::nolimits);

  static ParVector<TwoMesonRhoKStarCurrent,double> interfaceKPhase
    ("KPhase",
     "Phase of the weight of the different resonances for the K pi channel",
     &TwoMesonRhoKStarCurrent::_kphase, -1, 0., 0, 0,
     false, false, Interface::nolimits);
  
  static Switch<TwoMesonRhoKStarCurrent,int> interfacePiModel
    ("PiModel",
     "The model to use for the propagator for the pion modes.",
     &TwoMesonRhoKStarCurrent::_pimodel, 0, false, false);
  static SwitchOption interfacePiModelKuhn
    (interfacePiModel,
     "Kuhn",
     "The model of Kuhn and Santamaria",
     0);
  static SwitchOption interfacePiModelGounaris
    (interfacePiModel,
     "Gounaris",
     "The model of Gounaris and Sakurai.",
     1);
  
  static Switch<TwoMesonRhoKStarCurrent,int> interfaceKModel
    ("KModel",
     "The model to use for the propagator for the kaon modes.",
     &TwoMesonRhoKStarCurrent::_kmodel, 0, false, false);
  static SwitchOption interfaceKModelKuhn
    (interfaceKModel,
     "Kuhn",
     "The model of Kuhn and Santamaria",
     0);
  static SwitchOption interfaceKModelGounaris
    (interfaceKModel,
     "Gounaris",
     "The model of Gounaris and Sakurai.",
     1);

  static ClassDocumentation<TwoMesonRhoKStarCurrent> documentation
    ("The TwoMesonRhoKStarCurrent class is designed to implement weak"
     "decay to two scalar mesons using the models of either Kuhn and "
     "Santamaria (Z. Phys. C48, 445 (1990)) or Gounaris and Sakurai Phys. Rev. "
     "Lett. 21, 244 (1968).  The mixing parameters are taken from "
     "Phys. Rev. D61:112002,2000 (CLEO), although the PDG values for the "
     "masses and widths are used, for the decay pi+/- pi0."
     " The decay K pi is assumed to  be dominated by the lowest lying K* resonance.",
     "The weak "
     "decay current to two scalar mesons is implemented "
     "using the models of either Kuhn and "
     "Santamaria \\cite{Kuhn:1990ad} or Gounaris and Sakurai \\cite{Gounaris:1968mw}. "
     "The mixing parameters are taken from "
     "\\cite{Asner:1999kj}, although the PDG values for the "
     "masses and widths are used, for the decay pi+/- pi0."
     " The decay K pi is assumed to  be dominated by the lowest lying K* resonance.",
     "%\\cite{Kuhn:1990ad}\n"
     "\\bibitem{Kuhn:1990ad}\n"
     "  J.~H.~Kuhn and A.~Santamaria,\n"
     "  %``Tau decays to pions,''\n"
     "  Z.\\ Phys.\\  C {\\bf 48}, 445 (1990).\n"
     "  %%CITATION = ZEPYA,C48,445;%%\n"
     "%\\cite{Gounaris:1968mw}\n"
     "\\bibitem{Gounaris:1968mw}\n"
     "  G.~J.~Gounaris and J.~J.~Sakurai,\n"
     "   ``Finite width corrections to the vector meson dominance prediction for rho\n"
     "  %$\\to$ e+ e-,''\n"
     "  Phys.\\ Rev.\\ Lett.\\  {\\bf 21}, 244 (1968).\n"
     "  %%CITATION = PRLTA,21,244;%%\n"
     "%\\cite{Asner:1999kj}\n"
     "\\bibitem{Asner:1999kj}\n"
     "  D.~M.~Asner {\\it et al.}  [CLEO Collaboration],\n"
     "   ``Hadronic structure in the decay tau- --> nu/tau pi- pi0 pi0 and the  sign\n"
     "  %of the tau neutrino helicity,''\n"
     "  Phys.\\ Rev.\\  D {\\bf 61}, 012002 (2000)\n"
     "  [arXiv:hep-ex/9902022].\n"
     "  %%CITATION = PHRVA,D61,012002;%%\n"
     );



}

// complete the construction of the decay mode for integration
bool TwoMesonRhoKStarCurrent::createMode(int icharge, unsigned int imode,
					 DecayPhaseSpaceModePtr mode,
					 unsigned int iloc,unsigned int,
					 DecayPhaseSpaceChannelPtr phase,Energy upp) {
  if(abs(icharge)!=3) return false; 
  // make sure that the decays are kinematically allowed
  tPDPtr part[2];
  if(imode==0) {
    part[0]=getParticleData(ParticleID::piplus);
    part[1]=getParticleData(ParticleID::pi0);
  }
  else if(imode==1) {
    part[0]=getParticleData(ParticleID::Kplus);
    part[1]=getParticleData(ParticleID::pi0);
  }
  else if(imode==2) {
    part[0]=getParticleData(ParticleID::K0);
    part[1]=getParticleData(ParticleID::piplus);
  }
  else if(imode==3) {
    part[0]=getParticleData(ParticleID::Kplus);
    part[1]=getParticleData(ParticleID::K0);
  }
  else if(imode==4) {
    part[0]=getParticleData(ParticleID::eta);
    part[1]=getParticleData(ParticleID::Kplus);
  }
  Energy min(part[0]->massMin()+part[1]->massMin());
  if(min>upp) return false;
  DecayPhaseSpaceChannelPtr newchannel;
  // set the resonances
  // two pion or  K+ K0 decay
  tPDPtr res[3];
  if(imode==0||imode==3) {
    res[0]=getParticleData(213);
    res[1]=getParticleData(100213);
    res[2]=getParticleData(30213);
  }
  // K+ pi0 or K0 pi+ or K eta decay
  else if(imode==1||imode==2||imode==4) {
    res[0]=getParticleData(323);
    res[1]=getParticleData(100323);
    res[2]=getParticleData(30323);
  }
  else {
    throw Exception() << "Failure of initialisation in TwoMesonRhoKStarCurrent" 
		      << Exception::abortnow;
  }
  if(icharge==-3) {
    for(unsigned int ix=0;ix<3;++ix) {
      if(res[ix]&&res[ix]->CC()) res[ix]=res[ix]->CC();
    }
  }
  // create the channels
  for(unsigned int ix=0;ix<3;++ix) {
    if(res[ix]) {
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(res[ix],0,0.0,iloc,iloc+1);
      mode->addChannel(newchannel);
    }
  }
  // reset the masses in the intergrators if needed
  // for the rho 
  if(_rhoparameters&&(imode==0||imode==3)) {
    for(unsigned int ix=0;ix<3;++ix) {
      if(ix<_rhomasses.size()&&res[ix]) {
	mode->resetIntermediate(res[ix],_rhomasses[ix],_rhowidths[ix]);
      }
    }
  }
  // for the K*
  else if(_kstarparameters&&imode!=0&&imode!=3) {
    for(unsigned int ix=0;ix<3;++ix) {
      if(ix<_kstarmasses.size()&&res[ix]) {
	mode->resetIntermediate(res[ix],_kstarmasses[ix],_kstarwidths[ix]);
      }
    }
  }
  // return if successful
  return true;
}

// the particles produced by the current
tPDVector TwoMesonRhoKStarCurrent::particles(int icharge, unsigned int imode,
					    int,int) {
  if(abs(icharge)!=3) return tPDVector();
  tPDVector output(2);
  if(imode==0) {
    output[0]=getParticleData(ParticleID::piplus);
    output[1]=getParticleData(ParticleID::pi0);
  }
  else if(imode==1) {
    output[0]=getParticleData(ParticleID::Kplus);
    output[1]=getParticleData(ParticleID::pi0);
  }
  else if(imode==2) {
    output[0]=getParticleData(ParticleID::K0);
    output[1]=getParticleData(ParticleID::piplus);
  }
  else if(imode==3) {
    output[0]=getParticleData(ParticleID::Kplus);
    output[1]=getParticleData(ParticleID::Kbar0);
  }
  else {
    output[0]=getParticleData(ParticleID::eta);
    output[1]=getParticleData(ParticleID::Kplus);
  }
  if(icharge==-3) {
    for(unsigned int ix=0;ix<output.size();++ix) {
      if(output[ix]->CC()) output[ix]=output[ix]->CC();
    }
  }
  return output;
}

// hadronic current   
vector<LorentzPolarizationVectorE> 
TwoMesonRhoKStarCurrent::current(const int imode, const int ichan,
				 Energy & scale,const ParticleVector & outpart,
				 DecayIntegrator::MEOption meopt) const {
  useMe();
  if(meopt==DecayIntegrator::Terminate) {
    for(unsigned int ix=0;ix<2;++ix)
      ScalarWaveFunction::constructSpinInfo(outpart[ix],outgoing,true);
    return vector<LorentzPolarizationVectorE>(1,LorentzPolarizationVectorE());
  }
  // momentum difference and sum of the mesons
  Lorentz5Momentum pdiff(outpart[0]->momentum()-outpart[1]->momentum());
  Lorentz5Momentum psum (outpart[0]->momentum()+outpart[1]->momentum());
  psum.rescaleMass();
  scale=psum.mass();
  // mass2 of vector intermediate state
  Energy2 q2(psum.m2());
  double dot(psum*pdiff/q2);
  psum *=dot;
  LorentzPolarizationVector vect;
  // calculate the current
  Complex FPI(0.),denom(0.);
  // rho
  if(imode==0||imode==3) {
    if(ichan<0) {
      for(unsigned int ix=0;ix<_piwgt.size()&&ix<3;++ix) {
	FPI+=_piwgt[ix]*BreitWigner(q2,_pimodel,0,ix);
	denom+=_piwgt[ix];
      }
    }
    else if(ichan<int(_piwgt.size())&&ichan<3) {
      FPI=_piwgt[ichan]*BreitWigner(q2,_pimodel,0,ichan);
      for(unsigned int ix=0;ix<_piwgt.size()&&ix<3;++ix) denom+=_piwgt[ix];
    }
  }
  // K*
  else {
    if(ichan<0) {
      for(unsigned int ix=0;ix<_kwgt.size()&&ix<3;++ix) {
	FPI+=_kwgt[ix]*BreitWigner(q2,_kmodel,1,ix);
	denom+=_kwgt[ix];
      }
    }
    else if (ichan<int(_kwgt.size())&&ichan<3) {
      FPI=_kwgt[ichan]*BreitWigner(q2,_kmodel,1,ichan);
      for(unsigned int ix=0;ix<_kwgt.size()&&ix<3;++ix) denom+=_kwgt[ix];
    }
  }
  // additional prefactors
  FPI/=denom;
  // pion mode
  if(imode==0)           FPI *= sqrt(2.0);
  // single kaon modes
  else if(imode==1)      FPI *= sqrt(0.5);
  else if(imode==2)      FPI *= 1.       ;
    // two kaon modes
  else if(imode==3)      FPI *= 1.       ;
  // the kaon eta mode
  else if(imode==4)      FPI *=sqrt(1.5);
  // compute the current
  pdiff-=psum;
  return vector<LorentzPolarizationVectorE>(1,FPI*pdiff);
}
   
bool TwoMesonRhoKStarCurrent::accept(vector<int> id) {
  bool allowed(false);
  // check there are only two particles
  if(id.size()!=2){return false;}
  // pion modes
  if((id[0]==ParticleID::piminus && id[1]==ParticleID::pi0)     ||
     (id[0]==ParticleID::pi0     && id[1]==ParticleID::piminus) ||
     (id[0]==ParticleID::piplus  && id[1]==ParticleID::pi0)     ||
     (id[0]==ParticleID::pi0     && id[1]==ParticleID::piplus)) allowed=true;
  // single charged kaon
  else if((id[0]==ParticleID::Kminus && id[1]==ParticleID::pi0)    ||
	  (id[0]==ParticleID::pi0    && id[1]==ParticleID::Kminus) ||
	  (id[0]==ParticleID::Kplus  && id[1]==ParticleID::pi0)    ||
	  (id[0]==ParticleID::pi0    && id[1]==ParticleID::Kplus)) allowed=true;
  // single neutral kaon
  else if((id[0]==ParticleID::piminus && id[1]==ParticleID::Kbar0)   ||
	  (id[0]==ParticleID::Kbar0   && id[1]==ParticleID::piminus) ||
	  (id[0]==ParticleID::piplus  && id[1]==ParticleID::K0)      ||
	  (id[0]==ParticleID::K0      && id[1]==ParticleID::piplus)) allowed=true;
  // two kaons
  else if((id[0]==ParticleID::Kminus && id[1]==ParticleID::K0)     ||
	  (id[0]==ParticleID::K0     && id[1]==ParticleID::Kminus) ||
	  (id[0]==ParticleID::Kplus  && id[1]==ParticleID::Kbar0)  ||
	  (id[0]==ParticleID::Kbar0  && id[1]==ParticleID::Kplus)) allowed=true;
  // charged kaon and eta
  else if((id[0]==ParticleID::Kminus && id[1]==ParticleID::eta)    ||
	  (id[0]==ParticleID::eta    && id[1]==ParticleID::Kminus) ||
	  (id[0]==ParticleID::Kplus  && id[1]==ParticleID::eta)    ||
	  (id[0]==ParticleID::eta    && id[1]==ParticleID::Kplus)) allowed=true;
  return allowed;
}

// the decay mode
unsigned int TwoMesonRhoKStarCurrent::decayMode(vector<int> idout) {
  unsigned int imode(0),nkaon(0);
  for(unsigned int ix=0;ix<idout.size();++ix) {
    if(abs(idout[ix])==ParticleID::K0) {
      imode=2;
      ++nkaon;
    }
    else if (abs(idout[ix])==ParticleID::Kplus) {
      imode=1;
      ++nkaon;
    }
    else if (idout[ix]==ParticleID::eta) {
      imode=4;
      break;
    }
  }
  if(nkaon==2) imode=3;
  return imode;
}

// output the information for the database
void TwoMesonRhoKStarCurrent::dataBaseOutput(ofstream & output,bool header,
					     bool create) const {
  if(header) output << "update decayers set parameters=\"";
  if(create) output << "create Herwig::TwoMesonRhoKStarCurrent " 
		    << name() << " HwWeakCurrents.so\n";
  unsigned int ix;
  for(ix=0;ix<_rhomasses.size();++ix) {
    if(ix<3)  output << "newdef ";
    else      output << "insert ";
    output << name() << ":RhoMasses " << ix << " " << _rhomasses[ix]/MeV << "\n";
  }
  for(ix=0;ix<_rhowidths.size();++ix) {
    if(ix<3) output << "newdef ";
    else     output << "insert ";
    output << name() << ":RhoWidths " << ix << " " << _rhowidths[ix]/MeV << "\n";
  }
  for(ix=0;ix<_kstarmasses.size();++ix) {
    if(ix<2) output << "newdef ";
    else     output << "insert ";
    output << name() << ":KstarMasses " << ix << " " << _kstarmasses[ix]/MeV << "\n";
  }
  for(ix=0;ix<_kstarwidths.size();++ix) {
    if(ix<2) output << "newdef ";
    else     output << "insert ";
    output << name() << ":KstarWidths " << ix << " " << _kstarwidths[ix]/MeV << "\n";
  }
  output << "newdef " << name() << ":RhoParameters " << _rhoparameters << "\n";
  output << "newdef " << name() << ":KstarParameters " << _kstarparameters << "\n";
  for(ix=0;ix<_piwgt.size();++ix) {
    if(ix<3) output << "newdef ";
    else     output << "insert ";
    output << name() << ":PiMagnitude " << ix << " " << _pimag[ix]   << "\n";
    if(ix<3) output << "newdef ";
    else     output << "insert ";
    output << name() << ":PiPhase "     << ix << " " << _piphase[ix] << "\n";
  }
  for(ix=0;ix<_kwgt.size();++ix) {
    if(ix<3) output << "newdef ";
    else     output << "insert ";
    output << name() << ":KMagnitude " << ix << " " << _kmag[ix]   << "\n";
    if(ix<3) output << "newdef ";
    else     output << "insert ";
    output << name() << ":KPhase "     << ix << " " << _kphase[ix] << "\n";
  }
  output << "newdef " << name() << ":PiModel " << _pimodel << "\n";
  output << "newdef " << name() << ":KModel  " << _kmodel  << "\n";
  WeakDecayCurrent::dataBaseOutput(output,false,false);
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}
#line 1 "./TwoPionPhotonCurrent.cc"
// -*- C++ -*-
//
// TwoPionPhotonCurrent.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the TwoPionPhotonCurrent class.
//
//  Author: Peter Richardson
//

#include "TwoPionPhotonCurrent.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/StandardModel/StandardModelBase.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Interface/Switch.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

TwoPionPhotonCurrent::TwoPionPhotonCurrent() {
  // modes handled
  addDecayMode(2,-1);
  setInitialModes(1);
  // weight of the resonances in the current
  _resweights.push_back(1.0);_resweights.push_back(-0.1);_resweights.push_back(0.0);
  // parameters of the rho resonaces
  _rhoparameters=true;
  _rhomasses.push_back(0.773*GeV);_rhomasses.push_back(1.70*GeV);
  _rhowidths.push_back(0.145*GeV);_rhowidths.push_back(0.26*GeV);
  // parameters fo the omega resonance
  _omegaparameters=true;
  _omegamass=782*MeV;_omegawidth=8.5*MeV;
  // couplings
  _grho   = 0.11238947*GeV2;
  _grhoomegapi = 12.924/GeV;
  // parameters for the resonance used in the integration
  _intmass = 1.2*GeV;
  _intwidth = 0.35*GeV;
}

void TwoPionPhotonCurrent::doinit() {
  WeakDecayCurrent::doinit();
  // set up the rho masses and widths
  tPDPtr temp;
  for(unsigned int ix=0;ix<3;++ix) {
    if(ix==0)      temp = getParticleData(-213);
    else if(ix==1) temp = getParticleData(-100213);
    else if(ix==2) temp = getParticleData(-30213);
    // if using local values
    if(!_rhoparameters&&ix<_rhomasses.size()) {
      _rhomasses[ix]=temp->mass();
      _rhowidths[ix]=temp->width();
    }
    else if(ix>=_rhomasses.size()) {
      _rhomasses.push_back(temp->mass());
      _rhowidths.push_back(temp->width());
    }
  }
  // set up the omega masses and widths
  if(!_omegaparameters) {
    temp = getParticleData(ParticleID::omega);
    _omegamass  = temp->mass();
    _omegawidth = temp->width();
  }
}

void TwoPionPhotonCurrent::persistentOutput(PersistentOStream & os) const {
  os << ounit(_grho,GeV2) << ounit(_grhoomegapi,1/GeV) << _resweights 
     << _rhoparameters << ounit(_rhomasses,GeV) 
     << ounit(_rhowidths,GeV) << _omegaparameters << ounit(_omegamass,GeV) 
     << ounit(_omegawidth,GeV) << ounit(_intmass,GeV) 
     << ounit(_intwidth,GeV) ;
}

void TwoPionPhotonCurrent::persistentInput(PersistentIStream & is, int) { 
  is >> iunit(_grho,GeV2) >> iunit(_grhoomegapi,1/GeV) >> _resweights 
     >> _rhoparameters >> iunit(_rhomasses,GeV) 
     >> iunit(_rhowidths,GeV) >> _omegaparameters >> iunit(_omegamass,GeV) 
     >> iunit(_omegawidth,GeV) >> iunit(_intmass,GeV)
     >> iunit(_intwidth,GeV);
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<TwoPionPhotonCurrent,WeakDecayCurrent>
describeHerwigTwoPionPhotonCurrent("Herwig::TwoPionPhotonCurrent", "HwWeakCurrents.so");

void TwoPionPhotonCurrent::Init() {

  static ParVector<TwoPionPhotonCurrent,double> interfacereswgt
    ("Weights",
     "The weights of the different resonances for the decay tau -> nu pi pi gamma",
     &TwoPionPhotonCurrent::_resweights,
     0, 0, 0, -1000, 1000, false, false, true);
    
  static Switch<TwoPionPhotonCurrent,bool> interfaceRhoParameters
    ("RhoParameters",
     "Use local values for the rho meson masses and widths",
     &TwoPionPhotonCurrent::_rhoparameters, true, false, false);

  static SwitchOption interfaceRhoParameterstrue
    (interfaceRhoParameters,
     "Local",
     "Use local values",
     true);
  static SwitchOption interfaceRhoParametersParticleData
    (interfaceRhoParameters,
     "ParticleData",
     "Use the value from the particle data objects",
     false);
  
  static Switch<TwoPionPhotonCurrent,bool> interfaceomegaParameters
    ("omegaParameters",
     "Use local values for the omega meson masses and widths",
     &TwoPionPhotonCurrent::_omegaparameters, true, false, false);
  static SwitchOption interfaceomegaParameterstrue
    (interfaceomegaParameters,
     "Local",
     "Use local values",
     true);
  static SwitchOption interfaceomegaParametersParticleData
    (interfaceomegaParameters,
     "ParticleData",
     "Use the value from the particle data objects",
     false);

  static ParVector<TwoPionPhotonCurrent,Energy> interfaceRhoMasses
    ("RhoMasses",
     "The masses of the different rho resonances for the decay tau ->  pi pi photon",
     &TwoPionPhotonCurrent::_rhomasses, MeV, -1, 773.*MeV, ZERO, 10000.*MeV,
     false, false, true);

  static ParVector<TwoPionPhotonCurrent,Energy> interfaceRhoWidths
    ("RhoWidths",
     "The widths of the different rho resonances for the decay tau -> nu pi pi photon",
     &TwoPionPhotonCurrent::_rhowidths, MeV, -1, 145.*MeV, ZERO, 1000.*MeV,
     false, false, true);

  static Parameter<TwoPionPhotonCurrent,Energy> interfaceomegamass
    ("omegamass",
     "The mass of the omega",
     &TwoPionPhotonCurrent::_omegamass, GeV, 0.782*GeV, ZERO, 1.0*GeV,
     false, false, true);
  
  static Parameter<TwoPionPhotonCurrent,Energy> interfaceomegawidth
    ("omegawidth",
     "The width of the omega for the decay tau- -> pi pi photon",
     &TwoPionPhotonCurrent::_omegawidth, GeV, 0.0085*GeV, ZERO, 1.*GeV,
     false, false, false);
  
  static ClassDocumentation<TwoPionPhotonCurrent> documentation
    ("The TwoPionPhotonCurrent class implements the decay "
     "tau+/- -> pi+/- pi0 gamma via an omega.",
     "The decay $\\tau^\\pm \\to \\omega \\to \\pi^\\pm \\pi^0 \\gamma$ "
     "is modelled after \\cite{Jadach:1993hs}.",
     "  %\\cite{Jadach:1993hs}\n"
     "\\bibitem{Jadach:1993hs}\n"
     "  S.~Jadach, Z.~Was, R.~Decker and J.~H.~Kuhn,\n"
     "  %``The Tau Decay Library Tauola: Version 2.4,''\n"
     "  Comput.\\ Phys.\\ Commun.\\  {\\bf 76}, 361 (1993).\n"
     "  %%CITATION = CPHCB,76,361;%%\n"
     );

  static Parameter<TwoPionPhotonCurrent,Energy2> interfacegrho
    ("grho",
     "The rho meson decay constant.",
     &TwoPionPhotonCurrent::_grho, GeV2, 0.11238947*GeV2, -1.*GeV2, 1.*GeV2,
     false, false, false);

  static Parameter<TwoPionPhotonCurrent,InvEnergy> interfacegrhoomegapi
    ("grhoomegapi",
     "The rho-omega-pi coupling",
     &TwoPionPhotonCurrent::_grhoomegapi, 1./GeV, 12.924/GeV,
     -100./GeV, 100./GeV,
     false, false, false);

  static Parameter<TwoPionPhotonCurrent,Energy> interfaceIntegrationMass
    ("IntegrationMass",
     "Mass of the pseudoresonance used to improve integration effciency",
     &TwoPionPhotonCurrent::_intmass, GeV, 1.4*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static Parameter<TwoPionPhotonCurrent,Energy> interfaceIntegrationWidth
    ("IntegrationWidth",
     "Width of the pseudoresonance used to improve integration effciency",
     &TwoPionPhotonCurrent::_intwidth, GeV, 0.5*GeV, ZERO, 10.0*GeV,
     false, false, true);
}

// complete the construction of the decay mode for integration
bool TwoPionPhotonCurrent::createMode(int icharge, unsigned int,
				      DecayPhaseSpaceModePtr mode,
				      unsigned int iloc,unsigned int ires,
				      DecayPhaseSpaceChannelPtr phase,Energy upp) {
  if(icharge!=3&&icharge!=-3){return false;}
  // check that the mode is are kinematical allowed
  Energy min(getParticleData(ParticleID::piplus)->mass()+
	     getParticleData(ParticleID::pi0)->mass());
  if(min>upp) return false;
  // set up the integration channels;
  tPDPtr omega(getParticleData(ParticleID::omega));
  tPDPtr W(getParticleData(ParticleID::Wplus));
  if(icharge<0) W=W->CC();
  DecayPhaseSpaceChannelPtr newchannel; 
  newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
  newchannel->addIntermediate(W,0,0.0,-ires-1,iloc);
  newchannel->addIntermediate(omega,0,0.0,iloc+1,iloc+2);
  mode->addChannel(newchannel);
  // reset the masses and widths of the resonances if needed
  mode->resetIntermediate(W,_intmass,_intwidth);
  // set up the omega masses and widths
  if(_omegaparameters) mode->resetIntermediate(omega,_omegamass,_omegawidth);
  return true;
}

// the particles produced by the current
tPDVector TwoPionPhotonCurrent::particles(int icharge, unsigned int,int,int) {
  tPDVector extpart;
  if(abs(icharge)!=3) return extpart;
  if(icharge==3)       extpart.push_back(getParticleData(ParticleID::piplus));
  else if(icharge==-3) extpart.push_back(getParticleData(ParticleID::piminus));
  extpart.push_back(getParticleData(ParticleID::pi0));
  extpart.push_back(getParticleData(ParticleID::gamma));
  return extpart;
}


// the hadronic currents    
vector<LorentzPolarizationVectorE> 
TwoPionPhotonCurrent::current(const int, const int,Energy & scale,
			      const ParticleVector & decay,
			      DecayIntegrator::MEOption meopt) const {
  useMe();
  vector<LorentzPolarizationVector> temp;
  VectorWaveFunction::
    calculateWaveFunctions(temp,decay[2],outgoing,true);
  if(meopt==DecayIntegrator::Terminate) {
    for(unsigned int ix=0;ix<2;++ix)
      ScalarWaveFunction::constructSpinInfo(decay[ix],outgoing,true);
    VectorWaveFunction::constructSpinInfo(temp,decay[2],
					  outgoing,true,true);
    return vector<LorentzPolarizationVectorE>(1,LorentzPolarizationVectorE());
  }
  // locate the particles
  Lorentz5Momentum pout(decay[1]->momentum()+decay[2]->momentum()+
			decay[0]->momentum());
  // overall hadronic mass
  pout.rescaleMass();
  scale=pout.mass();
  Energy2 q2(pout.m2());
  // mass of the omega
  pout = decay[1]->momentum()+decay[2]->momentum();
  pout.rescaleMass();
  Energy2 s2(pout.m2());
  // compute the prefactor
  complex<InvEnergy3> prefactor(-FFunction(ZERO)*FFunction(q2)*scale*
			    sqrt(Constants::twopi*generator()->standardModel()->alphaEM())*
			    BreitWigner(s2,10));
  // dot products which don't depend on the polarization vector
  Energy2 dot12(decay[2]->momentum()*decay[1]->momentum());
  Energy2 dot13(decay[2]->momentum()*decay[0]->momentum());
  Energy2 dot23(decay[1]->momentum()*decay[0]->momentum());
  Energy2 mpi2 = sqr(decay[0]->mass());
  vector<LorentzPolarizationVectorE> ret(3);
  for(unsigned int ix=0;ix<3;++ix) {
    if(ix!=1) {
      // obtain the dot products we need
      complex<Energy> dote2 = temp[ix]*decay[1]->momentum();
      complex<Energy> dote3 = temp[ix]*decay[0]->momentum();
      // now compute the coefficients
      complex<Energy4> coeffa = mpi2*dot13-dot12*(dot23-dot13);
      complex<Energy3> coeffb = dote2*dot13-dote3*dot12;
      complex<Energy3> coeffc = dote2*dot23-dote3*(mpi2+dot12);
      // finally compute the current
      ret[ix]= prefactor*(coeffa*temp[ix]
			   -coeffb*decay[1]->momentum()
			   +coeffc*decay[2]->momentum());
    }
    else{ret[ix]=LorentzPolarizationVectorE();}
  }
  return ret;
}

bool TwoPionPhotonCurrent::accept(vector<int> id) {
  if(id.size()!=3){return false;}
  unsigned int npiplus(0),npi0(0),ngamma(0);
  for(unsigned int ix=0;ix<id.size();++ix) {
    if(abs(id[ix])==ParticleID::piplus) ++npiplus;
    else if(id[ix]==ParticleID::gamma)  ++ngamma;
    else if(id[ix]==ParticleID::pi0)    ++npi0;
  }
  return npiplus==1&&ngamma==1&&npi0==1;
}

unsigned int TwoPionPhotonCurrent::decayMode(vector<int>) {
  return 0;
}

// output the information for the database
void TwoPionPhotonCurrent::dataBaseOutput(ofstream & output,bool header,
					  bool create) const {
  if(header) output << "update decayers set parameters=\"";
  if(create) output << "create Herwig::TwoPionPhotonCurrent " << name() 
		    << " HwWeakCurrents.so\n";
  output << "newdef " << name() << ":RhoParameters "    << _rhoparameters << "\n";
  output << "newdef " << name() << ":omegaParameters "    << _omegaparameters << "\n";
  output << "newdef " << name() << ":omegamass "    << _omegamass/GeV << "\n";
  output << "newdef " << name() << ":omegawidth "    << _omegawidth/GeV << "\n";
  output << "newdef " << name() << ":grho "    << _grho/GeV2 << "\n";
  output << "newdef " << name() << ":grhoomegapi "    << _grhoomegapi*GeV << "\n";
  output << "newdef " << name() << ":IntegrationMass "  << _intmass/GeV  << "\n";
  output << "newdef " << name() << ":IntegrationWidth " << _intwidth/GeV  << "\n";
  unsigned int ix;
  for(ix=0;ix<_resweights.size();++ix) {
    if(ix<3) output << "newdef " << name() << ":Weights " << ix 
		    << " " << _resweights[ix] << "\n";
    else     output << "insert " << name() << ":Weights " << ix 
		    << " " << _resweights[ix] << "\n";
  }
  for(ix=0;ix<_rhomasses.size();++ix) {
    if(ix<2) output << "newdef " << name() << ":RhoMasses " << ix 
		    << " " << _rhomasses[ix]/MeV << "\n";
    else     output << "insert " << name() << ":RhoMasses " << ix 
		    << " " << _rhomasses[ix]/MeV << "\n";
  }
  for(ix=0;ix<_rhowidths.size();++ix) {
    if(ix<2) output << "newdef " << name() << ":RhoWidths " << ix 
		    << " " << _rhowidths[ix]/MeV << "\n";
    else     output << "insert " << name() << ":RhoWidths " << ix 
		    << " " << _rhowidths[ix]/MeV << "\n";
  }
  WeakDecayCurrent::dataBaseOutput(output,false,false);
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}
#line 1 "./VectorMesonCurrent.cc"
// -*- C++ -*-
//
// VectorMesonCurrent.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the VectorMesonCurrent class.
//

#include "VectorMesonCurrent.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/StandardModel/StandardModelBase.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/Helicity/WaveFunction/VectorWaveFunction.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

void VectorMesonCurrent::doinit() {
  unsigned int isize=numberOfModes();
  if(_id.size()!=isize||_decay_constant.size()!=isize)
    {throw InitException() << "Inconsistent parameters in VectorMesonCurrent::doinit()"
			   << Exception::abortnow;}
  WeakDecayCurrent::doinit();
}
VectorMesonCurrent::VectorMesonCurrent()  {
  _id.push_back(213);_decay_constant.push_back(0.1764*GeV2);
  addDecayMode(2,-1);
  _id.push_back(113);_decay_constant.push_back(0.1764*GeV2);
  addDecayMode(1,-1);
  _id.push_back(113);_decay_constant.push_back(0.1764*GeV2);
  addDecayMode(2,-2);
  _id.push_back(223);_decay_constant.push_back(0.1764*GeV2);
  addDecayMode(1,-1);
  _id.push_back(223);_decay_constant.push_back(0.1764*GeV2);
  addDecayMode(2,-2);
  _id.push_back(333);_decay_constant.push_back(0.2380*GeV2);
  addDecayMode(3,-3);
  _id.push_back(313);_decay_constant.push_back(0.2019*GeV2);
  addDecayMode(1,-3);
  _id.push_back(323);_decay_constant.push_back(0.2019*GeV2);
  addDecayMode(2,-3);
  _id.push_back(20213);_decay_constant.push_back(0.4626*GeV2);
  addDecayMode(2,-1);
  _id.push_back(20113);_decay_constant.push_back(0.4626*GeV2);
  addDecayMode(1,-1);
  _id.push_back(20113);_decay_constant.push_back(0.4626*GeV2);
  addDecayMode(2,-2);
  _id.push_back(413);_decay_constant.push_back(0.402*GeV2);
  addDecayMode(4,-1);
  _id.push_back(423);_decay_constant.push_back(0.402*GeV2);
  addDecayMode(4,-2);
  _id.push_back(433);_decay_constant.push_back(0.509*GeV2);
  addDecayMode(4,-3);
  _id.push_back(443);_decay_constant.push_back(1.223*GeV2);
  addDecayMode(4,-4);
  _id.push_back(100443);_decay_constant.push_back(1.08*GeV2);
  addDecayMode(4,-4);
  _id.push_back(10433);_decay_constant.push_back(0.397*GeV2);
  addDecayMode(4,-3);
  // initial size of the vectors
  _initsize=_id.size();
  setInitialModes(_initsize);
}

void VectorMesonCurrent::persistentOutput(PersistentOStream & os) const {
  os << _id << ounit(_decay_constant,GeV2);
}

void VectorMesonCurrent::persistentInput(PersistentIStream & is, int) {
  is >> _id >> iunit(_decay_constant,GeV2);
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<VectorMesonCurrent,WeakDecayCurrent>
describeHerwigVectorMesonCurrent("Herwig::VectorMesonCurrent", "HwWeakCurrents.so");

void VectorMesonCurrent::Init() {

  static ClassDocumentation<VectorMesonCurrent> documentation
    ("The VectorMesonCurrent class implements the current"
     " for the decay of the weak current into a (pseudo)vector meson.");

  static ParVector<VectorMesonCurrent,int> interfaceID
    ("ID",
     "The PDG code for the outgoing meson.",
     &VectorMesonCurrent::_id,
     0, 0, 0, -1000000, 1000000, false, false, true);

  static ParVector<VectorMesonCurrent,Energy2> interfaceDecay_Constant
    ("Decay_Constant",
     "The decay constant for the meson.",
     &VectorMesonCurrent::_decay_constant, GeV2, -1, 1.0*GeV2,-10.0*GeV2, 10.0*GeV2,
     false, false, true);

}

// create the decay phase space mode
bool VectorMesonCurrent::createMode(int icharge, unsigned int imode,
				    DecayPhaseSpaceModePtr mode,
				    unsigned int iloc,unsigned int ires,
				    DecayPhaseSpaceChannelPtr phase,Energy upp) {
  tPDPtr part(getParticleData(_id[imode]));
  // check the mode has the correct charge
  if(abs(icharge)!=abs(int(getParticleData(_id[imode])->iCharge()))) return false;
  // check if the particle is kinematically allowed
  Energy min=part->massMin();
  if(min>upp) return false;
  // construct the mode
  DecayPhaseSpaceChannelPtr newchannel(new_ptr(DecayPhaseSpaceChannel(*phase)));
  newchannel->resetDaughter(-ires,iloc);
  mode->addChannel(newchannel);
  return true;
}

// outgoing particles 
tPDVector VectorMesonCurrent::particles(int icharge, unsigned int imode, int iq, int ia) {
  tPDPtr part(getParticleData(_id[imode]));
  tPDVector output;
  if(icharge==int(part->iCharge())) {
    if(icharge==0) {
      int iqb,iab;
      decayModeInfo(imode,iqb,iab);
      if(iq==iqb&&ia==iab) output.push_back(part);
      else                 output.push_back(part->CC());
    }
    else output.push_back(part);
  }
  else if(icharge==-int(part->iCharge())) output.push_back(part->CC());
  return output;
}

vector<LorentzPolarizationVectorE> 
VectorMesonCurrent::current(const int imode, const int, 
			    Energy & scale,const ParticleVector & decay,
			    DecayIntegrator::MEOption meopt) const {
  // set up the spin information for the particle and calculate the wavefunctions
  vector<LorentzPolarizationVector> temp;
  VectorWaveFunction::
    calculateWaveFunctions(temp,decay[0],outgoing,false);
  if(meopt==DecayIntegrator::Terminate) {
    VectorWaveFunction::constructSpinInfo(temp,decay[0],
					  outgoing,true,false);
    return vector<LorentzPolarizationVectorE>(1,LorentzPolarizationVectorE());
  }
  scale=decay[0]->mass();
  // polarization vector
  Energy fact(_decay_constant[imode]/scale);
  // quarks in the current
  int iq,ia;
  decayModeInfo(imode,iq,ia);
  if(abs(iq)==abs(ia)&&abs(iq)<3) {
    fact *= sqrt(0.5);
    if(decay[0]->id()==ParticleID::rho0&&abs(iq)==1) fact=-fact;
  }
  // normalise the current
  vector<LorentzPolarizationVectorE> returnval(3);
  for(unsigned int ix=0;ix<3;++ix) {
    returnval[ix] = temp[ix] * fact;
  }
  // return the answer
  return returnval;
}

bool VectorMesonCurrent::accept(vector<int> id) {
  if(id.size()!=1) return false;
  int idtemp(abs(id[0]));
  for(unsigned int ix=0;ix<_id.size();++ix) {
    if(abs(_id[ix])==idtemp) return true;
  }
  return false;
}

unsigned int VectorMesonCurrent::decayMode(vector<int> idout) {
  int idtemp(abs(idout[0])); unsigned int ix(0);
  bool found(false);
  do {
    if(idtemp==abs(_id[ix])) found=true;
    else                     ++ix;
  }
  while(!found);
  return ix;
}

void VectorMesonCurrent::dataBaseOutput(ofstream & output,
					bool header,bool create) const {
  if(header) output << "update decayers set parameters=\"";
  if(create) output << "create Herwig::VectorMesonCurrent " << name() 
		    << " HwWeakCurrents.so\n";
  for(unsigned int ix=0;ix<_id.size();++ix) {
    if(ix<_initsize) {
      output << "newdef " << name() << ":ID " << ix 
	     << " " << _id[ix] << "\n";
      output << "newdef " << name() << ":Decay_Constant " << ix 
	     << " " << _decay_constant[ix]/GeV2 << "\n";
    }
    else {
      output << "insert " << name() << ":ID " << ix 
	     << " " << _id[ix] << "\n";
      output << "insert " << name() << ":Decay_Constant " << ix 
	     << " " << _decay_constant[ix]/GeV2 << "\n";
    }
  }
  WeakDecayCurrent::dataBaseOutput(output,false,false);
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}
#line 1 "./FivePionCurrent.cc"
// -*- C++ -*-
//
// FivePionCurrent.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the FivePionCurrent class.
//

#include "FivePionCurrent.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Interface/Switch.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"

using namespace Herwig;
using namespace ThePEG::Helicity;

FivePionCurrent::FivePionCurrent() {
  // set the number of modes
  addDecayMode(2,-1);
  addDecayMode(2,-1);
  addDecayMode(2,-1);
  setInitialModes(3);
  // masses of the intermediates
  _rhomass    = 776*MeV;
  _a1mass     = 1260*MeV;
  _omegamass  = 782*MeV;
  _sigmamass  = 800*MeV;
  // widths of the intermediates
  _rhowidth   = 150*MeV;
  _a1width    = 400*MeV;
  _omegawidth = 8.5*MeV;
  _sigmawidth = 600*MeV;
  // use local values of the resonance parameters
  _localparameters=true;
  // include the rho Breit-Wigners in omega decay
  _rhoomega = true;
  // Normalisation parameters for the different currents
  _c =4.*GeV2;
  _c0=3.;
  // various meson coupling constants
  _fomegarhopi=0.07/MeV;
  _grhopipi=6.0;
  _garhopi=6.*GeV;
  _faaf=4.*GeV;
  _ffpipi=5.*GeV;
  _presigma = ZERO;
  _preomega = ZERO;
}

inline void FivePionCurrent::doinit() {
  WeakDecayCurrent::doinit();
  if(!_localparameters) {
    _rhomass    = getParticleData(ParticleID::rhominus)->mass();
    _rhowidth   = getParticleData(ParticleID::rhominus)->width();
    _omegamass  = getParticleData(ParticleID::omega)->mass();
    _omegawidth = getParticleData(ParticleID::omega)->width();
    _sigmamass  = getParticleData(9000221)->mass();
    _sigmawidth = getParticleData(9000221)->width();
    _a1mass    = getParticleData(ParticleID::a_1minus)->mass();
    _a1width   = getParticleData(ParticleID::a_1minus)->width();
  }
  // prefactors
  _presigma =  _c/sqr(sqr(_a1mass)*_sigmamass*_rhomass)*_faaf*_ffpipi*
    _garhopi*_grhopipi;      
  _preomega = _c0*_fomegarhopi*sqr(_grhopipi/(sqr(_rhomass)*_omegamass));
}

void FivePionCurrent::persistentOutput(PersistentOStream & os) const {
  static const InvEnergy7 InvGeV7 = pow<-7,1>(GeV);
  static const InvEnergy3 InvGeV3 = pow<-3,1>(GeV);
  os << ounit(_rhomass,GeV)  << ounit(_a1mass,GeV) << ounit(_omegamass,GeV) 
     << ounit(_sigmamass,GeV) << ounit(_rhowidth,GeV)  
     << ounit(_a1width,GeV) << ounit(_omegawidth,GeV) << ounit(_sigmawidth,GeV) 
     << _localparameters << ounit(_c,GeV2) << _c0 
     << ounit(_fomegarhopi,1/GeV) << _grhopipi << ounit(_garhopi,GeV) 
     << ounit(_faaf,GeV) << ounit(_ffpipi,GeV)
     << ounit(_preomega,InvGeV7) << ounit(_presigma,InvGeV3) << _rhoomega;
}

void FivePionCurrent::persistentInput(PersistentIStream & is, int) {
  static const InvEnergy7 InvGeV7 = pow<-7,1>(GeV);
  static const InvEnergy3 InvGeV3 = pow<-3,1>(GeV);
  is >> iunit(_rhomass,GeV)  >> iunit(_a1mass,GeV) >> iunit(_omegamass,GeV) 
     >> iunit(_sigmamass,GeV) >> iunit(_rhowidth,GeV)  
     >> iunit(_a1width,GeV) >> iunit(_omegawidth,GeV) >> iunit(_sigmawidth,GeV) 
     >> _localparameters >> iunit(_c,GeV2) >> _c0 
     >> iunit(_fomegarhopi,1/GeV) >> _grhopipi >> iunit(_garhopi,GeV) 
     >> iunit(_faaf,GeV) >> iunit(_ffpipi,GeV) 
     >> iunit(_preomega,InvGeV7) >> iunit(_presigma,InvGeV3) >> _rhoomega;
}

// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<FivePionCurrent,WeakDecayCurrent>
describeHerwigFivePionCurrent("Herwig::FivePionCurrent", "HwWeakCurrents.so");

void FivePionCurrent::Init() {

  static ClassDocumentation<FivePionCurrent> documentation
    ("The FivePionCurrent class implements the model of hep-ph/0602162",
     "The model of \\cite{Kuhn:2006nw} was used for the hadronic five pion current.",
     "\\bibitem{Kuhn:2006nw} J.~H.~Kuhn and Z.~Was, hep-ph/0602162, (2006).");

  static Parameter<FivePionCurrent,Energy> interfaceRhoMass
    ("RhoMass",
     "The mass of the rho meson",
     &FivePionCurrent::_rhomass, MeV, 776*MeV, 500*MeV, 1000*MeV,
     false, false, Interface::limited);

  static Parameter<FivePionCurrent,Energy> interfaceA1Mass
    ("A1Mass",
     "The mass of the a_1 meson",
     &FivePionCurrent::_a1mass, MeV, 1260*MeV, 1000*MeV, 1500*MeV,
     false, false, Interface::limited);

  static Parameter<FivePionCurrent,Energy> interfaceOmegaMass
    ("OmegaMass",
     "The mass of the omega meson",
     &FivePionCurrent::_omegamass, MeV, 782*MeV, 600*MeV, 900*MeV,
     false, false, Interface::limited);

  static Parameter<FivePionCurrent,Energy> interfaceSigmaMass
    ("SigmaMass",
     "The mass of the sigma meson",
     &FivePionCurrent::_sigmamass, MeV, 800*MeV, 400*MeV, 1200*MeV,
     false, false, Interface::limited);

  static Parameter<FivePionCurrent,Energy> interfaceRhoWidth
    ("RhoWidth",
     "The width of the rho meson",
     &FivePionCurrent::_rhowidth, MeV, 150*MeV, 100*MeV, 300*MeV,
     false, false, Interface::limited);

  static Parameter<FivePionCurrent,Energy> interfaceA1Width
    ("A1Width",
     "The width of the a_1 meson",
     &FivePionCurrent::_a1width, MeV, 400*MeV, 100*MeV, 800*MeV,
     false, false, Interface::limited);

  static Parameter<FivePionCurrent,Energy> interfaceOmegaWidth
    ("OmegaWidth",
     "The width of the omega meson",
     &FivePionCurrent::_omegawidth, MeV, 8.5*MeV, 1.0*MeV, 20.0*MeV,
     false, false, Interface::limited);

  static Parameter<FivePionCurrent,Energy> interfaceSigmaWidth
    ("SigmaWidth",
     "The width of the sigma meson",
     &FivePionCurrent::_sigmawidth, MeV, 600*MeV, 100*MeV, 1200*MeV,
     false, false, Interface::limited);

  static Switch<FivePionCurrent,bool> interfaceLocalParameters
    ("LocalParameters",
     "Use local values of the meson masses and widths or those from the"
     " ParticleData objects.",
     &FivePionCurrent::_localparameters, true, false, false);
  static SwitchOption interfaceLocalParametersLocal
    (interfaceLocalParameters,
     "Local",
     "Use local values",
     true);
  static SwitchOption interfaceLocalParametersParticleData
    (interfaceLocalParameters,
     "ParticleData",
     "Use values from the particle data objects",
     false);

  static Switch<FivePionCurrent,bool> interfaceRhoOmega
    ("RhoOmega",
     "Option for the treatment of the rho Breit-Wigners in the omega decay",
     &FivePionCurrent::_rhoomega, true, false, false);
  static SwitchOption interfaceRhoOmegaInclude
    (interfaceRhoOmega,
     "Yes",
     "Include the rho Breit-Wigners",
     true);
  static SwitchOption interfaceRhoOmegaOmit
    (interfaceRhoOmega,
     "No",
     "Don't include the rho Breit-Wigners",
     false);

  static Parameter<FivePionCurrent,Energy2> interfaceC
    ("C",
     "The normalisation parameter for the a_1 sigma current",
     &FivePionCurrent::_c, GeV2, 4.0*GeV2, 0.1*GeV2, 20.0*GeV2,
     false, false, Interface::limited);

  static Parameter<FivePionCurrent,double> interfaceC0
    ("C0",
     "The normalisation constant for the omega-rho current",
     &FivePionCurrent::_c0, 3., 0.1, 10.0,
     false, false, Interface::limited);

  static Parameter<FivePionCurrent,InvEnergy> interfacegomegarhopi
    ("fomegarhopi",
     "The coupling of omega-rho-pi",
     &FivePionCurrent::_fomegarhopi, 1./MeV, 0.07/MeV, 0.01/MeV, 0.2/MeV,
     false, false, Interface::limited);

  static Parameter<FivePionCurrent,double> interfacegrhopipi
    ("grhopipi",
     "The coupling for rho-pi-pi",
     &FivePionCurrent::_grhopipi, 6.0, 1.0, 20.0,
     false, false, Interface::limited);

  static Parameter<FivePionCurrent,Energy> interfacegarhopi
    ("garhopi",
     "The coupling of a-rho-pi",
     &FivePionCurrent::_garhopi, GeV, 6.0*GeV, 0.1*GeV, 20.0*GeV,
     false, false, Interface::limited);

  static Parameter<FivePionCurrent,Energy> interfacefaaf
    ("faaf",
     "The coupling of a-a-f",
     &FivePionCurrent::_faaf, GeV, 4.0*GeV, 0.1*GeV, 20.0*GeV,
     false, false, Interface::limited);

  static Parameter<FivePionCurrent,Energy> interfaceffpipi
    ("ffpipi",
     "The coupling of f-pi-pi",
     &FivePionCurrent::_ffpipi, GeV, 5.0*GeV, 0.1*GeV, 20.0*GeV,
     false, false, Interface::limited);
}

bool FivePionCurrent::accept(vector<int> id) {
  bool allowed(false);
  // check five products
  if(id.size()!=5){return false;}
  int npiminus=0,npiplus=0,npi0=0;
  for(unsigned int ix=0;ix<id.size();++ix) {
    if(id[ix]==ParticleID:: piplus){++npiplus;}
    else if(id[ix]==ParticleID::piminus){++npiminus;}
    else if(id[ix]==ParticleID::pi0){++npi0;}
  }
  if(npiplus>npiminus) swap(npiplus,npiminus);
  if(     npiminus==3&&npiplus==2&&npi0==0) allowed=true;
  else if(npiminus==2&&npiplus==1&&npi0==2) allowed=true;
  else if(npiminus==1&&npiplus==0&&npi0==4) allowed=true;
  return allowed;
}

bool FivePionCurrent::createMode(int icharge, unsigned int imode,
				 DecayPhaseSpaceModePtr mode,
				 unsigned int iloc,unsigned int ires,
				 DecayPhaseSpaceChannelPtr phase,Energy upp) {
  // check the charge
  if(abs(icharge)!=3) return false;
  // check that the modes are kinematical allowed
  Energy min(ZERO);
  // 3 pi- 2pi+
  if(imode==0) {
    min=5.*getParticleData(ParticleID::piplus)->mass();
  }
  // 2pi- pi+ 2pi0
  else if(imode==1) {
    min=3.*getParticleData(ParticleID::piplus)->mass()
      +2.*getParticleData(ParticleID::pi0)->mass();
  }
  // pi- 4pi0
  else {
    min=   getParticleData(ParticleID::piplus)->mass()
      +4.*getParticleData(ParticleID::pi0)->mass();
  }
  if(min>upp) return false;
  // intermediates for the channels
  tPDPtr omega(getParticleData(ParticleID::omega)),rhop,rhom,
    rho0(getParticleData(ParticleID::rho0)),a1m,a10(getParticleData(ParticleID::a_10)),
    sigma(getParticleData(9000221));
  if(icharge==3) {
    rhop = getParticleData(ParticleID::rhominus);
    rhom = getParticleData(ParticleID::rhoplus);
    a1m  = getParticleData(ParticleID::a_1plus);
  }
  else {
    rhop = getParticleData(ParticleID::rhoplus);
    rhom = getParticleData(ParticleID::rhominus);
    a1m  = getParticleData(ParticleID::a_1minus);
  }
  DecayPhaseSpaceChannelPtr newchannel;
  // all charged mode
  if(imode==0) {
    if(sigma) {
      // 1st two a_1 sigma channels
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc+3, iloc+4);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc  );
      newchannel->addIntermediate(rho0   ,0,0.0, iloc+1, iloc+2);
      mode->addChannel(newchannel);
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc+3, iloc+4);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+1);
      newchannel->addIntermediate(rho0   ,0,0.0, iloc  , iloc+2);
      mode->addChannel(newchannel);
      // 2nd two a_1 sigma channels
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc+3, iloc  );
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+4);
      newchannel->addIntermediate(rho0   ,0,0.0, iloc+1, iloc+2);
      mode->addChannel(newchannel);
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc+3, iloc  );
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+1);
      newchannel->addIntermediate(rho0   ,0,0.0, iloc+4, iloc+2);
      mode->addChannel(newchannel);
      // 3rd two a_1 sigma channels
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc+3, iloc+1);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc  );
      newchannel->addIntermediate(rho0   ,0,0.0, iloc+4, iloc+2);
      mode->addChannel(newchannel);
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc+3, iloc+1);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+4);
      newchannel->addIntermediate(rho0   ,0,0.0, iloc  , iloc+2);
      mode->addChannel(newchannel);
      // 4th two a_1 sigma channels
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc+2, iloc+4);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc  );
      newchannel->addIntermediate(rho0   ,0,0.0, iloc+1, iloc+3);
      mode->addChannel(newchannel);
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc+2, iloc+4);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+1);
      newchannel->addIntermediate(rho0   ,0,0.0, iloc  , iloc+3);
      mode->addChannel(newchannel);
      // 5th two a_1 sigma channels
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc+2, iloc  );
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+4);
      newchannel->addIntermediate(rho0   ,0,0.0, iloc+1, iloc+3);
      mode->addChannel(newchannel);
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc+2, iloc  );
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+1);
      newchannel->addIntermediate(rho0   ,0,0.0, iloc+4, iloc+3);
      mode->addChannel(newchannel);
      // 6th two a_1 sigma channels
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc+2, iloc+1);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc  );
      newchannel->addIntermediate(rho0   ,0,0.0, iloc+4, iloc+3);
      mode->addChannel(newchannel);
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc+2, iloc+1);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+4);
      newchannel->addIntermediate(rho0   ,0,0.0, iloc  , iloc+3);
      mode->addChannel(newchannel);
    }
  }
  // 2 neutral mode
  else if(imode==1) {
    // first three omega channels
    newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
    newchannel->addIntermediate(rhom   ,0,0.0, iloc+3, iloc+4);
    newchannel->addIntermediate(omega  ,0,0.0,-ires-3, iloc+2);
    newchannel->addIntermediate(rho0   ,0,0.0, iloc  , iloc+1);
    mode->addChannel(newchannel);
    newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
    newchannel->addIntermediate(rhom   ,0,0.0, iloc+3, iloc+4);
    newchannel->addIntermediate(omega  ,0,0.0,-ires-3, iloc+1);
    newchannel->addIntermediate(rhop   ,0,0.0, iloc  , iloc+2);
    mode->addChannel(newchannel);
    newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
    newchannel->addIntermediate(rhom   ,0,0.0, iloc+3, iloc+4);
    newchannel->addIntermediate(omega  ,0,0.0,-ires-3, iloc  );
    newchannel->addIntermediate(rhom   ,0,0.0, iloc+1, iloc+2);
    mode->addChannel(newchannel);
    // second three omega channels
    newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
    newchannel->addIntermediate(rhom   ,0,0.0, iloc+1, iloc+4);
    newchannel->addIntermediate(omega  ,0,0.0,-ires-3, iloc+2);
    newchannel->addIntermediate(rho0   ,0,0.0, iloc  , iloc+3);
    mode->addChannel(newchannel);
    newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
    newchannel->addIntermediate(rhom   ,0,0.0, iloc+1, iloc+4);
    newchannel->addIntermediate(omega  ,0,0.0,-ires-3, iloc+3);
    newchannel->addIntermediate(rhop   ,0,0.0, iloc  , iloc+2);
    mode->addChannel(newchannel);
    newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
    newchannel->addIntermediate(rhom   ,0,0.0, iloc+1, iloc+4);
    newchannel->addIntermediate(omega  ,0,0.0,-ires-3, iloc  );
    newchannel->addIntermediate(rhom   ,0,0.0, iloc+3, iloc+2);
    mode->addChannel(newchannel);
    // third  three omega channels
    newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
    newchannel->addIntermediate(rhom   ,0,0.0, iloc+3, iloc+2);
    newchannel->addIntermediate(omega  ,0,0.0,-ires-3, iloc+4);
    newchannel->addIntermediate(rho0   ,0,0.0, iloc  , iloc+1);
    mode->addChannel(newchannel);
    newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
    newchannel->addIntermediate(rhom   ,0,0.0, iloc+3, iloc+2);
    newchannel->addIntermediate(omega  ,0,0.0,-ires-3, iloc+1);
    newchannel->addIntermediate(rhop   ,0,0.0, iloc  , iloc+4);
    mode->addChannel(newchannel);
    newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
    newchannel->addIntermediate(rhom   ,0,0.0, iloc+3, iloc+2);
    newchannel->addIntermediate(omega  ,0,0.0,-ires-3, iloc  );
    newchannel->addIntermediate(rhom   ,0,0.0, iloc+1, iloc+4);
    mode->addChannel(newchannel);
    // fourth three omega channels
    newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
    newchannel->addIntermediate(rhom   ,0,0.0, iloc+1, iloc+2);
    newchannel->addIntermediate(omega  ,0,0.0,-ires-3, iloc+4);
    newchannel->addIntermediate(rho0   ,0,0.0, iloc  , iloc+3);
    mode->addChannel(newchannel);
    newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
    newchannel->addIntermediate(rhom   ,0,0.0, iloc+1, iloc+2);
    newchannel->addIntermediate(omega  ,0,0.0,-ires-3, iloc+3);
    newchannel->addIntermediate(rhop   ,0,0.0, iloc  , iloc+4);
    mode->addChannel(newchannel);
    newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
    newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
    newchannel->addIntermediate(rhom   ,0,0.0, iloc+1, iloc+2);
    newchannel->addIntermediate(omega  ,0,0.0,-ires-3, iloc  );
    newchannel->addIntermediate(rhom   ,0,0.0, iloc+3, iloc+4);
    mode->addChannel(newchannel);
    // first two sigma a_1 channels
    if(sigma) {
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc  , iloc+1);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+2);
      newchannel->addIntermediate(rhom   ,0,0.0, iloc+3, iloc+4);
      mode->addChannel(newchannel);
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc  , iloc+1);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+4);
      newchannel->addIntermediate(rhom   ,0,0.0, iloc+3, iloc+2);
      mode->addChannel(newchannel);
      // second two sigma a_1 channels
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc  , iloc+3);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+2);
      newchannel->addIntermediate(rhom   ,0,0.0, iloc+1, iloc+4);
      mode->addChannel(newchannel);
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc  , iloc+3);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+4);
      newchannel->addIntermediate(rhom   ,0,0.0, iloc+1, iloc+2);
      mode->addChannel(newchannel);
      // third two sigma a_1 channels
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc+2, iloc+4);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+1);
      newchannel->addIntermediate(rho0   ,0,0.0, iloc  , iloc+3);
      mode->addChannel(newchannel);
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc+2, iloc+4);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+3);
      newchannel->addIntermediate(rho0   ,0,0.0, iloc  , iloc+1);
      mode->addChannel(newchannel);
    }
  }
  // 4 neutral mode
  else {
    if(sigma) {
      // 1st two sigma a_1 channels
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc+3, iloc+4);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+1);
      newchannel->addIntermediate(rhom   ,0,0.0, iloc  , iloc+2);
      mode->addChannel(newchannel);
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc+3, iloc+4);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc  );
      newchannel->addIntermediate(rhom   ,0,0.0, iloc+1, iloc+2);
      mode->addChannel(newchannel);
      // 2nd two sigma a_1 channels
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc+3, iloc  );
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+1);
      newchannel->addIntermediate(rhom   ,0,0.0, iloc+4, iloc+2);
      mode->addChannel(newchannel);
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc+3, iloc  );
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+4);
      newchannel->addIntermediate(rhom   ,0,0.0, iloc+1, iloc+2);
      mode->addChannel(newchannel);
      // 3rd two sigma a_1 channels
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc  , iloc+4);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+1);
      newchannel->addIntermediate(rhom   ,0,0.0, iloc+3, iloc+2);
      mode->addChannel(newchannel);
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc  , iloc+4);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+3);
      newchannel->addIntermediate(rhom   ,0,0.0, iloc+1, iloc+2);
      mode->addChannel(newchannel);
      // 4th two sigma a_1 channels
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc+1, iloc+4);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+3);
      newchannel->addIntermediate(rhom   ,0,0.0, iloc  , iloc+2);
      mode->addChannel(newchannel);
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc+1, iloc+4);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc  );
      newchannel->addIntermediate(rhom   ,0,0.0, iloc+3, iloc+2);
      mode->addChannel(newchannel);
      // 5th two sigma a_1 channels
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc+3, iloc+1);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+4);
      newchannel->addIntermediate(rhom   ,0,0.0, iloc  , iloc+2);
      mode->addChannel(newchannel);
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc+3, iloc+1);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc  );
      newchannel->addIntermediate(rhom   ,0,0.0, iloc+4, iloc+2);
      mode->addChannel(newchannel);
      // 6th two sigma a_1 channels
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc  , iloc+1);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+3);
      newchannel->addIntermediate(rhom   ,0,0.0, iloc+4, iloc+2);
      mode->addChannel(newchannel);
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-1,-ires-2);
      newchannel->addIntermediate(sigma  ,0,0.0, iloc  , iloc+1);
      newchannel->addIntermediate(a1m    ,0,0.0,-ires-3, iloc+4);
      newchannel->addIntermediate(rhom   ,0,0.0, iloc+3, iloc+2);
      mode->addChannel(newchannel);
    }
  }
  // reset the parameters of the resonances if using local values
  if(_localparameters) {
    mode->resetIntermediate(rhom,_rhomass,_rhowidth);
    mode->resetIntermediate(rhop,_rhomass,_rhowidth);
    mode->resetIntermediate(rho0,_rhomass,_rhowidth);
    mode->resetIntermediate(omega,_omegamass,_omegawidth);
    mode->resetIntermediate(a1m,_a1mass,_a1width);
    mode->resetIntermediate(a10,_a1mass,_a1width);
    if(sigma) mode->resetIntermediate(sigma,_sigmamass,_sigmawidth);
  }
  // return if successful
  return true;
}

// the particles produced by the current
tPDVector FivePionCurrent::particles(int icharge, unsigned int imode,int,int) {
  // particle data objects for the pions
  tPDPtr piplus (getParticleData(ParticleID::piplus ));
  tPDPtr pi0    (getParticleData(ParticleID::pi0    ));
  tPDPtr piminus(getParticleData(ParticleID::piminus));
  if(icharge==3) swap(piplus,piminus);
  tPDVector output(5);
  // all charged
  if(imode==0) {
    output[0]=piminus;
    output[1]=piminus;
    output[2]=piplus;;
    output[3]=piplus;
    output[4]=piminus;
  }
  // two neutral
  else if(imode==1) {
    output[0]=piplus;
    output[1]=piminus;
    output[2]=pi0;;
    output[3]=piminus;
    output[4]=pi0;
  }
  // four neutral
  else {
    output[0]=pi0;
    output[1]=pi0;
    output[2]=piminus;;
    output[3]=pi0;
    output[4]=pi0;
  }
  return output;
}

// the decay mode
unsigned int FivePionCurrent::decayMode(vector<int> idout) {
  unsigned int npi(0);
  for(unsigned int ix=0;ix<idout.size();++ix) {
    if(abs(idout[ix])==ParticleID::pi0) ++npi;
  }
  return npi/2;
}

// output the information for the database
void FivePionCurrent::dataBaseOutput(ofstream & output,bool header,bool create) const {
  if(header) output << "update decayers set parameters=\"";
  if(create) output << "create Herwig::FivePionCurrent " << name() 
		    << " HwWeakCurrents.so\n";
  output << "newdef " << name() << ":RhoMass "    << _rhomass/MeV << "\n";
  output << "newdef " << name() << ":A1Mass  "    << _a1mass/MeV  << "\n";
  output << "newdef " << name() << ":SigmaMass  " << _sigmamass/MeV  << "\n";
  output << "newdef " << name() << ":OmegaMass  " << _omegamass/MeV  << "\n";
  output << "newdef " << name() << ":RhoWidth "    << _rhowidth/MeV << "\n";
  output << "newdef " << name() << ":A1Width  "    << _a1width/MeV  << "\n";
  output << "newdef " << name() << ":SigmaWidth  " << _sigmawidth/MeV  << "\n";
  output << "newdef " << name() << ":OmegaWidth  " << _omegawidth/MeV  << "\n";
  output << "newdef " << name() << ":LocalParameters " <<  _localparameters << "\n";
  output << "newdef " << name() << ":RhoOmega " << _rhoomega << "\n";
  output << "newdef " << name() << ":C " << _c/GeV2 << "\n";
  output << "newdef " << name() << ":C0 " << _c0 << "\n";
  output << "newdef " << name() << ":fomegarhopi " <<_fomegarhopi*MeV << "\n";
  output << "newdef " << name() << ":grhopipi " <<_grhopipi << "\n";
  output << "newdef " << name() << ":garhopi " << _garhopi/GeV << "\n";
  output << "newdef " << name() << ":faaf " <<_faaf/GeV << "\n";
  output << "newdef " << name() << ":ffpipi " << _ffpipi/GeV << "\n";
  WeakDecayCurrent::dataBaseOutput(output,false,false);
  if(header) output << "\n\" where BINARY ThePEGName=\"" << fullName() << "\";\n";
}

vector<LorentzPolarizationVectorE> 
FivePionCurrent::current(const int imode,const int ichan,
			 Energy & scale,const ParticleVector & decay,
			 DecayIntegrator::MEOption meopt) const {
  useMe();
  if(meopt==DecayIntegrator::Terminate) {
    for(unsigned int ix=0;ix<5;++ix)
      ScalarWaveFunction::constructSpinInfo(decay[ix],outgoing,true);
    return vector<LorentzPolarizationVectorE>(1,LorentzPolarizationVectorE());
  }
  LorentzVector<complex<InvEnergy2> > output;
  Lorentz5Momentum q1(decay[0]->momentum());
  Lorentz5Momentum q2(decay[1]->momentum());
  Lorentz5Momentum q3(decay[2]->momentum());
  Lorentz5Momentum q4(decay[3]->momentum());
  Lorentz5Momentum q5(decay[4]->momentum());
  // total momentum
  Lorentz5Momentum Q(q1+q2+q3+q4+q5);
  Q.rescaleMass();
  scale=Q.mass();
  // decide which decay mode
  if(imode==0) {
    if(ichan<0) {
      output=
	a1SigmaCurrent(0,Q,q1,q2,q3,q4,q5)+
	a1SigmaCurrent(0,Q,q5,q2,q3,q4,q1)+
	a1SigmaCurrent(0,Q,q1,q5,q3,q4,q2)+
	a1SigmaCurrent(0,Q,q1,q2,q4,q3,q5)+
	a1SigmaCurrent(0,Q,q5,q2,q4,q3,q1)+
	a1SigmaCurrent(0,Q,q1,q5,q4,q3,q2);
    }
    else if(ichan==0 ) output=a1SigmaCurrent(2,Q,q1,q2,q3,q4,q5);
    else if(ichan==1 ) output=a1SigmaCurrent(1,Q,q1,q2,q3,q4,q5);
    else if(ichan==2 ) output=a1SigmaCurrent(2,Q,q5,q2,q3,q4,q1);
    else if(ichan==3 ) output=a1SigmaCurrent(1,Q,q5,q2,q3,q4,q1);
    else if(ichan==4 ) output=a1SigmaCurrent(2,Q,q1,q5,q3,q4,q2);
    else if(ichan==5 ) output=a1SigmaCurrent(1,Q,q1,q5,q3,q4,q2);
    else if(ichan==6 ) output=a1SigmaCurrent(2,Q,q1,q2,q4,q3,q5);
    else if(ichan==7 ) output=a1SigmaCurrent(1,Q,q1,q2,q4,q3,q5);
    else if(ichan==8 ) output=a1SigmaCurrent(2,Q,q5,q2,q4,q3,q1);
    else if(ichan==9 ) output=a1SigmaCurrent(1,Q,q5,q2,q4,q3,q1);
    else if(ichan==10) output=a1SigmaCurrent(2,Q,q1,q5,q4,q3,q2);
    else if(ichan==11) output=a1SigmaCurrent(1,Q,q1,q5,q4,q3,q2);
    // identical particle symmetry factor
    output/=sqrt(12.);
  }
  else if(imode==1) {
    if(ichan<0) {
      output=
	rhoOmegaCurrent(0,Q,q1,q2,q3,q4,q5)
	+rhoOmegaCurrent(0,Q,q1,q4,q3,q2,q5)
	+rhoOmegaCurrent(0,Q,q1,q2,q5,q4,q3)
	+rhoOmegaCurrent(0,Q,q1,q4,q5,q2,q3)
	+a1SigmaCurrent(0,Q,q2,q4,q1,q3,q5)
	+a1SigmaCurrent(0,Q,q3,q5,q2,q1,q4)
	+a1SigmaCurrent(0,Q,q3,q5,q4,q1,q2);
    }
    else if(ichan==0 ) output=rhoOmegaCurrent(3,Q,q1,q2,q3,q4,q5);
    else if(ichan==1 ) output=rhoOmegaCurrent(2,Q,q1,q2,q3,q4,q5);
    else if(ichan==2 ) output=rhoOmegaCurrent(1,Q,q1,q2,q3,q4,q5);
    else if(ichan==3 ) output=rhoOmegaCurrent(3,Q,q1,q4,q3,q2,q5);
    else if(ichan==4 ) output=rhoOmegaCurrent(2,Q,q1,q4,q3,q2,q5);
    else if(ichan==5 ) output=rhoOmegaCurrent(1,Q,q1,q4,q3,q2,q5);
    else if(ichan==6 ) output=rhoOmegaCurrent(3,Q,q1,q2,q5,q4,q3);
    else if(ichan==7 ) output=rhoOmegaCurrent(2,Q,q1,q2,q5,q4,q3);
    else if(ichan==8 ) output=rhoOmegaCurrent(1,Q,q1,q2,q5,q4,q3);
    else if(ichan==9 ) output=rhoOmegaCurrent(3,Q,q1,q4,q5,q2,q3);
    else if(ichan==10) output=rhoOmegaCurrent(2,Q,q1,q4,q5,q2,q3);
    else if(ichan==11) output=rhoOmegaCurrent(1,Q,q1,q4,q5,q2,q3);
    else if(ichan==12) output=a1SigmaCurrent(2,Q,q3,q5,q4,q1,q2);
    else if(ichan==13) output=a1SigmaCurrent(1,Q,q3,q5,q4,q1,q2);
    else if(ichan==14) output=a1SigmaCurrent(2,Q,q3,q5,q2,q1,q4);
    else if(ichan==15) output=a1SigmaCurrent(1,Q,q3,q5,q2,q1,q4);
    else if(ichan==16) output=a1SigmaCurrent(2,Q,q2,q4,q1,q3,q5);
    else if(ichan==17) output=a1SigmaCurrent(1,Q,q2,q4,q1,q3,q5);
    // identical particle symmetry factor
    output/=2.;
  }
  else if(imode==2) {
    if(ichan<0) {
      output=
	a1SigmaCurrent(0,Q,q1,q2,q3,q4,q5)+
	a1SigmaCurrent(0,Q,q5,q2,q3,q4,q1)+
	a1SigmaCurrent(0,Q,q2,q4,q3,q1,q5)+
	a1SigmaCurrent(0,Q,q1,q4,q3,q2,q5)+
	a1SigmaCurrent(0,Q,q1,q5,q3,q4,q2)+
	a1SigmaCurrent(0,Q,q4,q5,q3,q1,q2);
    }
    else if(ichan==0 ) output=a1SigmaCurrent(1,Q,q1,q2,q3,q4,q5);
    else if(ichan==1 ) output=a1SigmaCurrent(2,Q,q1,q2,q3,q4,q5);
    else if(ichan==2 ) output=a1SigmaCurrent(1,Q,q5,q2,q3,q4,q1);
    else if(ichan==3 ) output=a1SigmaCurrent(2,Q,q5,q2,q3,q4,q1);
    else if(ichan==4 ) output=a1SigmaCurrent(2,Q,q2,q4,q3,q1,q5);
    else if(ichan==5 ) output=a1SigmaCurrent(1,Q,q2,q4,q3,q1,q5);
    else if(ichan==6 ) output=a1SigmaCurrent(1,Q,q1,q4,q3,q2,q5);
    else if(ichan==7 ) output=a1SigmaCurrent(2,Q,q1,q4,q3,q2,q5);
    else if(ichan==8 ) output=a1SigmaCurrent(1,Q,q1,q5,q3,q4,q2);
    else if(ichan==9 ) output=a1SigmaCurrent(2,Q,q1,q5,q3,q4,q2);
    else if(ichan==10) output=a1SigmaCurrent(2,Q,q4,q5,q3,q1,q2);
    else if(ichan==11) output=a1SigmaCurrent(1,Q,q4,q5,q3,q1,q2);
    // identical particle symmetry factor
    output/=sqrt(24.);
  }
  else {
    throw DecayIntegratorError() << "Unknown decay mode in the " 
				 << "FivePionCurrent::"
				 << "hadronCurrent()" << Exception::abortnow;
  }
  // normalise and return the current
  return vector<LorentzPolarizationVectorE>(1, output * pow<3,1>(scale));
}
#line 1 "./KPiCurrent.cc"
// -*- C++ -*-
//
// KPiCurrent.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the KPiCurrent class.
//

#include "KPiCurrent.h"
#include "ThePEG/Utilities/DescribeClass.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Interface/Switch.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "ThePEG/Helicity/WaveFunction/ScalarWaveFunction.h"

using namespace Herwig;
using namespace ThePEG::Helicity;
using ThePEG::Helicity::outgoing;
using ThePEG::Helicity::ScalarWaveFunction;

KPiCurrent::KPiCurrent() :
  _localparameters(true),_transverse(false), _cV(1.),_cS(0.2),
  _mpi(ZERO), _mK(ZERO) {
  // set up for the modes in the base class
  addDecayMode(2,-3);
  addDecayMode(2,-3);
  setInitialModes(2);
  // parameters for the vector resonances
  _vecmag  .push_back(1.);_vecmag  .push_back(-0.135);
  _vecphase.push_back(0.);_vecphase.push_back(180.  );
  _vecmass .push_back(891.6*MeV);_vecmass .push_back(1412.*MeV);
  _vecwidth.push_back( 50. *MeV);_vecwidth.push_back( 227.*MeV);
  // parameters for the scalar resonances
  _scamag  .push_back(0.);_scamag  .push_back(1.);
  _scaphase.push_back(0.);_scaphase.push_back(0.);
  _scamass .push_back(841.*MeV);_scamass .push_back(1429.*MeV);
  _scawidth.push_back(618.*MeV);_scawidth.push_back( 287.*MeV);
}

void KPiCurrent::persistentOutput(PersistentOStream & os) const {
  os << _cV << _cS << _localparameters 
     << ounit(_mpi,GeV) << ounit(_mK,GeV) 
     << _resmap
     << _vecmag << _vecphase << _vecwgt 
     << ounit(_vecmass,MeV) << ounit(_vecwidth,MeV)
     << _scamag << _scaphase << _scawgt 
     << ounit(_scamass,MeV) << ounit(_scawidth,MeV)
     << _transverse;
}

void KPiCurrent::persistentInput(PersistentIStream & is, int) {
  is >> _cV >> _cS >> _localparameters 
     >> iunit(_mpi,GeV) >> iunit(_mK,GeV) 
     >> _resmap
     >> _vecmag >> _vecphase >> _vecwgt 
     >> iunit(_vecmass,MeV) >> iunit(_vecwidth,MeV) 
     >> _scamag >> _scaphase >> _scawgt 
     >> iunit(_scamass,MeV) >> iunit(_scawidth,MeV)
     >> _transverse;
}

void KPiCurrent::doinit() {
  WeakDecayCurrent::doinit();
  // check consistency of parametrers
  if(_vecmass.size()!=_vecwidth.size()||
     _scamass.size()!=_scawidth.size()) {
    throw InitException() << "Inconsistent parameters in KPiCurrent"
			  << "doinit()" << Exception::abortnow;
  }
  // the resonances
  tPDPtr vec[3]={getParticleData(-323   ),getParticleData(-100323),
		 getParticleData(-30323 )};
  tPDPtr sca[3]={getParticleData(-9000321),getParticleData(-10321)};
  // reset the masses in the form-factors if needed
  if(_localparameters) {
    if(_vecmass.size()<3) {
      for(unsigned int ix=_vecmass.size();ix<3;++ix) {
	if(vec[ix]) {
	  _vecmass.push_back( vec[ix]->mass() );
	  _vecwidth.push_back(vec[ix]->width());
	}
      }
    }
    if(_scamass.size()<2) {
      for(unsigned int ix=_scamass.size();ix<2;++ix) {
	if(sca[ix]) {
	  _scamass.push_back( sca[ix]->mass() );
	  _scawidth.push_back(sca[ix]->width());
	}
      }
    }
  }
  else {
    _vecmass.clear();_vecwidth.clear();
    for(unsigned int ix=0;ix<3;++ix) {
      if(vec[ix]) {
	_vecmass .push_back(vec[ix]->mass() );
	_vecwidth.push_back(vec[ix]->width());
      }
    }
    _scamass.clear();_scawidth.clear();
    for(unsigned int ix=0;ix<2;++ix) {
      if(sca[ix]) {
	_scamass .push_back(sca[ix]->mass() );
	_scawidth.push_back(sca[ix]->width());
      }
    }
  }
  _mpi=getParticleData(ParticleID::piplus)->mass();
  _mK =getParticleData(ParticleID::K0    )->mass();
  // weight for the vector channels
  if(_vecmag.size()!=_vecphase.size())
    throw InitException() << "The vectors containing the weights and phase for the"
			  << "vector channel must be the same size in"
			  << "KPiCurrent::doinit()" 
			  << Exception::runerror;
  _vecwgt.resize(_vecmag.size());
  for(unsigned int ix=0;ix<_vecwgt.size();++ix) {
    double angle = _vecphase[ix]/180.*Constants::pi;
    _vecwgt[ix] = _vecmag[ix]*(cos(angle)+Complex(0.,1.)*sin(angle));
  }
  // weight for the scalar channels
  if(_scamag.size()!=_scaphase.size())
    throw InitException() << "The vectors containing the weights and phase for the"
			  << "scalar channel must be the same size in"
			  << "KPiCurrent::doinit()" 
			  << Exception::runerror;
  _scawgt.resize(_scamag.size());
  for(unsigned int ix=0;ix<_scawgt.size();++ix) {
    double angle = _scaphase[ix]/180.*Constants::pi;
    _scawgt[ix] = _scamag[ix]*(cos(angle)+Complex(0.,1.)*sin(angle));
  }
  // mapping for the resonaces
  int ires(-1);
  for(unsigned int ix=0;ix<3;++ix) {
    if(vec[ix]) ++ires;
    if(ires<int(_vecwgt.size())) _resmap.push_back(ires);
  }
  if(_resmap.size()<_vecwgt.size()) {
    for(unsigned int ix=_resmap.size();ix<_vecwgt.size();++ix) {
      _resmap.push_back(-1);
    }
  }
  ires=-1;
  for(unsigned int ix=0;ix<2;++ix) {
    if(sca[ix]) ++ires;
    if(ires<int(_scawgt.size())) _resmap.push_back(ires);
  }
  if(_resmap.size()<_vecwgt.size()+_scawgt.size()) {
    for(unsigned int ix=_resmap.size()-_scawgt.size();
	ix<_scawgt.size();++ix) {
      _resmap.push_back(-1);
    }
  }
}
// The following static variable is needed for the type
// description system in ThePEG.
DescribeClass<KPiCurrent,WeakDecayCurrent>
describeHerwigKPiCurrent("Herwig::KPiCurrent", "HwWeakCurrents.so");

void KPiCurrent::Init() {

  static ClassDocumentation<KPiCurrent> documentation
    ("The KPiCurrent class",
     "The K pi weak current has the form of \\cite{Finkemeier:1996dh}.",
     "%\\cite{Finkemeier:1996dh}\n"
     "\\bibitem{Finkemeier:1996dh}\n"
     "  M.~Finkemeier and E.~Mirkes,\n"
     "  %``The scalar contribution to tau --> K pi nu/tau,''\n"
     "  Z.\\ Phys.\\  C {\\bf 72}, 619 (1996)\n"
     "  [arXiv:hep-ph/9601275].\n"
     "  %%CITATION = ZEPYA,C72,619;%%\n"
     );

  static Parameter<KPiCurrent,double> interfacecV
    ("cV",
     "The weight for the vector contribution",
     &KPiCurrent::_cV, 1., 0., 10.0,
     false, false, Interface::limited);

  static Parameter<KPiCurrent,double> interfacecS
    ("cS",
     "The weight for the scalar contribution",
     &KPiCurrent::_cS, 0.2, -10.0, 10.0,
     false, false, Interface::limited);
  
  static ParVector<KPiCurrent,double> interfaceVectorMagnitude
    ("VectorMagnitude",
     "Magnitude of the weight for the different vector resonances",
     &KPiCurrent::_vecmag, -1, 0., 0, 0,
     false, false, Interface::nolimits);

  static ParVector<KPiCurrent,double> interfaceVectorPhase
    ("VectorPhase",
     "Phase of the weight of the different vector resonances",
     &KPiCurrent::_vecphase, -1, 0., 0, 0,
     false, false, Interface::nolimits);
  
  static ParVector<KPiCurrent,double> interfaceScalarMagnitude
    ("ScalarMagnitude",
     "Magnitude of the weight for the different scalar resonances",
     &KPiCurrent::_scamag, -1, 0., 0, 0,
     false, false, Interface::nolimits);

  static ParVector<KPiCurrent,double> interfaceScalarPhase
    ("ScalarPhase",
     "Phase of the weight of the different scalar resonances",
     &KPiCurrent::_scaphase, -1, 0., 0, 0,
     false, false, Interface::nolimits);

  static Switch<KPiCurrent,bool> interfaceLocalParameters
    ("LocalParameters",
     "Use local values for the masses and widths of the resonances or those"
     " from the ParticleData objects",
     &KPiCurrent::_localparameters, true, false, false);
  static SwitchOption interfaceLocalParametersLocal
    (interfaceLocalParameters,
     "Local",
     "Use local values",
     true);
  static SwitchOption interfaceLocalParametersParticleData
    (interfaceLocalParameters,
     "ParticleData",
     "Use the values from the particle data objects",
     false);

  static Switch<KPiCurrent,bool> interfaceTransverse
    ("Transverse",
     "Form of the vector projection operator.",
     &KPiCurrent::_transverse, false, false, false);
  static SwitchOption interfaceTransverseTransverse
    (interfaceTransverse,
     "Transverse",
     "Use 1/Q^2 in the projection operator to force it to be transverse",
     true);
  static SwitchOption interfaceTransverseMass
    (interfaceTransverse,
     "Mass",
     "Use the on-shell mass in the projection operator",
     false);

  static ParVector<KPiCurrent,Energy> interfaceVectorMass
    ("VectorMass",
     "Masses of the vector resonances",
     &KPiCurrent::_vecmass, MeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static ParVector<KPiCurrent,Energy> interfaceVectorWidth
    ("VectorWidth",
     "Widths of the vector resonances",
     &KPiCurrent::_vecwidth, MeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static ParVector<KPiCurrent,Energy> interfaceScalarMass
    ("ScalarMass",
     "Masses of the scalar resonances",
     &KPiCurrent::_scamass, MeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static ParVector<KPiCurrent,Energy> interfaceScalarWidth
    ("ScalarWidth",
     "Widths of the scalar resonances",
     &KPiCurrent::_scawidth, MeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);
}

bool KPiCurrent::accept(vector<int> id) {
  bool allowed(false);
  // check there are only two particles
  if(id.size()!=2){return false;}
  if      ((id[0]==ParticleID::Kminus && id[1]==ParticleID::pi0)    ||
	  (id[0]==ParticleID::pi0    && id[1]==ParticleID::Kminus) ||
	  (id[0]==ParticleID::Kplus  && id[1]==ParticleID::pi0)    ||
	  (id[0]==ParticleID::pi0    && id[1]==ParticleID::Kplus)) allowed=true;
  // single neutral kaon
  else if((id[0]==ParticleID::piminus && id[1]==ParticleID::Kbar0)   ||
	  (id[0]==ParticleID::Kbar0   && id[1]==ParticleID::piminus) ||
	  (id[0]==ParticleID::piplus  && id[1]==ParticleID::K0)      ||
	  (id[0]==ParticleID::K0      && id[1]==ParticleID::piplus)) allowed=true;
  return allowed;
}

tPDVector KPiCurrent::particles(int icharge, unsigned int imode, int,int) {
  if(abs(icharge)!=3) return tPDVector();
  tPDVector output(2);
  if(imode==0) {
    output[0]=getParticleData(ParticleID::Kplus);
    output[1]=getParticleData(ParticleID::pi0);
  }
  else if(imode==1) {
    output[0]=getParticleData(ParticleID::K0);
    output[1]=getParticleData(ParticleID::piplus);
  }
  if(icharge==-3) {
    for(unsigned int ix=0;ix<output.size();++ix) {
      if(output[ix]->CC()) output[ix]=output[ix]->CC();
    }
  }
  return output;
}

unsigned int KPiCurrent::decayMode(vector<int> id) {
  unsigned int imode(0);
  for(unsigned int ix=0;ix<id.size();++ix) {
    if(abs(id[ix])==ParticleID::K0) imode=1;
  }
  return imode;
}

bool KPiCurrent::createMode(int icharge,unsigned int imode,
			    DecayPhaseSpaceModePtr mode,
			    unsigned int iloc,unsigned int,
			    DecayPhaseSpaceChannelPtr phase,Energy upp) {
  if(abs(icharge)!=3) return false; 
  // make sure that the decays are kinematically allowed
  tPDPtr part[2];
  if(imode==0) {
    part[0]=getParticleData(ParticleID::Kplus);
    part[1]=getParticleData(ParticleID::pi0);
  }
  else if(imode==1) {
    part[0]=getParticleData(ParticleID::K0);
    part[1]=getParticleData(ParticleID::piplus);
  }
  else {
    return false;
  }
  Energy min(part[0]->massMin()+part[1]->massMin());
  if(min>upp) return false;
  DecayPhaseSpaceChannelPtr newchannel;
  // possible resonances
  tPDPtr res[5]={getParticleData(-323   ),getParticleData(-100323),
		 getParticleData(-30323 ),getParticleData(-9000321),
		 getParticleData(-10321)};
  // create the channels
  for(unsigned int ix=0;ix<5;++ix) {
    if(res[ix]) {
      newchannel=new_ptr(DecayPhaseSpaceChannel(*phase));
      newchannel->addIntermediate(res[ix],0,0.0,iloc,iloc+1);
      mode->addChannel(newchannel);
    }
  }
  // reset the masses in the intergrators if needed
  if(_localparameters) {
    // for the vectors
    for(unsigned int ix=0;ix<3;++ix) {
      if(ix<_vecmass.size()&&res[ix]) {
	mode->resetIntermediate(res[ix],_vecmass[ix],_vecwidth[ix]);
      }
    }
    // for the scalars
    for(unsigned int ix=3;ix<5;++ix) {
      if(ix-3<_scamass.size()&&res[ix]) {
	mode->resetIntermediate(res[ix],_scamass[ix-3],_scawidth[ix-3]);
      }
    }
  }
  return true;
}

void KPiCurrent::dataBaseOutput(ofstream & output,bool header,
				bool create) const {
  if(header) output << "update decayers set parameters=\"";
  if(create) output << "create Herwig::KPiCurrent " << name() 
		    << " HeWeakCurrents.so\n";
  output << "newdef " << name() << ":LocalParameters " << _localparameters << "\n";
  output << "newdef " << name() << ":Transverse "      << _transverse << "\n";
  output << "newdef " << name() << ":cV " << _cV << "\n";
  output << "newdef " << name() << ":cS " << _cS << "\n";
  for(unsigned int ix=0;ix<_vecmag.size();++ix) {
    if(ix<2) output << "newdef ";
    else     output << "insert ";
    output << name() << ":VectorMagnitude " << ix << " " << _vecmag[ix]   << "\n";
    if(ix<3) output << "newdef ";
    else     output << "insert ";
    output << name() << ":VectorPhase "     << ix << " " << _vecphase[ix] << "\n";
  }
  for(unsigned int ix=0;ix<_scamag.size();++ix) {
    if(ix<2) output << "newdef ";
    else     output << "insert ";
    output << name() << ":ScalarMagnitude " << ix << " " << _scamag[ix]  << "\n";
    if(ix<2) output << "newdef ";
    else     output << "insert ";
    output << name() << ":ScalarPhase "     << ix << " " << _scaphase[ix] << "\n";
  }
  for(unsigned int ix=0;ix<_vecmass.size();++ix) {
    if(ix<2) output << "newdef ";
    else     output << "insert ";
    output << name() << ":VectorMass "  << ix << " " << _vecmass[ix]/MeV  << "\n";
    if(ix<2) output << "newdef ";
    else     output << "insert ";
    output << name() << ":VectorWidth " << ix << " " << _vecwidth[ix]/MeV << "\n";
  }
  for(unsigned int ix=0;ix<_scamass.size();++ix) {
    if(ix<2) output << "newdef ";
    else     output << "insert ";
    output << name() << ":ScalarMass "  << ix << " " << _scamass[ix]/MeV  << "\n";
    if(ix<2) output << "newdef ";
    else     output << "insert ";
    output << name() << ":ScalarWidth " << ix << " " << _scawidth[ix]/MeV << "\n";
  }
  WeakDecayCurrent::dataBaseOutput(output,false,false);
  if(header) output << "\n\" where BINARY ThePEGName=\"" 
		    << fullName() << "\";" << endl;
}

vector<LorentzPolarizationVectorE> 
KPiCurrent::current(const int imode, const int ichan, Energy & scale,
		    const ParticleVector & decay,
		    DecayIntegrator::MEOption meopt) const {
  useMe();
  if(meopt==DecayIntegrator::Terminate) {
    for(unsigned int ix=0;ix<2;++ix)
      ScalarWaveFunction::constructSpinInfo(decay[ix],outgoing,true);
    return vector<LorentzPolarizationVectorE>(1,LorentzPolarizationVectorE());
  }
  // momentum difference and sum of the mesons
  Lorentz5Momentum pdiff(decay[0]->momentum()-decay[1]->momentum());
  Lorentz5Momentum psum (decay[0]->momentum()+decay[1]->momentum());
  psum.rescaleMass();
  scale=psum.mass();
  // mass2 of intermediate state
  Energy2 q2 (psum.m2());
  Energy2 dot(psum*pdiff);
  // contribution of the vector resonances
  Complex vnorm(0.),gterm(0.),sterm(0.),snorm(0.);
  complex<InvEnergy2> qterm(ZERO);
  for(unsigned int ix=0;ix<_vecwgt.size();++ix) {
    vnorm += _vecwgt[ix];
    if(ichan<0||_resmap[ix]==ichan) {
      Complex bw=_vecwgt[ix]*pWaveBreitWigner(q2,ix);
      gterm +=bw;
      qterm += _transverse ? bw/sqr(scale) : bw/sqr(_vecmass[ix]);
    }
  }
  // contribution of the scalar resonances
  for(unsigned int ix=0;ix<_scawgt.size();++ix) {
    snorm += _scawgt[ix];
    if(ichan<0||_resmap[ix+_vecwgt.size()]==ichan) {
      sterm+=_scawgt[ix]*sWaveBreitWigner(q2,ix);
    }
  }
  // compute the current
  gterm *=_cV/vnorm;
  Complex qtermnew = qterm*_cV*dot/vnorm;
  sterm *= _cS/snorm;
  LorentzPolarizationVectorE output=gterm*pdiff+(-qtermnew+sterm)*psum;
  // return the answer
  if(imode==0) output *= sqrt(0.5);
  return vector<LorentzPolarizationVectorE>(1,output);
}
#line 1 "./KaonThreeMesonCurrent.cc"
// -*- C++ -*-
//
// KaonThreeMesonCurrent.cc is a part of Herwig - A multi-purpose Monte Carlo event generator
// Copyright (C) 2002-2019 The Herwig Collaboration
//
// Herwig is licenced under version 3 of the GPL, see COPYING for details.
// Please respect the MCnet academic guidelines, see GUIDELINES for details.
//
//
// This is the implementation of the non-inlined, non-templated member
// functions of the KaonThreeMesonCurrent class.
//

#include "KaonThreeMesonCurrent.h"
#include "ThePEG/Interface/ClassDocumentation.h"
#include "ThePEG/Interface/Switch.h"
#include "ThePEG/Interface/Parameter.h"
#include "ThePEG/Interface/ParVector.h"
#include "ThePEG/Persistency/PersistentOStream.h"
#include "ThePEG/Persistency/PersistentIStream.h"
#include "Herwig/PDT/ThreeBodyAllOnCalculator.h"
#include "ThePEG/Utilities/DescribeClass.h"

using namespace Herwig;

DescribeClass<KaonThreeMesonCurrent,ThreeMesonCurrentBase>
describeHerwigKaonThreeMesonCurrent("Herwig::KaonThreeMesonCurrent",
				    "HwWeakCurrents.so");
HERWIG_INTERPOLATOR_CLASSDESC(KaonThreeMesonCurrent,Energy,Energy2)

IBPtr KaonThreeMesonCurrent::clone() const {
  return new_ptr(*this);
}

IBPtr KaonThreeMesonCurrent::fullclone() const {
  return new_ptr(*this);
}

KaonThreeMesonCurrent::KaonThreeMesonCurrent() {
  // rho parameters
  // use local values
  _rhoparameters=true;
  // rho parameters for axial-vector pieces
  _rho1wgts.push_back( 1.0  );   _rho1wgts.push_back(-0.145);
  _rho1wgts.push_back(0.);
  _rho1mass.push_back(0.773*GeV);_rho1mass.push_back(1.370*GeV);
  _rho1mass.push_back(1.750*GeV);
  _rho1width.push_back(0.145*GeV);_rho1width.push_back(0.510*GeV);
  _rho1width.push_back(0.120*GeV);
  // rho parameters for vector pieces
  _rho2wgts.push_back( 1.0  );   _rho2wgts.push_back(-0.25 );
  _rho2wgts.push_back(-0.038);
  _rho2mass.push_back(0.773*GeV);_rho2mass.push_back(1.500*GeV);
  _rho2mass.push_back(1.750*GeV);
  _rho2width.push_back(0.145*GeV);_rho2width.push_back(0.220*GeV);
  _rho2width.push_back(0.120*GeV);
  // K* parameters
  _kstarparameters=true;
  // K* parameters for the axial-vector pieces
  _kstar1wgts.push_back( 1.0  );   _kstar1wgts.push_back(-0.135);
  _kstar1wgts.push_back(0.);
  _kstar1mass.push_back(0.892*GeV);_kstar1mass.push_back(1.412*GeV);
  _kstar1mass.push_back(1.714*GeV);
  _kstar1width.push_back(0.050*GeV);_kstar1width.push_back(0.227*GeV);
  _kstar1width.push_back(0.323*GeV);
  // K* parameters for vector pieces
  _kstar2wgts.push_back( 1.0  );   _kstar2wgts.push_back(-0.25 );
  _kstar2wgts.push_back(-0.038);
  _kstar2mass.push_back(0.892*GeV);_kstar2mass.push_back(1.412*GeV);
  _kstar2mass.push_back(1.714*GeV);
  _kstar2width.push_back(0.050*GeV);_kstar2width.push_back(0.227*GeV);
  _kstar2width.push_back(0.323*GeV);
  // a_1 parameters
  _a1parameters = true;
  _initializea1 = false;
  _a1opt        = true;
  _a1mass  = 1.251*GeV;
  _a1width = 0.475*GeV;
  double a1q2in[200]={0,15788.6,31577.3,47365.9,63154.6,78943.2,94731.9,110521,
		       126309,142098,157886,173675,189464,205252,221041,236830,
		       252618,268407,284196,299984,315773,331562,347350,363139,
		       378927,394716,410505,426293,442082,457871,473659,489448,
		       505237,521025,536814,552603,568391,584180,599969,615757,
		       631546,647334,663123,678912,694700,710489,726278,742066,
		       757855,773644,789432,805221,821010,836798,852587,868375,
		       884164,899953,915741,931530,947319,963107,978896,994685,
		       1.01047e+06,1.02626e+06,1.04205e+06,1.05784e+06,1.07363e+06,
		       1.08942e+06,1.10521e+06,1.12099e+06,1.13678e+06,1.15257e+06,
		       1.16836e+06,1.18415e+06,1.19994e+06,1.21573e+06,1.23151e+06,
		       1.2473e+06,1.26309e+06,1.27888e+06,1.29467e+06,1.31046e+06,
		       1.32625e+06,1.34203e+06,1.35782e+06,1.37361e+06,1.3894e+06,
		       1.40519e+06,1.42098e+06,1.43677e+06,1.45256e+06,1.46834e+06
		       ,1.48413e+06,1.49992e+06,1.51571e+06,1.5315e+06,1.54729e+06,
		       1.56308e+06,1.57886e+06,1.59465e+06,1.61044e+06,1.62623e+06,
		       1.64202e+06,1.65781e+06,1.6736e+06,1.68939e+06,1.70517e+06,
		       1.72096e+06,1.73675e+06,1.75254e+06,1.76833e+06,1.78412e+06,
		       1.79991e+06,1.81569e+06,1.83148e+06,1.84727e+06,1.86306e+06,
		       1.87885e+06,1.89464e+06,1.91043e+06,1.92621e+06,1.942e+06,
		       1.95779e+06,1.97358e+06,1.98937e+06,2.00516e+06,2.02095e+06,
		       2.03674e+06,2.05252e+06,2.06831e+06,2.0841e+06,2.09989e+06,
		       2.11568e+06,2.13147e+06,2.14726e+06,2.16304e+06,2.17883e+06,
		       2.19462e+06,2.21041e+06,2.2262e+06,2.24199e+06,2.25778e+06,
		       2.27356e+06,2.28935e+06,2.30514e+06,2.32093e+06,2.33672e+06,
		       2.35251e+06,2.3683e+06,2.38409e+06,2.39987e+06,2.41566e+06,
		       2.43145e+06,2.44724e+06,2.46303e+06,2.47882e+06,2.49461e+06,
		       2.51039e+06,2.52618e+06,2.54197e+06,2.55776e+06,2.57355e+06,
		       2.58934e+06,2.60513e+06,2.62092e+06,2.6367e+06,2.65249e+06,
		       2.66828e+06,2.68407e+06,2.69986e+06,2.71565e+06,2.73144e+06,
		       2.74722e+06,2.76301e+06,2.7788e+06,2.79459e+06,2.81038e+06,
		       2.82617e+06,2.84196e+06,2.85774e+06,2.87353e+06,2.88932e+06,
		       2.90511e+06,2.9209e+06,2.93669e+06,2.95248e+06,2.96827e+06,
		       2.98405e+06,2.99984e+06,3.01563e+06,3.03142e+06,3.04721e+06,
		       3.063e+06,3.07879e+06,3.09457e+06,3.11036e+06,3.12615e+06,
		       3.14194e+06};
  double a1widthin[200]={0,0,0,0,0,0,0,0,0,0,0,0,0.00153933,0.0136382,0.0457614,
			 0.105567,0.199612,0.333825,0.513831,0.745192,1.0336,1.38501,
			 1.80581,2.30295,2.88403,3.5575,4.33278,5.22045,6.23243,
			 7.38223,8.68521,10.1589,11.8234,13.7018,15.8206,18.2107,
			 20.9078,23.9533,27.3954,31.2905,35.7038,40.7106,46.3984,
			 52.8654,60.2207,68.581,78.0637,88.7754,100.794,114.145,
			 128.783,144.574,161.299,178.683,196.426,214.248,231.908,
			 249.221,266.059,282.336,298.006,313.048,327.46,341.254,
			 354.448,367.066,379.133,390.677,401.726,412.304,422.439,
			   432.155,441.474,450.419,459.01,467.267,475.207,482.847,
			 490.203,497.29,504.121,510.71,517.068,523.207,529.138,
			 534.869,540.411,545.776,550.961,556.663,560.851,565.566,
			 570.137,574.569,578.869,583.041,587.091,591.023,594.843,
			 598.553,602.16,605.664,609.072,612.396,615.626,618.754,
			 621.796,624.766,627.656,630.47,633.21,635.878,638.5,
			 641.006,643.471,645.873,648.213,650.493,652.715,654.88,
			 656.99,659.047,661.052,663.007,664.963,666.771,668.6,
			 670.351,672.075,673.828,675.397,676.996,678.567,680.083,
			 681.589,683.023,684.457,685.825,687.18,688.499,689.789,
			 691.058,692.284,693.501,694.667,695.82,696.947,698.05,
			 699.129,700.186,701.221,702.234,703.226,704.198,705.158,
			 706.085,707.001,707.899,708.78,709.644,710.474,711.334,
			 712.145,712.943,713.727,714.505,715.266,716.015,716.751,
			 717.474,718.183,718.88,719.645,720.243,720.91,721.565,
			 722.211,722.851,723.473,724.094,724.697,725.296,725.886,
			 726.468,727.041,727.608,728.166,728.718,729.262,729.808,
			 730.337,730.856,731.374,731.883,732.386,732.884,733.373,
			 733.859,734.339,734.813};
  
  vector<double> tmp1(a1widthin,a1widthin+200);
  _a1runwidth.clear();
  std::transform(tmp1.begin(), tmp1.end(),
		 back_inserter(_a1runwidth),
		 [](double x){return x*GeV;});
  
  vector<double> tmp2(a1q2in,a1q2in+200);
  _a1runq2.clear();
  std::transform(tmp2.begin(), tmp2.end(),
		 back_inserter(_a1runq2),
		 [](double x){return x*GeV2;});


  // K_1 parameters
  _k1parameters=true;
  _k1mass.push_back(1.270*GeV);_k1width.push_back(0.090*GeV);
  _k1mass.push_back(1.402*GeV);_k1width.push_back(0.174*GeV);
  _k1wgta.push_back(0.33);_k1wgta.push_back(1.);
  _k1wgtb.push_back(1.00);_k1wgtb.push_back(0.);
  // parameters for the T_omega function
  _omegaopt=true;
  _epsomega=0.05;
  _omegamass  = 0.782*GeV;
  _omegawidth = 0.00843*GeV;
  _phimass    = 1.020*GeV;
  _phiwidth   = 0.00443*GeV;
  _omegaKstarwgt=1./sqrt(2.);
  // the pion decay constant
  _fpi=130.7*MeV/sqrt(2.);
  _mpi=ZERO;_mK=ZERO;
  _maxmass=ZERO;
  _maxcalc=ZERO;
}


void KaonThreeMesonCurrent::persistentOutput(PersistentOStream & os) const {
  os << _a1runinter
     << _rho1wgts << ounit(_rho1mass,GeV) << ounit(_rho1width,GeV) 
     << _rho2wgts << ounit(_rho2mass,GeV) << ounit(_rho2width,GeV)
     << _kstar1wgts << ounit(_kstar1mass,GeV) << ounit(_kstar1width,GeV) 
     << _kstar2wgts << ounit(_kstar2mass,GeV) << ounit(_kstar2width,GeV) 
     << ounit(_a1mass,GeV) << ounit(_a1width,GeV) << ounit(_k1mass,GeV) 
     << ounit(_k1width,GeV) << _k1wgta << _k1wgtb 
     << ounit(_a1runwidth,GeV) << ounit(_a1runq2,GeV2) << _epsomega 
     << ounit(_omegamass,GeV) << ounit(_omegawidth,GeV) 
     << ounit(_phimass,GeV) << ounit(_phiwidth,GeV) << _omegaKstarwgt 
     << ounit(_fpi,GeV) << ounit(_mpi,GeV) << ounit(_mK,GeV) 
     << _initializea1 << _rhoparameters << _kstarparameters << _a1parameters 
     << _k1parameters << _a1opt << _omegaopt 
     << ounit(_maxmass,GeV) << ounit(_maxcalc,GeV);
}

void KaonThreeMesonCurrent::persistentInput(PersistentIStream & is, int) {
  is >> _a1runinter
     >> _rho1wgts >> iunit(_rho1mass,GeV) >> iunit(_rho1width,GeV) 
     >> _rho2wgts >> iunit(_rho2mass,GeV) >> iunit(_rho2width,GeV) 
     >> _kstar1wgts >> iunit(_kstar1mass,GeV) >> iunit(_kstar1width,GeV) 
     >> _kstar2wgts >> iunit(_kstar2mass,GeV) >> iunit(_kstar2width,GeV) 
     >> iunit(_a1mass,GeV) >> iunit(_a1width,GeV) >> iunit(_k1mass,GeV) 
     >> iunit(_k1width,GeV) >> _k1wgta >> _k1wgtb 
     >> iunit(_a1runwidth,GeV) >> iunit(_a1runq2,GeV2) >> _epsomega 
     >> iunit(_omegamass,GeV) >> iunit(_omegawidth,GeV) 
     >> iunit(_phimass,GeV) >> iunit(_phiwidth,GeV) >> _omegaKstarwgt 
     >> iunit(_fpi,GeV) >> iunit(_mpi,GeV) >> iunit(_mK,GeV) 
     >> _initializea1 >> _rhoparameters >> _kstarparameters >> _a1parameters 
     >> _k1parameters >> _a1opt >> _omegaopt 
     >> iunit(_maxmass,GeV) >> iunit(_maxcalc,GeV);
}


void KaonThreeMesonCurrent::Init() {

  static ClassDocumentation<KaonThreeMesonCurrent> documentation
    ("The KaonThreeMesonCurrent class implements the model of "
     "Z. Phys.  C 69 (1996) 243 [arXiv:hep-ph/9503474]"
     " for the weak current with three "
     "mesons, at least one of which is a kaon",
     "The KaonThreeMesonCurrent class implements the model of "
     "\\cite{Finkemeier:1995sr} for the weak current with three "
     "mesons, at least one of which is a kaon.",
     "\\bibitem{Finkemeier:1995sr}\n"
     "M.~Finkemeier and E.~Mirkes,\n"
     "Z.\\ Phys.\\  C {\\bf 69} (1996) 243 [arXiv:hep-ph/9503474].\n"
     " %%CITATION = ZEPYA,C69,243;%%\n"

);

  static Switch<KaonThreeMesonCurrent,bool> interfaceInitializea1
    ("Initializea1",
     "Initialise the calculation of the a_1 running width",
     &KaonThreeMesonCurrent::_initializea1, false, false, false);
  static SwitchOption interfaceInitializea1Initialization
    (interfaceInitializea1,
     "Yes",
     "Initialize the calculation",
     true);
  static SwitchOption interfaceInitializea1NoInitialization
    (interfaceInitializea1,
     "No",
     "Use the default values",
     false);

  static Parameter<KaonThreeMesonCurrent,Energy> interfaceA1Width
    ("A1Width",
     "The a_1 width if using local values.",
     &KaonThreeMesonCurrent::_a1width, GeV, 0.599*GeV, ZERO, 10.0*GeV,
     false, false, false);
  
  static Parameter<KaonThreeMesonCurrent,Energy> interfaceA1Mass
    ("A1Mass",
     "The a_1 mass if using local values.",
     &KaonThreeMesonCurrent::_a1mass, GeV, 1.251*GeV, ZERO, 10.0*GeV,
     false, false, false);

  static Parameter<KaonThreeMesonCurrent,Energy> interfaceFPi
    ("FPi",
     "The pion decay constant",
     &KaonThreeMesonCurrent::_fpi, MeV, 92.4*MeV, ZERO, 200.0*MeV,
     false, false, true);

  static ParVector<KaonThreeMesonCurrent,Energy> interfaceRhoAxialMasses
    ("RhoAxialMasses",
     "The masses for the rho resonances if used local values",
     &KaonThreeMesonCurrent::_rho1mass, GeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, true);
  
  static ParVector<KaonThreeMesonCurrent,Energy> interfaceRhoAxialWidths
    ("RhoAxialWidths",
     "The widths for the rho resonances if used local values",
     &KaonThreeMesonCurrent::_rho1width, GeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, true);
  
  static ParVector<KaonThreeMesonCurrent,Energy> interfaceRhoVectorMasses
    ("RhoVectorMasses",
     "The masses for the rho resonances if used local values",
     &KaonThreeMesonCurrent::_rho2mass, GeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, true);
  
  static ParVector<KaonThreeMesonCurrent,Energy> interfaceRhoVectorWidths
    ("RhoVectorWidths",
     "The widths for the rho resonances if used local values",
     &KaonThreeMesonCurrent::_rho2width, GeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, true);
  
  static ParVector<KaonThreeMesonCurrent,Energy> interfaceKstarAxialMasses
    ("KstarAxialMasses",
     "The masses for the Kstar resonances if used local values",
     &KaonThreeMesonCurrent::_kstar1mass, GeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, true);
  
  static ParVector<KaonThreeMesonCurrent,Energy> interfaceKstarAxialWidths
    ("KstarAxialWidths",
     "The widths for the Kstar resonances if used local values",
     &KaonThreeMesonCurrent::_kstar1width, GeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static ParVector<KaonThreeMesonCurrent,Energy> interfaceKstarVectorMasses
    ("KstarVectorMasses",
     "The masses for the Kstar resonances if used local values",
     &KaonThreeMesonCurrent::_kstar2mass, GeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, true);
  
  static ParVector<KaonThreeMesonCurrent,Energy> interfaceKstarVectorWidths
    ("KstarVectorWidths",
     "The widths for the Kstar resonances if used local values",
     &KaonThreeMesonCurrent::_kstar2width, GeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static ParVector<KaonThreeMesonCurrent,double> interfaceAxialRhoWeight
    ("AxialRhoWeight",
     "The weights of the different rho resonances in the F1,2,3 form factor",
     &KaonThreeMesonCurrent::_rho1wgts,
     0, 0, 0, -1000, 1000, false, false, true);
  
  static ParVector<KaonThreeMesonCurrent,double> interfaceAxialKStarWeight
    ("AxialKStarWeight",
     "The weights of the different Kstar resonances in the F1,2,3 form factor",
     &KaonThreeMesonCurrent::_kstar1wgts,
     0, 0, 0, -1000, 1000, false, false, true);

  static ParVector<KaonThreeMesonCurrent,double> interfaceVectorRhoWeight
    ("VectorRhoWeight",
     "The weights of the different rho resonances in the F1,2,3 form factor",
     &KaonThreeMesonCurrent::_rho2wgts,
     0, 0, 0, -1000, 1000, false, false, true);
  
  static ParVector<KaonThreeMesonCurrent,double> interfaceVectorKStarWeight
    ("VectorKStarWeight",
     "The weights of the different Kstar resonances in the F1,2,3 form factor",
     &KaonThreeMesonCurrent::_kstar2wgts,
     0, 0, 0, -1000, 1000, false, false, true);
  
  static Switch<KaonThreeMesonCurrent,bool> interfaceRhoParameters
    ("RhoParameters",
     "Use local values of the rho meson masses and widths",
     &KaonThreeMesonCurrent::_rhoparameters, true, false, false);
  static SwitchOption interfaceRhoParameterstrue
    (interfaceRhoParameters,
     "Local",
     "Use local values of the parameters",
     true);
  static SwitchOption interfaceRhoParametersParticleData
    (interfaceRhoParameters,
     "ParticleData",
     "Use the masses and widths from the particle data objects",
     false);
  
  static Switch<KaonThreeMesonCurrent,bool> interfaceKstarParameters
    ("KstarParameters",
     "Use local values of the rho meson masses and widths",
     &KaonThreeMesonCurrent::_kstarparameters, true, false, false);
  static SwitchOption interfaceKstarParameterstrue
    (interfaceKstarParameters,
     "Local",
     "Use local values of the parameters",
     true);
  static SwitchOption interfaceKstarParametersParticleData
    (interfaceKstarParameters,
     "ParticleData",
     "Use the masses and widths from the particle data objects",
     false);
  
  static Switch<KaonThreeMesonCurrent,bool> interfacea1Parameters
    ("a1Parameters",
     "Use local values of the rho meson masses and widths",
     &KaonThreeMesonCurrent::_a1parameters, true, false, false);
  static SwitchOption interfacea1Parameterstrue
    (interfacea1Parameters,
     "Local",
     "Use local values of the parameters",
     true);
  static SwitchOption interfacea1ParametersParticleData
    (interfacea1Parameters,
     "ParticleData",
     "Use the masses and widths from the particle data objects",
     false);
  
  static Switch<KaonThreeMesonCurrent,bool> interfaceK1Parameters
    ("K1Parameters",
     "Use local values of the rho meson masses and widths",
     &KaonThreeMesonCurrent::_k1parameters, true, false, false);
  static SwitchOption interfaceK1Parameterstrue
    (interfaceK1Parameters,
     "Local",
     "Use local values of the parameters",
     true);
  static SwitchOption interfaceK1ParametersParticleData
    (interfaceK1Parameters,
     "ParticleData",
     "Use the masses and widths from the particle data objects",
     false);
  
  static Switch<KaonThreeMesonCurrent,bool> interfacea1WidthOption
    ("a1WidthOption",
     "Option for the treatment of the a1 width",
     &KaonThreeMesonCurrent::_a1opt, true, false, false);
  static SwitchOption interfacea1WidthOptionLocal
    (interfacea1WidthOption,
     "Local",
     "Use a calculation of the running width based on the parameters as"
     " interpolation table.",
     true);
  static SwitchOption interfacea1WidthOptionParam
    (interfacea1WidthOption,
     "Kuhn",
     "Use the parameterization of Kuhn and Santamaria for default parameters."
     " This should only be used for testing vs TAUOLA",
     false);

  static ParVector<KaonThreeMesonCurrent,Energy> interfacea1RunningWidth
    ("a1RunningWidth",
     "The values of the a_1 width for interpolation to giving the running width.",
     &KaonThreeMesonCurrent::_a1runwidth, GeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, true);

  static ParVector<KaonThreeMesonCurrent,Energy2> interfacea1RunningQ2
    ("a1RunningQ2",
     "The values of the q^2 for interpolation to giving the running width.",
     &KaonThreeMesonCurrent::_a1runq2, GeV2, -1, 1.0*GeV2, ZERO, 10.0*GeV2,
     false, false, true);

  static ParVector<KaonThreeMesonCurrent,Energy> interfaceK1Masses
    ("K1Masses",
     "Masses of the K_1 mesons",
     &KaonThreeMesonCurrent::_k1mass, GeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static ParVector<KaonThreeMesonCurrent,Energy> interfaceK1Widths
    ("K1Widths",
     "Widths of the K_1 mesons",
     &KaonThreeMesonCurrent::_k1width, GeV, -1, 1.0*GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static ParVector<KaonThreeMesonCurrent,double> interfaceK1WeightKStarPi
    ("K1WeightKStarPi",
     "The relative weights for the K_1 resonances in the K* pi final-state",
     &KaonThreeMesonCurrent::_k1wgta, -1, 1.0, 0, 10.0,
     false, false, Interface::limited);

  static ParVector<KaonThreeMesonCurrent,double> interfaceK1WeightRhoK
    ("K1WeightRhoK",
     "The relative weights for the K_1 resonances in the rho K final-state",
     &KaonThreeMesonCurrent::_k1wgtb, -1, 1.0, 0, 10.0,
     false, false, Interface::limited);

  static Parameter<KaonThreeMesonCurrent,double> interfaceEpsOmega
    ("EpsOmega",
     "The omega-phi mixing ",
     &KaonThreeMesonCurrent::_epsomega, 0.05, 0.0, 1.0,
     false, false, Interface::limited);

  static Parameter<KaonThreeMesonCurrent,Energy> interfaceOmegaMass
    ("OmegaMass",
     "The mass of the omega meson",
     &KaonThreeMesonCurrent::_omegamass, GeV, 0.782*GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<KaonThreeMesonCurrent,Energy> interfaceOmegaWidth
    ("OmegaWidth",
     "The width of the omega meson",
     &KaonThreeMesonCurrent::_omegawidth, GeV, 0.00843*GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<KaonThreeMesonCurrent,Energy> interfacePhiMass
    ("PhiMass",
     "The mass of the phi meson",
     &KaonThreeMesonCurrent::_phimass, GeV, 1.020*GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<KaonThreeMesonCurrent,Energy> interfacePhiWidth
    ("PhiWidth",
     "The width of the phi meson",
     &KaonThreeMesonCurrent::_phiwidth, GeV, 0.00443*GeV, ZERO, 10.0*GeV,
     false, false, Interface::limited);

  static Parameter<KaonThreeMesonCurrent,double> interfaceOmegaKStarWeight
    ("OmegaKStarWeight",
     "The relative weight of the omega-phi and K* terms",
     &KaonThreeMesonCurrent::_omegaKstarwgt, 1./sqrt(2.), 0.0, 100.0,
     false, false, Interface::limited);
  
  static Switch<KaonThreeMesonCurrent,bool> interfaceOmegaParameters
    ("OmegaParameters",
     "Use local values of the omega/phi meson masses and widths",
     &KaonThreeMesonCurrent::_omegaopt, true, false, false);
  static SwitchOption interfaceOmegaParameterstrue
    (interfaceOmegaParameters,
     "Local",
     "Use local values of the parameters",
     true);
  static SwitchOption interfaceOmegaParametersParticleData
    (interfaceOmegaParameters,
     "ParticleData",
     "Use the masses and widths from the particle data objects",
     false);

}

void KaonThreeMesonCurrent::inita1Width(int iopt) {
  if(iopt==-1) {
    _maxcalc=_maxmass;
    if(!_initializea1||_maxmass==ZERO) return; 
    // parameters for the table of values
    Energy2 step(sqr(_maxmass)/199.);
    // integrator to perform the integral
    vector<double> inweights;inweights.push_back(0.5);inweights.push_back(0.5);
    vector<int> intype;intype.push_back(2);intype.push_back(3);
    Energy mrho(getParticleData(ParticleID::rhoplus)->mass()),
      wrho(getParticleData(ParticleID::rhoplus)->width());
    vector<Energy> inmass(2,mrho),inwidth(2,wrho);
    vector<double> inpow(2,0.0);
    ThreeBodyAllOnCalculator<KaonThreeMesonCurrent> 
      widthgen(inweights,intype,inmass,inwidth,inpow,*this,0,_mpi,_mpi,_mpi);
    // normalisation constant to give physical width if on shell
    double a1const(_a1width/(widthgen.partialWidth(sqr(_a1mass))));
    // loop to give the values
    _a1runq2.clear();_a1runwidth.clear();
    for(Energy2 moff2 = ZERO; moff2<=sqr(_maxmass); moff2+=step) {
      _a1runwidth.push_back(widthgen.partialWidth(moff2)*a1const);
      _a1runq2.push_back(moff2);
    }
  }
  // set up the interpolator
  else if(iopt==0) {
    _a1runinter = make_InterpolatorPtr(_a1runwidth,_a1runq2,3);
  }
}
  
// modes handled by this class
bool KaonThreeMesonCurrent::acceptMode(int imode) const { 
  return imode>=2&&imode<=11&&imode!=8;
}



// complete the construction of the decay mode for integration
bool KaonThreeMesonCurrent::createMode(int icharge, unsigned int imode,
					  DecayPhaseSpaceModePtr mode,
					  unsigned int iloc,unsigned int ires,
					  DecayPhaseSpaceChannelPtr phase,Energy upp) {
  int iq(0),ia(0);
  if(!acceptMode(imode)) return false;
  tPDVector extpart(particles(1,imode,iq,ia));
  Energy min(ZERO);
  for(unsigned int ix=0;ix<extpart.size();++ix) min+=extpart[ix]->massMin();
  if(min>upp) return false;
  // the particles we will use a lot
  tPDPtr a1,k1[2];
  if(icharge==-3) {
    a1    = getParticleData(ParticleID::a_1minus);
    k1[0] = getParticleData(ParticleID::K_1minus);
    k1[1] = getParticleData(ParticleID::Kstar_1minus);
  }
  else if(icharge==3) {
    a1    = getParticleData(ParticleID::a_1plus);
    k1[0] = getParticleData(ParticleID::K_1plus);
    k1[1] = getParticleData(ParticleID::Kstar_1plus);
  }
  else {
    return false;
  }
  _maxmass=max(_maxmass,upp);
  // the rho0 resonances
  tPDPtr rho0[3]  ={getParticleData( 113),getParticleData( 100113),
		    getParticleData( 30113)};
  // the charged rho resonances
  tPDPtr rhoc[3]  ={getParticleData(-213),getParticleData(-100213),
		    getParticleData(-30213)};
  // the K*0 resonances
  tPDPtr Kstar0[3]={getParticleData( 313),getParticleData( 100313),
		    getParticleData( 30313)};
  // the charged K* resonances
  tPDPtr Kstarc[3]={getParticleData(-323),getParticleData(-100323),
		    getParticleData(-30323)};
  if(icharge==3) {
    for(unsigned int ix=0;ix<3;++ix) {
      if(rhoc[ix]) rhoc[ix]=rhoc[ix]->CC();
      if(Kstar0[ix]) Kstar0[ix]=Kstar0[ix]->CC();
      if(Kstarc[ix]) Kstarc[ix]=Kstarc[ix]->CC();
    }
  }
  DecayPhaseSpaceChannelPtr newchannel;
  if(imode==2) {
    // channels for K- pi- K+
    for(unsigned int ix=0;ix<3;++ix) {
      if(Kstar0[ix]) {
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(a1        ,0,0.0,-ires-1,iloc);
	newchannel->addIntermediate(Kstar0[ix],0,0.0, iloc+1,iloc+2);
	mode->addChannel(newchannel);
      }
      if(rho0[ix]) {
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(a1      ,0,0.0,-ires-1,iloc+1);
	newchannel->addIntermediate(rho0[ix],0,0.0,iloc,iloc+2);
	mode->addChannel(newchannel);
      }
      for(unsigned int iy=0;iy<3;++iy) {
	if(!rhoc[ix]) continue;
	if(Kstar0[iy]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(rhoc[ix]  ,0,0.0,-ires-1,iloc);
	  newchannel->addIntermediate(Kstar0[iy],0,0.0, iloc+1,iloc+2);
	  mode->addChannel(newchannel);
	}
      }
    }
  }
  else if(imode==3) {
    // channels for K0 pi- K0bar
    for(unsigned int ix=0;ix<3;++ix) {
      if(Kstarc[ix]) {
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(a1        ,0,0.0,-ires-1,iloc);
	newchannel->addIntermediate(Kstarc[ix],0,0.0, iloc+1,iloc+2);
	mode->addChannel(newchannel);
      }
      if(rho0[ix]) {
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(a1      ,0,0.0,-ires-1,iloc+1);
	newchannel->addIntermediate(rho0[ix],0,0.0, iloc,iloc+2);
	mode->addChannel(newchannel);
      }
      for(unsigned int iy=0;iy<3;++iy) {
	if(!rhoc[ix]) continue;
	if(Kstarc[iy]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(rhoc[ix]  ,0,0.0,-ires-1,iloc);
	  newchannel->addIntermediate(Kstarc[iy],0,0.0, iloc+1,iloc+2);
	  mode->addChannel(newchannel);
	}
      }
    }
  }
  else if(imode==4) {
    // channels for K- pi0 K0
    for(unsigned int ix=0;ix<3;++ix) {
      if(Kstar0[ix]) {
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(a1        ,0,0.0,-ires-1,iloc);
	newchannel->addIntermediate(Kstar0[ix],0,0.0, iloc+1,iloc+2);
	mode->addChannel(newchannel);
      }
      if(Kstarc[ix]) {
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(a1        ,0,0.0,-ires-1,iloc+2);
	newchannel->addIntermediate(Kstarc[ix],0,0.0, iloc  ,iloc+1);
	mode->addChannel(newchannel);
      }
      if(rhoc[ix]) {
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(a1      ,0,0.0,-ires-1,iloc+1);
	newchannel->addIntermediate(rhoc[ix],0,0.0, iloc,iloc+2);
	mode->addChannel(newchannel);
      }
      for(unsigned int iy=0;iy<3;++iy) {
	if(!rhoc[ix]) continue;
	if(Kstar0[iy]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(rhoc[ix]  ,0,0.0,-ires-1,iloc);
	  newchannel->addIntermediate(Kstar0[iy],0,0.0, iloc+1,iloc+2);
	  mode->addChannel(newchannel);
	}
	if(Kstarc[iy]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(rhoc[ix]  ,0,0.0,-ires-1,iloc+2);
	  newchannel->addIntermediate(Kstarc[iy],0,0.0, iloc  ,iloc+1);
	  mode->addChannel(newchannel);
	}
      }
    }
  }
  else if(imode==5) {  
    // channels for pi0 pi0 K-
    for(unsigned int ix=0;ix<3;++ix) {
      if(!Kstarc[ix]) continue;
      for(unsigned int ik=0;ik<2;++ik) {
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(k1[ik]    ,0,0.0,-ires-1,iloc);
	newchannel->addIntermediate(Kstarc[ix],0,0.0, iloc+1,iloc+2);
	mode->addChannel(newchannel);
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(k1[ik]    ,0,0.0,-ires-1,iloc+1);
	newchannel->addIntermediate(Kstarc[ix],0,0.0, iloc,iloc+2);
	mode->addChannel(newchannel);
      }
      for(unsigned int iy=0;iy<3;++iy) {
	if(!Kstarc[iy]) continue;
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(Kstarc[ix],0,0.0,-ires-1,iloc  );
	newchannel->addIntermediate(Kstarc[iy],0,0.0, iloc+1,iloc+2);
	mode->addChannel(newchannel);
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(Kstarc[ix],0,0.0,-ires-1,iloc+1);
	newchannel->addIntermediate(Kstarc[iy],0,0.0, iloc  ,iloc+2);
	mode->addChannel(newchannel);
      }
    }
  }
  else if(imode==6) {
    // channels for K- pi- pi+
    for(unsigned int ix=0;ix<3;++ix) {
      for(unsigned int ik=0;ik<2;++ik) {
	if(rho0[ix]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(k1[ik]  ,0,0.0,-ires-1,iloc);
	  newchannel->addIntermediate(rho0[ix],0,0.0, iloc+1,iloc+2);
	  mode->addChannel(newchannel);
	}
	if(Kstar0[ix]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(k1[ik]    ,0,0.0,-ires-1,iloc+1);
	  newchannel->addIntermediate(Kstar0[ix],0,0.0, iloc,iloc+2);
	  mode->addChannel(newchannel);
	}
      }
      for(unsigned int iy=0;iy<3;++iy) {
	if(!Kstarc[ix]) continue;
	if(rho0[iy]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(Kstarc[ix],0,0.0,-ires-1,iloc);
	  newchannel->addIntermediate(rho0[iy]  ,0,0.0, iloc+1,iloc+2);
	  mode->addChannel(newchannel);
	}
	if(Kstar0[iy]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(Kstarc[ix],0,0.0,-ires-1,iloc+1);
	  newchannel->addIntermediate(Kstar0[iy],0,0.0, iloc,iloc+2);
	  mode->addChannel(newchannel);
	}
      }
    }
  }
  else if(imode==7) {
    // channels for pi- kbar0 pi0
    for(unsigned int ix=0;ix<3;++ix) {
      for(unsigned int ik=0;ik<2;++ik) {
	if(rhoc[ix]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(k1[ik]  ,0,0.0,-ires-1,iloc+1);
	  newchannel->addIntermediate(rhoc[ix],0,0.0, iloc,iloc+2);
	  mode->addChannel(newchannel);
	}
	if(Kstar0[ix]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(k1[ik]  ,0,0.0,-ires-1,iloc);
	  newchannel->addIntermediate(Kstar0[ix],0,0.0,iloc+1,iloc+2);
	  mode->addChannel(newchannel);
	}
	if(Kstarc[ix]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(k1[ik]  ,0,0.0,-ires-1,iloc+2);
	  newchannel->addIntermediate(Kstarc[ix],0,0.0, iloc,iloc+1);
	  mode->addChannel(newchannel);
	}
      }
      for(unsigned int iy=0;iy<3;++iy) {
	if(!Kstarc[ix]) continue;
	if(Kstar0[iy]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(Kstarc[ix],0,0.0,-ires-1,iloc);
	  newchannel->addIntermediate(Kstar0[iy],0,0.0, iloc+1,iloc+2);
	  mode->addChannel(newchannel);
	}
	if(rhoc[iy]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(Kstarc[ix],0,0.0,-ires-1,iloc+1);
	  newchannel->addIntermediate(rhoc[iy]  ,0,0.0, iloc,iloc+2);
	  mode->addChannel(newchannel);
	}
	if(Kstarc[iy]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(Kstarc[ix],0,0.0,-ires-1,iloc+2);
	  newchannel->addIntermediate(Kstarc[iy],0,0.0, iloc  ,iloc+1);
	  mode->addChannel(newchannel);
	}
      }
    }
  }
  else if(imode==9||imode==10) {
    // channels for K_S0 pi- K_S0 and K_L0 pi- K_L0 
    for(unsigned int ix=0;ix<3;++ix) {
      if(Kstarc[ix]) {
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(a1        ,0,0.0,-ires-1,iloc);
	newchannel->addIntermediate(Kstarc[ix],0,0.0, iloc+1,iloc+2);
	mode->addChannel(newchannel);
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(a1        ,0,0.0,-ires-1,iloc+2);
	newchannel->addIntermediate(Kstarc[ix],0,0.0, iloc  ,iloc+1);
	mode->addChannel(newchannel);
      }
      for(unsigned int iy=0;iy<3;++iy) {
	if(!rhoc[ix]) continue;
	if(Kstarc[iy]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(rhoc[ix]  ,0,0.0,-ires-1,iloc);
	  newchannel->addIntermediate(Kstarc[iy],0,0.0, iloc+1,iloc+2);
	  mode->addChannel(newchannel);
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(rhoc[ix]  ,0,0.0,-ires-1,iloc+2);
	  newchannel->addIntermediate(Kstarc[iy],0,0.0, iloc  ,iloc+1);
	  mode->addChannel(newchannel);
	}
      }
    }
  }
  else if(imode==11) {
    // channels for K_S0 pi- K_L0
    for(unsigned int ix=0;ix<3;++ix) {
      if(Kstarc[ix]) {
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(a1        ,0,0.0,-ires-1,iloc);
	newchannel->addIntermediate(Kstarc[ix],0,0.0, iloc+1,iloc+2);
	mode->addChannel(newchannel);
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(a1        ,0,0.0,-ires-1,iloc+2);
	newchannel->addIntermediate(Kstarc[ix],0,0.0, iloc  ,iloc+1);
	mode->addChannel(newchannel);
      }
      if(rho0[ix]) {
	newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	newchannel->addIntermediate(a1      ,0,0.0,-ires-1,iloc+1);
	newchannel->addIntermediate(rho0[ix],0,0.0, iloc,iloc+2);
	mode->addChannel(newchannel);
      }
      for(unsigned int iy=0;iy<3;++iy) {
	if(!rhoc[ix]) continue;
	if(Kstarc[iy]) {
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(rhoc[ix]  ,0,0.0,-ires-1,iloc);
	  newchannel->addIntermediate(Kstarc[iy],0,0.0, iloc+1,iloc+2);
	  mode->addChannel(newchannel);
	  newchannel= new_ptr(DecayPhaseSpaceChannel(*phase));
	  newchannel->addIntermediate(rhoc[ix]  ,0,0.0,-ires-1,iloc+2);
	  newchannel->addIntermediate(Kstarc[iy],0,0.0, iloc  ,iloc+1);
	  mode->addChannel(newchannel);
	}
      }
    }
  }
  if(_rhoparameters) {
    for(unsigned int ix=0;ix<_rho1mass.size();++ix) {
      if(rhoc[ix]) mode->resetIntermediate(rhoc[ix],_rho1mass[ix],
					   _rho1width[ix]);
      if(rho0[ix]) mode->resetIntermediate(rho0[ix],_rho1mass[ix],
					   _rho1width[ix]);
    }
  }
  // K star parameters in the base class
  if(_kstarparameters) {
    for(unsigned int ix=0;ix<_kstar1mass.size();++ix) {
      if(Kstarc[ix]) mode->resetIntermediate(Kstarc[ix],_kstar1mass[ix],
					     _kstar1width[ix]);
      if(Kstar0[ix]) mode->resetIntermediate(Kstar0[ix],_kstar1mass[ix],
					     _kstar1width[ix]);
    }
  }
  return true;
}

void KaonThreeMesonCurrent::dataBaseOutput(ofstream & os,
					   bool header,bool create) const {
  if(header) os << "update decayers set parameters=\"";
  if(create) os << "create Herwig::KaonThreeMesonCurrent " 
		<< name() << " HwWeakCurrents.so\n";
  for(unsigned int ix=0;ix<_rho1wgts.size();++ix) {
    if(ix<3) {
      os << "newdef " << name() << ":AxialRhoWeight " << ix 
	 << " " << _rho1wgts[ix] << "\n";
    }
    else {
      os << "insert " << name() << ":AxialRhoWeight " << ix 
	 << " " << _rho1wgts[ix] << "\n";
    }
  }
  for(unsigned int ix=0;ix<_kstar1wgts.size();++ix) {
    if(ix<3) {
      os << "newdef " << name() << ":AxialKStarWeight " << ix 
	 << " " << _kstar1wgts[ix] << "\n";}
    else {
      os << "insert " << name() << ":AxialKStarWeight " << ix 
	 << " " << _kstar1wgts[ix] << "\n";
    }
  }
  for(unsigned int ix=0;ix<_rho2wgts.size();++ix) {
    if(ix<3) {
      os << "newdef " << name() << ":VectorRhoWeight " << ix 
	 << " " << _rho2wgts[ix] << "\n";
    }
    else {
      os << "insert " << name() << ":VectorRhoWeight " << ix 
	 << " " << _rho2wgts[ix] << "\n";
    }
  }
  for(unsigned int ix=0;ix<_kstar2wgts.size();++ix) {
    if(ix<3) {
      os << "newdef " << name() << ":VectorKStarWeight " << ix 
	 << " " << _kstar2wgts[ix] << "\n";}
    else {
      os << "insert " << name() << ":VectorKStarWeight " << ix 
	 << " " << _kstar2wgts[ix] << "\n";
    }
  }
  os << "newdef " << name() << ":OmegaKStarWeight " << _omegaKstarwgt << "\n";
  os << "newdef " << name() << ":EpsOmega " << _epsomega << "\n";
  os << "newdef " << name() << ":Initializea1 " << _initializea1 << "\n";
  os << "newdef " << name() << ":RhoParameters " << _rhoparameters << "\n";
  os << "newdef " << name() << ":KstarParameters " << _kstarparameters << "\n";
  os << "newdef " << name() << ":a1Parameters " << _a1parameters << "\n";
  os << "newdef " << name() << ":K1Parameters " << _k1parameters << "\n";
  os << "newdef " << name() << ":OmegaParameters " << _omegaopt << "\n";
  os << "newdef " << name() << ":a1WidthOption " << _a1opt << "\n";
  for(unsigned int ix=0;ix<_a1runwidth.size();++ix) {
    os << "newdef " << name() << ":a1RunningWidth " << ix 
	   << " " << _a1runwidth[ix]/GeV << "\n";
  }
  for(unsigned int ix=0;ix<_a1runq2.size();++ix) {
    os << "newdef " << name() << ":a1RunningQ2 " << ix 
	   << " " << _a1runq2[ix]/GeV2 << "\n";
  }
  os << "newdef " << name() << ":A1Width " << _a1width/GeV << "\n";
  os << "newdef " << name() << ":A1Mass " << _a1mass/GeV << "\n";
  os << "newdef " << name() << ":OmegaWidth " << _omegawidth/GeV << "\n";
  os << "newdef " << name() << ":OmegaMass " << _omegamass/GeV << "\n";
  os << "newdef " << name() << ":PhiWidth " << _phiwidth/GeV << "\n";
  os << "newdef " << name() << ":PhiMass " << _phimass/GeV << "\n";
  os << "newdef " << name() << ":FPi " << _fpi/MeV << "\n";
  for(unsigned int ix=0;ix<_k1mass.size();++ix) {
    if(ix<2) {
      os << "newdef " << name() << ":K1Masses " << ix 
	 << " " << _k1mass[ix]/GeV << "\n";
    }
    else {
      os << "insert " << name() << ":K1Masses " << ix 
	 << " " << _k1mass[ix]/GeV << "\n";
    }
  }
  for(unsigned int ix=0;ix<_k1width.size();++ix) {
    if(ix<2) {
      os << "newdef " << name() << ":K1Widths " << ix 
	 << " " << _k1width[ix]/GeV << "\n";
    }
    else {
      os << "insert " << name() << ":K1Widths " << ix 
	 << " " << _k1width[ix]/GeV << "\n";
    }
  }
  for(unsigned int ix=0;ix<_rho1mass.size();++ix) {
    if(ix<3) os << "newdef " << name() << ":RhoAxialMasses " << ix 
		<< " " << _rho1mass[ix]/GeV << "\n";
    else     os << "insert " << name() << ": RhoAxialMasses" << ix 
		<< " " << _rho1mass[ix]/GeV << "\n";
  }
  for(unsigned int ix=0;ix<_rho1width.size();++ix) {
    if(ix<3) os << "newdef " << name() << ":RhoAxialWidths " << ix 
		    << " " << _rho1width[ix]/GeV << "\n";
    else     os << "insert " << name() << ":RhoAxialWidths " << ix 
		    << " " << _rho1width[ix]/GeV << "\n";
  }
  for(unsigned int ix=0;ix<_rho2mass.size();++ix) {
    if(ix<3) os << "newdef " << name() << ":RhoVectorMasses " << ix 
		<< " " << _rho2mass[ix]/GeV << "\n";
    else     os << "insert " << name() << ": RhoVectorMasses" << ix 
		<< " " << _rho2mass[ix]/GeV << "\n";
  }
  for(unsigned int ix=0;ix<_rho2width.size();++ix) {
    if(ix<3) os << "newdef " << name() << ":RhoVectorWidths " << ix 
		    << " " << _rho2width[ix]/GeV << "\n";
    else     os << "insert " << name() << ":RhoVectorWidths " << ix 
		    << " " << _rho2width[ix]/GeV << "\n";
  }
  for(unsigned int ix=0;ix<_kstar1mass.size();++ix) {
    if(ix<3) os << "newdef " << name() << ":KstarAxialMasses " << ix 
		<< " " << _kstar1mass[ix]/GeV << "\n";
    else     os << "insert " << name() << ": KstarAxialMasses" << ix 
		<< " " << _kstar1mass[ix]/GeV << "\n";
  }
  for(unsigned int ix=0;ix<_kstar1width.size();++ix) {
    if(ix<3) os << "newdef " << name() << ":KstarAxialWidths " << ix 
		    << " " << _kstar1width[ix]/GeV << "\n";
    else     os << "insert " << name() << ":KstarAxialWidths " << ix 
		    << " " << _kstar1width[ix]/GeV << "\n";
  }
  for(unsigned int ix=0;ix<_kstar2mass.size();++ix) {
    if(ix<3) os << "newdef " << name() << ":KstarVectorMasses " << ix 
		<< " " << _kstar2mass[ix]/GeV << "\n";
    else     os << "insert " << name() << ": KstarVectorMasses" << ix 
		<< " " << _kstar2mass[ix]/GeV << "\n";
  }
  for(unsigned int ix=0;ix<_kstar2width.size();++ix) {
    if(ix<3) os << "newdef " << name() << ":KstarVectorWidths " << ix 
		    << " " << _kstar2width[ix]/GeV << "\n";
    else     os << "insert " << name() << ":KstarVectorWidths " << ix 
		    << " " << _kstar2width[ix]/GeV << "\n";
  }
  for(unsigned int ix=0;ix<_k1wgta.size();++ix) {
    if(ix<2) os << "newdef " << name() << ":K1WeightKStarPi " << ix
		<< " " << _k1wgta[ix] << "\n";
    else     os << "insert " << name() << ":K1WeightKStarPi " << ix
		<< " " << _k1wgta[ix] << "\n";
  }
  for(unsigned int ix=0;ix<_k1wgtb.size();++ix) {
    if(ix<2) os << "newdef " << name() << ":K1WeightRhoK " << ix
		<< " " << _k1wgtb[ix] << "\n";
    else     os << "insert " << name() << ":K1WeightRhoK " << ix
		<< " " << _k1wgtb[ix] << "\n";
  }
  ThreeMesonCurrentBase::dataBaseOutput(os,false,false);
  if(header) os << "\n\" where BINARY ThePEGName=\"" 
		<< fullName() << "\";" << endl;
}  

void KaonThreeMesonCurrent::doinit() {
  ThreeMesonCurrentBase::doinit();
  // the particles we will use a lot
  tPDPtr a1(getParticleData(ParticleID::a_1minus)),
    pi0(getParticleData(ParticleID::pi0)),
    piplus(getParticleData(ParticleID::piplus)),
    piminus(getParticleData(ParticleID::piminus));
  tPDPtr k1[2]={getParticleData(ParticleID::K_1minus),
		getParticleData(ParticleID::Kstar_1minus)};
  // masses for the running widths
  _mpi=piplus->mass();
  _mK=getParticleData(ParticleID::K0)->mass();
  // the charged rho resonances
  tPDPtr rhoc[3]={getParticleData(-213),getParticleData(-100213),
 		  getParticleData(-30213)};
  // the charged K* resonances
  tPDPtr Kstarc[3]={getParticleData(-323),getParticleData(-100323),
 		    getParticleData(-30323)};
  if(!_a1parameters) {
    _a1mass=a1->mass();
    _a1width=a1->width();
  }
  // mass and width of the k_1
  if(!_k1parameters) {
    for(unsigned int ix=0;ix<2;++ix) {
      _k1mass[ix]  = k1[ix]->mass();
      _k1width[ix] = k1[ix]->width();
    }
  }
  // initialise the a_1 running width calculation
  inita1Width(-1);
  // rho parameters in the base classs
  tcPDPtr temp;
  unsigned int ix;
  if(_rhoparameters&&_rho1mass.size()<3) {
    ix = _rho1mass.size();
    _rho1mass.resize(3);
    _rho1width.resize(3);
    for(;ix<3;++ix) {
      if(rhoc[ix]) {
	_rho1mass [ix]=rhoc[ix]->mass();
	_rho1width[ix]=rhoc[ix]->width();
      }
    }
  }
  else if(!_rhoparameters) {
    _rho1mass.resize(3);_rho1width.resize(3);
    for(ix=0;ix<3;++ix) {
      if(rhoc[ix]) {
	_rho1mass[ix]=rhoc[ix]->mass();
	_rho1width[ix]=rhoc[ix]->width();
      }
    }
  }
  // K star parameters in the base class
  if(_kstarparameters&&_kstar1mass.size()<3) {
    ix = _kstar1mass.size();
    _kstar1mass.resize(3);_kstar1width.resize(3);
    for(;ix<3;++ix) {
      if(Kstarc[ix]) {
	_kstar1mass[ix]=Kstarc[ix]->mass();
	_kstar1width[ix]=Kstarc[ix]->width();
      }
    }
  }
  else if(!_kstarparameters) {
    _kstar1mass.resize(3);_kstar1width.resize(3);
    for(ix=0;ix<3;++ix) {
      if(Kstarc[ix]) {
	_kstar1mass[ix]=Kstarc[ix]->mass();
	_kstar1width[ix]=Kstarc[ix]->width();
      }
    }
  }
  // rho parameters here
  if(_rhoparameters&&_rho2mass.size()<3) {
    ix = _rho2mass.size();
    _rho2mass.resize(3);_rho2width.resize(3);
    for(;ix<3;++ix) {
      if(rhoc[ix]) {
	_rho2mass[ix]=rhoc[ix]->mass();
	_rho2width[ix]=rhoc[ix]->width();
      }
    }
  }
  else if(!_rhoparameters) {
    _rho2mass.resize(3);_rho2width.resize(3);
    for(ix=0;ix<3;++ix) {
      if(rhoc[ix]) {
	_rho2mass[ix]=rhoc[ix]->mass();
	_rho2width[ix]=rhoc[ix]->width();
      }
    }
  }
  // Kstar parameters here
  if(_kstarparameters&&_kstar2width.size()<3) {
    ix = _kstar2mass.size();
    _kstar2mass.resize(3);_kstar2width.resize(3);
    for(;ix<3;++ix) {
      if(Kstarc[ix]) {
	_kstar2mass[ix]=Kstarc[ix]->mass();
	_kstar2width[ix]=Kstarc[ix]->width();
      }
    }
  }
  else if(!_kstarparameters) {
    _kstar2mass.resize(3);_kstar2width.resize(3);
    for(ix=0;ix<3;++ix) {
      if(Kstarc[ix]) {
	_kstar2mass[ix]=Kstarc[ix]->mass();
	_kstar2width[ix]=Kstarc[ix]->width();
      }
    }
  }
  inita1Width(0);
}

KaonThreeMesonCurrent::FormFactors
KaonThreeMesonCurrent::calculateFormFactors(const int ichan,const int imode,
						 Energy2 q2,Energy2 s1,
						 Energy2 s2,Energy2 s3) const {
  useMe();
  Complex F1, F2, F5;
  // calculate the K- pi - K+ factor
  if(imode==2) {
    Complex a1fact(a1BreitWigner(q2)*sqrt(2.)/3.);
    if(ichan<0) {
      F1 = -a1fact*TKstar1(s1,-1);
      F2 =  a1fact*Trho1(s2,-1);
      F5 = Trho2(q2,-1)*TOmegaKStar(s1,s2,-1)*sqrt(2.);
    }
    else if(ichan%5==0) F1 = -a1fact*TKstar1(s1,    ichan/5);
    else if(ichan%5==1) F2 =  a1fact*Trho1(  s2,(ichan-1)/5);
    else if(ichan%5>=2) F5 = Trho2(q2,ichan/5)*TOmegaKStar(s1,s2,2*((ichan-2)%5))
      *sqrt(2.);
  }
  // calculate the K0 pi- K0bar
  else if(imode==3) {
    Complex a1fact(a1BreitWigner(q2)*sqrt(2.)/3.);
    if(ichan<0) {
      F1 =-a1fact*TKstar1(s1,-1);
      F2 = a1fact*Trho1  (s2,-1);
      F5 =-Trho2(q2,-1)*TOmegaKStar(s1,s2,-1)*sqrt(2.);
    }
    else if(ichan%5==0) F1 = -a1fact*TKstar1(s1,    ichan/5);
    else if(ichan%5==1) F2 =  a1fact*Trho1  (s2,(ichan-1)/5);
    else if(ichan%5>=2) F5 = -Trho2(q2,ichan/5)*TOmegaKStar(s1,s2,2*((ichan-2)%5))
      *sqrt(2.);
  }
  // calculate the K- pi0 k0
  else if(imode==4) {
    Complex a1fact(a1BreitWigner(q2)/3.);
    if(ichan<0) {
      F1 =  a1fact*( TKstar1(s1,-1)-TKstar1(s3,-1));
      F2 = -a1fact*(2.*Trho1(s2,-1)+TKstar1(s3,-1));
      F5 = Trho2(q2,-1)*(TKstar1(s3,-1)-TKstar1(s1,-1))/(1.+_omegaKstarwgt)/sqrt(2.);
    }
    else if(ichan%9==0) F1 =  a1fact*TKstar1(s1,ichan/9)/3.;
    else if(ichan%9==1) {
      F1 = +a1fact*TKstar1(s3,(ichan-1)/9)/3.;
      F2 = -a1fact*TKstar1(s3,(ichan-1)/9)/3.;
    }
    else if(ichan%9==2) F2 = -a1fact*2.*Trho1(s2,(ichan-2)/9)/3.;
    else if(ichan%9<6)  F5 =-Trho2(q2,ichan/9)*TKstar1(s1,(ichan-3)%9)
      /(1.+_omegaKstarwgt)/sqrt(2.);
    else                F5 = Trho2(q2,ichan/9)*TKstar1(s3,(ichan-6)%9)
      /(1.+_omegaKstarwgt)/sqrt(2.);
  }
  // calculate the pi0 pi0 K-
  else if(imode==5) {
    if(ichan<0) {
      Complex K1fact(TK1(q2,0,-1)/6.);
      F1 = K1fact*TKstar1(s1,-1);
      F2 =-K1fact*TKstar1(s2,-1);
      F5 =-0.25*TKstar2(q2,-1)*(TKstar1(s1,-1)-TKstar1(s2,-1));
    }
    else if(ichan%10==0) F1=  TK1(q2,0,0)/6.*TKstar1(s1,ichan/10);
    else if(ichan%10==1) F2= -TK1(q2,0,0)/6.*TKstar1(s2,ichan/10);
    else if(ichan%10==2) F1=  TK1(q2,0,1)/6.*TKstar1(s1,ichan/10);
    else if(ichan%10==3) F2= -TK1(q2,0,1)/6.*TKstar1(s2,ichan/10);
    else if(ichan%10<7 ) F5 =-sqrt(2.)/4*TKstar2(q2,ichan/10)*TKstar1(s1,(ichan-4)%10);
    else                 F5 = sqrt(2.)/4*TKstar2(q2,ichan/10)*TKstar1(s2,(ichan-7)%10);
  }
  // calculate the K- pi- pi+
  else if(imode==6) {
    double fact=sqrt(2.)/3.;
    if(ichan<0) {
      F1 = -fact*TK1(q2,1,-1)*Trho1(s1,-1);
      F2 =  fact*TK1(q2,0,-1)*TKstar1(s2,-1);
      F5 = -sqrt(0.5)*TKstar2(q2,-1)*(Trho1(s1,-1)+TKstar1(s2,-1));
    }
    else if(ichan%10==0) F1 = -fact*TK1(q2,1,0)*Trho1  (s1,ichan/10);
    else if(ichan%10==1) F2 =  fact*TK1(q2,0,0)*TKstar1(s2,ichan/10);
    else if(ichan%10==2) F1 = -fact*TK1(q2,1,1)*Trho1(  s1,ichan/10);
    else if(ichan%10==3) F2 =  fact*TK1(q2,0,1)*TKstar1(s2,ichan/10);
    else if(ichan%10<7)  F5 = -sqrt(0.5)*TKstar2(q2,ichan/10)*Trho1(  s1,(ichan-4)%10);
    else                 F5 = -sqrt(0.5)*TKstar2(q2,ichan/10)*TKstar1(s2,(ichan-7)%10);
  }
  // calculate the pi- K0bar pi0
  else if(imode==7) {
    if(ichan<0) {
      Complex K1facta(TK1(q2,0,-1)),K1factb(TK1(q2,1,-1));
      F1 = K1facta*(TKstar1(s1,-1)-TKstar1(s3,-1))/3.;
      F2 =-(2.*K1factb*Trho1(s2,-1)+K1facta*TKstar1(s3,-1))/3.;
      F5 = -0.5*TKstar2(q2,-1)*(2.*Trho1(s2,-1)+TKstar1(s1,-1)+TKstar1(s3,-1));
    }
    else if(ichan%15==0) F2 =-2.*TK1(q2,0,0)*Trho1  (s2,ichan/15)/3.;
    else if(ichan%15==1) F1 =    TK1(q2,1,0)*TKstar1(s1,ichan/15)/3.;
    else if(ichan%15==2) {
      F1 =-TK1(q2,1,0)*TKstar1(s3,ichan/15)/3.;
      F2 =-TK1(q2,1,0)*TKstar1(s3,ichan/15)/3.;
    }
    else if(ichan%15==3) F2 =-2.*TK1(q2,0,1)*Trho1  (s2,ichan/15)/3.;
    else if(ichan%15==4) F1 =    TK1(q2,1,1)*TKstar1(s1,ichan/15)/3.;
    else if(ichan%15==5) {
      F1 =-TK1(q2,1,1)*TKstar1(s3,ichan/15)/3.;
      F2 =-TK1(q2,1,1)*TKstar1(s3,ichan/15)/3.;
    }
    else if(ichan%15<9 ) F5 = -0.5*TKstar2(q2,ichan/15)*TKstar1(s1,(ichan- 6)%15);
    else if(ichan%15<12) F5 = -    TKstar2(q2,ichan/15)*Trho1  (s2,(ichan- 9)%15);
    else                 F5 = -0.5*TKstar2(q2,ichan/15)*TKstar1(s3,(ichan-12)%15);
  }
  // calculate the K_S0 pi- K_S0 or K_L0 pi- K_L0
  else if(imode==9||imode==10) {
    Complex a1fact(a1BreitWigner(q2)/6.);
    if(ichan<0) {
      F1 = a1fact*(TKstar1(s1,-1)+TKstar1(s3,-1));
      F2 = a1fact*TKstar1(s3,-1);
      F5 = 0.5*Trho2(q2,-1)*(TOmegaKStar(s1,s2,-1)-TOmegaKStar(s3,s2,-1));
    }
    else if(ichan%8==0) F1=a1fact*TKstar1(s1,ichan/8);
    else if(ichan%8==1) {
      F1 = a1fact*TKstar1(s3,ichan/8);
      F2 = a1fact*TKstar1(s3,ichan/8);
    }
    else if(ichan%8<5 ) F5 = -Trho2(q2,ichan/8)*TKstar1(s1,(ichan-2)%8)
      /(1.+_omegaKstarwgt)/2.;
    else                F5 =  Trho2(q2,ichan/8)*TKstar1(s3,(ichan-5)%8)
      /(1.+_omegaKstarwgt)/2.;
  }
  else if(imode==11) {
    Complex a1fact(a1BreitWigner(q2)/3./sqrt(2.));
    if(ichan<0) {
      F1 = -a1fact*(TKstar1(s1,-1)-TKstar1(s3,-1));
      F2 =  a1fact*(2.*Trho1(s2,-1)+TKstar1(s3,-1));
      F5 = -Trho2(q2,-1)*(TOmegaKStar(s1,s2,-1)+TOmegaKStar(s3,s2,-1))/sqrt(2.);
    }
    else if(ichan%9==0) F1 =-   a1fact*TKstar1(s1,ichan/9);
    else if(ichan%9==1) {
      F1 = a1fact*TKstar1(s3,ichan/9);
      F2 = a1fact*TKstar1(s3,ichan/9);
    }
    else if(ichan%9==2) F2 = 2.*a1fact*Trho1(  s2,ichan/9);
    else if(ichan%9<6 ) F5 = -sqrt(0.5)*TKstar2(q2,ichan/9)*
      TOmegaKStar(s1,s2,2*((ichan-3)%9))/sqrt(2.);
    else                F5 = -sqrt(0.5)*TKstar2(q2,ichan/9)*
      TOmegaKStar(s3,s2,2*((ichan-6)%9))/sqrt(2.);
  }
  return FormFactors(F1 / _fpi,
		     F2 / _fpi,
		     InvEnergy(),
		     InvEnergy(),
		     -F5 / sqr(Constants::twopi) / pow<3,1>(_fpi));
}

void KaonThreeMesonCurrent::doinitrun() {
  // set up the running a_1 width
  inita1Width(0);
  ThreeMesonCurrentBase::doinitrun();
}

void KaonThreeMesonCurrent::doupdate() {
  ThreeMesonCurrentBase::doupdate();
  // update running width if needed
  if ( !touched() ) return;
  if(_maxmass!=_maxcalc) inita1Width(-1);
}

double KaonThreeMesonCurrent::
threeBodyMatrixElement(const int       , const Energy2 q2,
		       const Energy2 s3, const Energy2 s2, 
		       const Energy2 s1, const Energy    , 
		       const Energy    , const Energy    ) const {
  Energy2 mpi2(sqr(_mpi));
  Complex propb(Trho1(s1,-1)),propa(Trho1(s2,-1)); 
  // the matrix element
  Energy2 output(ZERO); 
  // first resonance
  output+= ((s1-4.*mpi2)+0.25*(s3-s2)*(s3-s2)/q2)*real(propb*conj(propb)); 
  // second resonance
  output+= ((s2-4.*mpi2)+0.25*(s3-s1)*(s3-s1)/q2)*real(propa*conj(propa)); 
  // the interference term 
  output+= (0.5*q2-s3-0.5*mpi2+0.25*(s3-s2)*(s3-s1)/q2)*real(propa*conj(propb)+
							     propb*conj(propa)); 
  return output / sqr(_rho1mass[0]);
}

Complex KaonThreeMesonCurrent::Trho1(Energy2 q2,int ires) const {
  Complex output(0.);
  double norm(0.);
  for(unsigned int ix=0,N=_rho1wgts.size();ix<N;++ix) norm+=_rho1wgts[ix];
  if(ires<0) {
    for(unsigned int ix=0,N=_rho1wgts.size();ix<N;++ix) {
      output+=_rho1wgts[ix]*BWrho1(q2,ix);
    }
  }
  else {
    unsigned int temp(ires);
    if(temp<_rho1wgts.size()) output=_rho1wgts[temp]*BWrho1(q2,temp);
  }
  return output/norm;
}
  
Complex KaonThreeMesonCurrent::Trho2(Energy2 q2,int ires) const {
  Complex output(0.);
  double norm(0.);
  for(unsigned int ix=0,N=_rho2wgts.size();ix<N;++ix) norm+=_rho2wgts[ix];
  if(ires<0) {
    for(unsigned int ix=0,N=_rho2wgts.size();ix<N;++ix) {
      output+=_rho2wgts[ix]*BWrho2(q2,ix);
    }
  }
  else {
    unsigned int temp(ires);
    if(temp<_rho2wgts.size()) output=_rho2wgts[temp]*BWrho2(q2,temp);
  }
  return output/norm;
}
  
Complex KaonThreeMesonCurrent::TKstar1(Energy2 q2,int ires) const  {
  Complex output(0.);
  double norm(0.);
  for(unsigned int ix=0,N=_kstar1wgts.size();ix<N;++ix) norm+=_kstar1wgts[ix];
  if(ires<0) {
    for(unsigned int ix=0,N=_kstar1wgts.size();ix<N;++ix) {
      output+=_kstar1wgts[ix]*BWKstar1(q2,ix);
    }
  }
  else {
    unsigned int temp(ires);
    if(temp<_kstar1wgts.size()) output=_kstar1wgts[temp]*BWKstar1(q2,temp);
  }
  return output/norm;
}
  
Complex KaonThreeMesonCurrent::TKstar2(Energy2 q2,int ires) const {
  Complex output(0.);
  double norm(0.);
  for(unsigned int ix=0,N=_kstar2wgts.size();ix<N;++ix) norm+=_kstar2wgts[ix];
  if(ires<0) {
    for(unsigned int ix=0,N=_kstar2wgts.size();ix<N;++ix) {
      output+=_kstar2wgts[ix]*BWKstar2(q2,ix);
    }
  }
  else {
    unsigned int temp(ires);
    if(temp<_kstar2wgts.size()) output=_kstar2wgts[temp]*BWKstar2(q2,temp);
  }
  return output/norm;
}

Complex KaonThreeMesonCurrent::BWrho1(Energy2 q2, unsigned int ires) const {
  if(ires>=_rho1mass.size()) return 0.;
  Energy mass  = _rho1mass [ires];
  Energy width = _rho1width[ires];
  Energy q=sqrt(q2);
  Energy pcm0 = Kinematics::pstarTwoBodyDecay(mass,_mpi,_mpi);
  Energy pcm  = q<=2.*_mpi ? ZERO : Kinematics::pstarTwoBodyDecay(q,_mpi,_mpi);
  double ratio = Math::powi(pcm/pcm0, 3);
  Energy gam(width*mass*ratio/q);
  return sqr(mass)/(sqr(mass)-q2-Complex(0.,1.)*mass*gam);
} 

Complex KaonThreeMesonCurrent::BWrho2(Energy2 q2, unsigned int ires) const {
  if(ires>=_rho2mass.size()) return 0.;
  Energy mass  = _rho2mass [ires];
  Energy width = _rho2width[ires];
  Energy q=sqrt(q2);
  Energy pcm0 = Kinematics::pstarTwoBodyDecay(mass,_mpi,_mpi);
  Energy pcm  = q<=2.*_mpi ? ZERO : Kinematics::pstarTwoBodyDecay(q,_mpi,_mpi);
  double ratio(pcm/pcm0);ratio*=ratio*ratio;
  Energy gam(width*mass*ratio/q);
  return sqr(mass)/(sqr(mass)-q2-Complex(0.,1.)*mass*gam);
}
  
Complex KaonThreeMesonCurrent::BWKstar1(Energy2 q2, unsigned int ires) const {
  if(ires>=_kstar1mass.size()) return 0.;
  Energy mass  = _kstar1mass [ires];
  Energy width = _kstar1width[ires];
  Energy q=sqrt(q2);
  Energy pcm0 = Kinematics::pstarTwoBodyDecay(mass,_mK,_mpi);
  Energy pcm  = q<=_mpi+_mK ? ZERO : Kinematics::pstarTwoBodyDecay(q,_mK,_mpi);
  double ratio(pcm/pcm0);ratio*=ratio*ratio;
  Energy gam(width*mass*ratio/q);
  return sqr(mass)/(sqr(mass)-q2-Complex(0.,1.)*mass*gam);
}

Complex KaonThreeMesonCurrent::BWKstar2(Energy2 q2, unsigned int ires) const  {
  if(ires>=_kstar2mass.size()) return 0.;
  Energy mass  = _kstar2mass [ires];
  Energy width = _kstar2width[ires];
  Energy q=sqrt(q2);
  Energy pcm0 = Kinematics::pstarTwoBodyDecay(mass,_mK,_mpi);
  Energy pcm  = q<=_mpi+_mK ? ZERO : Kinematics::pstarTwoBodyDecay(q,_mK,_mpi);
  double ratio(pcm/pcm0);ratio*=ratio*ratio;
  Energy gam(width*mass*ratio/q);
  return sqr(mass)/(sqr(mass)-q2-Complex(0.,1.)*mass*gam);
}

Complex KaonThreeMesonCurrent::a1BreitWigner(Energy2 q2) const {
  Complex ii(0.,1.);
  Energy2 m2(_a1mass*_a1mass);
  Energy  q(sqrt(q2));
  return m2/(m2-q2-ii*q*a1Width(q2));
}
  
Complex KaonThreeMesonCurrent::TK1(Energy2 q2,unsigned int iopt,int ires) const {
  Complex denom(0),num(0.);
  if(iopt==0) {
    for(unsigned int ix=0;ix<_k1wgta.size();++ix) denom+=_k1wgta[ix];
    if(ires==-1) {
      for(unsigned int ix=0;ix<_k1wgta.size();++ix) 
	num+=_k1wgta[ix]*K1BreitWigner(q2,ix);
    }
    else {	
      num+=_k1wgta[ires]*K1BreitWigner(q2,ires);
    }
  }
  else if(iopt==1) {
    for(unsigned int ix=0;ix<_k1wgtb.size();++ix) denom+=_k1wgtb[ix];
    if(ires==-1) {
      for(unsigned int ix=0;ix<_k1wgtb.size();++ix) 
	num+=_k1wgtb[ix]*K1BreitWigner(q2,ix);
    }
    else {	
      num+=_k1wgtb[ires]*K1BreitWigner(q2,ires);
    }
  }
  else {
    return 0.;
  }
  return num/denom;
}

Complex KaonThreeMesonCurrent::K1BreitWigner(Energy2 q2,unsigned int ires) const {
  if(ires>=_k1mass.size()) return 0.;
  Energy2 m2=sqr(_k1mass[ires]),mg=_k1mass[ires]*_k1width[ires];
  return (-m2+Complex(0.,1.)*mg)/(q2-m2+Complex(0.,1.)*mg);
}

Energy KaonThreeMesonCurrent::a1Width(Energy2 q2) const {
  if(!_a1opt) return _a1mass*_a1width*g(q2)/g(_a1mass*_a1mass)/sqrt(q2);
  else        return (*_a1runinter)(q2);
}
  
double KaonThreeMesonCurrent::g(Energy2 q2) const {
  double output;
  if(q2<9.*_mpi*_mpi) {
    output=0.;
  }
  else if(q2<sqr(_rho1mass[0]+_mpi)) {
    double diff=(q2-9.*_mpi*_mpi)/GeV2;
      
    output=4.1*sqr(diff)*diff*(1.-3.3*diff+5.8*sqr(diff));
  }
  else {
    double ratio = q2/GeV2;
    output = ratio*(1.623+10.38/ratio-9.32/sqr(ratio)+0.65/(ratio*sqr(ratio)));
  }
  return output;
}
  
Complex KaonThreeMesonCurrent::Tomega(Energy2 q2, int ires) const {
  double denom=(1.+_epsomega);
  Complex num(0.);
  if(ires<0) num=OmegaPhiBreitWigner(q2,0)+_epsomega*OmegaPhiBreitWigner(q2,1);
  else if(ires==0) num=OmegaPhiBreitWigner(q2,0);
  else             num=OmegaPhiBreitWigner(q2,1);
  return num/denom;
}

Complex KaonThreeMesonCurrent::OmegaPhiBreitWigner(Energy2 q2, unsigned int ires) const {
  Energy2 m2,mg;
  if(ires==0) {
    m2=sqr(_omegamass);
    mg=_omegamass*_omegawidth;
  }
  else {
    m2=sqr(_phimass);
    mg=_phimass*_phiwidth;
  }
  return (-m2+Complex(0.,1.)*mg)/(q2-m2+Complex(0.,1.)*mg);
}

Complex KaonThreeMesonCurrent::TOmegaKStar(Energy2 s1,Energy2 s2,int ires) const {
  Complex output;
  if(ires<0)         output = _omegaKstarwgt*TKstar1(s1,-1)+Tomega(s2,-1);
  else if(ires%2==0) output = _omegaKstarwgt*TKstar1(s1,ires/2);
  else if(ires%2==1) output = Tomega(s2,ires/2);
  return output/(1.+_omegaKstarwgt);
}


